@@ -1,8 +1,8 @@
 /*
  *      pluginsymbols.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2008 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2008 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2008-2010 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2008-2010 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -19,7 +19,7 @@
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
  *      MA 02110-1301, USA.
  *
- * $Id: pluginsymbols.c 3090 2008-10-14 17:32:38Z ntrel $
+ * $Id: pluginsymbols.c 4552 2010-01-25 15:53:54Z ntrel $
  */
 
 /* Note: this file is for Doxygen only. */
@@ -55,10 +55,10 @@ const GeanyPlugin *geany_plugin;
 const GeanyData *geany_data;
 
 /** Geany owned function pointers, split into groups.
- * Example: @c geany_functions->p_document->new_file(NULL, NULL, NULL);
- *
- * Note: Usually plugins would use the pluginmacros.h file and just call:
- * @c p_document->new_file(NULL, NULL, NULL); */
+ * Example: @code #include "geanyfunctions.h"
+ * ...
+ * document_new_file(NULL, NULL, NULL); @endcode
+ * This is equivalent of @c geany_functions->p_document->document_new_file(NULL, NULL, NULL); */
 const GeanyFunctions *geany_functions;
 
 /** @deprecated Use @ref ui_add_document_sensitive() instead.
@@ -66,28 +66,33 @@ const GeanyFunctions *geany_functions;
 PluginFields *plugin_fields;
 
 /** An array for connecting GeanyObject events, which should be terminated with
- * @c {NULL, NULL, FALSE, NULL}. See @link signals Signal documentation @endlink. */
+ * @c {NULL, NULL, FALSE, NULL}. See @link signals Signal documentation @endlink.
+ * @see plugin_signal_connect(). */
 PluginCallback plugin_callbacks[];
 
-/** Most plugins should use the PLUGIN_KEY_GROUP() macro to define it. However,
- * its fields are not read until after plugin_init() is called for the plugin, so it
- * is possible to setup a variable number of keybindings, e.g. based on the
- * plugin's configuration file settings.
- * - The @c name field must not be empty or match Geany's default group name.
- * - The @c label field is set by Geany after plugin_init() is called to the name of the
- * plugin.
- * @note This is a single element array for implementation reasons,
- * but you can treat it like a pointer. */
-KeyBindingGroup plugin_key_group[1];
+/** Plugins must use the PLUGIN_KEY_GROUP() macro to define it.
+ * To setup a variable number of keybindings, e.g. based on the
+ * plugin's configuration file settings, use plugin_set_key_group() instead. */
+KeyBindingGroup *plugin_key_group;
 
 
-/** Called before showing the plugin preferences dialog to let the user set some basic
- * plugin configuration options. Can be omitted when not needed.
+/** Called before showing the plugin preferences dialog for multiple plugins.
+ * Can be omitted when not needed.
+ * The dialog will show all plugins that support this symbol together.
  * @param dialog The plugin preferences dialog widget - this should only be used to
  * connect the @c "response" signal. If settings should be read from the dialog, the
  * reponse will be either @c GTK_RESPONSE_OK or @c GTK_RESPONSE_APPLY.
- * @return A container widget holding preference widgets. */
-GtkWidget* plugin_configure(GtkDialog *dialog);
+ * @return A container widget holding preference widgets.
+ * @see plugin_configure_single(). */
+GtkWidget *plugin_configure(GtkDialog *dialog);
+
+/** Called when a plugin should show a preferences dialog, if plugin_configure() has not been
+ * implemented.
+ * @warning It's better to implement plugin_configure() instead, but this is simpler.
+ * This does not integrate as well with the multiple-plugin dialog.
+ * @param parent Pass this as the parent widget if showing a dialog.
+ * @see plugin_configure(). */
+void plugin_configure_single(GtkWidget *parent);
 
 /** Called after loading the plugin.
  * @param data The same as #geany_data. */
@@ -97,3 +102,9 @@ void plugin_init(GeanyData *data);
  * everything done in plugin_init() - e.g. destroy menu items, free memory. */
 void plugin_cleanup();
 
+/** Called whenever the plugin should show its documentation (if any). This may open a dialog,
+ * a browser with a website or a local installed HTML help file(see utils_start_browser())
+ * or something else.
+ * Can be omitted when not needed. */
+void plugin_help();
+
