@@ -2,8 +2,8 @@
  *      classbuilder.c - this file is part of Geany, a fast and lightweight IDE
  *
  *      Copyright 2007 Alexander Rodin <rodin(dot)alexander(at)gmail(dot)com>
- *      Copyright 2007-2009 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2007-2009 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2007-2010 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2007-2010 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -20,14 +20,13 @@
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
  *      MA 02110-1301, USA.
  *
- * $Id: classbuilder.c 3966 2009-07-14 15:06:20Z ntrel $
+ * $Id: classbuilder.c 4944 2010-05-22 19:39:38Z eht16 $
  */
 
 /* Class Builder - creates source files containing a new class interface and definition. */
 
 #include "geanyplugin.h"
 
-
 GeanyData		*geany_data;
 GeanyFunctions	*geany_functions;
 
@@ -35,7 +34,7 @@ GeanyFunctions	*geany_functions;
 PLUGIN_VERSION_CHECK(GEANY_API_VERSION)
 
 PLUGIN_SET_INFO(_("Class Builder"), _("Creates source files for new class types."), VERSION,
-	"Alexander Rodin")
+	"Alexander Rodin, Ondrej Donek, the Geany developer team")
 
 
 static GtkWidget *main_menu_item = NULL;
@@ -44,7 +43,8 @@ static GtkWidget *main_menu_item = NULL;
 enum
 {
 	GEANY_CLASS_TYPE_CPP,
-	GEANY_CLASS_TYPE_GTK
+	GEANY_CLASS_TYPE_GTK,
+	GEANY_CLASS_TYPE_PHP
 };
 
 typedef struct _ClassInfo	ClassInfo;
@@ -52,6 +52,9 @@ typedef struct _ClassInfo	ClassInfo;
 struct _ClassInfo
 {
 	gint type;
+	gchar *namespace;
+	gchar *namespace_up;
+	gchar *namespace_low;
 	gchar *class_name;
 	gchar *class_name_up;
 	gchar *class_name_low;
@@ -67,6 +70,11 @@ struct _ClassInfo
 	gchar *constructor_impl;
 	gchar *destructor_impl;
 	gchar *gtk_destructor_registration;
+	/* These are needed only for PHP classes */
+	gchar *namespace_decl;
+	gchar *implements_decl;
+	gchar *abstract_decl;
+	gchar *singleton_impl;
 };
 
 typedef struct _CreateClassDialog
@@ -83,9 +91,15 @@ typedef struct _CreateClassDialog
 	GtkWidget *create_constructor_box;
 	GtkWidget *create_destructor_box;
 	GtkWidget *gtk_constructor_type_entry;
+	/* These are needed only for PHP classes */
+	GtkWidget *class_namespace_entry;
+	GtkWidget *class_implements_entry;
+	GtkWidget *create_isabstract_box;
+	GtkWidget *create_issingleton_box;
 } CreateClassDialog;
 
 
+/* TODO make these templates configurable */
 static const gchar templates_cpp_class_header[] = "{fileheader}\n\n\
 #ifndef {header_guard}\n\
 #define {header_guard}\n\
@@ -115,34 +129,39 @@ static const gchar templates_gtk_class_header[] = "{fileheader}\n\n\
 #define __{header_guard}__\n\
 {base_include}\n\
 G_BEGIN_DECLS\n\
+\n\n\
+#define {namespace_up}TYPE_{class_name_up}				({namespace_low}{class_name_low}_get_type())\n\
+#define {namespace_up}{class_name_up}(obj)				(G_TYPE_CHECK_INSTANCE_CAST((obj),\\\n\
+			{namespace_up}TYPE_{class_name_up}, {namespace}{class_name}))\n\
+#define {namespace_up}{class_name_up}_CLASS(klass)		(G_TYPE_CHECK_CLASS_CAST((klass),\\\n\
+			{namespace_up}TYPE_{class_name_up}, {namespace}{class_name}Class))\n\
+#define {namespace_up}IS_{class_name_up}(obj)			(G_TYPE_CHECK_INSTANCE_TYPE((obj),\\\n\
+			{namespace_up}TYPE_{class_name_up}))\n\
+#define {namespace_up}IS_{class_name_up}_CLASS(klass)	(G_TYPE_CHECK_CLASS_TYPE((klass),\\\n\
+			{namespace_up}TYPE_{class_name_up}))\n\
+#define {namespace_up}{class_name_up}_GET_CLASS(obj)	(G_TYPE_INSTANCE_GET_CLASS((obj),\\\n\
+			{namespace_up}TYPE_{class_name_up}, {namespace}{class_name}Class))\n\
 \n\
-#define {class_name_up}_TYPE				({class_name_low}_get_type())\n\
-#define {class_name_up}(obj)				(G_TYPE_CHECK_INSTANCE_CAST((obj),\\\n\
-			{class_name_up}_TYPE, {class_name}))\n\
-#define {class_name_up}_CLASS(klass)		(G_TYPE_CHECK_CLASS_CAST((klass),\\\n\
-			{class_name_up}_TYPE, {class_name}Class))\n\
-#define IS_{class_name_up}(obj)			(G_TYPE_CHECK_INSTANCE_TYPE((obj),\\\n\
-			{class_name_up}_TYPE))\n\
-#define IS_{class_name_up}_CLASS(klass)	(G_TYPE_CHECK_CLASS_TYPE((klass),\\\n\
-			{class_name_up}_TYPE))\n\
+typedef struct _{namespace}{class_name}			{namespace}{class_name};\n\
+typedef struct _{namespace}{class_name}Class		{namespace}{class_name}Class;\n\
+typedef struct _{namespace}{class_name}Private		{namespace}{class_name}Private;\n\
 \n\
-typedef struct _{class_name}			{class_name};\n\
-typedef struct _{class_name}Class		{class_name}Class;\n\
-\n\
-struct _{class_name}\n\
+struct _{namespace}{class_name}\n\
 {\n\
 	{base_name} parent;\n\
 	/* add your public declarations here */\n\
+	\n\
+	{namespace}{class_name}Private *priv;\n\
 };\n\
 \n\
-struct _{class_name}Class\n\
+struct _{namespace}{class_name}Class\n\
 {\n\
 	{base_name}Class parent_class;\n\
 };\n\
-\n\
-GType		{class_name_low}_get_type		(void);\n\
+\n\n\
+GType		{namespace_low}{class_name_low}_get_type		(void);\n\
 {constructor_decl}\
-\n\
+\n\n\
 G_END_DECLS\n\
 \n\
 #endif /* __{header_guard}__ */\n\
@@ -151,41 +170,51 @@ G_END_DECLS\n\
 static const gchar templates_gtk_class_source[] = "{fileheader}\n\
 #include \"{header}\"\n\
 \n\
-typedef struct _{class_name}Private			{class_name}Private;\n\
-\n\
-#define {class_name_up}_GET_PRIVATE(obj)		(G_TYPE_INSTANCE_GET_PRIVATE((obj),\\\n\
-			{class_name_up}_TYPE, {class_name}Private))\n\
-\n\
-struct _{class_name}Private\n\
+struct _{namespace}{class_name}Private\n\
 {\n\
 	/* add your private declarations here */\n\
 };\n\
 \n\
 {destructor_decl}\
 \n\
-G_DEFINE_TYPE({class_name}, {class_name_low}, {base_gtype});\n\
+G_DEFINE_TYPE({namespace}{class_name}, {namespace_low}{class_name_low}, {base_gtype})\n\
 \n\n\
-static void {class_name_low}_class_init({class_name}Class *klass)\n\
+static void {namespace_low}{class_name_low}_class_init({namespace}{class_name}Class *klass)\n\
 {\n\
 	{gtk_destructor_registration}\n\
-	g_type_class_add_private((gpointer)klass, sizeof({class_name}Private));\n\
+	g_type_class_add_private((gpointer)klass, sizeof({namespace}{class_name}Private));\n\
 }\n\
 \n\
 {destructor_impl}\n\
 \n\
-static void {class_name_low}_init({class_name} *self)\n\
+static void {namespace_low}{class_name_low}_init({namespace}{class_name} *self)\n\
 {\n\
+	self->priv = G_TYPE_INSTANCE_GET_PRIVATE(self,\n\
+		{namespace_up}TYPE_{class_name_up}, {namespace}{class_name}Private);\n\
 	\n\
 }\n\
 \n\
 {constructor_impl}\n\
 ";
 
+static const gchar templates_php_class_source[] = "<?php\n\
+{fileheader}\n\
+{namespace_decl}\n\
+{base_include}\n\
+{abstract_decl}class {class_name}{base_decl}{implements_decl}\n\{\n\
+{singleton_impl}\
+{constructor_impl}\
+{destructor_impl}\n\
+	// ...\n\n\
+}\n\
+";
+
 
 static void cc_dlg_on_set_sensitive_toggled(GtkWidget *toggle_button, GtkWidget *target_widget);
 static void cc_dlg_on_class_name_entry_changed(GtkWidget *entry, CreateClassDialog *cc_dlg);
+static void cc_dlg_on_class_namespace_entry_changed(GtkWidget *entry, CreateClassDialog *cc_dlg);
 static void cc_dlg_on_base_name_entry_changed(GtkWidget *entry, CreateClassDialog *cc_dlg);
-static void cc_dlg_on_create_class(CreateClassDialog *cc_dlg);
+static gboolean create_class(CreateClassDialog *cc_dlg);
 
 
 /* The list must be ended with NULL as an extra check that arg_count is correct. */
@@ -237,6 +266,9 @@ get_template_class_header(ClassInfo *class_info)
 			utils_string_replace_all(template, "{fileheader}", fileheader);
 			utils_string_replace_all(template, "{header_guard}", class_info->header_guard);
 			utils_string_replace_all(template, "{base_include}", class_info->base_include);
+			utils_string_replace_all(template, "{namespace}", class_info->namespace);
+			utils_string_replace_all(template, "{namespace_up}", class_info->namespace_up);
+			utils_string_replace_all(template, "{namespace_low}", class_info->namespace_low);
 			utils_string_replace_all(template, "{class_name}", class_info->class_name);
 			utils_string_replace_all(template, "{class_name_up}", class_info->class_name_up);
 			utils_string_replace_all(template, "{class_name_low}", class_info->class_name_low);
@@ -282,6 +314,9 @@ get_template_class_source(ClassInfo *class_info)
 			template = g_string_new(templates_gtk_class_source);
 			utils_string_replace_all(template, "{fileheader}", fileheader);
 			utils_string_replace_all(template, "{header}", class_info->header);
+			utils_string_replace_all(template, "{namespace}", class_info->namespace);
+			utils_string_replace_all(template, "{namespace_up}", class_info->namespace_up);
+			utils_string_replace_all(template, "{namespace_low}", class_info->namespace_low);
 			utils_string_replace_all(template, "{class_name}", class_info->class_name);
 			utils_string_replace_all(template, "{class_name_up}", class_info->class_name_up);
 			utils_string_replace_all(template, "{class_name_low}", class_info->class_name_low);
@@ -295,6 +330,21 @@ get_template_class_source(ClassInfo *class_info)
 			utils_string_replace_all(template, "{gtk_destructor_registration}",
 					class_info->gtk_destructor_registration);
 			break;
+
+		case GEANY_CLASS_TYPE_PHP:
+			fileheader = templates_get_template_fileheader(GEANY_FILETYPES_PHP, class_info->source);
+			template = g_string_new(templates_php_class_source);
+			utils_string_replace_all(template, "{fileheader}", fileheader);
+			utils_string_replace_all(template, "{namespace_decl}", class_info->namespace_decl);
+			utils_string_replace_all(template, "{base_include}", class_info->base_include);
+			utils_string_replace_all(template, "{abstract_decl}", class_info->abstract_decl);
+			utils_string_replace_all(template, "{class_name}", class_info->class_name);
+			utils_string_replace_all(template, "{base_decl}", class_info->base_decl);
+			utils_string_replace_all(template, "{implements_decl}", class_info->implements_decl);
+			utils_string_replace_all(template, "{constructor_impl}", class_info->constructor_impl);
+			utils_string_replace_all(template, "{destructor_impl}", class_info->destructor_impl);
+			utils_string_replace_all(template, "{singleton_impl}", class_info->singleton_impl);
+			break;
 	}
 
 	g_free(fileheader);
@@ -329,6 +379,26 @@ void show_dialog_create_class(gint type)
 
 	main_box = ui_dialog_vbox_new(GTK_DIALOG(cc_dlg->dialog));
 
+	if (type == GEANY_CLASS_TYPE_PHP || type == GEANY_CLASS_TYPE_GTK)
+	{
+		frame = ui_frame_new_with_alignment(_("Namespace"), &align);
+		gtk_container_add(GTK_CONTAINER(main_box), frame);
+
+		vbox = gtk_vbox_new(FALSE, 10);
+		gtk_container_add(GTK_CONTAINER(align), vbox);
+
+		hbox = gtk_hbox_new(FALSE, 10);
+		gtk_container_add(GTK_CONTAINER(vbox), hbox);
+
+		label = gtk_label_new(_("Namespace:"));
+		gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
+
+		cc_dlg->class_namespace_entry = gtk_entry_new();
+		gtk_box_pack_start(GTK_BOX(hbox), cc_dlg->class_namespace_entry, TRUE, TRUE, 0);
+		g_signal_connect(cc_dlg->class_namespace_entry, "changed",
+				G_CALLBACK(cc_dlg_on_class_namespace_entry_changed), cc_dlg);
+	}
+
 	frame = ui_frame_new_with_alignment(_("Class"), &align);
 	gtk_container_add(GTK_CONTAINER(main_box), frame);
 
@@ -346,14 +416,17 @@ void show_dialog_create_class(gint type)
 	g_signal_connect(cc_dlg->class_name_entry, "changed",
 			G_CALLBACK(cc_dlg_on_class_name_entry_changed), cc_dlg);
 
-	hbox = gtk_hbox_new(FALSE, 10);
-	gtk_container_add(GTK_CONTAINER(vbox), hbox);
+	if (type != GEANY_CLASS_TYPE_PHP)
+	{
+		hbox = gtk_hbox_new(FALSE, 10);
+		gtk_container_add(GTK_CONTAINER(vbox), hbox);
 
-	label = gtk_label_new(_("Header file:"));
-	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
+		label = gtk_label_new(_("Header file:"));
+		gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
 
-	cc_dlg->header_entry = gtk_entry_new();
-	gtk_container_add(GTK_CONTAINER(hbox), cc_dlg->header_entry);
+		cc_dlg->header_entry = gtk_entry_new();
+		gtk_container_add(GTK_CONTAINER(hbox), cc_dlg->header_entry);
+	}
 
 	hbox = gtk_hbox_new(FALSE, 10);
 	gtk_container_add(GTK_CONTAINER(vbox), hbox);
@@ -384,9 +457,12 @@ void show_dialog_create_class(gint type)
 			G_CALLBACK(cc_dlg_on_base_name_entry_changed), (gpointer)cc_dlg);
 
 	hbox = gtk_hbox_new(FALSE, 10);
-	gtk_container_add(GTK_CONTAINER(vbox), hbox);
+ 	gtk_container_add(GTK_CONTAINER(vbox), hbox);
 
-	label = gtk_label_new(_("Base header:"));
+	if (type == GEANY_CLASS_TYPE_PHP)
+		label = gtk_label_new(_("Base source:"));
+	else
+		label = gtk_label_new(_("Base header:"));
 	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
 
 	cc_dlg->base_header_entry = gtk_entry_new();
@@ -394,9 +470,12 @@ void show_dialog_create_class(gint type)
 		gtk_entry_set_text(GTK_ENTRY(cc_dlg->base_header_entry), "glib-object.h");
 	gtk_container_add(GTK_CONTAINER(hbox), cc_dlg->base_header_entry);
 
-	cc_dlg->base_header_global_box = gtk_check_button_new_with_label(_("Global"));
-	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(cc_dlg->base_header_global_box), TRUE);
-	gtk_box_pack_end(GTK_BOX(hbox), cc_dlg->base_header_global_box, FALSE, FALSE, 0);
+	if (type != GEANY_CLASS_TYPE_PHP)
+	{
+		cc_dlg->base_header_global_box = gtk_check_button_new_with_label(_("Global"));
+		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(cc_dlg->base_header_global_box), TRUE);
+		gtk_box_pack_end(GTK_BOX(hbox), cc_dlg->base_header_global_box, FALSE, FALSE, 0);
+	}
 
 	if (type == GEANY_CLASS_TYPE_GTK)
 	{
@@ -411,6 +490,18 @@ void show_dialog_create_class(gint type)
 		gtk_container_add(GTK_CONTAINER(hbox), cc_dlg->base_gtype_entry);
 	}
 
+	if (type == GEANY_CLASS_TYPE_PHP)
+	{
+		hbox = gtk_hbox_new(FALSE, 10);
+		gtk_container_add(GTK_CONTAINER(vbox), hbox);
+
+		label = gtk_label_new(_("Implements:"));
+		gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
+
+		cc_dlg->class_implements_entry = gtk_entry_new();
+		gtk_container_add(GTK_CONTAINER(hbox), cc_dlg->class_implements_entry);
+	}
+
 	frame = ui_frame_new_with_alignment(_("Options"), &align);
 	gtk_container_add(GTK_CONTAINER(main_box), frame);
 
@@ -427,6 +518,18 @@ void show_dialog_create_class(gint type)
 	cc_dlg->create_destructor_box = gtk_check_button_new_with_label(_("Create destructor"));
 	gtk_container_add(GTK_CONTAINER(hbox), cc_dlg->create_destructor_box);
 
+	if (type == GEANY_CLASS_TYPE_PHP)
+	{
+		hbox = gtk_hbox_new(FALSE, 10);
+		gtk_container_add(GTK_CONTAINER(vbox), hbox);
+
+		cc_dlg->create_isabstract_box = gtk_check_button_new_with_label(_("Is abstract"));
+		gtk_container_add(GTK_CONTAINER(hbox), cc_dlg->create_isabstract_box);
+
+		cc_dlg->create_issingleton_box = gtk_check_button_new_with_label(_("Is singleton"));
+		gtk_container_add(GTK_CONTAINER(hbox), cc_dlg->create_issingleton_box);
+	}
+
 	if (type == GEANY_CLASS_TYPE_GTK)
 	{
 		hbox = gtk_hbox_new(FALSE, 10);
@@ -443,11 +546,14 @@ void show_dialog_create_class(gint type)
 	}
 
 	gtk_widget_show_all(cc_dlg->dialog);
-	if (gtk_dialog_run(GTK_DIALOG(cc_dlg->dialog)) == GTK_RESPONSE_OK)
-		cc_dlg_on_create_class(cc_dlg);
-
+	while (gtk_dialog_run(GTK_DIALOG(cc_dlg->dialog)) == GTK_RESPONSE_OK)
+	{
+		if (create_class(cc_dlg))
+			break;
+		else
+			gdk_beep();
+	}
 	gtk_widget_destroy(cc_dlg->dialog);
-/*	g_object_unref(G_OBJECT(cc_dlg->dialog));	*/
 }
 
 
@@ -463,32 +569,71 @@ static void cc_dlg_on_set_sensitive_toggled(GtkWidget *toggle_button, GtkWidget
 }
 
 
-static void cc_dlg_on_class_name_entry_changed(GtkWidget *entry, CreateClassDialog *cc_dlg)
+static void cc_dlg_update_file_names(CreateClassDialog *cc_dlg)
 {
+	gchar *class_name;
 	gchar *class_name_down;
-	gchar *class_header;
-	gchar *class_source;
+	gchar *class_header = NULL;
+	gchar *class_source = NULL;
 
-	g_return_if_fail(entry != NULL);
-	g_return_if_fail(GTK_IS_ENTRY(entry));
 	g_return_if_fail(cc_dlg != NULL);
 
-	class_name_down = g_ascii_strdown(gtk_entry_get_text(GTK_ENTRY(entry)), -1);
-	class_header = g_strconcat(class_name_down, ".h", NULL);
-	if (cc_dlg->class_type == GEANY_CLASS_TYPE_CPP)
-		class_source = g_strconcat(class_name_down, ".cpp", NULL);
-	else
-		class_source = g_strconcat(class_name_down, ".c", NULL);
+	class_name = g_strdup(gtk_entry_get_text(GTK_ENTRY(cc_dlg->class_name_entry)));
+	class_name_down = g_ascii_strdown(class_name, -1);
+	switch (cc_dlg->class_type)
+	{
+		case GEANY_CLASS_TYPE_CPP:
+		{
+			class_header = g_strconcat(class_name_down, ".h", NULL);
+			class_source = g_strconcat(class_name_down, ".cpp", NULL);
+			break;
+		}
+		case GEANY_CLASS_TYPE_GTK:
+		{
+			const gchar *namespace;
+			gchar *namespace_down;
 
-	gtk_entry_set_text(GTK_ENTRY(cc_dlg->header_entry), class_header);
-	gtk_entry_set_text(GTK_ENTRY(cc_dlg->source_entry), class_source);
+			namespace = gtk_entry_get_text(GTK_ENTRY(cc_dlg->class_namespace_entry));
+			namespace_down = g_ascii_strdown(namespace, -1);
+			class_header = g_strconcat(namespace_down, class_name_down, ".h", NULL);
+			class_source = g_strconcat(namespace_down, class_name_down, ".c", NULL);
+			g_free(namespace_down);
+			break;
+		}
+		case GEANY_CLASS_TYPE_PHP:
+		{
+			class_header = NULL;
+			class_source = g_strconcat(class_name, ".php", NULL);
+			break;
+		}
+	}
 
+	if (cc_dlg->header_entry != NULL && class_header != NULL)
+		gtk_entry_set_text(GTK_ENTRY(cc_dlg->header_entry), class_header);
+	if (cc_dlg->source_entry != NULL && class_source != NULL)
+		gtk_entry_set_text(GTK_ENTRY(cc_dlg->source_entry), class_source);
+
+	g_free(class_name);
 	g_free(class_name_down);
 	g_free(class_header);
 	g_free(class_source);
 }
 
 
+static void cc_dlg_on_class_name_entry_changed(GtkWidget *entry, CreateClassDialog *cc_dlg)
+{
+	cc_dlg_update_file_names(cc_dlg);
+}
+
+
+static void cc_dlg_on_class_namespace_entry_changed(GtkWidget *entry, CreateClassDialog *cc_dlg)
+{
+
+	if (cc_dlg->class_type == GEANY_CLASS_TYPE_GTK)
+		cc_dlg_update_file_names(cc_dlg);
+}
+
+
 static gchar* str_case_split(const gchar *str, gchar splitter)
 {
 	GString *result;
@@ -526,9 +671,16 @@ static void cc_dlg_on_base_name_entry_changed(GtkWidget *entry, CreateClassDialo
 		tmp = g_strdup("gtk/gtk.h");
 	else if (utils_str_equal(gtk_entry_get_text(GTK_ENTRY(entry)), "GObject"))
 		tmp = g_strdup("glib-object.h");
+	else if (cc_dlg->class_type == GEANY_CLASS_TYPE_PHP)
+		tmp = g_strconcat(gtk_entry_get_text(GTK_ENTRY(entry)), ".php", NULL);
 	else
 		tmp = g_strconcat(gtk_entry_get_text(GTK_ENTRY(entry)), ".h", NULL);
-	base_header = g_ascii_strdown(tmp, -1);
+
+	if (cc_dlg->class_type == GEANY_CLASS_TYPE_PHP)
+		base_header = g_strdup(tmp);
+	else
+		base_header = g_ascii_strdown(tmp, -1);
+
 	g_free(tmp);
 
 	gtk_entry_set_text(GTK_ENTRY(cc_dlg->base_header_entry), base_header);
@@ -556,17 +708,17 @@ static void cc_dlg_on_base_name_entry_changed(GtkWidget *entry, CreateClassDialo
 }
 
 
-static void cc_dlg_on_create_class(CreateClassDialog *cc_dlg)
+static gboolean create_class(CreateClassDialog *cc_dlg)
 {
 	ClassInfo *class_info;
 	GeanyDocument *doc;
 	gchar *text;
 	gchar *tmp;
 
-	g_return_if_fail(cc_dlg != NULL);
+	g_return_val_if_fail(cc_dlg != NULL, FALSE);
 
 	if (utils_str_equal(gtk_entry_get_text(GTK_ENTRY(cc_dlg->class_name_entry)), ""))
-		return;
+		return FALSE;
 
 	class_info = g_new0(ClassInfo, 1);
 	class_info->type = cc_dlg->class_type;
@@ -577,21 +729,33 @@ static void cc_dlg_on_create_class(CreateClassDialog *cc_dlg)
 	if (! utils_str_equal(gtk_entry_get_text(GTK_ENTRY(cc_dlg->base_name_entry)), ""))
 	{
 		class_info->base_name = g_strdup(gtk_entry_get_text(GTK_ENTRY(cc_dlg->base_name_entry)));
-		class_info->base_include = g_strdup_printf("\n#include %c%s%c\n",
-			gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(cc_dlg->base_header_global_box)) ?
+		if (class_info->type != GEANY_CLASS_TYPE_PHP)
+		{
+			class_info->base_include = g_strdup_printf("\n#include %c%s%c\n",
+				gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(cc_dlg->base_header_global_box)) ?
 				'<' : '\"',
-			gtk_entry_get_text(GTK_ENTRY(cc_dlg->base_header_entry)),
-			gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(cc_dlg->base_header_global_box)) ?
+				gtk_entry_get_text(GTK_ENTRY(cc_dlg->base_header_entry)),
+				gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(cc_dlg->base_header_global_box)) ?
 				'>' : '\"');
+		}
+		else
+		{
+			class_info->base_include = g_strdup_printf("\nrequire_once \"%s\";\n",
+				gtk_entry_get_text(GTK_ENTRY(cc_dlg->base_header_entry)));
+				class_info->base_decl = g_strdup_printf(" extends %s", class_info->base_name);
+		}
 	}
 	else
 	{
 		class_info->base_name = g_strdup("");
 		class_info->base_include = g_strdup("");
 	}
-	class_info->header = g_strdup(gtk_entry_get_text(GTK_ENTRY(cc_dlg->header_entry)));
-	class_info->header_guard = g_ascii_strup(class_info->header, -1);
-	g_strdelimit(class_info->header_guard, ".-", '_');
+	if (cc_dlg->header_entry != NULL)
+	{
+		class_info->header = g_strdup(gtk_entry_get_text(GTK_ENTRY(cc_dlg->header_entry)));
+		class_info->header_guard = g_ascii_strup(class_info->header, -1);
+		g_strdelimit(class_info->header_guard, ".-", '_');
+	}
 	switch (class_info->type)
 	{
 		case GEANY_CLASS_TYPE_CPP:
@@ -635,22 +799,40 @@ static void cc_dlg_on_create_class(CreateClassDialog *cc_dlg)
 		}
 		case GEANY_CLASS_TYPE_GTK:
 		{
+			class_info->namespace = g_strdup(gtk_entry_get_text(GTK_ENTRY(cc_dlg->class_namespace_entry)));
+			if (!NZV(class_info->namespace))
+			{
+				class_info->namespace_up = g_strdup("");
+				class_info->namespace_low = g_strdup("");
+			}
+			else
+			{
+				gchar *tmp_namespace;
+				gchar *tmp_namespace_split;
+
+				tmp_namespace_split = str_case_split(class_info->namespace, '_');
+				tmp_namespace = g_strconcat(tmp_namespace_split, "_", NULL);
+				class_info->namespace_up = g_ascii_strup(tmp_namespace, -1);
+				class_info->namespace_low = g_ascii_strdown(class_info->namespace_up, -1);
+				g_free(tmp_namespace);
+				g_free(tmp_namespace_split);
+			}
 			class_info->base_gtype = g_strdup(gtk_entry_get_text(
 					GTK_ENTRY(cc_dlg->base_gtype_entry)));
 			class_info->source = g_strdup(gtk_entry_get_text(GTK_ENTRY(cc_dlg->source_entry)));
 			if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(cc_dlg->create_constructor_box)))
 			{
-				class_info->constructor_decl = g_strdup_printf("%s*\t%s_new\t\t\t(void);\n",
+				class_info->constructor_decl = g_strdup_printf("%s*\t%s%s_new\t\t\t(void);\n",
 						gtk_entry_get_text(GTK_ENTRY(cc_dlg->gtk_constructor_type_entry)),
-						class_info->class_name_low);
+						class_info->namespace_low, class_info->class_name_low);
 				class_info->constructor_impl = g_strdup_printf("\n"
-						"%s *%s_new(void)\n"
+						"%s *%s%s_new(void)\n"
 						"{\n"
-						"\treturn g_object_new(%s_TYPE, NULL);\n"
+						"\treturn g_object_new(%sTYPE_%s, NULL);\n"
 						"}\n",
 						gtk_entry_get_text(GTK_ENTRY(cc_dlg->gtk_constructor_type_entry)),
-						class_info->class_name_low,
-						class_info->class_name_up);
+						class_info->namespace_low, class_info->class_name_low,
+						class_info->namespace_up, class_info->class_name_up);
 			}
 			else
 			{
@@ -662,25 +844,25 @@ static void cc_dlg_on_create_class(CreateClassDialog *cc_dlg)
 				class_info->gtk_destructor_registration =
 						g_strdup_printf("GObjectClass *g_object_class;\n\n"
 						"\tg_object_class = G_OBJECT_CLASS(klass);\n\n"
-						"\tg_object_class->finalize = %s_finalize;\n",
-						class_info->class_name_low);
+						"\tg_object_class->finalize = %s%s_finalize;\n",
+						class_info->namespace_low, class_info->class_name_low);
 				class_info->destructor_decl =
-						g_strdup_printf("static void %s_finalize  \t\t\t(GObject *object);\n",
-						class_info->class_name_low);
+						g_strdup_printf("static void %s%s_finalize\t\t\t(GObject *object);\n",
+						class_info->namespace_low, class_info->class_name_low);
 				class_info->destructor_impl = g_strdup_printf("\n"
-						"static void %s_finalize(GObject *object)\n"
+						"static void %s%s_finalize(GObject *object)\n"
 						"{\n"
-						"\t%s *self;\n\n"
+						"\t%s%s *self;\n\n"
 						"\tg_return_if_fail(object != NULL);\n"
-						"\tg_return_if_fail(IS_%s(object));\n\n"
-						"\tself = %s(object);\n\n"
-						"\tG_OBJECT_CLASS(%s_parent_class)->finalize(object);\n"
+						"\tg_return_if_fail(%sIS_%s(object));\n\n"
+						"\tself = %s%s(object);\n\n"
+						"\tG_OBJECT_CLASS(%s%s_parent_class)->finalize(object);\n"
 						"}\n",
-						class_info->class_name_low,
-						class_info->class_name,
-						class_info->class_name_up,
-						class_info->class_name_up,
-						class_info->class_name_low);
+						class_info->namespace_low,	class_info->class_name_low,
+						class_info->namespace,		class_info->class_name,
+						class_info->namespace_up,	class_info->class_name_up,
+						class_info->namespace_up,	class_info->class_name_up,
+						class_info->namespace_low,	class_info->class_name_low);
 			}
 			else
 			{
@@ -690,6 +872,107 @@ static void cc_dlg_on_create_class(CreateClassDialog *cc_dlg)
 			}
 			break;
 		}
+		case GEANY_CLASS_TYPE_PHP:
+		{
+			const gchar *tmp_str;
+
+			class_info->source = g_strdup(gtk_entry_get_text(GTK_ENTRY(cc_dlg->source_entry)));
+
+			tmp_str = gtk_entry_get_text(GTK_ENTRY(cc_dlg->class_namespace_entry));
+			if (! utils_str_equal(tmp_str, ""))
+				class_info->namespace_decl = g_strdup_printf("namespace %s;", tmp_str);
+			else
+				class_info->namespace_decl = g_strdup("");
+
+			tmp_str = gtk_entry_get_text(GTK_ENTRY(cc_dlg->class_implements_entry));
+			if (! utils_str_equal(tmp_str, ""))
+				class_info->implements_decl = g_strdup_printf(" implements %s", tmp_str);
+			else
+				class_info->implements_decl = g_strdup("");
+
+			if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(cc_dlg->create_constructor_box)) &&
+			    ! gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(cc_dlg->create_isabstract_box)))
+			{
+				class_info->constructor_impl = g_strdup_printf("\n"
+					"\t/**\n"
+					"\t * Constructor of class %s.\n"
+					"\t *\n"
+					"\t * @return void\n"
+					"\t */\n"
+					"\tpublic function __construct()\n"
+					"\t{\n"
+					"\t\t// ...\n"
+					"\t}\n",
+					class_info->class_name);
+			}
+			else if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(cc_dlg->create_constructor_box)) &&
+			         gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(cc_dlg->create_isabstract_box)))
+			{
+				class_info->constructor_impl = g_strdup_printf("\n"
+					"\t/**\n"
+					"\t * Constructor of class %s.\n"
+					"\t *\n"
+					"\t * @return void\n"
+					"\t */\n"
+					"\tprotected function __construct()\n"
+					"\t{\n"
+					"\t\t// ...\n"
+					"\t}\n",
+					class_info->class_name);
+			}
+			else
+				class_info->constructor_impl = g_strdup("");
+
+			if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(cc_dlg->create_destructor_box)))
+			{
+				class_info->destructor_impl = g_strdup_printf("\n"
+					"\t/**\n"
+					"\t * Destructor of class %s.\n"
+					"\t *\n"
+					"\t * @return void\n"
+					"\t */\n"
+					"\tpublic function __destruct()\n"
+					"\t{\n"
+					"\t\t// ...\n"
+					"\t}\n",
+					class_info->class_name);
+			}
+			else
+				class_info->destructor_impl = g_strdup("");
+
+			if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(cc_dlg->create_isabstract_box)))
+				class_info->abstract_decl = g_strdup("abstract ");
+			else
+				class_info->abstract_decl = g_strdup("");
+
+			if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(cc_dlg->create_issingleton_box)))
+			{
+				class_info->singleton_impl = g_strdup_printf("\n"
+					"\t/**\n"
+					"\t * Holds instance of self.\n"
+					"\t * \n"
+					"\t * @var %s\n"
+					"\t */\n"
+					"\tprotected static $kInstance = null;\n\n"
+					"\t/**\n"
+					"\t * Returns instance of self.\n"
+					"\t * \n"
+					"\t * @return %s\n"
+					"\t */\n"
+					"\tpublic static function getInstance() {\n"
+					"\t\tif(!(self::$kInstance instanceof %s)) {\n"
+					"\t\t\tself::$kInstance = new self();\n"
+					"\t\t}\n"
+					"\t\treturn self::$kInstance;\n"
+					"\t}\n",
+					class_info->class_name,
+					class_info->class_name,
+					class_info->class_name);
+			}
+			else
+				class_info->singleton_impl = g_strdup("");
+			break;
+		}
 	}
 
 	/* only create the files if the filename is not empty */
@@ -697,24 +980,28 @@ static void cc_dlg_on_create_class(CreateClassDialog *cc_dlg)
 	{
 		text = get_template_class_source(class_info);
 		doc = document_new_file(class_info->source, NULL, NULL);
-		sci_set_text(doc->editor->sci, text);
+		editor_insert_text_block(doc->editor, text, 0, -1, 0, TRUE);
 		g_free(text);
 	}
 
-	if (! utils_str_equal(class_info->header, ""))
+	if (! utils_str_equal(class_info->header, "") && class_info->type != GEANY_CLASS_TYPE_PHP)
 	{
 		text = get_template_class_header(class_info);
 		doc = document_new_file(class_info->header, NULL, NULL);
-		sci_set_text(doc->editor->sci, text);
+		editor_insert_text_block(doc->editor, text, 0, -1, 0, TRUE);
 		g_free(text);
 	}
 
-	free_pointers(17, tmp, class_info->class_name, class_info->class_name_up,
+	free_pointers(24, tmp, class_info->namespace, class_info->namespace_up,
+		class_info->namespace_low, class_info->class_name, class_info->class_name_up,
 		class_info->base_name, class_info->class_name_low, class_info->base_include,
 		class_info->header, class_info->header_guard, class_info->source, class_info->base_decl,
 		class_info->constructor_decl, class_info->constructor_impl,
 		class_info->gtk_destructor_registration, class_info->destructor_decl,
-		class_info->destructor_impl, class_info->base_gtype, class_info, NULL);
+		class_info->destructor_impl, class_info->base_gtype,
+		class_info->namespace_decl, class_info->implements_decl,
+		class_info->abstract_decl, class_info->singleton_impl, class_info, NULL);
+	return TRUE;
 }
 
 
@@ -734,12 +1021,21 @@ on_menu_create_gtk_class_activate      (GtkMenuItem     *menuitem,
 }
 
 
+static void
+on_menu_create_php_class_activate	(GtkMenuItem	*menuitem,
+					 gpointer	user_data)
+{
+	show_dialog_create_class(GEANY_CLASS_TYPE_PHP);
+}
+
+
 void plugin_init(GeanyData *data)
 {
 	GtkWidget *menu_create_class1;
 	GtkWidget *menu_create_class1_menu;
 	GtkWidget *menu_create_cpp_class;
 	GtkWidget *menu_create_gtk_class;
+	GtkWidget *menu_create_php_class;
 
 	menu_create_class1 = ui_image_menu_item_new (GTK_STOCK_ADD, _("Create Cla_ss"));
 	gtk_container_add (GTK_CONTAINER (geany->main_widgets->tools_menu), menu_create_class1);
@@ -753,16 +1049,21 @@ void plugin_init(GeanyData *data)
 	menu_create_gtk_class = gtk_menu_item_new_with_mnemonic (_("_GTK+ Class"));
 	gtk_container_add (GTK_CONTAINER (menu_create_class1_menu), menu_create_gtk_class);
 
+	menu_create_php_class = gtk_menu_item_new_with_mnemonic (_("_PHP Class"));
+	gtk_container_add (GTK_CONTAINER (menu_create_class1_menu), menu_create_php_class);
+
 	g_signal_connect(menu_create_cpp_class, "activate",
 		G_CALLBACK (on_menu_create_cpp_class_activate),
 		NULL);
 	g_signal_connect(menu_create_gtk_class, "activate",
 		G_CALLBACK (on_menu_create_gtk_class_activate),
 		NULL);
+	g_signal_connect(menu_create_php_class, "activate",
+		G_CALLBACK (on_menu_create_php_class_activate),
+		NULL);
 
 	gtk_widget_show_all(menu_create_class1);
 
-	ui_add_document_sensitive(menu_create_class1);
 	main_menu_item = menu_create_class1;
 }
 
