@@ -1,8 +1,8 @@
 /*
  *      filebrowser.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2007-2009 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2007-2009 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2007-2010 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2007-2010 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -19,7 +19,7 @@
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
  *      MA 02110-1301, USA.
  *
- * $Id: filebrowser.c 4041 2009-07-29 17:40:20Z ntrel $
+ * $Id: filebrowser.c 5133 2010-08-11 14:13:24Z ntrel $
  */
 
 /* Sidebar file browser plugin. */
@@ -29,13 +29,16 @@
 
 #include <gdk/gdkkeysyms.h>
 
+#ifdef G_OS_WIN32
+# include <windows.h>
+#endif
 
 GeanyPlugin		*geany_plugin;
 GeanyData		*geany_data;
 GeanyFunctions	*geany_functions;
 
 
-PLUGIN_VERSION_CHECK(131)
+PLUGIN_VERSION_CHECK(175)
 
 PLUGIN_SET_INFO(_("File Browser"), _("Adds a file browser tab to the sidebar."), VERSION,
 	_("The Geany developer team"))
@@ -85,6 +88,7 @@ static struct
 	GtkWidget *open;
 	GtkWidget *open_external;
 	GtkWidget *find_in_files;
+	GtkWidget *show_hidden_files;
 } popup_items;
 
 
@@ -98,16 +102,35 @@ PluginCallback plugin_callbacks[] =
 };
 
 
+#ifdef G_OS_WIN32
+static gboolean win32_check_hidden(const gchar *filename)
+{
+	DWORD attrs;
+	static wchar_t w_filename[MAX_PATH];
+	MultiByteToWideChar(CP_UTF8, 0, filename, -1, w_filename, sizeof(w_filename));
+	attrs = GetFileAttributesW(w_filename);
+	if (attrs != INVALID_FILE_ATTRIBUTES && attrs & FILE_ATTRIBUTE_HIDDEN)
+		return TRUE;
+	return FALSE;
+}
+#endif
+
+
 /* Returns: whether name should be hidden. */
-static gboolean check_hidden(const gchar *base_name)
+static gboolean check_hidden(const gchar *filename, const gchar *base_name)
 {
 	gsize len;
 
 	if (! NZV(base_name))
 		return FALSE;
 
+#ifdef G_OS_WIN32
+	if (win32_check_hidden(filename))
+		return TRUE;
+#else
 	if (base_name[0] == '.')
 		return TRUE;
+#endif
 
 	len = strlen(base_name);
 	if (base_name[len - 1] == '~')
@@ -131,7 +154,7 @@ static gboolean check_hidden(const gchar *base_name)
 }
 
 
-/* Returns: whether name has been removed by filter. */
+/* Returns: whether filename should be removed. */
 static gboolean check_filtered(const gchar *base_name)
 {
 	if (filter == NULL)
@@ -141,7 +164,6 @@ static gboolean check_filtered(const gchar *base_name)
 	{
 		return TRUE;
 	}
-
 	return FALSE;
 }
 
@@ -150,12 +172,10 @@ static gboolean check_filtered(const gchar *base_name)
 static void add_item(const gchar *name)
 {
 	GtkTreeIter iter;
-	gchar *fname, *utf8_name, *utf8_fullname, *sep;
+	gchar *fname, *utf8_name, *utf8_fullname;
+	const gchar *sep;
 	gboolean dir;
 
-	if (! show_hidden_files && check_hidden(name))
-		return;
-
 	sep = (utils_str_equal(current_dir, "/")) ? "" : G_DIR_SEPARATOR_S;
 	fname = g_strconcat(current_dir, sep, name, NULL);
 	dir = g_file_test(fname, G_FILE_TEST_IS_DIR);
@@ -163,6 +183,13 @@ static void add_item(const gchar *name)
 	utf8_name = utils_get_utf8_from_locale(name);
 	g_free(fname);
 
+	if (! show_hidden_files && check_hidden(utf8_fullname, name))
+	{
+		g_free(utf8_name);
+		g_free(utf8_fullname);
+		return;
+	}
+
 	if (dir)
 	{
 		if (last_dir_iter == NULL)
@@ -184,8 +211,6 @@ static void add_item(const gchar *name)
 		}
 		gtk_list_store_append(file_store, &iter);
 	}
-
-
 	gtk_list_store_set(file_store, &iter,
 		FILEVIEW_COLUMN_ICON, (dir) ? GTK_STOCK_DIRECTORY : GTK_STOCK_FILE,
 		FILEVIEW_COLUMN_NAME, utf8_name,
@@ -231,11 +256,15 @@ static void clear(void)
 }
 
 
+/* Reuses list to free each node, so list must be a variable */
+#define foreach_slist_free(node, list) \
+	for (node = list, list = NULL; g_slist_free_1(list), node != NULL; list = node, node = node->next)
+
 /* recreate the tree model from current_dir. */
 static void refresh(void)
 {
 	gchar *utf8_dir;
-	GSList *list;
+	GSList *list, *node;
 
 	/* don't clear when the new path doesn't exist */
 	if (! g_file_test(current_dir, G_FILE_TEST_EXISTS))
@@ -252,11 +281,16 @@ static void refresh(void)
 	list = utils_get_file_list(current_dir, NULL, NULL);
 	if (list != NULL)
 	{
-		g_slist_foreach(list, (GFunc) add_item, NULL);
-		g_slist_foreach(list, (GFunc) g_free, NULL);
-		g_slist_free(list);
+		/* free filenames & nodes as we go through the list */
+		foreach_slist_free(node, list)
+		{
+			gchar *fname = node->data;
+
+			add_item(fname);
+			g_free(fname);
+		}
 	}
-   	gtk_entry_completion_set_model(entry_completion, GTK_TREE_MODEL(file_store));
+	gtk_entry_completion_set_model(entry_completion, GTK_TREE_MODEL(file_store));
 }
 
 
@@ -309,6 +343,9 @@ static void on_current_path(void)
 
 static void on_go_up(void)
 {
+	gsize len = strlen(current_dir);
+	if (current_dir[len-1] == G_DIR_SEPARATOR)
+		current_dir[len-1] = '\0';
 	/* remove the highest directory part (which becomes the basename of current_dir) */
 	setptr(current_dir, g_path_get_dirname(current_dir));
 	refresh();
@@ -438,10 +475,11 @@ static void on_external_open(GtkMenuItem *menuitem, gpointer user_data)
 
 
 /* We use document_open_files() as it's more efficient. */
-static void open_selected_files(GList *list)
+static void open_selected_files(GList *list, gboolean do_not_focus)
 {
 	GSList *files = NULL;
 	GList *item;
+	GeanyDocument *doc;
 
 	for (item = list; item != NULL; item = g_list_next(item))
 	{
@@ -451,6 +489,10 @@ static void open_selected_files(GList *list)
 		files = g_slist_append(files, fname);
 	}
 	document_open_files(files, FALSE, NULL, NULL);
+	doc = document_get_current();
+	if (doc != NULL && ! do_not_focus)
+		keybindings_send_command(GEANY_KEY_GROUP_FOCUS, GEANY_KEYS_FOCUS_EDITOR);
+
 	g_slist_foreach(files, (GFunc) g_free, NULL);	/* free filenames */
 	g_slist_free(files);
 }
@@ -487,7 +529,7 @@ static void on_open_clicked(GtkMenuItem *menuitem, gpointer user_data)
 		}
 	}
 	else
-		open_selected_files(list);
+		open_selected_files(list, GPOINTER_TO_INT(user_data));
 
 	g_list_foreach(list, (GFunc) gtk_tree_path_free, NULL);
 	g_list_free(list);
@@ -503,7 +545,9 @@ static void on_find_in_files(GtkMenuItem *menuitem, gpointer user_data)
 	gboolean is_dir = FALSE;
 
 	treesel = gtk_tree_view_get_selection(GTK_TREE_VIEW(file_view));
-	if (! check_single_selection(treesel))
+	/* allow 0 or 1 selections */
+	if (gtk_tree_selection_count_selected_rows(treesel) > 0 &&
+		! check_single_selection(treesel))
 		return;
 
 	list = gtk_tree_selection_get_selected_rows(treesel, &model);
@@ -540,6 +584,12 @@ static void on_hide_sidebar(void)
 }
 
 
+static void on_show_preferences(void)
+{
+	plugin_show_configure(geany_plugin);
+}
+
+
 static GtkWidget *create_popup_menu(void)
 {
 	GtkWidget *item, *menu;
@@ -558,6 +608,15 @@ static GtkWidget *create_popup_menu(void)
 	g_signal_connect(item, "activate", G_CALLBACK(on_external_open), NULL);
 	popup_items.open_external = item;
 
+	item = gtk_separator_menu_item_new();
+	gtk_widget_show(item);
+	gtk_container_add(GTK_CONTAINER(menu), item);
+
+	item = gtk_image_menu_item_new_from_stock(GTK_STOCK_REFRESH, NULL);
+	gtk_widget_show(item);
+	gtk_container_add(GTK_CONTAINER(menu), item);
+	g_signal_connect(item, "activate", G_CALLBACK(refresh), NULL);
+
 	item = ui_image_menu_item_new(GTK_STOCK_FIND, _("_Find in Files"));
 	gtk_widget_show(item);
 	gtk_container_add(GTK_CONTAINER(menu), item);
@@ -572,6 +631,16 @@ static GtkWidget *create_popup_menu(void)
 	gtk_widget_show(item);
 	gtk_container_add(GTK_CONTAINER(menu), item);
 	g_signal_connect(item, "activate", G_CALLBACK(on_hidden_files_clicked), NULL);
+	popup_items.show_hidden_files = item;
+
+	item = gtk_separator_menu_item_new();
+	gtk_widget_show(item);
+	gtk_container_add(GTK_CONTAINER(menu), item);
+
+	item = gtk_image_menu_item_new_from_stock(GTK_STOCK_PREFERENCES, NULL);
+	gtk_widget_show(item);
+	gtk_container_add(GTK_CONTAINER(menu), item);
+	g_signal_connect(item, "activate", G_CALLBACK(on_show_preferences), NULL);
 
 	item = gtk_separator_menu_item_new();
 	gtk_widget_show(item);
@@ -614,6 +683,8 @@ static gboolean on_button_press(GtkWidget *widget, GdkEventButton *event, gpoint
 		if (popup_menu == NULL)
 			popup_menu = create_popup_menu();
 
+		gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(popup_items.show_hidden_files),
+			show_hidden_files);
 		gtk_menu_popup(GTK_MENU(popup_menu), NULL, NULL, NULL, NULL, event->button, event->time);
 		/* don't return TRUE here, unless the selection won't be changed */
 	}
@@ -623,15 +694,18 @@ static gboolean on_button_press(GtkWidget *widget, GdkEventButton *event, gpoint
 
 static gboolean on_key_press(GtkWidget *widget, GdkEventKey *event, gpointer data)
 {
-	if (event->keyval == GDK_Return
-		|| event->keyval == GDK_ISO_Enter
-		|| event->keyval == GDK_KP_Enter
-		|| event->keyval == GDK_space)
+	if (ui_is_keyval_enter_or_return(event->keyval))
 	{
 		on_open_clicked(NULL, NULL);
 		return TRUE;
 	}
 
+	if (event->keyval == GDK_space)
+	{
+		on_open_clicked(NULL, GINT_TO_POINTER(TRUE));
+		return TRUE;
+	}
+
 	if ((event->keyval == GDK_Up ||
 		event->keyval == GDK_KP_Up) &&
 		(event->state & GDK_MOD1_MASK))	/* FIXME: Alt-Up doesn't seem to work! */
@@ -707,12 +781,20 @@ static void on_filter_activate(GtkEntry *entry, gpointer user_data)
 }
 
 
+static void on_filter_clear(GtkEntry *entry, gint icon_pos,
+							GdkEvent *event, gpointer data)
+{
+	setptr(filter, g_strdup("*"));
+
+	refresh();
+}
+
+
 static void prepare_file_view(void)
 {
 	GtkCellRenderer *text_renderer, *icon_renderer;
 	GtkTreeViewColumn *column;
-	GtkTreeSelection *select;
-	PangoFontDescription *pfd;
+	GtkTreeSelection *selection;
 
 	file_store = gtk_list_store_new(FILEVIEW_N_COLUMNS, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING);
 
@@ -722,30 +804,28 @@ static void prepare_file_view(void)
 	icon_renderer = gtk_cell_renderer_pixbuf_new();
 	text_renderer = gtk_cell_renderer_text_new();
 	column = gtk_tree_view_column_new();
-    gtk_tree_view_column_pack_start(column, icon_renderer, FALSE);
-  	gtk_tree_view_column_set_attributes(column, icon_renderer, "stock-id", FILEVIEW_COLUMN_ICON, NULL);
-    gtk_tree_view_column_pack_start(column, text_renderer, TRUE);
-  	gtk_tree_view_column_set_attributes(column, text_renderer, "text", FILEVIEW_COLUMN_NAME, NULL);
+	gtk_tree_view_column_pack_start(column, icon_renderer, FALSE);
+	gtk_tree_view_column_set_attributes(column, icon_renderer, "stock-id", FILEVIEW_COLUMN_ICON, NULL);
+	gtk_tree_view_column_pack_start(column, text_renderer, TRUE);
+	gtk_tree_view_column_set_attributes(column, text_renderer, "text", FILEVIEW_COLUMN_NAME, NULL);
 	gtk_tree_view_append_column(GTK_TREE_VIEW(file_view), column);
 	gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(file_view), FALSE);
 
 	gtk_tree_view_set_enable_search(GTK_TREE_VIEW(file_view), TRUE);
 	gtk_tree_view_set_search_column(GTK_TREE_VIEW(file_view), FILEVIEW_COLUMN_NAME);
 
-	pfd = pango_font_description_from_string(geany->interface_prefs->tagbar_font);
-	gtk_widget_modify_font(file_view, pfd);
-	pango_font_description_free(pfd);
+	ui_widget_modify_font_from_string(file_view, geany->interface_prefs->tagbar_font);
 
 	/* GTK 2.12 tooltips */
 	if (gtk_check_version(2, 12, 0) == NULL)
 		g_object_set(file_view, "has-tooltip", TRUE, "tooltip-column", FILEVIEW_COLUMN_FILENAME, NULL);
 
 	/* selection handling */
-	select = gtk_tree_view_get_selection(GTK_TREE_VIEW(file_view));
-	gtk_tree_selection_set_mode(select, GTK_SELECTION_MULTIPLE);
+	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(file_view));
+	gtk_tree_selection_set_mode(selection, GTK_SELECTION_MULTIPLE);
 
 	g_signal_connect(file_view, "realize", G_CALLBACK(on_current_path), NULL);
-	g_signal_connect(select, "changed", G_CALLBACK(on_tree_selection_changed), NULL);
+	g_signal_connect(selection, "changed", G_CALLBACK(on_tree_selection_changed), NULL);
 	g_signal_connect(file_view, "button-press-event", G_CALLBACK(on_button_press), NULL);
 	g_signal_connect(file_view, "key-press-event", G_CALLBACK(on_key_press), NULL);
 }
@@ -759,34 +839,36 @@ static GtkWidget *make_toolbar(void)
 	gtk_toolbar_set_icon_size(GTK_TOOLBAR(toolbar), GTK_ICON_SIZE_MENU);
 	gtk_toolbar_set_style(GTK_TOOLBAR(toolbar), GTK_TOOLBAR_ICONS);
 
-	wid = (GtkWidget *) gtk_tool_button_new_from_stock(GTK_STOCK_GO_UP);
+	wid = GTK_WIDGET(gtk_tool_button_new_from_stock(GTK_STOCK_GO_UP));
 	ui_widget_set_tooltip_text(wid, _("Up"));
 	g_signal_connect(wid, "clicked", G_CALLBACK(on_go_up), NULL);
 	gtk_container_add(GTK_CONTAINER(toolbar), wid);
 
-	wid = (GtkWidget *) gtk_tool_button_new_from_stock(GTK_STOCK_REFRESH);
+	wid = GTK_WIDGET(gtk_tool_button_new_from_stock(GTK_STOCK_REFRESH));
 	ui_widget_set_tooltip_text(wid, _("Refresh"));
 	g_signal_connect(wid, "clicked", G_CALLBACK(refresh), NULL);
 	gtk_container_add(GTK_CONTAINER(toolbar), wid);
 
-	wid = (GtkWidget *) gtk_tool_button_new_from_stock(GTK_STOCK_HOME);
+	wid = GTK_WIDGET(gtk_tool_button_new_from_stock(GTK_STOCK_HOME));
 	ui_widget_set_tooltip_text(wid, _("Home"));
 	g_signal_connect(wid, "clicked", G_CALLBACK(on_go_home), NULL);
 	gtk_container_add(GTK_CONTAINER(toolbar), wid);
 
-	wid = (GtkWidget *) gtk_tool_button_new_from_stock(GTK_STOCK_JUMP_TO);
+	wid = GTK_WIDGET(gtk_tool_button_new_from_stock(GTK_STOCK_JUMP_TO));
 	ui_widget_set_tooltip_text(wid, _("Set path from document"));
 	g_signal_connect(wid, "clicked", G_CALLBACK(on_current_path), NULL);
 	gtk_container_add(GTK_CONTAINER(toolbar), wid);
 
-	wid = (GtkWidget *) gtk_separator_tool_item_new();
-	gtk_container_add(GTK_CONTAINER(toolbar), wid);
-
-	wid = (GtkWidget *) gtk_tool_button_new_from_stock(GTK_STOCK_CLEAR);
-	ui_widget_set_tooltip_text(wid, _("Clear the filter"));
-	g_signal_connect(wid, "clicked", G_CALLBACK(on_clear_filter), NULL);
-	gtk_container_add(GTK_CONTAINER(toolbar), wid);
+	if (gtk_check_version(2, 15, 2) != NULL)
+	{
+		wid = GTK_WIDGET(gtk_separator_tool_item_new());
+		gtk_container_add(GTK_CONTAINER(toolbar), wid);
 
+		wid = GTK_WIDGET(gtk_tool_button_new_from_stock(GTK_STOCK_CLEAR));
+		ui_widget_set_tooltip_text(wid, _("Clear the filter"));
+		g_signal_connect(wid, "clicked", G_CALLBACK(on_clear_filter), NULL);
+		gtk_container_add(GTK_CONTAINER(toolbar), wid);
+	}
 	return toolbar;
 }
 
@@ -800,6 +882,14 @@ static GtkWidget *make_filterbar(void)
 	label = gtk_label_new(_("Filter:"));
 
 	filter_entry = gtk_entry_new();
+
+	if (gtk_check_version(2, 15, 2) == NULL)
+	{
+		ui_entry_add_clear_icon(GTK_ENTRY(filter_entry));
+		g_signal_connect(filter_entry, "icon-release", G_CALLBACK(on_filter_clear), NULL);
+	}
+	ui_widget_set_tooltip_text(filter_entry,
+		_("Filter your files with usual wildcards"));
 	g_signal_connect(filter_entry, "activate", G_CALLBACK(on_filter_activate), NULL);
 
 	gtk_box_pack_start(GTK_BOX(filterbar), label, FALSE, FALSE, 0);
@@ -1025,6 +1115,38 @@ void plugin_init(GeanyData *data)
 }
 
 
+static void save_settings(void)
+{
+	GKeyFile *config = g_key_file_new();
+	gchar *data;
+	gchar *config_dir = g_path_get_dirname(config_file);
+
+	g_key_file_load_from_file(config, config_file, G_KEY_FILE_NONE, NULL);
+
+	g_key_file_set_string(config, "filebrowser", "open_command", open_cmd);
+	g_key_file_set_boolean(config, "filebrowser", "show_hidden_files", show_hidden_files);
+	g_key_file_set_boolean(config, "filebrowser", "hide_object_files", hide_object_files);
+	g_key_file_set_boolean(config, "filebrowser", "fb_follow_path", fb_follow_path);
+	g_key_file_set_boolean(config, "filebrowser", "fb_set_project_base_path",
+		fb_set_project_base_path);
+
+	if (! g_file_test(config_dir, G_FILE_TEST_IS_DIR) && utils_mkdir(config_dir, TRUE) != 0)
+	{
+		dialogs_show_msgbox(GTK_MESSAGE_ERROR,
+			_("Plugin configuration directory could not be created."));
+	}
+	else
+	{
+		/* write config to file */
+		data = g_key_file_to_data(config, NULL, NULL);
+		utils_write_file(config_file, data);
+		g_free(data);
+	}
+	g_free(config_dir);
+	g_key_file_free(config);
+}
+
+
 static struct
 {
 	GtkWidget *open_cmd_entry;
@@ -1040,10 +1162,6 @@ on_configure_response(GtkDialog *dialog, gint response, gpointer user_data)
 {
 	if (response == GTK_RESPONSE_OK || response == GTK_RESPONSE_APPLY)
 	{
-		GKeyFile *config = g_key_file_new();
-		gchar *data;
-		gchar *config_dir = g_path_get_dirname(config_file);
-
 		g_free(open_cmd);
 		open_cmd = g_strdup(gtk_entry_get_text(GTK_ENTRY(pref_widgets.open_cmd_entry)));
 		show_hidden_files = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(pref_widgets.show_hidden_checkbox));
@@ -1052,33 +1170,8 @@ on_configure_response(GtkDialog *dialog, gint response, gpointer user_data)
 		fb_set_project_base_path = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(
 			pref_widgets.set_project_base_path_checkbox));
 
-		g_key_file_load_from_file(config, config_file, G_KEY_FILE_NONE, NULL);
-
-		g_key_file_set_string(config, "filebrowser", "open_command", open_cmd);
-		g_key_file_set_boolean(config, "filebrowser", "show_hidden_files", show_hidden_files);
-		g_key_file_set_boolean(config, "filebrowser", "hide_object_files", hide_object_files);
-		g_key_file_set_boolean(config, "filebrowser", "fb_follow_path", fb_follow_path);
-		g_key_file_set_boolean(config, "filebrowser", "fb_set_project_base_path",
-			fb_set_project_base_path);
-
-		if (! g_file_test(config_dir, G_FILE_TEST_IS_DIR) && utils_mkdir(config_dir, TRUE) != 0)
-		{
-			dialogs_show_msgbox(GTK_MESSAGE_ERROR,
-				_("Plugin configuration directory could not be created."));
-		}
-		else
-		{
-			/* write config to file */
-			data = g_key_file_to_data(config, NULL, NULL);
-			utils_write_file(config_file, data);
-			g_free(data);
-		}
-
 		/* apply the changes */
 		refresh();
-
-		g_free(config_dir);
-		g_key_file_free(config);
 	}
 }
 
@@ -1086,12 +1179,14 @@ on_configure_response(GtkDialog *dialog, gint response, gpointer user_data)
 GtkWidget *plugin_configure(GtkDialog *dialog)
 {
 	GtkWidget *label, *entry, *checkbox_of, *checkbox_hf, *checkbox_fp, *checkbox_pb, *vbox;
+	GtkWidget *box;
 
 	vbox = gtk_vbox_new(FALSE, 6);
+	box = gtk_vbox_new(FALSE, 3);
 
 	label = gtk_label_new(_("External open command:"));
 	gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
-	gtk_container_add(GTK_CONTAINER(vbox), label);
+	gtk_box_pack_start(GTK_BOX(box), label, FALSE, FALSE, 0);
 
 	entry = gtk_entry_new();
 	gtk_widget_show(entry);
@@ -1101,13 +1196,15 @@ GtkWidget *plugin_configure(GtkDialog *dialog)
 		_("The command to execute when using \"Open with\". You can use %f and %d wildcards.\n"
 		  "%f will be replaced with the filename including full path\n"
 		  "%d will be replaced with the path name of the selected file without the filename"));
-	gtk_container_add(GTK_CONTAINER(vbox), entry);
+	gtk_box_pack_start(GTK_BOX(box), entry, FALSE, FALSE, 0);
 	pref_widgets.open_cmd_entry = entry;
 
+	gtk_box_pack_start(GTK_BOX(vbox), box, FALSE, FALSE, 6);
+
 	checkbox_hf = gtk_check_button_new_with_label(_("Show hidden files"));
 	gtk_button_set_focus_on_click(GTK_BUTTON(checkbox_hf), FALSE);
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkbox_hf), show_hidden_files);
-	gtk_box_pack_start(GTK_BOX(vbox), checkbox_hf, FALSE, FALSE, 5);
+	gtk_box_pack_start(GTK_BOX(vbox), checkbox_hf, FALSE, FALSE, 0);
 	pref_widgets.show_hidden_checkbox = checkbox_hf;
 
 	checkbox_of = gtk_check_button_new_with_label(_("Hide object files"));
@@ -1116,13 +1213,13 @@ GtkWidget *plugin_configure(GtkDialog *dialog)
 	ui_widget_set_tooltip_text(checkbox_of,
 		_("Don't show generated object files in the file browser, this includes "
 		  "*.o, *.obj. *.so, *.dll, *.a, *.lib"));
-	gtk_box_pack_start(GTK_BOX(vbox), checkbox_of, FALSE, FALSE, 5);
+	gtk_box_pack_start(GTK_BOX(vbox), checkbox_of, FALSE, FALSE, 0);
 	pref_widgets.hide_objects_checkbox = checkbox_of;
 
 	checkbox_fp = gtk_check_button_new_with_label(_("Follow the path of the current file"));
 	gtk_button_set_focus_on_click(GTK_BUTTON(checkbox_fp), FALSE);
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkbox_fp), fb_follow_path);
-	gtk_box_pack_start(GTK_BOX(vbox), checkbox_fp, FALSE, FALSE, 5);
+	gtk_box_pack_start(GTK_BOX(vbox), checkbox_fp, FALSE, FALSE, 0);
 	pref_widgets.follow_path_checkbox = checkbox_fp;
 
 	checkbox_pb = gtk_check_button_new_with_label(_("Use the project's base directory"));
@@ -1130,7 +1227,7 @@ GtkWidget *plugin_configure(GtkDialog *dialog)
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkbox_pb), fb_set_project_base_path);
 	ui_widget_set_tooltip_text(checkbox_pb,
 		_("Change the directory to the base directory of the currently opened project"));
-	gtk_box_pack_start(GTK_BOX(vbox), checkbox_pb, FALSE, FALSE, 5);
+	gtk_box_pack_start(GTK_BOX(vbox), checkbox_pb, FALSE, FALSE, 0);
 	pref_widgets.set_project_base_path_checkbox = checkbox_pb;
 
 	gtk_widget_show_all(vbox);
@@ -1142,6 +1239,8 @@ GtkWidget *plugin_configure(GtkDialog *dialog)
 
 void plugin_cleanup(void)
 {
+	save_settings();
+
 	g_free(config_file);
 	g_free(open_cmd);
 	g_free(filter);
