@@ -1,9 +1,9 @@
 /*
  *      htmlchars.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2009 Frank Lanitz <frank(at)frank(dot)uvena(dot)de>
- *      Copyright 2006-2009 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2007-2009 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2009-2010 Frank Lanitz <frank(at)frank(dot)uvena(dot)de>
+ *      Copyright 2006-2010 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2007-2010 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -20,7 +20,7 @@
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
  *      MA 02110-1301, USA.
  *
- * $Id: htmlchars.c 3966 2009-07-14 15:06:20Z ntrel $
+ * $Id: htmlchars.c 4861 2010-04-25 17:43:09Z eht16 $
  */
 
 /* HTML Characters plugin (Inserts HTML character entities like '&amp;') */
@@ -68,6 +68,9 @@ static GtkTreeView *sc_tree = NULL;
 static GtkWidget *menu_htmltoggle = NULL;
 static gboolean plugin_active = FALSE;
 
+/* Configuration file */
+static gchar *config_file = NULL;
+
 const gchar *chars[][2] ={
 	{ N_("HTML characters"), NULL },
 	{ "\"", "&quot;" },
@@ -342,12 +345,39 @@ PluginCallback plugin_callbacks[] =
 
 
 /* Functions to toggle the status of plugin */
-void set_status(gboolean new_status)
+static void set_status(gboolean new_status)
 {
-	/* No more function at the moment.*/
 	if (plugin_active != new_status)
+	{
+		GKeyFile *config = g_key_file_new();
+		gchar *data;
+		gchar *config_dir = g_path_get_dirname(config_file);
+
 		plugin_active = new_status;
-}
+
+		/* Now we save the new status into configuration file to
+		 * remember next time */
+		g_key_file_set_boolean(config, "general", "replacement_on_typing_active",
+			plugin_active);
+
+		if (!g_file_test(config_dir, G_FILE_TEST_IS_DIR)
+			&& utils_mkdir(config_dir, TRUE) != 0)
+		{
+			dialogs_show_msgbox(GTK_MESSAGE_ERROR,
+				_("Plugin configuration directory could not be created."));
+		}
+		else
+		{
+			/* write config to file */
+			data = g_key_file_to_data(config, NULL, NULL);
+			utils_write_file(config_file, data);
+			g_free(data);
+		}
+		g_free(config_dir);
+		g_key_file_free(config);
+	}
+}	
+
 
 static void toggle_status(G_GNUC_UNUSED GtkMenuItem * menuitem)
 {
@@ -368,7 +398,7 @@ static gboolean sc_insert(GtkTreeModel *model, GtkTreeIter *iter);
 
 /* Function takes over value of key which was pressed and returns
  * HTML/SGML entity if any */
-const gchar *get_entity(gchar *letter)
+static const gchar *get_entity(gchar *letter)
 {
 	guint i, len;
 
@@ -429,11 +459,16 @@ static gboolean ht_editor_notify_cb(GObject *object, GeanyEditor *editor,
 static void kbhtmltoggle_toggle(G_GNUC_UNUSED guint key_id)
 {
 	if (plugin_active == TRUE)
+	{
 		set_status(FALSE);
+	}
 	else
+	{
 		set_status(TRUE);
+	}
 }
 
+
 static void tools_show_dialog_insert_special_chars(void)
 {
 	if (sc_dialog == NULL)
@@ -451,7 +486,7 @@ static void tools_show_dialog_insert_special_chars(void)
 		gtk_box_set_spacing(GTK_BOX(vbox), 6);
 		gtk_widget_set_name(sc_dialog, "GeanyDialog");
 
-		height = GEANY_WINDOW_MINIMAL_HEIGHT;
+		height = GEANY_DEFAULT_DIALOG_HEIGHT;
 		gtk_window_set_default_size(GTK_WINDOW(sc_dialog), height * 8 / 10, height);
 		gtk_dialog_set_default_response(GTK_DIALOG(sc_dialog), GTK_RESPONSE_CANCEL);
 
@@ -530,6 +565,7 @@ static void sc_fill_store(GtkTreeStore *store)
 	}
 }
 
+
 /* just inserts the HTML_NAME coloumn of the selected row at current position
  * returns only TRUE if a valid selection(i.e. no category) could be found */
 static gboolean sc_insert(GtkTreeModel *model, GtkTreeIter *iter)
@@ -599,7 +635,7 @@ static void sc_on_tree_row_activated(GtkTreeView *treeview, GtkTreePath *path,
 }
 
 
-static void replace_special_character()
+static void replace_special_character(void)
 {
 	GeanyDocument *doc = NULL;
 	doc = document_get_current();
@@ -650,6 +686,7 @@ item_activate(GtkMenuItem *menuitem, gpointer gdata)
 	tools_show_dialog_insert_special_chars();
 }
 
+
 static void kb_activate(G_GNUC_UNUSED guint key_id)
 {
 	item_activate(NULL, NULL);
@@ -664,19 +701,38 @@ replace_special_character_activated(GtkMenuItem *menuitem, gpointer gdata)
 }
 
 
-
 static void kb_special_chars_replacement(G_GNUC_UNUSED guint key_id)
 {
 	replace_special_character();
 }
 
 
-/* Called by Geany to initialize the plugin */
+static void init_configuration(void)
+{
+	GKeyFile *config = g_key_file_new();
+
+	/* loading configurations from file ...*/
+	config_file = g_strconcat(geany->app->configdir, G_DIR_SEPARATOR_S,
+	"plugins", G_DIR_SEPARATOR_S,
+	"htmchars", G_DIR_SEPARATOR_S, "general.conf", NULL);
+
+	/* ... and initialising options from config file */
+	g_key_file_load_from_file(config, config_file, G_KEY_FILE_NONE, NULL);
+
+	plugin_active = utils_get_setting_boolean(config, "general",
+		"replacement_on_typing_active", FALSE);
+}
+
+
+/* Called by Geany to initialise the plugin */
 void plugin_init(GeanyData *data)
 {
 	GtkWidget *menu_item;
 	const gchar *menu_text = _("_Insert Special HTML Characters");
 
+	/* First we catch the configuration and initialize them */
+	init_configuration();
+
 	/* Add an item to the Tools menu for html chars dialog*/
 	menu_item = gtk_menu_item_new_with_mnemonic(menu_text);
 	gtk_widget_show(menu_item);
@@ -736,6 +792,9 @@ void plugin_cleanup(void)
 
 	if (sc_dialog != NULL)
 		gtk_widget_destroy(sc_dialog);
+
+	if (config_file != NULL)
+		g_free(config_file);
 }
 
 
