@@ -1,8 +1,8 @@
 /*
  *      saveactions.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2007-2009 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2007-2009 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2007-2010 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2007-2010 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -19,7 +19,7 @@
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
  *      MA 02110-1301, USA.
  *
- * $Id: saveactions.c 3966 2009-07-14 15:06:20Z ntrel $
+ * $Id: saveactions.c 4861 2010-04-25 17:43:09Z eht16 $
  */
 
 
@@ -250,7 +250,7 @@ static void instantsave_document_new_cb(GObject *obj, GeanyDocument *doc, gpoint
 		if (fd != -1)
 			close(fd); /* close the returned file descriptor as we only need the filename */
 
-		if (ft == NULL)
+		if (ft == NULL || ft->id == GEANY_FILETYPES_NONE)
 			/* ft is NULL when a new file without template was opened, so use the
 			 * configured default file type */
 			ft = filetypes_lookup_by_name(instantsave_default_ft);
@@ -278,7 +278,7 @@ PluginCallback plugin_callbacks[] =
 };
 
 
-gboolean auto_save(gpointer data)
+static gboolean auto_save(gpointer data)
 {
 	GeanyDocument *doc;
 	GeanyDocument *cur_doc = document_get_current();
@@ -316,7 +316,7 @@ gboolean auto_save(gpointer data)
 }
 
 
-void autosave_set_timeout(void)
+static void autosave_set_timeout(void)
 {
 	if (! enable_autosave)
 		return;
