@@ -1,8 +1,8 @@
 /*
  *      splitwindow.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2008-2009 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
- *      Copyright 2008-2009 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2008-2010 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2008-2010 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -19,7 +19,7 @@
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
  *      MA 02110-1301, USA.
  *
- * $Id: splitwindow.c 4012 2009-07-23 11:00:32Z ntrel $
+ * $Id: splitwindow.c 4518 2010-01-01 22:55:18Z eht16 $
  */
 
 /* Split Window plugin. */
@@ -39,6 +39,7 @@ PLUGIN_SET_INFO(_("Split Window"), _("Splits the editor view into two windows.")
 
 GeanyData		*geany_data;
 GeanyFunctions	*geany_functions;
+GeanyPlugin		*geany_plugin;
 
 
 /* Keybinding(s) */
@@ -50,9 +51,6 @@ enum
 	KB_COUNT
 };
 
-PLUGIN_KEY_GROUP(split_window, KB_COUNT);
-
-
 enum State
 {
 	STATE_SPLIT_HORIZONTAL,
@@ -88,69 +86,23 @@ static EditWindow edit_window = {NULL, NULL, NULL, NULL};
 static void on_unsplit(GtkMenuItem *menuitem, gpointer user_data);
 
 
-static gint sci_get_value(ScintillaObject *sci, gint message_id, gint param)
-{
-	return scintilla_send_message(sci, message_id, param, 0);
-}
-
-
-static void set_styles(ScintillaObject *oldsci, ScintillaObject *newsci)
-{
-	gint style_id;
-
-	for (style_id = 0; style_id <= 127; style_id++)
-	{
-		gint val;
-
-		val = sci_get_value(oldsci, SCI_STYLEGETFORE, style_id);
-		scintilla_send_message(newsci, SCI_STYLESETFORE, style_id, val);
-		val = sci_get_value(oldsci, SCI_STYLEGETBACK, style_id);
-		scintilla_send_message(newsci, SCI_STYLESETBACK, style_id, val);
-		val = sci_get_value(oldsci, SCI_STYLEGETBOLD, style_id);
-		scintilla_send_message(newsci, SCI_STYLESETBOLD, style_id, val);
-		val = sci_get_value(oldsci, SCI_STYLEGETITALIC, style_id);
-		scintilla_send_message(newsci, SCI_STYLESETITALIC, style_id, val);
-	}
-}
-
-
-static void update_font(ScintillaObject *current, ScintillaObject *sci)
-{
-	gint style_id;
-	gint size;
-	gchar font_name[1024]; /* should be big enough */
-
-	scintilla_send_message(current, SCI_STYLEGETFONT, 0, (sptr_t)font_name);
-	size = sci_get_value(current, SCI_STYLEGETSIZE, 0);
-
-	for (style_id = 0; style_id <= 127; style_id++)
-	{
-		sci_set_font(sci, style_id, font_name, size);
-	}
-
-	/* line number and braces */
-	sci_set_font(sci, STYLE_LINENUMBER, font_name, size);
-	sci_set_font(sci, STYLE_BRACELIGHT, font_name, size);
-	sci_set_font(sci, STYLE_BRACEBAD, font_name, size);
-}
-
-
 /* line numbers visibility */
-static void set_line_numbers(ScintillaObject * sci, gboolean set, gint extra_width)
+static void set_line_numbers(ScintillaObject * sci, gboolean set)
 {
 	if (set)
 	{
 		gchar tmp_str[15];
 		gint len = scintilla_send_message(sci, SCI_GETLINECOUNT, 0, 0);
 		gint width;
-		g_snprintf(tmp_str, 15, "_%d%d", len, extra_width);
+
+		g_snprintf(tmp_str, 15, "_%d", len);
 		width = scintilla_send_message(sci, SCI_TEXTWIDTH, STYLE_LINENUMBER, (sptr_t) tmp_str);
 		scintilla_send_message(sci, SCI_SETMARGINWIDTHN, 0, width);
 		scintilla_send_message(sci, SCI_SETMARGINSENSITIVEN, 0, FALSE); /* use default behaviour */
 	}
 	else
 	{
-		scintilla_send_message(sci, SCI_SETMARGINWIDTHN, 0, 0 );
+		scintilla_send_message(sci, SCI_SETMARGINWIDTHN, 0, 0);
 	}
 }
 
@@ -158,24 +110,20 @@ static void set_line_numbers(ScintillaObject * sci, gboolean set, gint extra_wid
 static void sync_to_current(ScintillaObject *sci, ScintillaObject *current)
 {
 	gpointer sdoc;
-	gint lexer;
 	gint pos;
 
 	/* set the new sci widget to view the existing Scintilla document */
 	sdoc = (gpointer) scintilla_send_message(current, SCI_GETDOCPOINTER, 0, 0);
 	scintilla_send_message(sci, SCI_SETDOCPOINTER, 0, (sptr_t) sdoc);
 
-	update_font(current, sci);
-	lexer = scintilla_send_message(current, SCI_GETLEXER, 0, 0);
-	scintilla_send_message(sci, SCI_SETLEXER, lexer, 0);
-	set_styles(current, sci);
-
+	highlighting_set_styles(sci, edit_window.editor->document->file_type);
 	pos = sci_get_current_position(current);
 	sci_set_current_position(sci, pos, TRUE);
 
 	/* override some defaults */
-	set_line_numbers(sci, TRUE, 0);
-	scintilla_send_message(sci, SCI_SETMARGINWIDTHN, 1, 0 ); /* hide marker margin */
+	set_line_numbers(sci, geany->editor_prefs->show_linenumber_margin);
+	scintilla_send_message(sci, SCI_SETMARGINWIDTHN, 1, 0 ); /* hide marker margin (no commands) */
+	scintilla_send_message(sci, SCI_SETMARGINWIDTHN, 2, 0 ); /* hide fold margin (no toggle callback) */
 }
 
 
@@ -264,6 +212,23 @@ static void on_refresh(void)
 }
 
 
+static void on_doc_menu_item_clicked(gpointer item, GeanyDocument *doc)
+{
+	if (doc->is_valid)
+		set_editor(&edit_window, doc->editor);
+}
+
+
+static void on_doc_menu_show(GtkMenu *menu)
+{
+	/* clear the old menu items */
+	gtk_container_foreach(GTK_CONTAINER(menu), (GtkCallback) gtk_widget_destroy, NULL);
+
+	ui_menu_add_document_items(menu, edit_window.editor->document,
+		G_CALLBACK(on_doc_menu_item_clicked));
+}
+
+
 static GtkWidget *create_toolbar(void)
 {
 	GtkWidget *toolbar, *item;
@@ -273,11 +238,17 @@ static GtkWidget *create_toolbar(void)
 	gtk_toolbar_set_icon_size(GTK_TOOLBAR(toolbar), GTK_ICON_SIZE_MENU);
 	gtk_toolbar_set_style(GTK_TOOLBAR(toolbar), GTK_TOOLBAR_ICONS);
 
-	item = ui_tool_button_new(GTK_STOCK_JUMP_TO, "",
-		_("Show the current document"));
+	tool_item = gtk_menu_tool_button_new(NULL, NULL);
+	gtk_tool_button_set_stock_id(GTK_TOOL_BUTTON(tool_item), GTK_STOCK_JUMP_TO);
+	item = (GtkWidget*)tool_item;
+	ui_widget_set_tooltip_text(item, _("Show the current document"));
 	gtk_container_add(GTK_CONTAINER(toolbar), item);
 	g_signal_connect(item, "clicked", G_CALLBACK(on_refresh), NULL);
 
+	item = gtk_menu_new();
+	gtk_menu_tool_button_set_menu(GTK_MENU_TOOL_BUTTON(tool_item), item);
+	g_signal_connect(item, "show", G_CALLBACK(on_doc_menu_show), NULL);
+
 	tool_item = gtk_tool_item_new();
 	gtk_tool_item_set_expand(tool_item, TRUE);
 	gtk_container_add(GTK_CONTAINER(toolbar), GTK_WIDGET(tool_item));
@@ -395,6 +366,7 @@ static void kb_activate(guint key_id)
 void plugin_init(GeanyData *data)
 {
 	GtkWidget *item, *menu;
+	GeanyKeyGroup *key_group;
 
 	menu_items.main = item = gtk_menu_item_new_with_mnemonic(_("_Split Window"));
 	gtk_menu_shell_append(GTK_MENU_SHELL(geany_data->main_widgets->tools_menu), item);
@@ -423,12 +395,13 @@ void plugin_init(GeanyData *data)
 	set_state(STATE_UNSPLIT);
 
 	/* setup keybindings */
-	keybindings_set_item(plugin_key_group, KB_SPLIT_HORIZONTAL, kb_activate,
+	key_group = plugin_set_key_group(geany_plugin, "split_window", KB_COUNT, NULL);
+	keybindings_set_item(key_group, KB_SPLIT_HORIZONTAL, kb_activate,
 		0, 0, "split_horizontal", _("Split Horizontally"), menu_items.horizontal);
-	keybindings_set_item(plugin_key_group, KB_SPLIT_VERTICAL, kb_activate,
+	keybindings_set_item(key_group, KB_SPLIT_VERTICAL, kb_activate,
 		0, 0, "split_vertical", _("Split Vertically"), menu_items.vertical);
-	keybindings_set_item(plugin_key_group, KB_SPLIT_UNSPLIT, kb_activate,
-		0, 0, "split_unsplit", _("Unsplit"), menu_items.unsplit);
+	keybindings_set_item(key_group, KB_SPLIT_UNSPLIT, kb_activate,
+		0, 0, "split_unsplit", _("_Unsplit"), menu_items.unsplit);
 }
 
 
