@@ -1,8 +1,8 @@
 /*
  *      callbacks.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2005-2009 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2006-2009 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2005-2010 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2006-2010 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: callbacks.c 4014 2009-07-23 16:01:45Z ntrel $
+ * $Id: callbacks.c 4894 2010-05-09 15:48:55Z eht16 $
  */
 
 /*
@@ -52,7 +52,7 @@
 #include "build.h"
 #include "prefs.h"
 #include "templates.h"
-#include "treeviews.h"
+#include "sidebar.h"
 #include "keybindings.h"
 #include "encodings.h"
 #include "search.h"
@@ -65,6 +65,7 @@
 #include "plugins.h"
 #include "log.h"
 #include "toolbar.h"
+#include "pluginutils.h"
 
 
 #ifdef HAVE_VTE
@@ -76,9 +77,6 @@
 #endif
 
 
-/* flag to indicate the explicit change of a toggle button of the toolbar menu and so the
- * toggled callback should ignore the change since it is not triggered by the user */
-static gboolean ignore_toolbar_toggle = FALSE;
 
 /* flag to indicate that an insert callback was triggered from the file menu,
  * so we need to store the current cursor position in editor_info.click_pos. */
@@ -181,7 +179,7 @@ on_save1_activate                      (GtkMenuItem     *menuitem,
 
 	if (doc != NULL && cur_page >= 0)
 	{
-		if (doc->file_name == NULL)
+		if (document_need_save_as(doc))
 			dialogs_show_save_as();
 		else
 			document_save_file(doc, FALSE);
@@ -203,23 +201,33 @@ on_save_all1_activate                  (GtkMenuItem     *menuitem,
 {
 	gint i, max = gtk_notebook_get_n_pages(GTK_NOTEBOOK(main_widgets.notebook));
 	GeanyDocument *doc, *cur_doc = document_get_current();
+	gint count = 0;
 
 	for (i = 0; i < max; i++)
 	{
 		doc = document_get_from_page(i);
 		if (! doc->changed)
 			continue;
-		if (doc->file_name == NULL)
+		if (document_need_save_as(doc))
 		{
 			/* display unnamed document */
 			gtk_notebook_set_current_page(GTK_NOTEBOOK(main_widgets.notebook),
 				document_get_notebook_page(doc));
-			dialogs_show_save_as();
+			if (dialogs_show_save_as())
+				count++;
 		}
 		else
-			document_save_file(doc, FALSE);
+		{
+			if (document_save_file(doc, FALSE))
+				count++;
+		}
 	}
-	treeviews_update_tag_list(cur_doc, TRUE);
+	if (!count)
+		return;
+
+	ui_set_statusbar(FALSE, ngettext("%d file saved.", "%d files saved.", count), count);
+	/* saving may have changed window title, sidebar for another doc, so update */
+	sidebar_update_tag_list(cur_doc, TRUE);
 	ui_set_window_title(cur_doc);
 }
 
@@ -272,9 +280,18 @@ void
 on_edit1_activate                      (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
+	GtkWidget *item;
 	GeanyDocument *doc = document_get_current();
+
 	ui_update_menu_copy_items(doc);
 	ui_update_insert_include_item(doc, 1);
+
+	item = ui_lookup_widget(main_widgets.window, "plugin_preferences1");
+#ifndef HAVE_PLUGINS
+	gtk_widget_hide(item);
+#else
+	gtk_widget_set_sensitive(item, plugins_have_preferences());
+#endif
 }
 
 
@@ -453,7 +470,7 @@ on_reload_as_activate                  (GtkMenuItem     *menuitem,
 	GeanyDocument *doc = document_get_current();
 	gchar *base_name;
 	gint i = GPOINTER_TO_INT(user_data);
-	gchar *charset = NULL;
+	const gchar *charset = NULL;
 
 	g_return_if_fail(doc != NULL);
 	g_return_if_fail(doc->file_name != NULL);
@@ -464,6 +481,8 @@ on_reload_as_activate                  (GtkMenuItem     *menuitem,
 			return;
 		charset = encodings[i].charset;
 	}
+	else
+		charset = doc->encoding;
 
 	base_name = g_path_get_basename(doc->file_name);
 	if (dialogs_show_question_full(NULL, _("_Reload"), GTK_STOCK_CANCEL,
@@ -478,42 +497,6 @@ on_reload_as_activate                  (GtkMenuItem     *menuitem,
 }
 
 
-void
-on_images_and_text2_activate           (GtkCheckMenuItem *menuitem,
-                                        gpointer          user_data)
-{
-	if (ignore_toolbar_toggle || ! gtk_check_menu_item_get_active(menuitem))
-		return;
-
-	gtk_toolbar_set_style(GTK_TOOLBAR(main_widgets.toolbar), GTK_TOOLBAR_BOTH);
-	toolbar_prefs.icon_style = GTK_TOOLBAR_BOTH;
-}
-
-
-void
-on_images_only2_activate               (GtkCheckMenuItem *menuitem,
-                                        gpointer          user_data)
-{
-	if (ignore_toolbar_toggle || ! gtk_check_menu_item_get_active(menuitem))
-		return;
-
-	gtk_toolbar_set_style(GTK_TOOLBAR(main_widgets.toolbar), GTK_TOOLBAR_ICONS);
-	toolbar_prefs.icon_style = GTK_TOOLBAR_ICONS;
-}
-
-
-void
-on_text_only2_activate                 (GtkCheckMenuItem *menuitem,
-                                        gpointer          user_data)
-{
-	if (ignore_toolbar_toggle || ! gtk_check_menu_item_get_active(menuitem))
-		return;
-
-	gtk_toolbar_set_style(GTK_TOOLBAR(main_widgets.toolbar), GTK_TOOLBAR_TEXT);
-	toolbar_prefs.icon_style = GTK_TOOLBAR_TEXT;
-}
-
-
 void
 on_change_font1_activate               (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
@@ -530,6 +513,7 @@ on_toolbutton_new_clicked              (GtkAction       *action,
 	document_new_file(NULL, NULL, NULL);
 }
 
+
 /* open file */
 void
 on_toolbutton_open_clicked             (GtkAction       *action,
@@ -590,42 +574,8 @@ on_toolbutton_search_clicked           (GtkAction       *action,
 		if (search_data.search_bar)
 			ui_set_search_entry_background(entry, result);
 	}
-}
-
-
-void
-on_toolbar_large_icons1_activate       (GtkCheckMenuItem *menuitem,
-                                        gpointer          user_data)
-{
-	if (ignore_toolbar_toggle || ! gtk_check_menu_item_get_active(menuitem))
-		return;
-
-	toolbar_prefs.icon_size = GTK_ICON_SIZE_LARGE_TOOLBAR;
-	gtk_toolbar_set_icon_size(GTK_TOOLBAR(main_widgets.toolbar), toolbar_prefs.icon_size);
-}
-
-
-void
-on_toolbar_small_icons1_activate       (GtkCheckMenuItem *menuitem,
-                                        gpointer          user_data)
-{
-	if (ignore_toolbar_toggle || ! gtk_check_menu_item_get_active(menuitem))
-		return;
-
-	toolbar_prefs.icon_size = GTK_ICON_SIZE_SMALL_TOOLBAR;
-	gtk_toolbar_set_icon_size(GTK_TOOLBAR(main_widgets.toolbar), toolbar_prefs.icon_size);
-}
-
-
-void
-on_very_small_icons1_activate          (GtkCheckMenuItem *menuitem,
-                                        gpointer          user_data)
-{
-	if (ignore_toolbar_toggle || ! gtk_check_menu_item_get_active(menuitem))
-		return;
-
-	toolbar_prefs.icon_size = GTK_ICON_SIZE_MENU;
-	gtk_toolbar_set_icon_size(GTK_TOOLBAR(main_widgets.toolbar), toolbar_prefs.icon_size);
+	else
+		on_find1_activate(NULL, NULL);
 }
 
 
@@ -655,6 +605,7 @@ on_zoom_in1_activate                   (GtkMenuItem     *menuitem,
 	sci_zoom_in(doc->editor->sci);
 }
 
+
 /* zoom out from menu bar and popup menu */
 void
 on_zoom_out1_activate                   (GtkMenuItem     *menuitem,
@@ -735,12 +686,14 @@ on_notebook1_switch_page_after         (GtkNotebook     *notebook,
 
 	if (doc != NULL)
 	{
-		treeviews_select_openfiles_item(doc);
-		document_set_text_changed(doc, doc->changed);	/* also sets window title and status bar */
+		sidebar_select_openfiles_item(doc);
+		ui_save_buttons_toggle(doc->changed);
+		ui_set_window_title(doc);
+		ui_update_statusbar(doc, -1);
 		ui_update_popup_reundo_items(doc);
 		ui_document_show_hide(doc); /* update the document menu */
 		build_menu_update(doc);
-		treeviews_update_tag_list(doc, FALSE);
+		sidebar_update_tag_list(doc, FALSE);
 
 		/* We delay the check to avoid weird fast, unintended switching of notebook pages when
 		 * the 'file has changed' dialog is shown while the switch event is not yet completely
@@ -777,18 +730,26 @@ on_tv_notebook_switch_page_after       (GtkNotebook     *notebook,
 }
 
 
+static void convert_eol(gint mode)
+{
+	GeanyDocument *doc = document_get_current();
+
+	g_return_if_fail(doc != NULL);
+
+	sci_convert_eols(doc->editor->sci, mode);
+	sci_set_eol_mode(doc->editor->sci, mode);
+	ui_update_statusbar(doc, -1);
+}
+
+
 void
 on_crlf_activate                       (GtkCheckMenuItem *menuitem,
                                         gpointer          user_data)
 {
-	GeanyDocument *doc;
 	if (ignore_callback || ! gtk_check_menu_item_get_active(menuitem))
 		return;
-	doc = document_get_current();
-	g_return_if_fail(doc != NULL);
 
-	sci_convert_eols(doc->editor->sci, SC_EOL_CRLF);
-	sci_set_eol_mode(doc->editor->sci, SC_EOL_CRLF);
+	convert_eol(SC_EOL_CRLF);
 }
 
 
@@ -796,14 +757,10 @@ void
 on_lf_activate                         (GtkCheckMenuItem *menuitem,
                                         gpointer          user_data)
 {
-	GeanyDocument *doc;
 	if (ignore_callback || ! gtk_check_menu_item_get_active(menuitem))
 		return;
-	doc = document_get_current();
-	g_return_if_fail(doc != NULL);
 
-	sci_convert_eols(doc->editor->sci, SC_EOL_LF);
-	sci_set_eol_mode(doc->editor->sci, SC_EOL_LF);
+	convert_eol(SC_EOL_LF);
 }
 
 
@@ -811,14 +768,10 @@ void
 on_cr_activate                         (GtkCheckMenuItem *menuitem,
                                         gpointer          user_data)
 {
-	GeanyDocument *doc;
 	if (ignore_callback || ! gtk_check_menu_item_get_active(menuitem))
 		return;
-	doc = document_get_current();
-	g_return_if_fail(doc != NULL);
 
-	sci_convert_eols(doc->editor->sci, SC_EOL_CR);
-	sci_set_eol_mode(doc->editor->sci, SC_EOL_CR);
+	convert_eol(SC_EOL_CR);
 }
 
 
@@ -841,32 +794,7 @@ toolbar_popup_menu                     (GtkWidget *widget,
 {
 	if (event->button == 3)
 	{
-		GtkWidget *w;
-
-		ignore_toolbar_toggle = TRUE;
-
-		switch (toolbar_prefs.icon_style)
-		{
-			case 0: w = ui_lookup_widget(ui_widgets.toolbar_menu, "images_only2"); break;
-			case 1: w = ui_lookup_widget(ui_widgets.toolbar_menu, "text_only2"); break;
-			default: w = ui_lookup_widget(ui_widgets.toolbar_menu, "images_and_text2"); break;
-		}
-		gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(w), TRUE);
-
-		switch (toolbar_prefs.icon_size)
-		{
-			case GTK_ICON_SIZE_LARGE_TOOLBAR:
-					w = ui_lookup_widget(ui_widgets.toolbar_menu, "large_icons1"); break;
-			case GTK_ICON_SIZE_SMALL_TOOLBAR:
-					w = ui_lookup_widget(ui_widgets.toolbar_menu, "small_icons1"); break;
-			default: w = ui_lookup_widget(ui_widgets.toolbar_menu, "very_small_icons1"); break;
-		}
-		gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(w), TRUE);
-
-		ignore_toolbar_toggle = FALSE;
-
 		gtk_menu_popup(GTK_MENU(ui_widgets.toolbar_menu), NULL, NULL, NULL, NULL, event->button, event->time);
-
 		return TRUE;
 	}
 	return FALSE;
@@ -958,10 +886,11 @@ void
 on_show_messages_window1_toggled       (GtkCheckMenuItem *checkmenuitem,
                                         gpointer          user_data)
 {
-	if (ignore_callback) return;
+	if (ignore_callback)
+		return;
 
 	ui_prefs.msgwindow_visible = (ui_prefs.msgwindow_visible) ? FALSE : TRUE;
-	ui_widget_show_hide(ui_lookup_widget(main_widgets.window, "scrolledwindow1"), ui_prefs.msgwindow_visible);
+	msgwin_show_hide(ui_prefs.msgwindow_visible);
 }
 
 
@@ -1121,8 +1050,15 @@ on_goto_tag_activate                   (GtkMenuItem     *menuitem,
 
 	g_return_if_fail(doc != NULL);
 
-	sci_set_current_position(doc->editor->sci, editor_info.click_pos, FALSE);
-	symbols_goto_tag(editor_info.current_word, definition);
+	/* update cursor pos for navigating back afterwards */
+	if (!sci_has_selection(doc->editor->sci))
+		sci_set_current_position(doc->editor->sci, editor_info.click_pos, FALSE);
+
+	/* use the keybinding callback as it checks for selections as well as current word */
+	if (definition)
+		keybindings_send_command(GEANY_KEY_GROUP_GOTO, GEANY_KEYS_GOTO_TAGDEFINITION);
+	else
+		keybindings_send_command(GEANY_KEY_GROUP_GOTO, GEANY_KEYS_GOTO_TAGDECLARATION);
 }
 
 
@@ -1240,21 +1176,45 @@ on_find_in_files1_activate             (GtkMenuItem     *menuitem,
 }
 
 
+static void get_line_and_offset_from_text(const gchar *text, gint *line_no, gint *offset)
+{
+	if (*text == '+' || *text == '-')
+	{
+		*line_no = atoi(text + 1);
+		*offset = (*text == '+') ? 1 : -1;
+	}
+	else
+	{
+		*line_no = atoi(text) - 1;
+		*offset = 0;
+	}
+}
+
+
 void
 on_go_to_line_activate                 (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
-	static gdouble val = 1;
+	static gchar value[16] = "";
+	gchar *result;
 
-	if (dialogs_show_input_numeric(_("Go to Line"), _("Enter the line you want to go to:"),
-			&val, 1, 100000000, 1))
+	result = dialogs_show_input_goto_line(
+		_("Go to Line"), _("Enter the line you want to go to:"), value);
+	if (result != NULL)
 	{
 		GeanyDocument *doc = document_get_current();
+		gint offset;
+		gint line_no;
 
 		g_return_if_fail(doc != NULL);
 
-		if (! editor_goto_line(doc->editor, (gint) val - 1))
+		get_line_and_offset_from_text(result, &line_no, &offset);
+		if (! editor_goto_line(doc->editor, line_no, offset))
 			utils_beep();
+		/* remember value for future calls */
+		g_snprintf(value, sizeof(value), "%s", result);
+
+		g_free(result);
 	}
 }
 
@@ -1263,11 +1223,16 @@ void
 on_toolbutton_goto_entry_activate(GtkAction *action, const gchar *text, gpointer user_data)
 {
 	GeanyDocument *doc = document_get_current();
+	gint offset;
+	gint line_no;
 
 	g_return_if_fail(doc != NULL);
 
-	if (! editor_goto_line(doc->editor, atoi(text) - 1))
+	get_line_and_offset_from_text(text, &line_no, &offset);
+	if (! editor_goto_line(doc->editor, line_no, offset))
 		utils_beep();
+	else
+		keybindings_send_command(GEANY_KEY_GROUP_FOCUS, GEANY_KEYS_FOCUS_EDITOR);
 }
 
 
@@ -1283,6 +1248,8 @@ on_toolbutton_goto_clicked             (GtkAction       *action,
 
 		on_toolbutton_goto_entry_activate(NULL, text, NULL);
 	}
+	else
+		on_go_to_line_activate(NULL, NULL);
 }
 
 
@@ -1290,24 +1257,9 @@ void
 on_help1_activate                      (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
-	gint skip;
 	gchar *uri;
 
-#ifdef G_OS_WIN32
-	skip = 8;
-	uri = g_strconcat("file:///", app->docdir, "/Manual.html", NULL);
-	g_strdelimit(uri, "\\", '/'); /* replace '\\' by '/' */
-#else
-	skip = 7;
-	uri = g_strconcat("file://", app->docdir, "index.html", NULL);
-#endif
-
-	if (! g_file_test(uri + skip, G_FILE_TEST_IS_REGULAR))
-	{	/* fall back to online documentation if it is not found on the hard disk */
-		g_free(uri);
-		uri = g_strconcat(GEANY_HOMEPAGE, "manual/", VERSION, "/index.html", NULL);
-	}
-
+	uri = utils_get_help_url(NULL);
 	utils_open_browser(uri);
 	g_free(uri);
 }
@@ -1350,7 +1302,7 @@ on_comments_function_activate          (GtkMenuItem     *menuitem,
 	line = symbols_get_current_function(doc, &cur_tag);
 	pos = sci_get_position_from_line(doc->editor->sci, line - 1);
 
-	text = templates_get_template_function(doc->file_type->id, cur_tag);
+	text = templates_get_template_function(doc, cur_tag);
 
 	sci_insert_text(doc->editor->sci, pos, text);
 	g_free(text);
@@ -1372,7 +1324,10 @@ on_comments_multiline_activate         (GtkMenuItem     *menuitem,
 
 	verify_click_pos(doc); /* make sure that the click_pos is valid */
 
-	editor_insert_multiline_comment(doc->editor);
+	if (doc->file_type->comment_open)
+		editor_insert_multiline_comment(doc->editor);
+	else
+		utils_beep();
 }
 
 
@@ -1385,7 +1340,7 @@ on_comments_gpl_activate               (GtkMenuItem     *menuitem,
 
 	g_return_if_fail(doc != NULL);
 
-	text = templates_get_template_licence(FILETYPE_ID(doc->file_type), GEANY_TEMPLATE_GPL);
+	text = templates_get_template_licence(doc, GEANY_TEMPLATE_GPL);
 
 	verify_click_pos(doc); /* make sure that the click_pos is valid */
 
@@ -1404,7 +1359,7 @@ on_comments_bsd_activate               (GtkMenuItem     *menuitem,
 
 	g_return_if_fail(doc != NULL);
 
-	text = templates_get_template_licence(FILETYPE_ID(doc->file_type), GEANY_TEMPLATE_BSD);
+	text = templates_get_template_licence(doc, GEANY_TEMPLATE_BSD);
 
 	verify_click_pos(doc); /* make sure that the click_pos is valid */
 
@@ -1423,7 +1378,7 @@ on_comments_changelog_activate         (GtkMenuItem     *menuitem,
 
 	g_return_if_fail(doc != NULL);
 
-	text = templates_get_template_changelog();
+	text = templates_get_template_changelog(doc);
 	sci_insert_text(doc->editor->sci, 0, text);
 	/* sets the cursor to the right position to type the changelog text,
 	 * the template has 21 chars + length of name and email */
@@ -1439,7 +1394,7 @@ on_comments_fileheader_activate        (GtkMenuItem     *menuitem,
 {
 	GeanyDocument *doc = document_get_current();
 	gchar *text;
-	gchar *fname;
+	const gchar *fname;
 	GeanyFiletype *ft;
 
 	g_return_if_fail(doc != NULL);
@@ -1454,23 +1409,23 @@ on_comments_fileheader_activate        (GtkMenuItem     *menuitem,
 }
 
 
-static void
-on_custom_date_input_response(const gchar *input)
-{
-	setptr(ui_prefs.custom_date_format, g_strdup(input));
-}
-
-
 void
 on_insert_date_activate                (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
 	GeanyDocument *doc = document_get_current();
-	gchar *format;
+	const gchar *format = NULL;
 	gchar *time_str;
 
 	g_return_if_fail(doc != NULL);
 
+	/* set default value */
+	if (utils_str_equal("", ui_prefs.custom_date_format))
+	{
+		g_free(ui_prefs.custom_date_format);
+		ui_prefs.custom_date_format = g_strdup("%d.%m.%Y");
+	}
+
 	if (utils_str_equal(_("dd.mm.yyyy"), (gchar*) user_data))
 		format = "%d.%m.%Y";
 	else if (utils_str_equal(_("mm.dd.yyyy"), (gchar*) user_data))
@@ -1487,16 +1442,11 @@ on_insert_date_activate                (GtkMenuItem     *menuitem,
 		format = ui_prefs.custom_date_format;
 	else
 	{
-		/* set default value */
-		if (utils_str_equal("", ui_prefs.custom_date_format))
-		{
-			g_free(ui_prefs.custom_date_format);
-			ui_prefs.custom_date_format = g_strdup("%d.%m.%Y");
-		}
-
-		dialogs_show_input(_("Custom Date Format"),
-			_("Enter here a custom date and time format. You can use any conversion specifiers which can be used with the ANSI C strftime function."),
-			ui_prefs.custom_date_format, FALSE, &on_custom_date_input_response);
+		setptr(ui_prefs.custom_date_format,
+			dialogs_show_input(_("Custom Date Format"),
+				_("Enter here a custom date and time format. "
+				"You can use any conversion specifiers which can be used with the ANSI C strftime function."),
+				ui_prefs.custom_date_format));
 		return;
 	}
 
@@ -1624,19 +1574,28 @@ on_menu_select_all1_activate           (GtkMenuItem     *menuitem,
 
 void
 on_menu_show_sidebar1_toggled          (GtkCheckMenuItem *checkmenuitem,
-                                        gpointer         user_data)
+                                        gpointer          user_data)
 {
 	if (ignore_callback)
 		return;
 
 	ui_prefs.sidebar_visible = ! ui_prefs.sidebar_visible;
 
-	if ((! interface_prefs.sidebar_openfiles_visible && ! interface_prefs.sidebar_symbol_visible))
+	if (! interface_prefs.sidebar_openfiles_visible && ! interface_prefs.sidebar_symbol_visible)
 	{
 		interface_prefs.sidebar_openfiles_visible = TRUE;
 		interface_prefs.sidebar_symbol_visible = TRUE;
 	}
 
+#if GTK_CHECK_VERSION(2, 14, 0)
+	/* if window has input focus, set it back to the editor before toggling off */
+	if (! ui_prefs.sidebar_visible &&
+		gtk_container_get_focus_child(GTK_CONTAINER(main_widgets.sidebar_notebook)) != NULL)
+	{
+		keybindings_send_command(GEANY_KEY_GROUP_FOCUS, GEANY_KEYS_FOCUS_EDITOR);
+	}
+#endif
+
 	ui_sidebar_show_hide();
 }
 
@@ -1699,14 +1658,6 @@ on_menu_toggle_line_commentation1_activate
 }
 
 
-void
-on_menu_duplicate_line1_activate       (GtkMenuItem     *menuitem,
-                                        gpointer         user_data)
-{
-	keybindings_send_command(GEANY_KEY_GROUP_EDITOR, GEANY_KEYS_EDITOR_DUPLICATELINE);
-}
-
-
 void
 on_menu_increase_indent1_activate      (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
@@ -1866,7 +1817,7 @@ on_menu_open_selected_file1_activate   (GtkMenuItem     *menuitem,
 
 	if (sel != NULL)
 	{
-		gchar *locale_filename, *filename;
+		gchar *locale_filename, *filename = NULL;
 
 		if (g_path_is_absolute(sel))
 			filename = g_strdup(sel);
@@ -1874,7 +1825,10 @@ on_menu_open_selected_file1_activate   (GtkMenuItem     *menuitem,
 		{	/* relative filename, add the path of the current file */
 			gchar *path;
 
-			path = g_path_get_dirname(doc->file_name);
+			path = utils_get_current_file_dir_utf8();
+			if (!path)
+				path = g_get_current_dir();
+
 			filename = g_build_path(G_DIR_SEPARATOR_S, path, sel, NULL);
 
 			if (! g_file_test(filename, G_FILE_TEST_EXISTS) &&
@@ -2143,6 +2097,7 @@ on_line_breaking1_activate             (GtkMenuItem     *menuitem,
 	doc->editor->line_breaking = !doc->editor->line_breaking;
 }
 
+
 void
 on_replace_spaces_activate             (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
@@ -2219,6 +2174,7 @@ on_send_selection_to_vte1_activate     (GtkMenuItem     *menuitem,
 #endif
 }
 
+
 gboolean on_window_state_event         (GtkWidget           *widget,
                                         GdkEventWindowState *event,
                                         gpointer             user_data)
@@ -2246,7 +2202,17 @@ void
 on_customize_toolbar1_activate         (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
-	toolbar_configure();
+	GtkWidget *widget;
+	GtkNotebook *notebook;
+
+	prefs_show_dialog();
+
+	/* select the KB page */
+	widget = ui_lookup_widget(ui_widgets.prefs_dialog, "vbox15");
+	notebook = GTK_NOTEBOOK(ui_lookup_widget(ui_widgets.prefs_dialog, "notebook2"));
+
+	if (notebook != NULL && widget != NULL)
+		gtk_notebook_set_current_page(notebook, gtk_notebook_page_num(notebook, widget));
 }
 
 
@@ -2254,6 +2220,114 @@ void
 on_button_customize_toolbar_clicked    (GtkButton       *button,
                                         gpointer         user_data)
 {
-	toolbar_configure();
+	toolbar_configure(GTK_WINDOW(ui_widgets.prefs_dialog));
+}
+
+
+void
+on_cut_current_line_s_1_activate       (GtkMenuItem     *menuitem,
+                                        gpointer         user_data)
+{
+	keybindings_send_command(GEANY_KEY_GROUP_CLIPBOARD, GEANY_KEYS_CLIPBOARD_CUTLINE);
+}
+
+
+void
+on_copy_current_line_s_1_activate      (GtkMenuItem     *menuitem,
+                                        gpointer         user_data)
+{
+	keybindings_send_command(GEANY_KEY_GROUP_CLIPBOARD, GEANY_KEYS_CLIPBOARD_COPYLINE);
+}
+
+
+void
+on_delete_current_line_s_1_activate    (GtkMenuItem     *menuitem,
+                                        gpointer         user_data)
+{
+	keybindings_send_command(GEANY_KEY_GROUP_EDITOR, GEANY_KEYS_EDITOR_DELETELINE);
+}
+
+
+void
+on_duplicate_line_or_selection1_activate
+                                        (GtkMenuItem     *menuitem,
+                                        gpointer         user_data)
+{
+	keybindings_send_command(GEANY_KEY_GROUP_EDITOR, GEANY_KEYS_EDITOR_DUPLICATELINE);
+}
+
+
+void
+on_select_current_line_s_1_activate    (GtkMenuItem     *menuitem,
+                                        gpointer         user_data)
+{
+	keybindings_send_command(GEANY_KEY_GROUP_SELECT, GEANY_KEYS_SELECT_LINE);
+}
+
+
+void
+on_select_current_paragraph1_activate  (GtkMenuItem     *menuitem,
+                                        gpointer         user_data)
+{
+	keybindings_send_command(GEANY_KEY_GROUP_SELECT, GEANY_KEYS_SELECT_PARAGRAPH);
+}
+
+
+void
+on_insert_alternative_white_space1_activate
+                                        (GtkMenuItem     *menuitem,
+                                        gpointer         user_data)
+{
+	keybindings_send_command(GEANY_KEY_GROUP_INSERT, GEANY_KEYS_INSERT_ALTWHITESPACE);
+}
+
+
+void
+on_go_to_next_marker1_activate         (GtkMenuItem     *menuitem,
+                                        gpointer         user_data)
+{
+	keybindings_send_command(GEANY_KEY_GROUP_GOTO, GEANY_KEYS_GOTO_NEXTMARKER);
+}
+
+
+void
+on_go_to_previous_marker1_activate     (GtkMenuItem     *menuitem,
+                                        gpointer         user_data)
+{
+	keybindings_send_command(GEANY_KEY_GROUP_GOTO, GEANY_KEYS_GOTO_PREVIOUSMARKER);
+}
+
+
+void
+on_reflow_lines_block1_activate        (GtkMenuItem     *menuitem,
+                                        gpointer         user_data)
+{
+	keybindings_send_command(GEANY_KEY_GROUP_FORMAT, GEANY_KEYS_FORMAT_REFLOWPARAGRAPH);
+}
+
+
+void
+on_transpose_current_line1_activate    (GtkMenuItem     *menuitem,
+                                        gpointer         user_data)
+{
+	keybindings_send_command(GEANY_KEY_GROUP_EDITOR, GEANY_KEYS_EDITOR_TRANSPOSELINE);
+}
+
+
+void
+on_smart_line_indent1_activate         (GtkMenuItem     *menuitem,
+                                        gpointer         user_data)
+{
+	keybindings_send_command(GEANY_KEY_GROUP_FORMAT, GEANY_KEYS_FORMAT_AUTOINDENT);
+}
+
+
+void
+on_plugin_preferences1_activate        (GtkMenuItem     *menuitem,
+                                        gpointer         user_data)
+{
+#ifdef HAVE_PLUGINS
+	plugin_show_configure(NULL);
+#endif
 }
 
