@@ -1,8 +1,8 @@
 /*
  *      callbacks.h - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2005-2008 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2006-2008 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2005-2010 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2006-2010 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -18,21 +18,13 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: callbacks.h 3094 2008-10-15 11:59:32Z ntrel $
+ * $Id: callbacks.h 4818 2010-04-11 21:56:23Z eht16 $
  */
 
 
 #include "geany.h" /* necessary for interface.c */
 
 
-typedef struct
-{
-	GeanyDocument *last_doc;
-} CallbacksData;
-
-extern CallbacksData callbacks_data;
-
-
 gboolean
 on_exit_clicked                        (GtkWidget *widget, gpointer gdata);
 
@@ -65,13 +57,11 @@ on_change_font1_activate               (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
 
 void
-on_toolbutton_close_clicked            (GtkToolButton   *toolbutton,
+on_toolbutton_close_clicked            (GtkAction       *action,
                                         gpointer         user_data);
 
 void
-on_notebook1_switch_page               (GtkNotebook     *notebook,
-                                        GtkNotebookPage *page,
-                                        guint            page_num,
+on_toolbutton_close_all_clicked        (GtkAction       *action,
                                         gpointer         user_data);
 
 void
@@ -84,30 +74,26 @@ on_close1_activate                     (GtkMenuItem     *menuitem,
 
 void
 on_close_all1_activate                 (GtkMenuItem     *menuitem,
-                                        gpointer         user_data);
+                                        gpointer          user_data);
 
 void
-on_crlf_activate                       (GtkMenuItem     *menuitem,
-                                        gpointer         user_data);
+on_crlf_activate                       (GtkCheckMenuItem *menuitem,
+                                        gpointer          user_data);
 
 void
-on_lf_activate                         (GtkMenuItem     *menuitem,
-                                        gpointer         user_data);
+on_lf_activate                         (GtkCheckMenuItem *menuitem,
+                                        gpointer          user_data);
 
 void
-on_cr_activate                         (GtkMenuItem     *menuitem,
-                                        gpointer         user_data);
+on_cr_activate                         (GtkCheckMenuItem *menuitem,
+                                        gpointer          user_data);
 
 void
 on_replace_tabs_activate               (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
 
 void
-on_toolbutton_search_clicked           (GtkToolButton   *toolbutton,
-                                        gpointer         user_data);
-
-void
-on_toolbar_search_entry_activate       (GtkEntry        *entry,
+on_toolbutton_search_clicked           (GtkAction       *action,
                                         gpointer         user_data);
 
 gboolean
@@ -115,18 +101,6 @@ toolbar_popup_menu                     (GtkWidget *widget,
                                         GdkEventButton *event,
                                         gpointer user_data);
 
-void
-on_images_and_text2_activate           (GtkMenuItem     *menuitem,
-                                        gpointer         user_data);
-
-void
-on_images_only2_activate               (GtkMenuItem     *menuitem,
-                                        gpointer         user_data);
-
-void
-on_text_only2_activate                 (GtkMenuItem     *menuitem,
-                                        gpointer         user_data);
-
 void
 on_hide_toolbar1_activate              (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
@@ -164,8 +138,9 @@ on_zoom_out1_activate                  (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
 
 void
-on_toolbar_search_entry_changed        (GtkEditable     *editable,
-                                        gpointer         user_data);
+on_toolbar_search_entry_changed        (GtkAction *action,
+                                        const gchar *text,
+                                        gpointer user_data);
 
 void
 on_toggle_case1_activate               (GtkMenuItem     *menuitem,
@@ -254,45 +229,39 @@ on_find_in_files1_activate             (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
 
 void
-on_toolbutton_new_clicked              (GtkToolButton   *toolbutton,
+on_toolbutton_new_clicked              (GtkAction       *action,
                                         gpointer         user_data);
 
 void
-on_toolbutton_open_clicked             (GtkToolButton   *toolbutton,
+on_toolbutton_open_clicked             (GtkAction       *action,
                                         gpointer         user_data);
 
 void
-on_toolbutton_save_clicked             (GtkToolButton   *toolbutton,
+on_toolbutton_save_clicked             (GtkAction       *action,
                                         gpointer         user_data);
 
 void
-on_toolbutton_quit_clicked             (GtkToolButton   *toolbutton,
+on_toolbutton_quit_clicked             (GtkAction       *action,
                                         gpointer         user_data);
 
 void
-on_toolbutton_reload_clicked           (GtkToolButton   *toolbutton,
+on_toolbutton_preferences_clicked      (GtkAction       *action,
+                                        gpointer         user_data);
+
+void
+on_toolbutton_reload_clicked           (GtkAction       *action,
                                         gpointer         user_data);
 
 void
 on_go_to_line_activate                 (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
 
-void
-on_goto_line_dialog_response           (GtkDialog *dialog,
-                                        gint response,
-                                        gpointer user_data);
-
-void
-on_goto_line_entry_activate            (GtkEntry        *entry,
-                                        gpointer         user_data);
-
-
 void
 on_help1_activate                      (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
 
 void
-on_compile_button_clicked              (GtkToolButton   *toolbutton,
+on_toolbutton_compile_clicked          (GtkAction       *action,
                                         gpointer         user_data);
 
 void
@@ -340,14 +309,6 @@ void
 on_insert_date_activate                (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
 
-void
-on_toolbar_large_icons1_activate       (GtkMenuItem     *menuitem,
-                                        gpointer         user_data);
-
-void
-on_toolbar_small_icons1_activate       (GtkMenuItem     *menuitem,
-                                        gpointer         user_data);
-
 void
 on_set_file_readonly1_toggled          (GtkCheckMenuItem *checkmenuitem,
                                         gpointer         user_data);
@@ -386,19 +347,14 @@ on_menu_unfold_all1_activate           (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
 
 void
-on_toolbutton_goto_entry_activate      (GtkEntry        *entry,
+on_toolbutton_goto_entry_activate      (GtkAction *action, const gchar *text, gpointer user_data);
+
+void
+on_toolbutton_goto_clicked             (GtkAction       *action,
                                         gpointer         user_data);
 
 void
-on_toolbutton_goto_clicked             (GtkToolButton   *toolbutton,
-                                        gpointer         user_data);
-
-void
-on_run_button_clicked                  (GtkToolButton   *toolbutton,
-                                        gpointer         user_data);
-
-void
-on_go_to_line1_activate                (GtkMenuItem     *menuitem,
+on_toolbutton_run_clicked              (GtkAction       *action,
                                         gpointer         user_data);
 
 void
@@ -411,10 +367,6 @@ void
 on_menu_remove_indicators1_activate    (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
 
-void
-on_encoding_change                     (GtkMenuItem     *menuitem,
-                                        gpointer         user_data);
-
 void
 on_reload_as_activate                  (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
@@ -443,10 +395,6 @@ void
 on_menu_uncomment_line1_activate       (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
 
-void
-on_menu_duplicate_line1_activate       (GtkMenuItem     *menuitem,
-                                        gpointer         user_data);
-
 void
 on_menu_increase_indent1_activate      (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
@@ -553,12 +501,12 @@ on_tv_notebook_switch_page_after       (GtkNotebook     *notebook,
                                         gpointer         user_data);
 
 void
-on_tabs1_activate                      (GtkMenuItem     *menuitem,
-                                        gpointer         user_data);
+on_tabs1_activate                      (GtkCheckMenuItem *menuitem,
+                                        gpointer          user_data);
 
 void
-on_spaces1_activate                    (GtkMenuItem     *menuitem,
-                                        gpointer         user_data);
+on_spaces1_activate                    (GtkCheckMenuItem *menuitem,
+                                        gpointer          user_data);
 
 
 void
@@ -604,8 +552,91 @@ on_find_document_usage1_activate       (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
 
 void
-on_tabs_and_spaces1_activate           (GtkMenuItem     *menuitem,
-                                        gpointer         user_data);
+on_tabs_and_spaces1_activate           (GtkCheckMenuItem *menuitem,
+                                        gpointer          user_data);
 void
 on_debug_messages1_activate            (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
+
+void
+on_menu_show_white_space1_toggled      (GtkCheckMenuItem *checkmenuitem,
+                                        gpointer         user_data);
+
+void
+on_menu_show_line_endings1_toggled     (GtkCheckMenuItem *checkmenuitem,
+                                        gpointer         user_data);
+
+void
+on_menu_show_indentation_guides1_toggled
+                                        (GtkCheckMenuItem *checkmenuitem,
+                                        gpointer         user_data);
+
+void
+on_send_selection_to_vte1_activate     (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+gboolean on_window_state_event         (GtkWidget           *widget,
+                                        GdkEventWindowState *event,
+                                        gpointer             user_data);
+
+void
+on_customize_toolbar1_activate         (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_button_customize_toolbar_clicked    (GtkButton       *button,
+                                        gpointer         user_data);
+
+void
+on_cut_current_line_s_1_activate       (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_copy_current_line_s_1_activate      (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_delete_current_line_s_1_activate    (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_duplicate_line_or_selection1_activate
+                                        (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_select_current_line_s_1_activate    (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_select_current_paragraph1_activate  (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_insert_alternative_white_space1_activate
+                                        (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_go_to_next_marker1_activate         (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_go_to_previous_marker1_activate     (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_reflow_lines_block1_activate        (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_transpose_current_line1_activate    (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_smart_line_indent1_activate         (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_plugin_preferences1_activate        (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
