@@ -1,8 +1,8 @@
 /*
  *      dialogs.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2005-2009 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2006-2009 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2005-2010 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2006-2010 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: dialogs.c 4010 2009-07-21 22:12:13Z eht16 $
+ * $Id: dialogs.c 4929 2010-05-16 18:22:24Z eht16 $
  */
 
 /*
@@ -65,7 +65,11 @@ enum
 	GEANY_RESPONSE_VIEW
 };
 
-#if ! GEANY_USE_WIN32_DIALOG
+
+static gboolean handle_save_as(const gchar *utf8_filename, gboolean open_new_tab,
+	gboolean rename_file);
+
+
 static GtkWidget *add_file_open_extra_widget(void);
 
 
@@ -84,11 +88,11 @@ on_file_open_dialog_response           (GtkDialog *dialog,
 		gint encoding_idx = gtk_combo_box_get_active(GTK_COMBO_BOX(
 						ui_lookup_widget(GTK_WIDGET(dialog), "encoding_combo")));
 		GeanyFiletype *ft = NULL;
-		gchar *charset = NULL;
+		const gchar *charset = NULL;
 		gboolean ro = (response == GEANY_RESPONSE_VIEW);	/* View clicked */
 
 		/* ignore detect from file item */
-		if (filetype_idx > 0 && filetype_idx < GEANY_MAX_BUILT_IN_FILETYPES)
+		if (filetype_idx > 0)
 			ft = g_slist_nth_data(filetypes_by_title, filetype_idx);
 		if (encoding_idx >= 0 && encoding_idx < GEANY_ENCODINGS_MAX)
 			charset = encodings[encoding_idx].charset;
@@ -203,7 +207,6 @@ static void create_open_file_dialog(void)
 	g_signal_connect(ui_widgets.open_filesel, "response",
 				G_CALLBACK(on_file_open_dialog_response), NULL);
 }
-#endif
 
 
 /* This shows the file selection dialog to open a file. */
@@ -221,34 +224,35 @@ void dialogs_show_open_file()
 
 	setptr(initdir, utils_get_locale_from_utf8(initdir));
 
-#if GEANY_USE_WIN32_DIALOG
-	win32_show_file_dialog(TRUE, initdir);
-#else /* X11, not win32: use GTK_FILE_CHOOSER */
-
-	/* We use the same file selection widget each time, so first of all we create it
-	 * if it hasn't already been created. */
-	if (ui_widgets.open_filesel == NULL)
-		create_open_file_dialog();
-
-	if (initdir != NULL)
-	{
-		if (g_path_is_absolute(initdir))
-			gtk_file_chooser_set_current_folder(
-				GTK_FILE_CHOOSER(ui_widgets.open_filesel), initdir);
-	}
-
-	if (app->project && NZV(app->project->base_path))
-		gtk_file_chooser_add_shortcut_folder(GTK_FILE_CHOOSER(ui_widgets.open_filesel),
-			app->project->base_path, NULL);
-
-	gtk_file_chooser_unselect_all(GTK_FILE_CHOOSER(ui_widgets.open_filesel));
-	gtk_window_present(GTK_WINDOW(ui_widgets.open_filesel));
+#ifdef G_OS_WIN32
+	if (interface_prefs.use_native_windows_dialogs)
+		win32_show_document_open_dialog(GTK_WINDOW(main_widgets.window), _("Open File"), initdir);
+	else
 #endif
+	{
+		/* We use the same file selection widget each time, so first of all we create it
+		 * if it hasn't already been created. */
+		if (ui_widgets.open_filesel == NULL)
+			create_open_file_dialog();
+
+		if (initdir != NULL)
+		{
+			if (g_path_is_absolute(initdir))
+				gtk_file_chooser_set_current_folder(
+					GTK_FILE_CHOOSER(ui_widgets.open_filesel), initdir);
+		}
+
+		if (app->project && NZV(app->project->base_path))
+			gtk_file_chooser_add_shortcut_folder(GTK_FILE_CHOOSER(ui_widgets.open_filesel),
+				app->project->base_path, NULL);
+
+		gtk_file_chooser_unselect_all(GTK_FILE_CHOOSER(ui_widgets.open_filesel));
+		gtk_window_present(GTK_WINDOW(ui_widgets.open_filesel));
+	}
 	g_free(initdir);
 }
 
 
-#if ! GEANY_USE_WIN32_DIALOG
 static GtkWidget *add_file_open_extra_widget()
 {
 	GtkWidget *expander, *vbox, *table, *check_hidden;
@@ -321,19 +325,15 @@ static GtkWidget *add_file_open_extra_widget()
 
 	return expander;
 }
-#endif
 
 
-#if ! GEANY_USE_WIN32_DIALOG
 static void on_save_as_new_tab_toggled(GtkToggleButton *togglebutton, gpointer user_data)
 {
 	gtk_widget_set_sensitive(GTK_WIDGET(user_data),
 		! gtk_toggle_button_get_active(togglebutton));
 }
-#endif
 
 
-#if ! GEANY_USE_WIN32_DIALOG
 static gboolean handle_save_as(const gchar *utf8_filename, gboolean open_new_tab, gboolean rename_file)
 {
 	GeanyDocument *doc = document_get_current();
@@ -412,10 +412,8 @@ on_file_save_dialog_response           (GtkDialog *dialog,
 	if (success)
 		gtk_widget_hide(ui_widgets.save_filesel);
 }
-#endif
 
 
-#if ! GEANY_USE_WIN32_DIALOG
 static void create_save_file_dialog(void)
 {
 	GtkWidget *vbox, *check_open_new_tab, *rename_btn;
@@ -472,15 +470,15 @@ static void create_save_file_dialog(void)
 
 	gtk_window_set_transient_for(GTK_WINDOW(ui_widgets.save_filesel), GTK_WINDOW(main_widgets.window));
 }
-#endif
 
 
-#if ! GEANY_USE_WIN32_DIALOG
 static gboolean gtk_show_save_as(void)
 {
 	GeanyDocument *doc = document_get_current();
 	gint resp;
 
+	g_return_val_if_fail(doc != NULL, FALSE);
+
 	if (G_UNLIKELY(ui_widgets.save_filesel == NULL))
 		create_save_file_dialog();
 
@@ -511,8 +509,7 @@ static gboolean gtk_show_save_as(void)
 	{
 		gchar *fname = NULL;
 
-		if (doc->file_type != NULL && doc->file_type->id != GEANY_FILETYPES_NONE &&
-			doc->file_type->extension != NULL)
+		if (doc->file_type != NULL && doc->file_type->extension != NULL)
 			fname = g_strconcat(GEANY_STRING_UNTITLED, ".",
 								doc->file_type->extension, NULL);
 		else
@@ -536,23 +533,29 @@ static gboolean gtk_show_save_as(void)
 
 	return (resp == GTK_RESPONSE_ACCEPT);
 }
-#endif
 
 
 /**
- *  Show the Save As dialog for the current notebook page.
+ *  Shows the Save As dialog for the current notebook page.
  *
  *  @return @c TRUE if the file was saved, otherwise @c FALSE.
  **/
 gboolean dialogs_show_save_as()
 {
-	gboolean result;
+	gboolean result = FALSE;
 
-#if GEANY_USE_WIN32_DIALOG
-	result = win32_show_file_dialog(FALSE, utils_get_default_dir_utf8());
-#else
-	result = gtk_show_save_as();
+#ifdef G_OS_WIN32
+	if (interface_prefs.use_native_windows_dialogs)
+	{
+		GeanyDocument *doc = document_get_current();
+		gchar *utf8_name = win32_show_document_save_as_dialog(GTK_WINDOW(main_widgets.window),
+						_("Save File"), DOC_FILENAME(doc));
+		if (utf8_name != NULL)
+			result = handle_save_as(utf8_name, FALSE, FALSE);
+	}
+	else
 #endif
+	result = gtk_show_save_as();
 	return result;
 }
 
@@ -577,7 +580,7 @@ static void show_msgbox_dialog(GtkWidget *dialog, GtkMessageType type, GtkWindow
 			break;
 	}
 	gtk_window_set_title(GTK_WINDOW(dialog), title);
-	if (parent == NULL)
+	if (parent == NULL || GTK_IS_DIALOG(parent))
 	{
 		GdkPixbuf *pb = ui_new_pixbuf_from_inline(GEANY_IMAGE_LOGO);
 		gtk_window_set_icon(GTK_WINDOW(dialog), pb);
@@ -592,7 +595,7 @@ static void show_msgbox_dialog(GtkWidget *dialog, GtkMessageType type, GtkWindow
 
 
 /**
- *  Show a message box of the type @a type with @a text.
+ *  Shows a message box of the type @a type with @a text.
  *  On Unix-like systems a GTK message dialog box is shown, on Win32 systems a native Windows
  *  message dialog box is shown.
  *
@@ -606,12 +609,12 @@ void dialogs_show_msgbox(GtkMessageType type, const gchar *text, ...)
 #ifndef G_OS_WIN32
 	GtkWidget *dialog;
 #endif
-	gchar string[512];
+	gchar *string;
 	va_list args;
 	GtkWindow *parent = (main_status.main_window_realized) ? GTK_WINDOW(main_widgets.window) : NULL;
 
 	va_start(args, text);
-	g_vsnprintf(string, 511, text, args);
+	string = g_strdup_vprintf(text, args);
 	va_end(args);
 
 #ifdef G_OS_WIN32
@@ -621,6 +624,7 @@ void dialogs_show_msgbox(GtkMessageType type, const gchar *text, ...)
                                   type, GTK_BUTTONS_OK, "%s", string);
 	show_msgbox_dialog(dialog, type, parent);
 #endif
+	g_free(string);
 }
 
 
@@ -697,11 +701,11 @@ gboolean dialogs_show_unsaved_file(GeanyDocument *doc)
 #endif
 	g_free(msg);
 
-	switch(ret)
+	switch (ret)
 	{
 		case GTK_RESPONSE_YES:
 		{
-			if (doc->file_name == NULL)
+			if (document_need_save_as(doc))
 			{
 				ret = dialogs_show_save_as();
 			}
@@ -821,8 +825,8 @@ on_input_dialog_response(GtkDialog *dialog,
 	if (response == GTK_RESPONSE_ACCEPT)
 	{
 		const gchar *str = gtk_entry_get_text(GTK_ENTRY(entry));
-		InputCallback input_cb =
-			(InputCallback) g_object_get_data(G_OBJECT(dialog), "input_cb");
+		GeanyInputCallback input_cb =
+			(GeanyInputCallback) g_object_get_data(G_OBJECT(dialog), "input_cb");
 
 		if (persistent)
 		{
@@ -831,37 +835,37 @@ on_input_dialog_response(GtkDialog *dialog,
 		}
 		input_cb(str);
 	}
-
-	if (persistent)
-		gtk_widget_hide(GTK_WIDGET(dialog));
-	else
-		gtk_widget_destroy(GTK_WIDGET(dialog));
+	gtk_widget_hide(GTK_WIDGET(dialog));
 }
 
 
 static void add_input_widgets(GtkWidget *dialog, GtkWidget *vbox,
-		const gchar *label_text, const gchar *default_text, gboolean persistent)
+		const gchar *label_text, const gchar *default_text, gboolean persistent,
+		GCallback insert_text_cb)
 {
-	GtkWidget *label, *entry;
+	GtkWidget *entry;
 
-	label = gtk_label_new(label_text);
-	gtk_label_set_line_wrap(GTK_LABEL(label), TRUE);
-	gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
-	gtk_container_add(GTK_CONTAINER(vbox), label);
+	if (label_text)
+	{
+		GtkWidget *label = gtk_label_new(label_text);
+		gtk_label_set_line_wrap(GTK_LABEL(label), TRUE);
+		gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
+		gtk_container_add(GTK_CONTAINER(vbox), label);
+	}
 
 	if (persistent)	/* remember previous entry text in a combo box */
 	{
 		GtkWidget *combo = gtk_combo_box_entry_new_text();
 
 		entry = GTK_BIN(combo)->child;
-		ui_entry_add_clear_icon(entry);
+		ui_entry_add_clear_icon(GTK_ENTRY(entry));
 		g_object_set_data(G_OBJECT(dialog), "combo", combo);
 		gtk_container_add(GTK_CONTAINER(vbox), combo);
 	}
 	else
 	{
 		entry = gtk_entry_new();
-		ui_entry_add_clear_icon(entry);
+		ui_entry_add_clear_icon(GTK_ENTRY(entry));
 		gtk_container_add(GTK_CONTAINER(vbox), entry);
 	}
 
@@ -872,6 +876,8 @@ static void add_input_widgets(GtkWidget *dialog, GtkWidget *vbox,
 	gtk_entry_set_max_length(GTK_ENTRY(entry), 255);
 	gtk_entry_set_width_chars(GTK_ENTRY(entry), 30);
 
+	if (insert_text_cb != NULL)
+		g_signal_connect(entry, "insert-text", insert_text_cb, NULL);
 	g_signal_connect(entry, "activate", G_CALLBACK(on_input_entry_activate), dialog);
 	g_signal_connect(dialog, "show", G_CALLBACK(on_input_dialog_show), entry);
 	g_signal_connect(dialog, "response", G_CALLBACK(on_input_dialog_response), entry);
@@ -883,9 +889,9 @@ static void add_input_widgets(GtkWidget *dialog, GtkWidget *vbox,
  * 	in this case the dialog returned is not destroyed on a response,
  * 	and can be reshown.
  * Returns: the dialog widget. */
-GtkWidget *
-dialogs_show_input(const gchar *title, const gchar *label_text, const gchar *default_text,
-						gboolean persistent, InputCallback input_cb)
+static GtkWidget *
+dialogs_show_input_full(const gchar *title, const gchar *label_text, const gchar *default_text,
+						gboolean persistent, GeanyInputCallback input_cb, GCallback insert_text_cb)
 {
 	GtkWidget *dialog, *vbox;
 
@@ -899,20 +905,66 @@ dialogs_show_input(const gchar *title, const gchar *label_text, const gchar *def
 	g_object_set_data(G_OBJECT(dialog), "has_combo", GINT_TO_POINTER(persistent));
 	g_object_set_data(G_OBJECT(dialog), "input_cb", (gpointer) input_cb);
 
-	add_input_widgets(dialog, vbox, label_text, default_text, persistent);
+	add_input_widgets(dialog, vbox, label_text, default_text, persistent, insert_text_cb);
 
 	if (persistent)
+	{
+		/* override default handler */
 		g_signal_connect(dialog, "delete-event", G_CALLBACK(gtk_widget_hide_on_delete), NULL);
-	else
-		g_signal_connect(dialog, "delete-event", G_CALLBACK(gtk_widget_destroy), NULL);
-
+		gtk_widget_show_all(dialog);
+		return dialog;
+	}
 	gtk_widget_show_all(dialog);
-	return dialog;
+	gtk_dialog_run(GTK_DIALOG(dialog));
+	gtk_widget_destroy(dialog);
+	return NULL;
+}
+
+
+/* Remember previous entry text in a combo box.
+ * Returns: the dialog widget. */
+GtkWidget *
+dialogs_show_input_persistent(const gchar *title, const gchar *label_text, const gchar *default_text,
+		GeanyInputCallback input_cb)
+{
+	return dialogs_show_input_full(title, label_text, default_text, TRUE, input_cb, NULL);
+}
+
+
+/* ugly hack - user_data not supported for callback */
+static gchar *dialog_input = NULL;
+
+static void on_dialog_input(const gchar *str)
+{
+	dialog_input = g_strdup(str);
+}
+
+
+/* Returns: newly allocated string - a copy of either the entry text or default_text. */
+gchar *dialogs_show_input(const gchar *title, const gchar *label_text,
+	const gchar *default_text)
+{
+	dialog_input = NULL;
+	dialogs_show_input_full(title, label_text, default_text, FALSE, on_dialog_input, NULL);
+	return NVL(dialog_input, g_strdup(default_text));
+}
+
+
+/* Returns: newly allocated copy of the entry text or NULL on cancel.
+ * Specialised variant for Goto Line dialog. */
+gchar *dialogs_show_input_goto_line(const gchar *title, const gchar *label_text,
+	const gchar *default_text)
+{
+	dialog_input = NULL;
+	dialogs_show_input_full(
+		title, label_text, default_text, FALSE, on_dialog_input,
+		G_CALLBACK(ui_editable_insert_text_callback));
+	return dialog_input;
 }
 
 
 /**
- *  Show an input box to enter a numerical value using a GtkSpinButton.
+ *  Shows an input box to enter a numerical value using a GtkSpinButton.
  *  If the dialog is aborted, @a value remains untouched.
  *
  *  @param title The dialog title.
@@ -950,7 +1002,7 @@ gboolean dialogs_show_input_numeric(const gchar *title, const gchar *label_text,
 	gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
 
 	spin = gtk_spin_button_new_with_range(min, max, step);
-	ui_entry_add_clear_icon(spin);
+	ui_entry_add_clear_icon(GTK_ENTRY(spin));
 	gtk_spin_button_set_value(GTK_SPIN_BUTTON(spin), *value);
 	g_signal_connect(spin, "activate", G_CALLBACK(on_input_numeric_activate), dialog);
 
@@ -1050,8 +1102,7 @@ void dialogs_show_file_properties(GeanyDocument *doc)
 
 	gtk_window_set_default_size(GTK_WINDOW(dialog), 300, -1);
 
-	title = g_strdup_printf("<b>%s</b>", base_name);
-	label = gtk_label_new(title);
+	label = ui_label_new_bold(base_name);
 	gtk_label_set_selectable(GTK_LABEL(label), TRUE);
 	gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
 	image = gtk_image_new_from_stock("gtk-file", GTK_ICON_SIZE_BUTTON);
@@ -1061,7 +1112,6 @@ void dialogs_show_file_properties(GeanyDocument *doc)
 	gtk_container_add(GTK_CONTAINER(hbox), image);
 	gtk_container_add(GTK_CONTAINER(hbox), label);
 	gtk_container_add(GTK_CONTAINER(vbox), hbox);
-	g_free(title);
 
 	table = gtk_table_new(8, 2, FALSE);
 	gtk_table_set_row_spacings(GTK_TABLE(table), 10);
@@ -1345,19 +1395,44 @@ void dialogs_show_file_properties(GeanyDocument *doc)
 }
 
 
-static gboolean show_question(GtkWidget *parent, const gchar *yes_btn, const gchar *no_btn,
-							  const gchar *question_text, const gchar *extra_text)
+/* extra_text can be NULL; otherwise it is displayed below main_text.
+ * if parent is NULL, main_widgets.window will be used
+ * btn_1, btn_2, btn_3 can be NULL.
+ * returns response_1, response_2 or response_3 */
+static gint show_prompt(GtkWidget *parent,
+		const gchar *btn_1, GtkResponseType response_1,
+		const gchar *btn_2, GtkResponseType response_2,
+		const gchar *btn_3, GtkResponseType response_3,
+		const gchar *question_text, const gchar *extra_text)
 {
 	gboolean ret = FALSE;
-#ifdef G_OS_WIN32
-	gchar *string = (extra_text == NULL) ? g_strdup(question_text) :
-		g_strconcat(question_text, "\n\n", extra_text, NULL);
-
-	ret = win32_message_dialog(parent, GTK_MESSAGE_QUESTION, string);
-	g_free(string);
-#else
 	GtkWidget *dialog;
+	GtkWidget *btn;
 
+	if (btn_2 == NULL)
+	{
+		btn_2 = GTK_STOCK_NO;
+		response_2 = GTK_RESPONSE_NO;
+	}
+	if (btn_3 == NULL)
+	{
+		btn_3 = GTK_STOCK_YES;
+		response_3 = GTK_RESPONSE_YES;
+	}
+
+#ifdef G_OS_WIN32
+	/* our native dialog code doesn't support custom buttons */
+	if (btn_3 == GTK_STOCK_YES && btn_2 == GTK_STOCK_NO && btn_1 == NULL)
+	{
+		gchar *string = (extra_text == NULL) ? g_strdup(question_text) :
+			g_strconcat(question_text, "\n\n", extra_text, NULL);
+
+		ret = win32_message_dialog(parent, GTK_MESSAGE_QUESTION, string);
+		ret = ret ? response_3 : response_2;
+		g_free(string);
+		return ret;
+	}
+#endif
 	if (parent == NULL && main_status.main_window_realized)
 		parent = main_widgets.window;
 
@@ -1366,7 +1441,7 @@ static gboolean show_question(GtkWidget *parent, const gchar *yes_btn, const gch
 		GTK_BUTTONS_NONE, "%s", question_text);
 	gtk_widget_set_name(dialog, "GeanyDialog");
 	gtk_window_set_title(GTK_WINDOW(dialog), _("Question"));
-	if (parent == NULL)
+	if (parent == NULL || GTK_IS_DIALOG(parent))
 	{
 		GdkPixbuf *pb = ui_new_pixbuf_from_inline(GEANY_IMAGE_LOGO);
 		gtk_window_set_icon(GTK_WINDOW(dialog), pb);
@@ -1378,21 +1453,27 @@ static gboolean show_question(GtkWidget *parent, const gchar *yes_btn, const gch
 		gtk_message_dialog_format_secondary_text(GTK_MESSAGE_DIALOG(dialog),
 			"%s", extra_text);
 
-	/* For a cancel button, use cancel response so user can press escape to cancel */
-	gtk_dialog_add_button(GTK_DIALOG(dialog), no_btn,
-		utils_str_equal(no_btn, GTK_STOCK_CANCEL) ? GTK_RESPONSE_CANCEL : GTK_RESPONSE_NO);
-	gtk_dialog_add_button(GTK_DIALOG(dialog), yes_btn, GTK_RESPONSE_YES);
+	if (btn_1 != NULL)
+		gtk_dialog_add_button(GTK_DIALOG(dialog), btn_1, response_1);
 
-	if (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_YES)
-		ret = TRUE;
+	/* For a cancel button, use cancel response so user can press escape to cancel */
+	btn = gtk_dialog_add_button(GTK_DIALOG(dialog), btn_2,
+		utils_str_equal(btn_2, GTK_STOCK_CANCEL) ? GTK_RESPONSE_CANCEL : response_2);
+	/* we don't want a default, but we need to override the apply button as default */
+	gtk_widget_grab_default(btn);
+	gtk_dialog_add_button(GTK_DIALOG(dialog), btn_3, response_3);
+
+	ret = gtk_dialog_run(GTK_DIALOG(dialog));
 	gtk_widget_destroy(dialog);
-#endif
+
+	if (ret == GTK_RESPONSE_CANCEL)
+		ret = response_2;
 	return ret;
 }
 
 
 /**
- *  Show a question message box with @a text and Yes/No buttons.
+ *  Shows a question message box with @a text and Yes/No buttons.
  *  On Unix-like systems a GTK message dialog box is shown, on Win32 systems a native Windows
  *  message dialog box is shown.
  *
@@ -1403,16 +1484,21 @@ static gboolean show_question(GtkWidget *parent, const gchar *yes_btn, const gch
  **/
 gboolean dialogs_show_question(const gchar *text, ...)
 {
-	gboolean ret = FALSE;
-	gchar string[512];
+	gchar *string;
 	va_list args;
 	GtkWidget *parent = (main_status.main_window_realized) ? main_widgets.window : NULL;
+	gint result;
 
 	va_start(args, text);
-	g_vsnprintf(string, 511, text, args);
+	string = g_strdup_vprintf(text, args);
 	va_end(args);
-	ret = show_question(parent, GTK_STOCK_YES, GTK_STOCK_NO, string, NULL);
-	return ret;
+	result = show_prompt(parent,
+		NULL, GTK_RESPONSE_NONE,
+		GTK_STOCK_NO, GTK_RESPONSE_NO,
+		GTK_STOCK_YES, GTK_RESPONSE_YES,
+		string, NULL);
+	g_free(string);
+	return (result == GTK_RESPONSE_YES);
 }
 
 
@@ -1422,20 +1508,44 @@ gboolean dialogs_show_question(const gchar *text, ...)
 gboolean dialogs_show_question_full(GtkWidget *parent, const gchar *yes_btn, const gchar *no_btn,
 	const gchar *extra_text, const gchar *main_text, ...)
 {
-	gboolean ret = FALSE;
-	gchar string[512];
+	gint result;
+	gchar *string;
 	va_list args;
 
-	if (!yes_btn)
-		yes_btn = GTK_STOCK_YES;
-	if (!no_btn)
-		no_btn = GTK_STOCK_NO;
-
 	va_start(args, main_text);
-	g_vsnprintf(string, 511, main_text, args);
+	string = g_strdup_vprintf(main_text, args);
 	va_end(args);
-	ret = show_question(parent, yes_btn, no_btn, string, extra_text);
-	return ret;
+	result = show_prompt(parent,
+		NULL, GTK_RESPONSE_NONE,
+		no_btn, GTK_RESPONSE_NO,
+		yes_btn, GTK_RESPONSE_YES,
+		string, extra_text);
+	g_free(string);
+	return (result == GTK_RESPONSE_YES);
+}
+
+
+/* extra_text can be NULL; otherwise it is displayed below main_text.
+ * if parent is NULL, main_widgets.window will be used
+ * btn_1, btn_2, btn_3 can be NULL.
+ * returns response_1, response_2 or response_3 */
+gint dialogs_show_prompt(GtkWidget *parent,
+		const gchar *btn_1, GtkResponseType response_1,
+		const gchar *btn_2, GtkResponseType response_2,
+		const gchar *btn_3, GtkResponseType response_3,
+		const gchar *extra_text, const gchar *main_text, ...)
+{
+	gchar *string;
+	va_list args;
+	gint result;
+
+	va_start(args, main_text);
+	string = g_strdup_vprintf(main_text, args);
+	va_end(args);
+	result = show_prompt(parent, btn_1, response_1, btn_2, response_2, btn_3, response_3,
+				string, extra_text);
+	g_free(string);
+	return result;
 }
 
 
