@@ -1,8 +1,8 @@
 /*
  *      document.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2005-2009 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2006-2009 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2005-2010 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2006-2010 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: document.c 4093 2009-08-16 16:20:41Z eht16 $
+ * $Id: document.c 4940 2010-05-22 19:38:49Z eht16 $
  */
 
 /*
@@ -41,7 +41,6 @@
 # include <sys/types.h>
 #endif
 
-#include <ctype.h>
 #include <stdlib.h>
 
 /* gstdio.h also includes sys/stat.h */
@@ -66,7 +65,7 @@
 #include "dialogs.h"
 #include "msgwindow.h"
 #include "templates.h"
-#include "treeviews.h"
+#include "sidebar.h"
 #include "ui_utils.h"
 #include "utils.h"
 #include "encodings.h"
@@ -90,11 +89,11 @@ GeanyFilePrefs file_prefs;
  * file by then.
  *
  * You also need to check @c GeanyDocument::is_valid when iterating over this array,
- * although usually you would just use the documents_foreach() macro.
+ * although usually you would just use the foreach_document() macro.
  *
  * Never assume that the order of document pointers is the same as the order of notebook tabs.
  * Notebook tabs can be reordered. Use @c document_get_from_page(). */
-GPtrArray *documents_array;
+GPtrArray *documents_array = NULL;
 
 
 /* an undo action, also used for redo actions */
@@ -114,19 +113,20 @@ static gboolean update_tags_from_buffer(GeanyDocument *doc);
 
 /* ignore the case of filenames and paths under WIN32, causes errors if not */
 #ifdef G_OS_WIN32
-#define filenamecmp(a,b)	utils_str_casecmp((a), (b))
+#define filenamecmp(a, b)	utils_str_casecmp((a), (b))
 #else
-#define filenamecmp(a,b)	strcmp((a), (b))
+#define filenamecmp(a, b)	strcmp((a), (b))
 #endif
 
 /**
- * Find and retrieve a document with the given filename from the document list.
+ * Finds a document whose @c real_path field matches the given filename.
  *
  * @param realname The filename to search, which should be identical to the
  * string returned by @c tm_get_real_path().
  *
  * @return The matching document, or @c NULL.
  * @note This is only really useful when passing a @c TMWorkObject::file_name.
+ * @see GeanyDocument::real_path.
  * @see document_find_by_filename().
  *
  * @since 0.15
@@ -166,7 +166,7 @@ static gchar *get_real_path_from_utf8(const gchar *utf8_filename)
 
 
 /**
- *  Find and retrieve a document with the given filename from the document list.
+ *  Finds a document with the given filename.
  *  This matches either an exact GeanyDocument::file_name string, or variant
  *  filenames with relative elements in the path (e.g. @c "/dir/..//name" will
  *  match @c "/name").
@@ -222,7 +222,10 @@ GeanyDocument *document_find_by_sci(ScintillaObject *sci)
 }
 
 
-/* returns the index of the notebook page for the document. */
+/** Gets the notebook page index for a document.
+ * @param doc The document.
+ * @return The index.
+ * @since 0.19 */
 gint document_get_notebook_page(GeanyDocument *doc)
 {
 	g_return_val_if_fail(doc != NULL, -1);
@@ -233,7 +236,7 @@ gint document_get_notebook_page(GeanyDocument *doc)
 
 
 /**
- *  Find and retrieve the document for the given notebook page @a page_num.
+ *  Finds the document for the given notebook page @a page_num.
  *
  *  @param page_num The notebook page number to search.
  *
@@ -254,7 +257,7 @@ GeanyDocument *document_get_from_page(guint page_num)
 
 
 /**
- *  Find and retrieve the current document.
+ *  Finds the current document.
  *
  *  @return A pointer to the current document or @c NULL if there are no opened documents.
  **/
@@ -339,7 +342,7 @@ void document_update_tab_label(GeanyDocument *doc)
 
 
 /**
- *  Update the tab labels, the status bar, the window title and some save-sensitive buttons
+ *  Updates the tab labels, the status bar, the window title and some save-sensitive buttons
  *  according to the document's save state.
  *  This is called by Geany mostly when opening or saving files.
  *
@@ -467,7 +470,7 @@ static void monitor_file_changed_cb(G_GNUC_UNUSED GFileMonitor *monitor, G_GNUC_
 #endif
 
 
-void document_stop_file_monitoring(GeanyDocument *doc)
+static void document_stop_file_monitoring(GeanyDocument *doc)
 {
 	g_return_if_fail(doc != NULL);
 
@@ -513,19 +516,43 @@ static void monitor_file_setup(GeanyDocument *doc)
 }
 
 
+void document_try_focus(GeanyDocument *doc, GtkWidget *source_widget)
+{
+	/* doc might not be valid e.g. if user closed a tab whilst Geany is opening files */
+	if (DOC_VALID(doc))
+	{
+		GtkWidget *sci = GTK_WIDGET(doc->editor->sci);
+		GtkWidget *focusw = gtk_window_get_focus(GTK_WINDOW(main_widgets.window));
+
+		if (source_widget == NULL)
+			source_widget = doc->priv->tag_tree;
+
+		if (focusw == source_widget)
+			gtk_widget_grab_focus(sci);
+	}
+}
+
+
+static gboolean on_idle_focus(gpointer doc)
+{
+	document_try_focus(doc, NULL);
+	return FALSE;
+}
+
+
 /* Creates a new document and editor, adding a tab in the notebook.
  * @return The created document */
 static GeanyDocument *document_create(const gchar *utf8_filename)
 {
-	GeanyDocument *this;
+	GeanyDocument *doc;
 	gint new_idx;
 	gint cur_pages = gtk_notebook_get_n_pages(GTK_NOTEBOOK(main_widgets.notebook));
 
 	if (cur_pages == 1)
 	{
-		GeanyDocument *doc = document_get_current();
+		GeanyDocument *cur = document_get_current();
 		/* remove the empty document and open a new one */
-		if (doc != NULL && doc->file_name == NULL && ! doc->changed)
+		if (cur != NULL && cur->file_name == NULL && ! cur->changed)
 			document_remove_page(0);
 	}
 
@@ -537,38 +564,35 @@ static GeanyDocument *document_create(const gchar *utf8_filename)
 		new_idx = documents_array->len;
 		g_ptr_array_add(documents_array, new_doc);
 	}
-	this = documents[new_idx];
-	init_doc_struct(this);	/* initialize default document settings */
-	this->index = new_idx;
+	doc = documents[new_idx];
+	init_doc_struct(doc);	/* initialize default document settings */
+	doc->index = new_idx;
 
-	this->file_name = g_strdup(utf8_filename);
+	doc->file_name = g_strdup(utf8_filename);
 
-	this->editor = editor_create(this);
+	doc->editor = editor_create(doc);
 
-	editor_apply_update_prefs(this->editor);
+	sidebar_openfiles_add(doc);	/* sets doc->iter */
 
-	treeviews_openfiles_add(this);	/* sets this->iter */
-
-	notebook_new_tab(this);
+	notebook_new_tab(doc);
 
 	/* select document in sidebar */
 	{
 		GtkTreeSelection *sel;
 
 		sel = gtk_tree_view_get_selection(GTK_TREE_VIEW(tv.tree_openfiles));
-		gtk_tree_selection_select_iter(sel, &this->priv->iter);
+		gtk_tree_selection_select_iter(sel, &doc->priv->iter);
 	}
 
 	ui_document_buttons_update();
 
-	gtk_widget_grab_focus(GTK_WIDGET(this->editor->sci));
-
-	this->is_valid = TRUE;	/* do this last to prevent UI updating with NULL items. */
-	return this;
+	doc->is_valid = TRUE;	/* do this last to prevent UI updating with NULL items. */
+	return doc;
 }
 
+
 /**
- *  Close the given document.
+ *  Closes the given document.
  *
  *  @param doc The document to remove.
  *
@@ -585,7 +609,7 @@ gboolean document_close(GeanyDocument *doc)
 
 
 /**
- *  Remove the given notebook tab at @a page_num and clear all related information
+ *  Removes the given notebook tab at @a page_num and clears all related information
  *  in the document list.
  *
  *  @param page_num The notebook page number to remove.
@@ -616,10 +640,13 @@ gboolean document_remove_page(guint page_num)
 
 	doc->is_valid = FALSE;
 
-	notebook_remove_page(page_num);
-	treeviews_remove_document(doc);
-	navqueue_remove_file(doc->file_name);
-	msgwin_status_add(_("File %s closed."), DOC_FILENAME(doc));
+	if (! main_status.quitting)
+	{
+		notebook_remove_page(page_num);
+		sidebar_remove_document(doc);
+		navqueue_remove_file(doc->file_name);
+		msgwin_status_add(_("File %s closed."), DOC_FILENAME(doc));
+	}
 	g_free(doc->encoding);
 	g_free(doc->priv->saved_encoding.encoding);
 	g_free(doc->file_name);
@@ -642,10 +669,11 @@ gboolean document_remove_page(guint page_num)
 
 	if (gtk_notebook_get_n_pages(GTK_NOTEBOOK(main_widgets.notebook)) == 0)
 	{
-		treeviews_update_tag_list(NULL, FALSE);
+		sidebar_update_tag_list(NULL, FALSE);
 		/*on_notebook1_switch_page(GTK_NOTEBOOK(main_widgets.notebook), NULL, 0, NULL);*/
 		ui_set_window_title(NULL);
 		ui_save_buttons_toggle(FALSE);
+		ui_update_popup_reundo_items(NULL);
 		ui_document_buttons_update();
 		build_menu_update(NULL);
 	}
@@ -674,7 +702,7 @@ GeanyDocument *document_new_file_if_non_open(void)
 
 /**
  *  Creates a new document.
- *  After all, the "document-new" signal is emitted for plugins.
+ *  Afterwards, the @c "document-new" signal is emitted for plugins.
  *
  *  @param utf8_filename The file name in UTF-8 encoding, or @c NULL to open a file as "untitled".
  *  @param ft The filetype to set or @c NULL to detect it from @a filename if not @c NULL.
@@ -729,7 +757,9 @@ GeanyDocument *document_new_file(const gchar *utf8_filename, GeanyFiletype *ft,
 	ui_document_show_hide(doc); /* update the document menu */
 
 	sci_set_line_numbers(doc->editor->sci, editor_prefs.show_linenumber_margin, 0);
-	sci_goto_pos(doc->editor->sci, 0, TRUE);
+	/* bring it in front, jump to the start and grab the focus */
+	editor_goto_pos(doc->editor, 0, FALSE);
+	document_try_focus(doc, NULL);
 
 #if USE_GIO_FILEMON
 	monitor_file_setup(doc);
@@ -750,8 +780,8 @@ GeanyDocument *document_new_file(const gchar *utf8_filename, GeanyFiletype *ft,
 
 
 /**
- *  Open a document specified by @a locale_filename.
- *  After all, the "document-open" signal is emitted for plugins.
+ *  Opens a document specified by @a locale_filename.
+ *  Afterwards, the @c "document-open" signal is emitted for plugins.
  *
  *  @param locale_filename The filename of the document to load, in locale encoding.
  *  @param readonly Whether to open the document in read-only mode.
@@ -996,8 +1026,10 @@ static gboolean load_text_file(const gchar *locale_filename, const gchar *displa
 
 /* Sets the cursor position on opening a file. First it sets the line when cl_options.goto_line
  * is set, otherwise it sets the line when pos is greater than zero and finally it sets the column
- * if cl_options.goto_column is set. */
-static void set_cursor_position(GeanyEditor *editor, gint pos)
+ * if cl_options.goto_column is set.
+ *
+ * returns the new position which may have changed */
+static gint set_cursor_position(GeanyEditor *editor, gint pos)
 {
 	if (cl_options.goto_line >= 0)
 	{	/* goto line which was specified on command line and then undefine the line */
@@ -1013,11 +1045,14 @@ static void set_cursor_position(GeanyEditor *editor, gint pos)
 
 	if (cl_options.goto_column >= 0)
 	{	/* goto column which was specified on command line and then undefine the column */
-		gint cur_pos = sci_get_current_position(editor->sci);
-		sci_set_current_position(editor->sci, cur_pos + cl_options.goto_column, FALSE);
+
+		gint new_pos = sci_get_current_position(editor->sci) + cl_options.goto_column;
+		sci_set_current_position(editor->sci, new_pos, FALSE);
 		editor->scroll_percent = 0.5F;
 		cl_options.goto_column = -1;
+		return new_pos;
 	}
+	return sci_get_current_position(editor->sci);
 }
 
 
@@ -1069,6 +1104,11 @@ static GeanyIndentType detect_indent_type(GeanyEditor *editor)
 		gint pos = sci_get_position_from_line(sci, line);
 		gchar c;
 
+		/* most code will have indent total <= 24, otherwise it's more likely to be
+		 * alignment than indentation */
+		if (sci_get_line_indentation(sci, line) > 24)
+			continue;
+
 		c = sci_get_char_at(sci, pos);
 		if (c == '\t')
 			tabs++;
@@ -1093,24 +1133,25 @@ static GeanyIndentType detect_indent_type(GeanyEditor *editor)
 }
 
 
-static void set_indentation(GeanyEditor *editor)
+void document_apply_indent_settings(GeanyDocument *doc)
 {
-	const GeanyIndentPrefs *iprefs = editor_get_indent_prefs(editor);
+	const GeanyIndentPrefs *iprefs = editor_get_indent_prefs(NULL);
+	GeanyIndentType type = iprefs->type;
 
-	switch (FILETYPE_ID(editor->document->file_type))
+	switch (FILETYPE_ID(doc->file_type))
 	{
 		case GEANY_FILETYPES_MAKE:
 			/* force using tabs for indentation for Makefiles */
-			editor_set_indent_type(editor, GEANY_INDENT_TYPE_TABS);
+			editor_set_indent_type(doc->editor, GEANY_INDENT_TYPE_TABS);
 			return;
 		case GEANY_FILETYPES_F77:
 			/* force using spaces for indentation for Fortran 77 */
-			editor_set_indent_type(editor, GEANY_INDENT_TYPE_SPACES);
+			editor_set_indent_type(doc->editor, GEANY_INDENT_TYPE_SPACES);
 			return;
 	}
 	if (iprefs->detect_type)
 	{
-		GeanyIndentType type = detect_indent_type(editor);
+		type = detect_indent_type(doc->editor);
 
 		if (type != iprefs->type)
 		{
@@ -1131,10 +1172,10 @@ static void set_indentation(GeanyEditor *editor)
 			/* For translators: first wildcard is the indentation mode (Spaces, Tabs, Tabs
 			 * and Spaces), the second one is the filename */
 			ui_set_statusbar(TRUE, _("Setting %s indentation mode for %s."), name,
-				DOC_FILENAME(editor->document));
+				DOC_FILENAME(doc));
 		}
-		editor_set_indent_type(editor, type);
 	}
+	editor_set_indent_type(doc->editor, type);
 }
 
 
@@ -1150,7 +1191,7 @@ static gboolean auto_update_tag_list(gpointer data)
 		return TRUE;
 
 	if (update_tags_from_buffer(doc))
-		treeviews_update_tag_list(doc, TRUE);
+		sidebar_update_tag_list(doc, TRUE);
 
 	return TRUE;
 }
@@ -1204,117 +1245,110 @@ GeanyDocument *document_open_file_full(GeanyDocument *doc, const gchar *filename
 		if (doc != NULL)
 		{
 			ui_add_recent_file(utf8_filename);	/* either add or reorder recent item */
+			/* show the doc before reload dialog */
 			gtk_notebook_set_current_page(GTK_NOTEBOOK(main_widgets.notebook),
-					gtk_notebook_page_num(GTK_NOTEBOOK(main_widgets.notebook),
-					(GtkWidget*) doc->editor->sci));
-			g_free(utf8_filename);
-			g_free(locale_filename);
+				gtk_notebook_page_num(GTK_NOTEBOOK(main_widgets.notebook),
+				(GtkWidget*) doc->editor->sci));
 			document_check_disk_status(doc, TRUE);	/* force a file changed check */
-			set_cursor_position(doc->editor, pos);
-			return doc;
 		}
 	}
-	display_filename = utils_str_middle_truncate(utf8_filename, 100);
+	if (reload || doc == NULL)
+	{	/* doc possibly changed */
+		display_filename = utils_str_middle_truncate(utf8_filename, 100);
 
-	/* if default encoding for opening files is set, use it if no forced encoding is set */
-	if (file_prefs.default_open_encoding >= 0 && forced_enc == NULL)
-		forced_enc = encodings[file_prefs.default_open_encoding].charset;
+		if (! load_text_file(locale_filename, display_filename, &filedata, forced_enc))
+		{
+			g_free(display_filename);
+			g_free(utf8_filename);
+			g_free(locale_filename);
+			return NULL;
+		}
 
-	if (! load_text_file(locale_filename, display_filename, &filedata, forced_enc))
-	{
-		g_free(display_filename);
-		g_free(utf8_filename);
-		g_free(locale_filename);
-		return NULL;
+		if (! reload)
+		{
+			doc = document_create(utf8_filename);
+			g_return_val_if_fail(doc != NULL, NULL); /* really should not happen */
+
+			/* file exists on disk, set real_path */
+			setptr(doc->real_path, tm_get_real_path(locale_filename));
+
+			doc->priv->is_remote = utils_is_remote_path(locale_filename);
+			monitor_file_setup(doc);
+		}
+
+		sci_set_undo_collection(doc->editor->sci, FALSE); /* avoid creation of an undo action */
+		sci_empty_undo_buffer(doc->editor->sci);
+
+		/* add the text to the ScintillaObject */
+		sci_set_readonly(doc->editor->sci, FALSE);	/* to allow replacing text */
+		sci_set_text(doc->editor->sci, filedata.data);	/* NULL terminated data */
+		queue_colourise(doc);	/* Ensure the document gets colourised. */
+
+		/* detect & set line endings */
+		editor_mode = utils_get_line_endings(filedata.data, filedata.len);
+		sci_set_eol_mode(doc->editor->sci, editor_mode);
+		g_free(filedata.data);
+
+		sci_set_undo_collection(doc->editor->sci, TRUE);
+
+		doc->priv->mtime = filedata.mtime; /* get the modification time from file and keep it */
+		g_free(doc->encoding);	/* if reloading, free old encoding */
+		doc->encoding = filedata.enc;
+		doc->has_bom = filedata.bom;
+		store_saved_encoding(doc);	/* store the opened encoding for undo/redo */
+
+		doc->readonly = readonly || filedata.readonly;
+		sci_set_readonly(doc->editor->sci, doc->readonly);
+
+		/* update line number margin width */
+		doc->priv->line_count = sci_get_line_count(doc->editor->sci);
+		sci_set_line_numbers(doc->editor->sci, editor_prefs.show_linenumber_margin, 0);
+
+		if (! reload)
+		{
+
+			/* "the" SCI signal (connect after initial setup(i.e. adding text)) */
+			g_signal_connect(doc->editor->sci, "sci-notify", G_CALLBACK(editor_sci_notify_cb),
+				doc->editor);
+
+			use_ft = (ft != NULL) ? ft : filetypes_detect_from_document(doc);
+		}
+		else
+		{	/* reloading */
+			document_undo_clear(doc);
+
+			use_ft = ft;
+		}
+		/* update taglist, typedef keywords and build menu if necessary */
+		document_set_filetype(doc, use_ft);
+
+		/* set indentation settings after setting the filetype */
+		if (reload)
+			editor_set_indent_type(doc->editor, doc->editor->indent_type); /* resetup sci */
+		else
+			document_apply_indent_settings(doc);
+
+		document_set_text_changed(doc, FALSE);	/* also updates tab state */
+		ui_document_show_hide(doc);	/* update the document menu */
+
+		/* finally add current file to recent files menu, but not the files from the last session */
+		if (! main_status.opening_session_files)
+			ui_add_recent_file(utf8_filename);
+
+		if (! reload)
+			g_signal_emit_by_name(geany_object, "document-open", doc);
+
+		if (reload)
+			ui_set_statusbar(TRUE, _("File %s reloaded."), display_filename);
+		else
+			/* For translators: this is the status window message for opening a file. %d is the number
+			 * of the newly opened file, %s indicates whether the file is opened read-only
+			 * (it is replaced with the string ", read-only"). */
+			msgwin_status_add(_("File %s opened(%d%s)."),
+				display_filename, gtk_notebook_get_n_pages(GTK_NOTEBOOK(main_widgets.notebook)),
+				(readonly) ? _(", read-only") : "");
 	}
 
-	if (! reload)
-	{
-		doc = document_create(utf8_filename);
-		g_return_val_if_fail(doc != NULL, NULL); /* really should not happen */
-
-		/* file exists on disk, set real_path */
-		setptr(doc->real_path, tm_get_real_path(locale_filename));
-
-		doc->priv->is_remote = utils_is_remote_path(locale_filename);
-		monitor_file_setup(doc);
-	}
-
-	sci_set_undo_collection(doc->editor->sci, FALSE); /* avoid creation of an undo action */
-	sci_empty_undo_buffer(doc->editor->sci);
-
-	/* add the text to the ScintillaObject */
-	sci_set_readonly(doc->editor->sci, FALSE);	/* to allow replacing text */
-	sci_set_text(doc->editor->sci, filedata.data);	/* NULL terminated data */
-	queue_colourise(doc);	/* Ensure the document gets colourised. */
-
-	/* detect & set line endings */
-	editor_mode = utils_get_line_endings(filedata.data, filedata.len);
-	sci_set_eol_mode(doc->editor->sci, editor_mode);
-	g_free(filedata.data);
-
-	sci_set_undo_collection(doc->editor->sci, TRUE);
-
-	doc->priv->mtime = filedata.mtime; /* get the modification time from file and keep it */
-	g_free(doc->encoding);	/* if reloading, free old encoding */
-	doc->encoding = filedata.enc;
-	doc->has_bom = filedata.bom;
-	store_saved_encoding(doc);	/* store the opened encoding for undo/redo */
-
-	doc->readonly = readonly || filedata.readonly;
-	sci_set_readonly(doc->editor->sci, doc->readonly);
-
-	/* update line number margin width */
-	doc->priv->line_count = sci_get_line_count(doc->editor->sci);
-	sci_set_line_numbers(doc->editor->sci, editor_prefs.show_linenumber_margin, 0);
-
-	/* set the cursor position according to pos, cl_options.goto_line and cl_options.goto_column */
-	set_cursor_position(doc->editor, pos);
-
-	if (! reload)
-	{
-
-		/* "the" SCI signal (connect after initial setup(i.e. adding text)) */
-		g_signal_connect(doc->editor->sci, "sci-notify", G_CALLBACK(editor_sci_notify_cb),
-			doc->editor);
-
-		use_ft = (ft != NULL) ? ft : filetypes_detect_from_document(doc);
-	}
-	else
-	{	/* reloading */
-		document_undo_clear(doc);
-
-		use_ft = ft;
-	}
-	/* update taglist, typedef keywords and build menu if necessary */
-	document_set_filetype(doc, use_ft);
-
-	/* set indentation settings after setting the filetype */
-	if (reload)
-		editor_set_indent_type(doc->editor, doc->editor->indent_type); /* resetup sci */
-	else
-		set_indentation(doc->editor);
-
-	document_set_text_changed(doc, FALSE);	/* also updates tab state */
-	ui_document_show_hide(doc);	/* update the document menu */
-
-	/* finally add current file to recent files menu, but not the files from the last session */
-	if (! main_status.opening_session_files)
-		ui_add_recent_file(utf8_filename);
-
-	if (! reload)
-		g_signal_emit_by_name(geany_object, "document-open", doc);
-
-	if (reload)
-		ui_set_statusbar(TRUE, _("File %s reloaded."), display_filename);
-	else
-		/* For translators: this is the status window message for opening a file. %d is the number
-		 * of the newly opened file, %s indicates whether the file is opened read-only
-		 * (it is replaced with the string ", read-only"). */
-		msgwin_status_add(_("File %s opened(%d%s)."),
-			display_filename, gtk_notebook_get_n_pages(GTK_NOTEBOOK(main_widgets.notebook)),
-			(readonly) ? _(", read-only") : "");
-
 	g_free(display_filename);
 	g_free(utf8_filename);
 	g_free(locale_filename);
@@ -1323,6 +1357,14 @@ GeanyDocument *document_open_file_full(GeanyDocument *doc, const gchar *filename
 	 * based on a configurable interval */
 	/*g_timeout_add(10000, auto_update_tag_list, doc);*/
 
+	/* set the cursor position according to pos, cl_options.goto_line and cl_options.goto_column */
+	pos = set_cursor_position(doc->editor, pos);
+	/* now bring the file in front */
+	editor_goto_pos(doc->editor, pos, FALSE);
+
+	/* finally, let the editor widget grab the focus so you can start coding
+	 * right away */
+	g_idle_add(on_idle_focus, doc);
 	return doc;
 }
 
@@ -1383,6 +1425,7 @@ void document_open_files(const GSList *filenames, gboolean readonly, GeanyFilety
 	}
 }
 
+
 /**
  *  Reloads the document with the specified file encoding
  *  @a forced_enc or @c NULL to auto-detect the file encoding.
@@ -1464,21 +1507,24 @@ static void replace_header_filename(GeanyDocument *doc)
 	g_return_if_fail(doc != NULL);
 	g_return_if_fail(doc->file_type != NULL);
 
-	filebase = g_strconcat(GEANY_STRING_UNTITLED, ".", (doc->file_type)->extension, NULL);
+	if (doc->file_type->extension)
+		filebase = g_strconcat("\\<", GEANY_STRING_UNTITLED, "\\.\\w+", NULL);
+	else
+		filebase = g_strdup(GEANY_STRING_UNTITLED);
+
 	filename = g_path_get_basename(doc->file_name);
 
 	/* only search the first 3 lines */
 	ttf.chrg.cpMin = 0;
 	ttf.chrg.cpMax = sci_get_position_from_line(doc->editor->sci, 3);
-	ttf.lpstrText = (gchar*)filebase;
+	ttf.lpstrText = filebase;
 
-	if (sci_find_text(doc->editor->sci, SCFIND_MATCHCASE, &ttf) != -1)
+	if (search_find_text(doc->editor->sci, SCFIND_MATCHCASE | SCFIND_REGEXP, &ttf) != -1)
 	{
 		sci_set_target_start(doc->editor->sci, ttf.chrgText.cpMin);
 		sci_set_target_end(doc->editor->sci, ttf.chrgText.cpMax);
 		sci_replace_target(doc->editor->sci, filename, FALSE);
 	}
-
 	g_free(filebase);
 	g_free(filename);
 }
@@ -1516,6 +1562,16 @@ void document_rename_file(GeanyDocument *doc, const gchar *new_filename)
 }
 
 
+/* Return TRUE if the document hasn't been saved before, i.e. either the filename or
+ * the real_path is not set. */
+gboolean document_need_save_as(GeanyDocument *doc)
+{
+	g_return_val_if_fail(doc != NULL, FALSE);
+
+	return (doc->file_name == NULL || doc->real_path == NULL);
+}
+
+
 /**
  * Saves the document, detecting the filetype.
  *
@@ -1642,10 +1698,7 @@ static gchar *write_data_to_disk(GeanyDocument *doc, const gchar *locale_filenam
 	g_return_val_if_fail(doc != NULL, g_strdup(g_strerror(EINVAL)));
 	g_return_val_if_fail(data != NULL, g_strdup(g_strerror(EINVAL)));
 
-	/* we never use g_file_set_contents() for remote files as Geany only writes such files
-	 * 'into' the Fuse mountpoint, the file itself is then written by GVfs on the target
-	 * remote filesystem. */
-	if (! file_prefs.use_safe_file_saving || doc->priv->is_remote)
+	if (! file_prefs.use_safe_file_saving)
 	{
 		fp = g_fopen(locale_filename, "wb");
 		if (G_UNLIKELY(fp == NULL))
@@ -1685,10 +1738,12 @@ static gchar *write_data_to_disk(GeanyDocument *doc, const gchar *locale_filenam
 
 
 /**
- *  Save the document. Saving includes replacing tabs by spaces,
+ *  Saves the document. Saving includes replacing tabs by spaces,
  *  stripping trailing spaces and adding a final new line at the end of the file (all only if
- *  user enabled these features). The filetype is set again or auto-detected if it wasn't
- *  set yet. After all, the "document-save" signal is emitted for plugins.
+ *  user enabled these features). Then the @c "document-before-save" signal is emitted,
+ *  allowing plugins to modify the document before it is saved, and data is
+ *  actually written to disk. The filetype is set again or auto-detected if it wasn't set yet.
+ *  Afterwards, the @c "document-save" signal is emitted for plugins.
  *
  *  If the file is not modified, this functions does nothing unless force is set to @c TRUE.
  *
@@ -1727,6 +1782,9 @@ gboolean document_save_file(GeanyDocument *doc, gboolean force)
 	if (file_prefs.final_new_line)
 		editor_ensure_final_newline(doc->editor);
 
+	/* notify plugins which may wish to modify the document before it's saved */
+	g_signal_emit_by_name(geany_object, "document-before-save", doc);
+
 	len = sci_get_length(doc->editor->sci) + 1;
 	if (doc->has_bom && encodings_is_unicode_charset(doc->encoding))
 	{	/* always write a UTF-8 BOM because in this moment the text itself is still in UTF-8
@@ -1749,7 +1807,7 @@ gboolean document_save_file(GeanyDocument *doc, gboolean force)
 	if (doc->encoding != NULL && ! utils_str_equal(doc->encoding, "UTF-8") &&
 		! utils_str_equal(doc->encoding, encodings[GEANY_ENCODING_NONE].charset))
 	{
-		if  (! save_convert_to_encoding(doc, &data, &len))
+		if (! save_convert_to_encoding(doc, &data, &len))
 		{
 			g_free(data);
 			return FALSE;
@@ -1903,7 +1961,7 @@ gint document_find_text(GeanyDocument *doc, const gchar *text, gint flags, gbool
 	if (search_backwards)
 		search_pos = sci_search_prev(doc->editor->sci, flags, text);
 	else
-		search_pos = sci_search_next(doc->editor->sci, flags, text);
+		search_pos = search_find_next(doc->editor->sci, text, flags);
 
 	if (search_pos != -1)
 	{
@@ -1987,7 +2045,7 @@ gint document_replace_text(GeanyDocument *doc, const gchar *find_text, const gch
 		gint replace_len;
 		/* search next/prev will select matching text, which we use to set the replace target */
 		sci_target_from_selection(doc->editor->sci);
-		replace_len = sci_replace_target(doc->editor->sci, replace_text, flags & SCFIND_REGEXP);
+		replace_len = search_replace_target(doc->editor->sci, replace_text, flags & SCFIND_REGEXP);
 		/* select the replacement - find text will skip past the selected text */
 		sci_set_selection_start(doc->editor->sci, search_pos);
 		sci_set_selection_end(doc->editor->sci, search_pos + replace_len);
@@ -2060,54 +2118,12 @@ document_replace_range(GeanyDocument *doc, const gchar *find_text, const gchar *
 
 	sci = doc->editor->sci;
 
-	sci_start_undo_action(sci);
 	ttf.chrg.cpMin = start;
 	ttf.chrg.cpMax = end;
 	ttf.lpstrText = (gchar*)find_text;
 
-	while (TRUE)
-	{
-		gint search_pos;
-		gint find_len = 0, replace_len = 0;
-
-		search_pos = sci_find_text(sci, flags, &ttf);
-		find_len = ttf.chrgText.cpMax - ttf.chrgText.cpMin;
-		if (search_pos == -1)
-			break;	/* no more matches */
-		if (find_len == 0 && ! NZV(replace_text))
-			break;	/* nothing to do */
-
-		if (search_pos + find_len > end)
-			break;	/* found text is partly out of range */
-		else
-		{
-			gint movepastEOL = 0;
-
-			sci_set_target_start(sci, search_pos);
-			sci_set_target_end(sci, search_pos + find_len);
-
-			if (find_len <= 0)
-			{
-				gchar chNext = sci_get_char_at(sci, sci_get_target_end(sci));
-
-				if (chNext == '\r' || chNext == '\n')
-					movepastEOL = 1;
-			}
-			replace_len = sci_replace_target(sci, replace_text,
-				flags & SCFIND_REGEXP);
-			count++;
-			if (search_pos == end)
-				break;	/* Prevent hang when replacing regex $ */
-
-			/* make the next search start after the replaced text */
-			start = search_pos + replace_len + movepastEOL;
-			if (find_len == 0)
-				start = sci_get_position_after(sci, start);	/* prevent '[ ]*' regex rematching part of replaced text */
-			ttf.chrg.cpMin = start;
-			end += replace_len - find_len;	/* update end of range now text has changed */
-			ttf.chrg.cpMax = end;
-		}
-	}
+	sci_start_undo_action(sci);
+	count = search_replace_range(sci, &ttf, flags, replace_text);
 	sci_end_undo_action(sci);
 
 	if (count > 0)
@@ -2116,7 +2132,7 @@ document_replace_range(GeanyDocument *doc, const gchar *find_text, const gchar *
 			sci_goto_pos(sci, ttf.chrg.cpMin, TRUE);
 
 		if (new_range_end != NULL)
-			*new_range_end = end;
+			*new_range_end = ttf.chrg.cpMax;
 	}
 	return count;
 }
@@ -2224,8 +2240,8 @@ void document_replace_sel(GeanyDocument *doc, const gchar *find_text, const gcha
 }
 
 
-/* returns TRUE if at least one replacement was made. */
-gboolean document_replace_all(GeanyDocument *doc, const gchar *find_text, const gchar *replace_text,
+/* returns number of replacements made. */
+gint document_replace_all(GeanyDocument *doc, const gchar *find_text, const gchar *replace_text,
 		gint flags, gboolean escaped_chars)
 {
 	gint len, count;
@@ -2239,7 +2255,7 @@ gboolean document_replace_all(GeanyDocument *doc, const gchar *find_text, const
 			doc, find_text, replace_text, flags, 0, len, TRUE, NULL);
 
 	show_replace_summary(doc, count, find_text, replace_text, escaped_chars);
-	return (count > 0);
+	return count;
 }
 
 
@@ -2265,7 +2281,7 @@ static gboolean update_tags_from_buffer(GeanyDocument *doc)
 
 void document_update_tag_list(GeanyDocument *doc, gboolean update)
 {
-	/* We must call treeviews_update_tag_list() before returning,
+	/* We must call sidebar_update_tag_list() before returning,
 	 * to ensure that the symbol list is always updated properly (e.g.
 	 * when creating a new document with a partial filename set. */
 	gboolean success = FALSE;
@@ -2275,15 +2291,18 @@ void document_update_tag_list(GeanyDocument *doc, gboolean update)
 		! filetype_has_tags(doc->file_type) || ! doc->file_name)
 	{
 		/* set the default (empty) tag list */
-		treeviews_update_tag_list(doc, FALSE);
+		sidebar_update_tag_list(doc, FALSE);
 		return;
 	}
 
 	if (doc->tm_file == NULL)
 	{
 		gchar *locale_filename = utils_get_locale_from_utf8(doc->file_name);
+		const gchar *name;
 
-		doc->tm_file = tm_source_file_new(locale_filename, FALSE, doc->file_type->name);
+		/* lookup the name rather than using filetype name to support custom filetypes */
+		name = tm_source_file_get_lang_name(doc->file_type->lang);
+		doc->tm_file = tm_source_file_new(locale_filename, FALSE, name);
 		g_free(locale_filename);
 
 		if (doc->tm_file)
@@ -2307,7 +2326,7 @@ void document_update_tag_list(GeanyDocument *doc, gboolean update)
 		if (G_UNLIKELY(! success))
 			geany_debug("tag list updating failed");
 	}
-	treeviews_update_tag_list(doc, success);
+	sidebar_update_tag_list(doc, success);
 }
 
 
@@ -2428,7 +2447,11 @@ static void document_load_config(GeanyDocument *doc, GeanyFiletype *type,
 			tm_workspace_remove_object(doc->tm_file, TRUE, TRUE);
 			doc->tm_file = NULL;
 		}
-		highlighting_set_styles(doc->editor->sci, type->id);
+		/* load tags files before highlighting (some lexers highlight global typenames) */
+		if (type->id != GEANY_FILETYPES_NONE)
+			symbols_global_tags_loaded(type->id);
+
+		highlighting_set_styles(doc->editor->sci, type);
 		editor_set_indentation_guides(doc->editor);
 		build_menu_update(doc);
 		queue_colourise(doc);
@@ -2447,11 +2470,13 @@ static void document_load_config(GeanyDocument *doc, GeanyFiletype *type,
 void document_set_filetype(GeanyDocument *doc, GeanyFiletype *type)
 {
 	gboolean ft_changed;
+	GeanyFiletype *old_ft;
 
 	g_return_if_fail(doc);
 	if (type == NULL)
 		type = filetypes[GEANY_FILETYPES_NONE];
 
+	old_ft = doc->file_type;
 	geany_debug("%s : %s (%s)",
 		(doc->file_name != NULL) ? doc->file_name : "unknown",
 		(type->name != NULL) ? type->name : "unknown",
@@ -2459,6 +2484,9 @@ void document_set_filetype(GeanyDocument *doc, GeanyFiletype *type)
 
 	ft_changed = (doc->file_type != type); /* filetype has changed */
 	document_load_config(doc, type, ft_changed);
+
+	if (ft_changed)
+		g_signal_emit_by_name(geany_object, "document-filetype-set", doc, old_ft);
 }
 
 
@@ -2717,13 +2745,13 @@ static void document_redo_add(GeanyDocument *doc, guint type, gpointer data)
 
 
 /**
- *  Gets the status colour of the document, or @c NULL if default widget colouring should be used.
- *  Returned colours are red if the document has changes, green is the document is read-only
+ *  Gets the status color of the document, or @c NULL if default widget coloring should be used.
+ *  Returned colors are red if the document has changes, green if the document is read-only
  *  or simply @c NULL if the document is unmodified but writable.
  *
  *  @param doc The document to use.
  *
- *  @return The colour for the document or @c NULL if the default colour should be used. The colour
+ *  @return The color for the document or @c NULL if the default color should be used. The color
  *          object is owned by Geany and should not be modified or freed.
  *
  *  @since 0.16
@@ -2806,7 +2834,7 @@ gboolean document_account_for_unsaved(void)
 	for (p = 0; p < page_count; p++)
 	{
 		doc = document_get_from_page(p);
-		if (doc->changed)
+		if (DOC_VALID(doc) && doc->changed)
 		{
 			if (! dialogs_show_unsaved_file(doc))
 				return FALSE;
@@ -2839,9 +2867,9 @@ static void force_close_all(void)
 	}
 	main_status.closing_all = TRUE;
 
-	while (gtk_notebook_get_n_pages(GTK_NOTEBOOK(main_widgets.notebook)) > 0)
+	foreach_document(i)
 	{
-		document_remove_page(0);
+		document_close(documents[i]);
 	}
 
 	main_status.closing_all = FALSE;
@@ -2870,7 +2898,7 @@ static gboolean monitor_reload_file(GeanyDocument *doc)
 		base_name);
 
 	if (want_reload)
-		document_reload_file(doc, NULL);
+		document_reload_file(doc, doc->encoding);
 
 	g_free(base_name);
 	return want_reload;
@@ -2880,19 +2908,33 @@ static gboolean monitor_reload_file(GeanyDocument *doc)
 static gboolean monitor_resave_missing_file(GeanyDocument *doc)
 {
 	gboolean want_reload = FALSE;
+	gboolean file_saved = FALSE;
+	gint ret;
 
-	/* file is missing - set unsaved state */
-	document_set_text_changed(doc, TRUE);
-	/* don't prompt more than once */
-	setptr(doc->real_path, NULL);
-
-	if (dialogs_show_question_full(NULL, GTK_STOCK_SAVE, GTK_STOCK_CANCEL,
-		_("Try to resave the file?"),
-		_("File \"%s\" was not found on disk!"), doc->file_name))
+	ret = dialogs_show_prompt(NULL,
+		_("Close _without saving"), GTK_RESPONSE_CLOSE,
+		GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+		GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
+		NULL,
+		_("File \"%s\" was not found on disk! Try to resave the file?"),
+		doc->file_name);
+	if (ret == GTK_RESPONSE_ACCEPT)
 	{
-		dialogs_show_save_as();
+		file_saved = dialogs_show_save_as();
 		want_reload = TRUE;
 	}
+	else if (ret == GTK_RESPONSE_CLOSE)
+	{
+		document_close(doc);
+	}
+	if (ret != GTK_RESPONSE_CLOSE && ! file_saved)
+	{
+		/* file is missing - set unsaved state */
+		document_set_text_changed(doc, TRUE);
+		/* don't prompt more than once */
+		setptr(doc->real_path, NULL);
+	}
+
 	return want_reload;
 }
 
@@ -2950,11 +2992,13 @@ gboolean document_check_disk_status(GeanyDocument *doc, gboolean force)
 	}
 	g_free(locale_filename);
 
-	old_status = doc->priv->file_disk_status;
-	doc->priv->file_disk_status = FILE_OK;
-	if (old_status != doc->priv->file_disk_status)
-		ui_update_tab_status(doc);
-
+	if (DOC_VALID(doc))
+	{	/* doc can get invalid when a document was closed by monitor_resave_missing_file() */
+		old_status = doc->priv->file_disk_status;
+		doc->priv->file_disk_status = FILE_OK;
+		if (old_status != doc->priv->file_disk_status)
+			ui_update_tab_status(doc);
+	}
 	return ret;
 }
 
