@@ -1,8 +1,8 @@
 /*
  *      encodings.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2005-2009 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2006-2009 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2005-2010 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2006-2010 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- *  $Id: encodings.c 4010 2009-07-21 22:12:13Z eht16 $
+ *  $Id: encodings.c 5166 2010-08-16 11:47:50Z ntrel $
  */
 
 /*
@@ -232,6 +232,7 @@ const gchar *encodings_get_charset(const GeanyEncoding* enc)
 	return enc->charset;
 }
 
+
 static GtkWidget *radio_items[GEANY_ENCODINGS_MAX];
 
 
@@ -522,11 +523,11 @@ gchar *encodings_convert_to_utf8(const gchar *buffer, gsize size, gchar **used_e
 {
 	gchar *locale_charset = NULL;
 	gchar *regex_charset = NULL;
-	gchar *charset;
+	const gchar *charset;
 	gchar *utf8_content;
 	gboolean check_regex = FALSE;
 	gboolean check_locale = FALSE;
-	gint i, len;
+	gint i, len, preferred_charset;
 
 	if ((gint)size == -1)
 	{
@@ -546,31 +547,54 @@ gchar *encodings_convert_to_utf8(const gchar *buffer, gsize size, gchar **used_e
 	/* current locale is not UTF-8, we have to check this charset */
 	check_locale = ! g_get_charset((const gchar**) &charset);
 
-	for (i = 0; i < GEANY_ENCODINGS_MAX; i++)
+	/* First check for preferred charset, if specified */
+	preferred_charset = file_prefs.default_open_encoding;
+
+	if (preferred_charset == encodings[GEANY_ENCODING_NONE].idx ||
+		preferred_charset < 0 ||
+		preferred_charset >= GEANY_ENCODINGS_MAX)
 	{
-		if (G_UNLIKELY(i == encodings[GEANY_ENCODING_NONE].idx) || G_UNLIKELY(i == -1))
+		preferred_charset = -1;
+	}
+
+	/* -1 means "Preferred charset" */
+	for (i = -1; i < GEANY_ENCODINGS_MAX; i++)
+	{
+		if (G_UNLIKELY(i == encodings[GEANY_ENCODING_NONE].idx))
 			continue;
 
 		if (check_regex)
 		{
 			check_regex = FALSE;
 			charset = regex_charset;
-			i = -1;
+			i = -2; /* keep i below the start value to have it again at -1 on the next loop run */
 		}
 		else if (check_locale)
 		{
 			check_locale = FALSE;
 			charset = locale_charset;
-			i = -1;
+			i = -2; /* keep i below the start value to have it again at -1 on the next loop run */
 		}
-		else
+		else if (i == -1)
+		{
+			if (preferred_charset >= 0)
+			{
+				charset = encodings[preferred_charset].charset;
+				geany_debug("Using preferred charset: %s", charset);
+			}
+			else
+				continue;
+		}
+		else if (i >= 0)
 			charset = encodings[i].charset;
-
+		else /* in this case we have i == -2, continue to increase i and go ahead */
+			continue;
 
 		if (G_UNLIKELY(charset == NULL))
 			continue;
 
-		geany_debug("Trying to convert %" G_GSIZE_FORMAT " bytes of data from %s into UTF-8.", size, charset);
+		geany_debug("Trying to convert %" G_GSIZE_FORMAT " bytes of data from %s into UTF-8.",
+			size, charset);
 		utf8_content = encodings_convert_to_utf8_from_charset(buffer, size, charset, FALSE);
 
 		if (G_LIKELY(utf8_content != NULL))
@@ -636,7 +660,7 @@ GeanyEncodingIndex encodings_scan_unicode_bom(const gchar *string, gsize len, gu
 		if (bom_len)
 			*bom_len = 2;
 
-		if ((guchar)string[0]==0xfe && (guchar)string[1] == 0xff)
+		if ((guchar)string[0] == 0xfe && (guchar)string[1] == 0xff)
 		{
 			return GEANY_ENCODING_UTF_16BE; /* Big endian */
 		}
