@@ -1,8 +1,8 @@
 /*
  *      filetypes.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2005-2009 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2006-2009 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2005-2010 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2006-2010 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: filetypes.c 4048 2009-07-30 12:38:10Z ntrel $
+ * $Id: filetypes.c 5161 2010-08-13 17:00:36Z ntrel $
  */
 
 /**
@@ -26,6 +26,9 @@
  * Filetype detection, file extensions and filetype menu items.
  */
 
+/* Note: we use filetype_id for some function arguments, but GeanyFiletype is better; we should
+ * only use GeanyFiletype for API functions. */
+
 #include <string.h>
 #include <glib/gstdio.h>
 
@@ -60,7 +63,9 @@ typedef struct GeanyFiletypePrivate
 #ifdef HAVE_REGCOMP
 	regex_t		error_regex;
 	gboolean	error_regex_compiled;
+	gchar		*last_string; /* last one compiled */
 #endif
+	gboolean	custom;
 }
 GeanyFiletypePrivate;
 
@@ -112,7 +117,7 @@ static void init_builtin_filetypes(void)
 	ft->lang = -2;
 	ft->name = g_strdup(_("None"));
 	ft->title = g_strdup(_("None"));
-	ft->extension = g_strdup("*");
+	ft->extension = NULL;
 	ft->pattern = utils_strv_new("*", NULL);
 	ft->comment_open = NULL;
 	ft->comment_close = NULL;
@@ -375,6 +380,28 @@ static void init_builtin_filetypes(void)
 	ft->comment_close = NULL;
 	ft->group = GEANY_FILETYPE_GROUP_MISC;
 
+#define TXT2TAGS
+	ft = filetypes[GEANY_FILETYPES_TXT2TAGS];
+	ft->lang = 37;
+	ft->name = g_strdup("Txt2tags");
+	filetype_make_title(ft, TITLE_SOURCE_FILE);
+	ft->extension = g_strdup("t2t");
+	ft->pattern = utils_strv_new("*.t2t", "*.txt2tags", NULL);
+	ft->comment_open = NULL;
+	ft->comment_close = NULL;
+	ft->group = GEANY_FILETYPE_GROUP_MISC;
+
+#define ABC
+	ft = filetypes[GEANY_FILETYPES_ABC];
+	ft->lang = 38;
+	ft->name = g_strdup("Abc");
+	filetype_make_title(ft, TITLE_SOURCE_FILE);
+	ft->extension = g_strdup("abc");
+	ft->pattern = utils_strv_new("*.abc", "*.abp", NULL);
+	ft->comment_open = NULL;
+	ft->comment_close = NULL;
+	ft->group = GEANY_FILETYPE_GROUP_MISC;
+
 #define SH
 	ft = filetypes[GEANY_FILETYPES_SH];
 	ft->lang = 16;
@@ -478,6 +505,17 @@ static void init_builtin_filetypes(void)
 	ft->comment_close = NULL;
 	ft->group = GEANY_FILETYPE_GROUP_COMPILED;
 
+#define VERILOG
+	ft = filetypes[GEANY_FILETYPES_VERILOG];
+	ft->lang = 39;
+	ft->name = g_strdup("Verilog");
+	filetype_make_title(ft, TITLE_SOURCE_FILE);
+	ft->extension = g_strdup("v");
+	ft->pattern = utils_strv_new("*.v", "*.verilog", NULL);
+	ft->comment_open = g_strdup("/*");
+	ft->comment_close = g_strdup("*/");
+	ft->group = GEANY_FILETYPE_GROUP_COMPILED;
+
 #define DIFF
 	ft = filetypes[GEANY_FILETYPES_DIFF];
 	ft->lang = 20;
@@ -536,7 +574,7 @@ static void init_builtin_filetypes(void)
 
 #define R
 	ft = filetypes[GEANY_FILETYPES_R];
-	ft->lang = 34;
+	ft->lang = 40;
 	ft->name = g_strdup("R");
 	ft->title = g_strdup_printf(_("%s script file"), "R");
 	ft->extension = g_strdup("R");
@@ -620,8 +658,7 @@ static GeanyFiletype *filetype_new(void)
 	GeanyFiletype *ft = g_new0(GeanyFiletype, 1);
 
 	ft->lang = -2;	/* assume no tagmanager parser */
-	ft->programs = g_new0(struct build_programs, 1);
-	ft->actions = g_new0(struct build_actions, 1);
+	ft->project_list_entry = -1; /* no entry */
 
 	ft->priv = g_new0(GeanyFiletypePrivate, 1);
 	return ft;
@@ -656,6 +693,55 @@ static void filetype_add(GeanyFiletype *ft)
 }
 
 
+static void add_custom_filetype(const gchar *filename)
+{
+	gchar *fn = utils_strdupa(strstr(filename, ".") + 1);
+	gchar *dot = g_strrstr(fn, ".conf");
+	GeanyFiletype *ft;
+
+	g_return_if_fail(dot);
+
+	*dot = 0x0;
+
+	if (g_hash_table_lookup(filetypes_hash, fn))
+		return;
+
+	ft = filetype_new();
+	ft->name = g_strdup(fn);
+	filetype_make_title(ft, TITLE_FILE);
+	ft->pattern = g_new0(gchar*, 1);
+	ft->group = GEANY_FILETYPE_GROUP_CUSTOM;
+	ft->priv->custom = TRUE;
+	filetype_add(ft);
+	geany_debug("Added filetype %s (%d).", ft->name, ft->id);
+}
+
+
+static void init_custom_filetypes(const gchar *path)
+{
+	GDir *dir;
+	const gchar *filename;
+
+	g_return_if_fail(path);
+
+	dir = g_dir_open(path, 0, NULL);
+	if (dir == NULL)
+		return;
+
+	foreach_dir(filename, dir)
+	{
+		const gchar prefix[] = "filetypes.";
+
+		if (g_str_has_prefix(filename, prefix) &&
+			g_str_has_suffix(filename + strlen(prefix), ".conf"))
+		{
+			add_custom_filetype(filename);
+		}
+	}
+	g_dir_close(dir);
+}
+
+
 /* Create the filetypes array and fill it with the known filetypes. */
 void filetypes_init_types()
 {
@@ -679,6 +765,8 @@ void filetypes_init_types()
 	{
 		filetype_add(filetypes[ft_id]);
 	}
+	init_custom_filetypes(app->datadir);
+	init_custom_filetypes(utils_build_path(app->configdir, GEANY_FILEDEFS_SUBDIR, NULL));
 }
 
 
@@ -698,7 +786,7 @@ static void on_document_save(G_GNUC_UNUSED GObject *object, GeanyDocument *doc)
 		 * The user can do this manually with 'Tools->Reload Configuration' */
 		filetypes_load_config(GEANY_FILETYPES_NONE, TRUE);
 
-		documents_foreach(i)
+		foreach_document(i)
 			document_reload_config(documents[i]);
 	}
 }
@@ -739,22 +827,25 @@ static void create_set_filetype_menu(void)
 	create_sub_menu(filetype_menu, GEANY_FILETYPE_GROUP_SCRIPT, _("_Scripting Languages"));
 	create_sub_menu(filetype_menu, GEANY_FILETYPE_GROUP_MARKUP, _("_Markup Languages"));
 	create_sub_menu(filetype_menu, GEANY_FILETYPE_GROUP_MISC, _("M_iscellaneous Languages"));
+	create_sub_menu(filetype_menu, GEANY_FILETYPE_GROUP_CUSTOM, _("_Custom Filetypes"));
 
 	/* Append all filetypes to the filetype menu */
 	foreach_slist(node, filetypes_by_title)
 	{
 		GeanyFiletype *ft = node->data;
 
-		if (ft->id != GEANY_FILETYPES_NONE)
+		if (ft->group != GEANY_FILETYPE_GROUP_NONE)
 			create_radio_menu_item(group_menus[ft->group], ft);
+		else
+			create_radio_menu_item(filetype_menu, ft);
 	}
-	create_radio_menu_item(filetype_menu, filetypes[GEANY_FILETYPES_NONE]);
 }
 
 
 void filetypes_init()
 {
 	filetypes_init_types();
+
 	create_set_filetype_menu();
 	setup_config_file_menus();
 }
@@ -871,7 +962,7 @@ static GeanyFiletype *find_shebang(const gchar *utf8_filename, const gchar *line
 
 		if (strncmp(tmp, "env ", 4) == 0 && strlen(tmp) > 4)
 		{	/* skip "env" and read the following interpreter */
-			basename_interpreter +=4;
+			basename_interpreter += 4;
 		}
 
 		if (strncmp(basename_interpreter, "sh", 2) == 0)
@@ -976,7 +1067,7 @@ GeanyFiletype *filetypes_detect_from_document(GeanyDocument *doc)
 #ifdef HAVE_PLUGINS
 /* Currently only used by external plugins (e.g. geanyprj). */
 /**
- *  Detect filetype based on a shebang line in the file, or the filename extension.
+ *  Detects filetype based on a shebang line in the file or the filename extension.
  *
  *  @param utf8_filename The filename in UTF-8 encoding.
  *
@@ -1083,12 +1174,9 @@ static void filetype_free(gpointer data, G_GNUC_UNUSED gpointer user_data)
 	g_free(ft->comment_open);
 	g_free(ft->comment_close);
 	g_free(ft->context_action_cmd);
-	g_free(ft->programs->compiler);
-	g_free(ft->programs->linker);
-	g_free(ft->programs->run_cmd);
-	g_free(ft->programs->run_cmd2);
-	g_free(ft->programs);
-	g_free(ft->actions);
+	g_free(ft->filecmds);
+	g_free(ft->ftdefcmds);
+	g_free(ft->execcmds);
 	set_error_regex(ft, NULL);
 
 	g_strfreev(ft->pattern);
@@ -1160,57 +1248,46 @@ static void load_settings(gint ft_id, GKeyFile *config, GKeyFile *configh)
 		filetypes[ft_id]->context_action_cmd = result;
 	}
 
+	result = utils_get_setting_string(configh, "settings", "tag_parser", NULL);
+	if (!result)
+		result = utils_get_setting_string(config, "settings", "tag_parser", NULL);
+	if (result)
+	{
+		ft->lang = tm_source_file_get_named_lang(result);
+		if (ft->lang < 0)
+			geany_debug("Cannot find tag parser '%s' for custom filetype '%s'.", result, ft->name);
+		g_free(result);
+	}
+
+	result = utils_get_setting_string(configh, "settings", "lexer_filetype", NULL);
+	if (!result)
+		result = utils_get_setting_string(config, "settings", "lexer_filetype", NULL);
+	if (result)
+	{
+		ft->lexer_filetype = filetypes_lookup_by_name(result);
+		if (!ft->lexer_filetype)
+			geany_debug("Cannot find lexer filetype '%s' for custom filetype '%s'.", result, ft->name);
+		g_free(result);
+	}
+
 	/* read build settings */
-	result = g_key_file_get_string(configh, "build_settings", "compiler", NULL);
-	if (result == NULL) result = g_key_file_get_string(config, "build_settings", "compiler", NULL);
-	if (G_LIKELY(result != NULL))
-	{
-		filetypes[ft_id]->programs->compiler = result;
-		filetypes[ft_id]->actions->can_compile = TRUE;
-	}
-
-	result = g_key_file_get_string(configh, "build_settings", "linker", NULL);
-	if (result == NULL) result = g_key_file_get_string(config, "build_settings", "linker", NULL);
-	if (result != NULL)
-	{
-		filetypes[ft_id]->programs->linker = result;
-		filetypes[ft_id]->actions->can_link = TRUE;
-	}
-
-	result = g_key_file_get_string(configh, "build_settings", "run_cmd", NULL);
-	if (result == NULL) result = g_key_file_get_string(config, "build_settings", "run_cmd", NULL);
-	if (G_LIKELY(result != NULL))
-	{
-		filetypes[ft_id]->programs->run_cmd = result;
-		filetypes[ft_id]->actions->can_exec = TRUE;
-	}
-
-	result = g_key_file_get_string(configh, "build_settings", "run_cmd2", NULL);
-	if (result == NULL) result = g_key_file_get_string(config, "build_settings", "run_cmd2", NULL);
-	if (result != NULL)
-	{
-		filetypes[ft_id]->programs->run_cmd2 = result;
-		filetypes[ft_id]->actions->can_exec = TRUE;
-	}
-
-	result = g_key_file_get_string(configh, "build_settings", "error_regex", NULL);
-	if (result == NULL) result = g_key_file_get_string(config, "build_settings", "error_regex", NULL);
-	if (result != NULL)
-	{
-		set_error_regex(ft, result);
-	}
+	build_load_menu(config, GEANY_BCS_FT, (gpointer)ft);
+	build_load_menu(configh, GEANY_BCS_HOME_FT, (gpointer)ft);
 }
 
 
 /* simple wrapper function to print file errors in DEBUG mode */
 static void load_system_keyfile(GKeyFile *key_file, const gchar *file, GKeyFileFlags flags,
-								G_GNUC_UNUSED GError **just_for_compatibility)
+		GeanyFiletype *ft)
 {
 	GError *error = NULL;
 	gboolean done = g_key_file_load_from_file(key_file, file, flags, &error);
-	if (G_UNLIKELY(! done) && G_UNLIKELY(error != NULL))
+
+	if (error != NULL)
 	{
-		geany_debug("Failed to open %s (%s)", file, error->message);
+		if (!done && !ft->priv->custom)
+			geany_debug("Failed to open %s (%s)", file, error->message);
+
 		g_error_free(error);
 		error = NULL;
 	}
@@ -1224,10 +1301,12 @@ void filetypes_load_config(gint ft_id, gboolean reload)
 {
 	GKeyFile *config, *config_home;
 	GeanyFiletypePrivate *pft;
+	GeanyFiletype *ft;
 
 	g_return_if_fail(ft_id >= 0 && ft_id < (gint) filetypes_array->len);
 
-	pft = filetypes[ft_id]->priv;
+	ft = filetypes[ft_id];
+	pft = ft->priv;
 
 	/* when reloading, proceed only if the settings were already loaded */
 	if (reload && G_UNLIKELY(! pft->keyfile_loaded))
@@ -1248,7 +1327,7 @@ void filetypes_load_config(gint ft_id, gboolean reload)
 		gchar *f = g_strconcat(app->configdir,
 			G_DIR_SEPARATOR_S GEANY_FILEDEFS_SUBDIR G_DIR_SEPARATOR_S "filetypes.", ext, NULL);
 
-		load_system_keyfile(config, f0, G_KEY_FILE_KEEP_COMMENTS, NULL);
+		load_system_keyfile(config, f0, G_KEY_FILE_KEEP_COMMENTS, ft);
 		g_key_file_load_from_file(config_home, f, G_KEY_FILE_KEEP_COMMENTS, NULL);
 
 		g_free(ext);
@@ -1267,6 +1346,10 @@ void filetypes_load_config(gint ft_id, gboolean reload)
 gchar *filetypes_get_conf_extension(gint filetype_idx)
 {
 	gchar *result;
+	GeanyFiletype *ft = filetypes[filetype_idx];
+
+	if (ft->priv->custom)
+		return g_strconcat(ft->name, ".conf", NULL);
 
 	/* Handle any special extensions different from lowercase filetype->name */
 	switch (filetype_idx)
@@ -1274,7 +1357,7 @@ gchar *filetypes_get_conf_extension(gint filetype_idx)
 		case GEANY_FILETYPES_CPP: result = g_strdup("cpp"); break;
 		case GEANY_FILETYPES_CS: result = g_strdup("cs"); break;
 		case GEANY_FILETYPES_MAKE: result = g_strdup("makefile"); break;
-		default: result = g_ascii_strdown(filetypes[filetype_idx]->name, -1); break;
+		default: result = g_ascii_strdown(ft->name, -1); break;
 	}
 	return result;
 }
@@ -1284,38 +1367,27 @@ void filetypes_save_commands(void)
 {
 	gchar *conf_prefix = g_strconcat(app->configdir,
 		G_DIR_SEPARATOR_S GEANY_FILEDEFS_SUBDIR G_DIR_SEPARATOR_S "filetypes.", NULL);
-	gint i;
+	guint i;
 
-	for (i = 1; i < GEANY_MAX_BUILT_IN_FILETYPES; i++)
+	for (i = 1; i < filetypes_array->len; i++)
 	{
-		struct build_programs *bp = filetypes[i]->programs;
 		GKeyFile *config_home;
 		gchar *fname, *ext, *data;
 
-		if (! bp->modified)
-			continue;
-
-		ext = filetypes_get_conf_extension(i);
-		fname = g_strconcat(conf_prefix, ext, NULL);
-		g_free(ext);
-
-		config_home = g_key_file_new();
-		g_key_file_load_from_file(config_home, fname, G_KEY_FILE_KEEP_COMMENTS, NULL);
-
-		if (NZV(bp->compiler))
-			g_key_file_set_string(config_home, "build_settings", "compiler", bp->compiler);
-		if (NZV(bp->linker))
-			g_key_file_set_string(config_home, "build_settings", "linker", bp->linker);
-		if (NZV(bp->run_cmd))
-			g_key_file_set_string(config_home, "build_settings", "run_cmd", bp->run_cmd);
-		if (NZV(bp->run_cmd2))
-			g_key_file_set_string(config_home, "build_settings", "run_cmd2", bp->run_cmd2);
-
-		data = g_key_file_to_data(config_home, NULL, NULL);
-		utils_write_file(fname, data);
-		g_free(data);
-		g_key_file_free(config_home);
-		g_free(fname);
+		if (filetypes[i]->home_save_needed)
+		{
+			ext = filetypes_get_conf_extension(i);
+			fname = g_strconcat(conf_prefix, ext, NULL);
+			g_free(ext);
+			config_home = g_key_file_new();
+			g_key_file_load_from_file(config_home, fname, G_KEY_FILE_KEEP_COMMENTS, NULL);
+			build_save_menu(config_home, (gpointer)(filetypes[i]), GEANY_BCS_HOME_FT);
+			data = g_key_file_to_data(config_home, NULL, NULL);
+			utils_write_file(fname, data);
+			g_free(data);
+			g_key_file_free(config_home);
+			g_free(fname);
+		}
 	}
 	g_free(conf_prefix);
 }
@@ -1365,7 +1437,8 @@ GtkFileFilter *filetypes_create_file_filter(const GeanyFiletype *ft)
 }
 
 
-/* Indicates whether there is a tag parser for the filetype or not. */
+/* Indicates whether there is a tag parser for the filetype or not.
+ * Only works for custom filetypes if the filetype settings have been loaded. */
 gboolean filetype_has_tags(GeanyFiletype *ft)
 {
 	g_return_val_if_fail(ft != NULL, FALSE);
@@ -1374,7 +1447,7 @@ gboolean filetype_has_tags(GeanyFiletype *ft)
 }
 
 
-/** Find a filetype pointer from its @a name field.
+/** Finds a filetype pointer from its @a name field.
  * @param name Filetype name.
  * @return The filetype found, or @c NULL.
  *
@@ -1401,9 +1474,9 @@ static gchar *get_regex_match_string(const gchar *message, regmatch_t *pmatch, g
 }
 
 
-static void compile_regex(GeanyFiletype *ft, regex_t *regex)
+static void compile_regex(GeanyFiletype *ft, regex_t *regex, gchar *regstr)
 {
-	gint retval = regcomp(regex, ft->error_regex_string, REG_EXTENDED);
+	gint retval = regcomp(regex, regstr, REG_EXTENDED);
 
 	ft->priv->error_regex_compiled = (retval == 0);	/* prevent recompilation */
 
@@ -1422,22 +1495,41 @@ static void compile_regex(GeanyFiletype *ft, regex_t *regex)
 gboolean filetypes_parse_error_message(GeanyFiletype *ft, const gchar *message,
 		gchar **filename, gint *line)
 {
+	gchar *regstr;
+	gchar **tmp;
+	GeanyDocument *doc;
+#ifdef HAVE_REGCOMP
+	regex_t *regex;
+	regmatch_t pmatch[3];
+#endif
+	if (ft == NULL)
+	{
+		doc = document_get_current();
+		if (doc != NULL)
+			ft = doc->file_type;
+	}
+	tmp = build_get_regex(build_info.grp, ft, NULL);
+	if (tmp == NULL)
+		return FALSE;
+	regstr = *tmp;
 #ifndef HAVE_REGCOMP
-	if (!NZV(ft->error_regex_string))
+	if (!NZV(regstr))
 		geany_debug("No regex support - maybe you should configure with --enable-gnu-regex!");
 	return FALSE;
 #else
-	regex_t *regex = &ft->priv->error_regex;
-	regmatch_t pmatch[3];
+	regex = &ft->priv->error_regex;
 
 	*filename = NULL;
 	*line = -1;
 
-	if (!NZV(ft->error_regex_string))
+	if (!NZV(regstr))
 		return FALSE;
 
-	if (!ft->priv->error_regex_compiled)
-		compile_regex(ft, regex);
+	if (!ft->priv->error_regex_compiled || regstr != ft->priv->last_string)
+	{
+		compile_regex(ft, regex, regstr);
+		ft->priv->last_string = regstr;
+	}
 	if (!ft->priv->error_regex_compiled)	/* regex error */
 		return FALSE;
 
@@ -1479,6 +1571,18 @@ gboolean filetypes_parse_error_message(GeanyFiletype *ft, const gchar *message,
 }
 
 
+#ifdef G_OS_WIN32
+static void convert_filetype_extensions_to_lower_case(gchar **patterns, gsize len)
+{
+	guint i;
+	for (i = 0; i < len; i++)
+	{
+		setptr(patterns[i], g_ascii_strdown(patterns[i], -1));
+	}
+}
+#endif
+
+
 void filetypes_read_extensions(void)
 {
 	guint i;
@@ -1487,7 +1591,6 @@ void filetypes_read_extensions(void)
 		"filetype_extensions.conf", NULL);
 	gchar *userconfigfile = g_strconcat(app->configdir, G_DIR_SEPARATOR_S,
 		"filetype_extensions.conf", NULL);
-	gchar **list;
 	GKeyFile *sysconfig = g_key_file_new();
 	GKeyFile *userconfig = g_key_file_new();
 
@@ -1499,20 +1602,33 @@ void filetypes_read_extensions(void)
 	{
 		gboolean userset =
 			g_key_file_has_key(userconfig, "Extensions", filetypes[i]->name, NULL);
-		list = g_key_file_get_string_list(
+		gchar **list = g_key_file_get_string_list(
 			(userset) ? userconfig : sysconfig, "Extensions", filetypes[i]->name, &len, NULL);
+
 		if (G_LIKELY(list) && G_LIKELY(len > 0))
 		{
 			g_strfreev(filetypes[i]->pattern);
 			filetypes[i]->pattern = list;
+#ifdef G_OS_WIN32
+			convert_filetype_extensions_to_lower_case(filetypes[i]->pattern, len);
+#endif
 		}
-		else g_strfreev(list);
+		else
+			g_strfreev(list);
 	}
 
 	g_free(sysconfigfile);
 	g_free(userconfigfile);
 	g_key_file_free(sysconfig);
 	g_key_file_free(userconfig);
+
+	foreach_document(i)
+	{
+		GeanyDocument *doc = documents[i];
+		if (doc->file_type->id != GEANY_FILETYPES_NONE)
+			continue;
+		document_set_filetype(doc, filetypes_detect_from_document(doc));
+	}
 }
 
 
@@ -1527,3 +1643,35 @@ GeanyFiletype *filetypes_index(gint idx)
 {
 	return (idx >= 0 && idx < (gint) filetypes_array->len) ? filetypes[idx] : NULL;
 }
+
+
+void filetypes_reload(void)
+{
+	guint i;
+	GeanyDocument *current_doc;
+
+	/* save possibly changed commands before re-reading them */
+	filetypes_save_commands();
+
+	/* reload filetype configs */
+	for (i = 0; i < filetypes_array->len; i++)
+	{
+		/* filetypes_load_config() will skip not loaded filetypes */
+		filetypes_load_config(i, TRUE);
+	}
+
+	current_doc = document_get_current();
+	if (!current_doc)
+		return;
+
+	/* update document styling */
+	foreach_document(i)
+	{
+		if (current_doc != documents[i])
+			document_reload_config(documents[i]);
+	}
+	/* process the current document at last */
+	document_reload_config(current_doc);
+}
+
+
