@@ -1,8 +1,8 @@
 /*
  *      geanywraplabel.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2009 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2009 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2009-2010 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2009-2010 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- *  $Id: geanywraplabel.c 3708 2009-04-15 22:47:33Z eht16 $
+ *  $Id: geanywraplabel.c 4862 2010-04-25 17:43:39Z eht16 $
  */
 
 /*
@@ -33,8 +33,7 @@
 
 
 
-#define GEANY_WRAP_LABEL_GET_PRIVATE(obj)		(G_TYPE_INSTANCE_GET_PRIVATE((obj),\
-	GEANY_WRAP_LABEL_TYPE, GeanyWrapLabelPrivate))
+#define GEANY_WRAP_LABEL_GET_PRIVATE(obj)	(GEANY_WRAP_LABEL(obj)->priv)
 
 
 struct _GeanyWrapLabelClass
@@ -42,22 +41,23 @@ struct _GeanyWrapLabelClass
 	GtkLabelClass parent_class;
 };
 
-struct _GeanyWrapLabel
-{
-	GtkLabel parent;
-};
-
 typedef struct
 {
 	gsize wrap_width;
 } GeanyWrapLabelPrivate;
 
+struct _GeanyWrapLabel
+{
+	GtkLabel parent;
+	GeanyWrapLabelPrivate *priv;
+};
+
 
 static void geany_wrap_label_size_request	(GtkWidget *widget, GtkRequisition *req);
 static void geany_wrap_label_size_allocate	(GtkWidget *widget, GtkAllocation *alloc);
 static void geany_wrap_label_set_wrap_width	(GtkWidget *widget, gsize width);
 
-G_DEFINE_TYPE(GeanyWrapLabel, geany_wrap_label, GTK_TYPE_LABEL);
+G_DEFINE_TYPE(GeanyWrapLabel, geany_wrap_label, GTK_TYPE_LABEL)
 
 
 static void geany_wrap_label_class_init(GeanyWrapLabelClass *klass)
@@ -73,8 +73,12 @@ static void geany_wrap_label_class_init(GeanyWrapLabelClass *klass)
 
 static void geany_wrap_label_init(GeanyWrapLabel *self)
 {
-	GeanyWrapLabelPrivate *priv = GEANY_WRAP_LABEL_GET_PRIVATE(self);
+	GeanyWrapLabelPrivate *priv;
 
+	self->priv = G_TYPE_INSTANCE_GET_PRIVATE(self,
+		GEANY_WRAP_LABEL_TYPE, GeanyWrapLabelPrivate);
+
+	priv = self->priv;
 	priv->wrap_width = 0;
 }
 
