@@ -1,8 +1,8 @@
 /*
  *      highlighting.h - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2005-2008 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2006-2008 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2005-2010 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2006-2010 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: highlighting.h 2903 2008-08-26 12:43:46Z ntrel $
+ * $Id: highlighting.h 4552 2010-01-25 15:53:54Z ntrel $
  */
 
 
@@ -30,7 +30,7 @@
 
 
 /** Fields representing the different attributes of a Scintilla lexer style.
- * @see Scintilla messages @c SCI_STYLEGETFORE, etc, for use with ScintillaFuncs::send_message(). */
+ * @see Scintilla messages @c SCI_STYLEGETFORE, etc, for use with scintilla_send_message(). */
 typedef struct GeanyLexerStyle
 {
 	gint	foreground;	/**< Foreground text colour, in @c 0xBBGGRR format. */
@@ -41,9 +41,11 @@ typedef struct GeanyLexerStyle
 GeanyLexerStyle;
 
 
+void highlighting_init(void);
+
 void highlighting_init_styles(gint filetype_idx, GKeyFile *config, GKeyFile *configh);
 
-void highlighting_set_styles(ScintillaObject *sci, gint filetype_idx);
+void highlighting_set_styles(ScintillaObject *sci, GeanyFiletype *ft);
 
 const GeanyLexerStyle *highlighting_get_style(gint ft_id, gint style_id);
 
