@@ -1,8 +1,8 @@
 /*
  *      keyfile.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2005-2009 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2006-2009 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2005-2010 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2006-2010 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: keyfile.c 3997 2009-07-20 12:25:24Z ntrel $
+ * $Id: keyfile.c 5134 2010-08-11 14:16:42Z ntrel $
  */
 
 /*
@@ -98,7 +98,7 @@ GPtrArray *pref_groups = NULL;
 
 /* The group will be free'd on quitting.
  * @param for_prefs_dialog is whether the group also has Prefs dialog items. */
-void configuration_add_pref_group(struct GeanyPrefGroup *group, gboolean for_prefs_dialog)
+void configuration_add_pref_group(struct StashGroup *group, gboolean for_prefs_dialog)
 {
 	g_ptr_array_add(keyfile_groups, group);
 
@@ -109,7 +109,7 @@ void configuration_add_pref_group(struct GeanyPrefGroup *group, gboolean for_pre
 
 static void init_pref_groups(void)
 {
-	GeanyPrefGroup *group;
+	StashGroup *group;
 
 	group = stash_group_new(PACKAGE);
 	configuration_add_pref_group(group, TRUE);
@@ -122,7 +122,13 @@ static void init_pref_groups(void)
 	stash_group_add_toggle_button(group, &interface_prefs.notebook_double_click_hides_widgets,
 		"notebook_double_click_hides_widgets", FALSE, "check_double_click_hides_widgets");
 	stash_group_add_integer(group, &interface_prefs.tab_pos_sidebar, "tab_pos_sidebar", GTK_POS_TOP);
+	stash_group_add_radio_buttons(group, &interface_prefs.sidebar_pos,
+		"sidebar_pos", GTK_POS_LEFT,
+		"radio_sidebar_left", GTK_POS_LEFT,
+		"radio_sidebar_right", GTK_POS_RIGHT,
+		NULL);
 
+	/* editor display */
 	stash_group_add_toggle_button(group, &interface_prefs.highlighting_invert_all,
 		"highlighting_invert_all", FALSE, "check_highlighting_invert");
 
@@ -138,8 +144,6 @@ static void init_pref_groups(void)
 		"use_tab_to_indent", TRUE, "check_tab_key_indents");
 	stash_group_add_spin_button_integer(group, &editor_prefs.indentation->width,
 		"pref_editor_tab_width", 4, "spin_indent_width");
-	stash_group_add_spin_button_integer(group, &editor_prefs.indentation->hard_tab_width,
-		"indent_hard_tab_width", 8, "spin_tab_width");
 	stash_group_add_combo_box(group, (gint*)(void*)&editor_prefs.indentation->auto_indent_mode,
 		"indent_mode", GEANY_AUTOINDENT_CURRENTCHARS, "combo_auto_indent_mode");
 	stash_group_add_radio_buttons(group, (gint*)(void*)&editor_prefs.indentation->type,
@@ -148,6 +152,12 @@ static void init_pref_groups(void)
 		"radio_indent_tabs", GEANY_INDENT_TYPE_TABS,
 		"radio_indent_both", GEANY_INDENT_TYPE_BOTH,
 		NULL);
+	stash_group_add_radio_buttons(group, (gint*)(void*)&editor_prefs.show_virtual_space,
+		"virtualspace", GEANY_VIRTUAL_SPACE_SELECTION,
+		"radio_virtualspace_disabled", GEANY_VIRTUAL_SPACE_DISABLED,
+		"radio_virtualspace_selection", GEANY_VIRTUAL_SPACE_SELECTION,
+		"radio_virtualspace_always", GEANY_VIRTUAL_SPACE_ALWAYS,
+		NULL);
 	stash_group_add_toggle_button(group, &editor_prefs.autocomplete_doc_words,
 		"autocomplete_doc_words", FALSE, "check_autocomplete_doc_words");
 	stash_group_add_toggle_button(group, &editor_prefs.completion_drops_rest_of_word,
@@ -155,6 +165,8 @@ static void init_pref_groups(void)
 	stash_group_add_spin_button_integer(group, (gint*)&editor_prefs.autocompletion_max_entries,
 		"autocompletion_max_entries", GEANY_MAX_AUTOCOMPLETE_WORDS,
 		"spin_autocompletion_max_entries");
+	stash_group_add_string(group, &editor_prefs.color_scheme,
+		"color_scheme", NULL);
 
 	/* files */
 	stash_group_add_spin_button_integer(group, (gint*)&file_prefs.mru_length,
@@ -166,6 +178,7 @@ static void init_pref_groups(void)
 	group = stash_group_new(PACKAGE);
 	configuration_add_pref_group(group, FALSE);
 	stash_group_set_write_once(group, TRUE);
+
 	stash_group_add_boolean(group, &editor_prefs.show_scrollbars,
 		"show_editor_scrollbars", TRUE);
 	stash_group_add_boolean(group, &editor_prefs.brace_match_ltgt,
@@ -176,10 +189,15 @@ static void init_pref_groups(void)
 		"complete_snippets_whilst_editing", FALSE);
 	stash_group_add_boolean(group, &interface_prefs.show_symbol_list_expanders,
 		"show_symbol_list_expanders", TRUE);
+	stash_group_add_boolean(group, &interface_prefs.compiler_tab_autoscroll,
+		"compiler_tab_autoscroll", TRUE);
 	stash_group_add_boolean(group, &ui_prefs.allow_always_save,
 		"allow_always_save", FALSE);
 	stash_group_add_boolean(group, &file_prefs.use_safe_file_saving,
 		"use_safe_file_saving", FALSE);
+	/* for backwards-compatibility */
+	stash_group_add_integer(group, &editor_prefs.indentation->hard_tab_width,
+		"indent_hard_tab_width", 8);
 }
 
 
@@ -193,7 +211,7 @@ SettingAction;
 static void settings_action(GKeyFile *config, SettingAction action)
 {
 	guint i;
-	GeanyPrefGroup *group;
+	StashGroup *group;
 
 	foreach_ptr_array(group, i, keyfile_groups)
 	{
@@ -341,8 +359,14 @@ static void save_dialog_prefs(GKeyFile *config)
 	g_key_file_set_boolean(config, PACKAGE, "show_notebook_tabs", interface_prefs.show_notebook_tabs);
 	g_key_file_set_boolean(config, PACKAGE, "show_tab_cross", file_prefs.show_tab_cross);
 	g_key_file_set_boolean(config, PACKAGE, "tab_order_ltr", file_prefs.tab_order_ltr);
+	g_key_file_set_boolean(config, PACKAGE, "tab_order_beside", file_prefs.tab_order_beside);
 	g_key_file_set_integer(config, PACKAGE, "tab_pos_editor", interface_prefs.tab_pos_editor);
 	g_key_file_set_integer(config, PACKAGE, "tab_pos_msgwin", interface_prefs.tab_pos_msgwin);
+	g_key_file_set_boolean(config, PACKAGE, "msgwin_status_visible", interface_prefs.msgwin_status_visible);
+	g_key_file_set_boolean(config, PACKAGE, "msgwin_compiler_visible", interface_prefs.msgwin_compiler_visible);
+	g_key_file_set_boolean(config, PACKAGE, "msgwin_messages_visible", interface_prefs.msgwin_messages_visible);
+	g_key_file_set_boolean(config, PACKAGE, "msgwin_scribble_visible", interface_prefs.msgwin_scribble_visible);
+	g_key_file_set_boolean(config, PACKAGE, "use_native_windows_dialogs", interface_prefs.use_native_windows_dialogs);
 
 	/* display */
 	g_key_file_set_boolean(config, PACKAGE, "show_indent_guide", editor_prefs.show_indent_guide);
@@ -350,8 +374,9 @@ static void save_dialog_prefs(GKeyFile *config)
 	g_key_file_set_boolean(config, PACKAGE, "show_line_endings", editor_prefs.show_line_endings);
 	g_key_file_set_boolean(config, PACKAGE, "show_markers_margin", editor_prefs.show_markers_margin);
 	g_key_file_set_boolean(config, PACKAGE, "show_linenumber_margin", editor_prefs.show_linenumber_margin);
-	g_key_file_set_integer(config, PACKAGE, "long_line_type", editor_prefs.long_line_type);
-	g_key_file_set_integer(config, PACKAGE, "long_line_column", editor_prefs.long_line_column);
+	g_key_file_set_boolean(config, PACKAGE, "long_line_enabled", editor_prefs.long_line_global_enabled);
+	g_key_file_set_integer(config, PACKAGE, "long_line_type", editor_prefs.long_line_global_type);
+	g_key_file_set_integer(config, PACKAGE, "long_line_column", editor_prefs.long_line_global_column);
 	g_key_file_set_string(config, PACKAGE, "long_line_color", editor_prefs.long_line_color);
 
 	/* editor */
@@ -387,6 +412,8 @@ static void save_dialog_prefs(GKeyFile *config)
 	/* toolbar */
 	g_key_file_set_boolean(config, PACKAGE, "pref_toolbar_show", toolbar_prefs.visible);
 	g_key_file_set_boolean(config, PACKAGE, "pref_toolbar_append_to_menu", toolbar_prefs.append_to_menu);
+	g_key_file_set_boolean(config, PACKAGE, "pref_toolbar_use_gtk_default_style", toolbar_prefs.use_gtk_default_style);
+	g_key_file_set_boolean(config, PACKAGE, "pref_toolbar_use_gtk_default_icon", toolbar_prefs.use_gtk_default_icon);
 	g_key_file_set_integer(config, PACKAGE, "pref_toolbar_icon_style", toolbar_prefs.icon_style);
 	g_key_file_set_integer(config, PACKAGE, "pref_toolbar_icon_size", toolbar_prefs.icon_size);
 
@@ -401,12 +428,14 @@ static void save_dialog_prefs(GKeyFile *config)
 	g_key_file_set_string(config, PACKAGE, "pref_template_datetime", template_prefs.datetime_format);
 
 	/* tools settings */
-	g_key_file_set_string(config, "tools", "make_cmd", tool_prefs.make_cmd ? tool_prefs.make_cmd : "");
 	g_key_file_set_string(config, "tools", "term_cmd", tool_prefs.term_cmd ? tool_prefs.term_cmd : "");
 	g_key_file_set_string(config, "tools", "browser_cmd", tool_prefs.browser_cmd ? tool_prefs.browser_cmd : "");
 	g_key_file_set_string(config, "tools", "grep_cmd", tool_prefs.grep_cmd ? tool_prefs.grep_cmd : "");
 	g_key_file_set_string(config, PACKAGE, "context_action_cmd", tool_prefs.context_action_cmd);
 
+	/* build menu */
+	build_save_menu(config, NULL, GEANY_BCS_PREF);
+
 	/* printing */
 	g_key_file_set_string(config, "printing", "print_cmd", printing_prefs.external_print_cmd ? printing_prefs.external_print_cmd : "");
 	g_key_file_set_boolean(config, "printing", "use_gtk_printing", printing_prefs.use_gtk_printing);
@@ -425,6 +454,9 @@ static void save_dialog_prefs(GKeyFile *config)
 
 		if (!g_key_file_has_key(config, "VTE", "emulation", NULL))	/* hidden */
 			g_key_file_set_string(config, "VTE", "emulation", vc->emulation);
+		if (!g_key_file_has_key(config, "VTE", "send_selection_unsafe", NULL))	/* hidden */
+			g_key_file_set_boolean(config, "VTE", "send_selection_unsafe",
+				vc->send_selection_unsafe);
 		g_key_file_set_string(config, "VTE", "font", vc->font);
 		g_key_file_set_boolean(config, "VTE", "scroll_on_key", vc->scroll_on_key);
 		g_key_file_set_boolean(config, "VTE", "scroll_on_out", vc->scroll_on_out);
@@ -463,7 +495,7 @@ static void save_ui_prefs(GKeyFile *config)
 		GtkTextIter start, end, iter;
 		GtkTextMark *mark;
 
-		buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(ui_lookup_widget(main_widgets.window, "textview_scribble")));
+		buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(msgwindow.scribble));
 		gtk_text_buffer_get_bounds(buffer, &start, &end);
 		scribble_text = gtk_text_buffer_get_text(buffer, &start, &end, FALSE);
 		g_key_file_set_string(config, PACKAGE, "scribble_text", scribble_text);
@@ -481,10 +513,7 @@ static void save_ui_prefs(GKeyFile *config)
 				gtk_paned_get_position(GTK_PANED(ui_lookup_widget(main_widgets.window, "hpaned1"))));
 		g_key_file_set_integer(config, PACKAGE, "msgwindow_position",
 				gtk_paned_get_position(GTK_PANED(ui_lookup_widget(main_widgets.window, "vpaned1"))));
-	}
 
-	if (prefs.save_winpos)
-	{
 		gtk_window_get_position(GTK_WINDOW(main_widgets.window), &ui_prefs.geometry[0], &ui_prefs.geometry[1]);
 		gtk_window_get_size(GTK_WINDOW(main_widgets.window), &ui_prefs.geometry[2], &ui_prefs.geometry[3]);
 		if (gdk_window_get_state(main_widgets.window->window) & GDK_WINDOW_STATE_MAXIMIZED)
@@ -595,14 +624,6 @@ void configuration_load_session_files(GKeyFile *config, gboolean read_recent_fil
 }
 
 
-#define GEANY_GET_SETTING(propertyname, value, default_value) \
-	if (g_object_class_find_property( \
-		G_OBJECT_GET_CLASS(G_OBJECT(gtk_settings_get_default())), propertyname)) \
-			g_object_get(G_OBJECT(gtk_settings_get_default()), propertyname, &value, \
-				NULL); \
-	else \
-		value = default_value;
-
 static void load_dialog_prefs(GKeyFile *config)
 {
 	gchar *tmp_string, *tmp_string2;
@@ -629,16 +650,28 @@ static void load_dialog_prefs(GKeyFile *config)
 	interface_prefs.sidebar_openfiles_visible = utils_get_setting_boolean(config, PACKAGE, "sidebar_openfiles_visible", TRUE);
 	interface_prefs.statusbar_visible = utils_get_setting_boolean(config, PACKAGE, "statusbar_visible", TRUE);
 	file_prefs.tab_order_ltr = utils_get_setting_boolean(config, PACKAGE, "tab_order_ltr", TRUE);
+	file_prefs.tab_order_beside = utils_get_setting_boolean(config, PACKAGE, "tab_order_beside", FALSE);
 	interface_prefs.show_notebook_tabs = utils_get_setting_boolean(config, PACKAGE, "show_notebook_tabs", TRUE);
 	file_prefs.show_tab_cross = utils_get_setting_boolean(config, PACKAGE, "show_tab_cross", TRUE);
 	interface_prefs.editor_font = utils_get_setting_string(config, PACKAGE, "editor_font", GEANY_DEFAULT_FONT_EDITOR);
 	interface_prefs.tagbar_font = utils_get_setting_string(config, PACKAGE, "tagbar_font", GEANY_DEFAULT_FONT_SYMBOL_LIST);
 	interface_prefs.msgwin_font = utils_get_setting_string(config, PACKAGE, "msgwin_font", GEANY_DEFAULT_FONT_MSG_WINDOW);
+	interface_prefs.msgwin_status_visible = utils_get_setting_boolean(config, PACKAGE, "msgwin_status_visible", TRUE);
+	interface_prefs.msgwin_compiler_visible = utils_get_setting_boolean(config, PACKAGE, "msgwin_compiler_visible", TRUE);
+	interface_prefs.msgwin_messages_visible = utils_get_setting_boolean(config, PACKAGE, "msgwin_messages_visible", TRUE);
+	interface_prefs.msgwin_scribble_visible = utils_get_setting_boolean(config, PACKAGE, "msgwin_scribble_visible", TRUE);
+	interface_prefs.use_native_windows_dialogs = utils_get_setting_boolean(config, PACKAGE, "use_native_windows_dialogs", FALSE);
 
 	/* display, editor */
-	editor_prefs.long_line_type = utils_get_setting_integer(config, PACKAGE, "long_line_type", 0);
+	editor_prefs.long_line_global_enabled = utils_get_setting_boolean(config, PACKAGE, "long_line_enabled", TRUE);
+	editor_prefs.long_line_global_type = utils_get_setting_integer(config, PACKAGE, "long_line_type", 0);
+	if (editor_prefs.long_line_global_type == 2) /* backward compatibility */
+	{
+		editor_prefs.long_line_global_type = 0;
+		editor_prefs.long_line_global_enabled = FALSE;
+	}
 	editor_prefs.long_line_color = utils_get_setting_string(config, PACKAGE, "long_line_color", "#C2EBC2");
-	editor_prefs.long_line_column = utils_get_setting_integer(config, PACKAGE, "long_line_column", 72);
+	editor_prefs.long_line_global_column = utils_get_setting_integer(config, PACKAGE, "long_line_column", 72);
 	editor_prefs.symbolcompletion_min_chars = utils_get_setting_integer(config, PACKAGE, "symbolcompletion_min_chars", GEANY_MIN_SYMBOLLIST_CHARS);
 	editor_prefs.symbolcompletion_max_height = utils_get_setting_integer(config, PACKAGE, "symbolcompletion_max_height", GEANY_MAX_SYMBOLLIST_HEIGHT);
 	editor_prefs.line_wrapping = utils_get_setting_boolean(config, PACKAGE, "line_wrapping", FALSE); /* default is off for better performance */
@@ -698,12 +731,13 @@ static void load_dialog_prefs(GKeyFile *config)
 	toolbar_prefs.visible = utils_get_setting_boolean(config, PACKAGE, "pref_toolbar_show", TRUE);
 	toolbar_prefs.append_to_menu = utils_get_setting_boolean(config, PACKAGE, "pref_toolbar_append_to_menu", FALSE);
 	{
-		GtkIconSize tb_iconsize;
-		GtkToolbarStyle tb_style;
-		GEANY_GET_SETTING("gtk-toolbar-style", tb_style, GTK_TOOLBAR_ICONS);
-		GEANY_GET_SETTING("gtk-toolbar-icon-size", tb_iconsize, GTK_ICON_SIZE_LARGE_TOOLBAR);
-		toolbar_prefs.icon_style = utils_get_setting_integer(config, PACKAGE, "pref_toolbar_icon_style", tb_style);
-		toolbar_prefs.icon_size = utils_get_setting_integer(config, PACKAGE, "pref_toolbar_icon_size", tb_iconsize);
+		toolbar_prefs.use_gtk_default_style = utils_get_setting_boolean(config, PACKAGE, "pref_toolbar_use_gtk_default_style", TRUE);
+		if (! toolbar_prefs.use_gtk_default_style)
+			toolbar_prefs.icon_style = utils_get_setting_integer(config, PACKAGE, "pref_toolbar_icon_style", GTK_TOOLBAR_ICONS);
+
+		toolbar_prefs.use_gtk_default_icon = utils_get_setting_boolean(config, PACKAGE, "pref_toolbar_use_gtk_default_icon", TRUE);
+		if (! toolbar_prefs.use_gtk_default_icon)
+			toolbar_prefs.icon_size = utils_get_setting_integer(config, PACKAGE, "pref_toolbar_icon_size", GTK_ICON_SIZE_LARGE_TOOLBAR);
 	}
 
 	/* VTE */
@@ -712,7 +746,7 @@ static void load_dialog_prefs(GKeyFile *config)
 	if (vte_info.load_vte)
 	{
 		struct passwd *pw = getpwuid(getuid());
-		gchar *shell = (pw != NULL) ? pw->pw_shell : "/bin/sh";
+		const gchar *shell = (pw != NULL) ? pw->pw_shell : "/bin/sh";
 
 		vc = g_new0(VteConfig, 1);
 		vte_info.dir = utils_get_setting_string(config, "VTE", "last_dir", NULL);
@@ -724,6 +758,8 @@ static void load_dialog_prefs(GKeyFile *config)
 			vte_info.dir = g_strdup("/");
 
 		vc->emulation = utils_get_setting_string(config, "VTE", "emulation", "xterm");
+		vc->send_selection_unsafe = utils_get_setting_boolean(config, "VTE",
+			"send_selection_unsafe", FALSE);
 		vc->shell = utils_get_setting_string(config, "VTE", "shell", shell);
 		vc->font = utils_get_setting_string(config, "VTE", "font", "Monospace 10");
 		vc->scroll_on_key = utils_get_setting_boolean(config, "VTE", "scroll_on_key", TRUE);
@@ -764,13 +800,18 @@ static void load_dialog_prefs(GKeyFile *config)
 	template_prefs.datetime_format = utils_get_setting_string(config, PACKAGE, "pref_template_datetime", "%d.%m.%Y %H:%M:%S %Z");
 
 	/* tools */
-	tool_prefs.make_cmd = utils_get_setting_string(config, "tools", "make_cmd", GEANY_DEFAULT_TOOLS_MAKE);
 	tool_prefs.term_cmd = utils_get_setting_string(config, "tools", "term_cmd", GEANY_DEFAULT_TOOLS_TERMINAL);
 	tool_prefs.browser_cmd = utils_get_setting_string(config, "tools", "browser_cmd", GEANY_DEFAULT_TOOLS_BROWSER);
 	tool_prefs.grep_cmd = utils_get_setting_string(config, "tools", "grep_cmd", GEANY_DEFAULT_TOOLS_GREP);
 
 	tool_prefs.context_action_cmd = utils_get_setting_string(config, PACKAGE, "context_action_cmd", "");
 
+	/* build menu */
+	build_set_group_count(GEANY_GBG_FT, utils_get_setting_integer(config, "build-menu", "number_ft_menu_items", 0));
+	build_set_group_count(GEANY_GBG_NON_FT, utils_get_setting_integer(config, "build-menu", "number_non_ft_menu_items", 0));
+	build_set_group_count(GEANY_GBG_EXEC, utils_get_setting_integer(config, "build-menu", "number_exec_menu_items", 0));
+	build_load_menu(config, GEANY_BCS_PREF, NULL);
+
 	/* printing */
 	tmp_string2 = g_find_program_in_path(GEANY_DEFAULT_TOOLS_PRINTCMD);
 #ifdef G_OS_WIN32
@@ -897,7 +938,7 @@ gboolean configuration_load(void)
 	if (! g_file_test(configfile, G_FILE_TEST_IS_REGULAR | G_FILE_TEST_IS_SYMLINK))
 	{	/* config file does not (yet) exist, so try to load a global config file which may be */
 		/* created by distributors */
-		geany_debug("No config file found, try to use global configuration.");
+		geany_debug("No user config file found, trying to use global configuration.");
 		setptr(configfile, g_strconcat(app->datadir, G_DIR_SEPARATOR_S "geany.conf", NULL));
 	}
 
@@ -1034,8 +1075,7 @@ void configuration_apply_settings(void)
 	{	/* update the scribble widget, because now it's realized */
 		GtkTextIter iter;
 		GtkTextBuffer *buffer =
-			gtk_text_view_get_buffer(GTK_TEXT_VIEW(ui_lookup_widget(main_widgets.window,
-				"textview_scribble")));
+			gtk_text_view_get_buffer(GTK_TEXT_VIEW(msgwindow.scribble));
 
 		gtk_text_buffer_set_text(buffer, scribble_text, -1);
 		gtk_text_buffer_get_iter_at_offset(buffer, &iter, scribble_pos);
@@ -1058,77 +1098,11 @@ void configuration_apply_settings(void)
 		ui_prefs.fullscreen = TRUE;
 		ui_set_fullscreen();
 	}
+
+	msgwin_show_hide_tabs();
 }
 
 
-#ifdef GEANY_DEBUG
-/* Geany data file generation is only available with a debug build of Geany. */
-
-static void generate_filetype_extensions(const gchar *output_dir);
-
-
-/* Generate the config files in "data/" from defaults */
-void configuration_generate_data_files(void)
-{
-	gchar *cur_dir, *gen_dir;
-
-	cur_dir = g_get_current_dir();
-	gen_dir = g_strconcat(cur_dir, G_DIR_SEPARATOR_S, "data", NULL);
-	g_free(cur_dir);
-
-	if (! g_file_test(gen_dir, G_FILE_TEST_IS_DIR))
-	{
-		g_print("%s does not exist!\n", gen_dir);
-		return;
-	}
-	g_print("Generating system files in %s:\n", gen_dir);
-	/* currently only filetype extensions are auto-generated. */
-	generate_filetype_extensions(gen_dir);
-	g_free(gen_dir);
-}
-
-
-/* This will write the default settings for the system filetype_extensions.conf */
-static void generate_filetype_extensions(const gchar *output_dir)
-{
-	guint i;
-	gchar *configfile = g_strconcat(output_dir, G_DIR_SEPARATOR_S, "filetype_extensions.conf", NULL);
-	gchar *data, *basename;
-	GKeyFile *config;
-
-	config = g_key_file_new();
-	g_key_file_set_comment(config, NULL, NULL,
-		"*** This file generated by: geany --generate-data-files ***", NULL);
-	/* add filetype keys */
-	for (i = 0; i < filetypes_array->len; i++)
-	{
-		g_key_file_set_string_list(config, "Extensions", filetypes[i]->name,
-			(const gchar**) filetypes[i]->pattern, g_strv_length(filetypes[i]->pattern));
-	}
-	/* add comment */
-	g_key_file_set_comment(config, "Extensions", NULL,
-		"Filetype extension configuration file for Geany\n"
-		"Insert as many items as you want, seperate them with a \";\".\n"
-		"See Geany's main documentation for details.", NULL);
-
-	/* write the file */
-	g_print("%s: ", G_STRFUNC);
-	data = g_key_file_to_data(config, NULL, NULL);
-	basename = g_path_get_basename(configfile);
-
-	if (utils_write_file(configfile, data) == 0)
-		g_print("wrote file %s.\n", basename);
-	else
-		g_print("*** ERROR: error writing file %s\n", basename);
-	g_free(basename);
-
-	g_free(data);
-	g_key_file_free(config);
-}
-
-#endif
-
-
 void configuration_init(void)
 {
 	keyfile_groups = g_ptr_array_new();
@@ -1140,7 +1114,7 @@ void configuration_init(void)
 void configuration_finalize(void)
 {
 	guint i;
-	GeanyPrefGroup *group;
+	StashGroup *group;
 
 	foreach_ptr_array(group, i, keyfile_groups)
 		stash_group_free(group);
@@ -1148,4 +1122,3 @@ void configuration_finalize(void)
 	g_ptr_array_free(keyfile_groups, TRUE);
 	g_ptr_array_free(pref_groups, TRUE);
 }
-
