@@ -1,8 +1,8 @@
 /*
  *      keyfile.h - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2005-2009 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2006-2009 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2005-2010 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2006-2010 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: keyfile.h 3707 2009-04-15 22:46:13Z eht16 $
+ * $Id: keyfile.h 4776 2010-03-18 17:04:17Z ntrel $
  */
 
 
@@ -33,9 +33,9 @@ void configuration_init(void);
 
 void configuration_finalize(void);
 
-struct GeanyPrefGroup;
+struct StashGroup;
 
-void configuration_add_pref_group(struct GeanyPrefGroup *group, gboolean for_prefs_dialog);
+void configuration_add_pref_group(struct StashGroup *group, gboolean for_prefs_dialog);
 
 void configuration_save(void);
 
@@ -55,9 +55,4 @@ void configuration_save_session_files(GKeyFile *config);
  * realisation of the main window */
 void configuration_apply_settings(void);
 
-#ifdef GEANY_DEBUG
-/* Generate the config files in "data/" from defaults */
-void configuration_generate_data_files(void);
-#endif
-
 #endif
