@@ -1,8 +1,8 @@
 /*
  *      main.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2005-2009 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2006-2009 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2005-2010 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2006-2010 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: main.c 4074 2009-08-12 18:53:35Z frlan $
+ * $Id: main.c 4957 2010-05-29 18:35:51Z eht16 $
  */
 
 /**
@@ -59,7 +59,7 @@
 #include "dialogs.h"
 #include "templates.h"
 #include "encodings.h"
-#include "treeviews.h"
+#include "sidebar.h"
 #include "notebook.h"
 #include "keybindings.h"
 #include "editor.h"
@@ -98,7 +98,6 @@ CommandLineOptions cl_options;	/* fields initialised in parse_command_line_optio
 static gboolean want_plugins;
 
 /* command-line options */
-static gboolean debug_mode = FALSE;
 static gboolean verbose_mode = FALSE;
 static gboolean ignore_global_tags = FALSE;
 static gboolean no_msgwin = FALSE;
@@ -108,7 +107,6 @@ static gchar *alternate_config = NULL;
 static gboolean no_vte = FALSE;
 static gchar *lib_vte = NULL;
 #endif
-static gboolean generate_datafiles = FALSE;
 static gboolean generate_tags = FALSE;
 static gboolean no_preprocessing = FALSE;
 static gboolean ft_names = FALSE;
@@ -123,13 +121,13 @@ static GOptionEntry entries[] =
 {
 	{ "column", 0, 0, G_OPTION_ARG_INT, &cl_options.goto_column, N_("Set initial column number for the first opened file (useful in conjunction with --line)"), NULL },
 	{ "config", 'c', 0, G_OPTION_ARG_FILENAME, &alternate_config, N_("Use an alternate configuration directory"), NULL },
-	{ "debug", 'd', G_OPTION_FLAG_HIDDEN, G_OPTION_ARG_NONE, &debug_mode, N_("Be verbose"), NULL },
 	{ "ft-names", 0, 0, G_OPTION_ARG_NONE, &ft_names, N_("Print internal filetype names"), NULL },
 	{ "generate-tags", 'g', 0, G_OPTION_ARG_NONE, &generate_tags, N_("Generate global tags file (see documentation)"), NULL },
 	{ "no-preprocessing", 'P', 0, G_OPTION_ARG_NONE, &no_preprocessing, N_("Don't preprocess C/C++ files when generating tags"), NULL },
-	{ "generate-data-files", 0, G_OPTION_FLAG_HIDDEN, G_OPTION_ARG_NONE, &generate_datafiles, NULL, NULL },
 #ifdef HAVE_SOCKET
 	{ "new-instance", 'i', 0, G_OPTION_ARG_NONE, &cl_options.new_instance, N_("Don't open files in a running instance, force opening a new instance"), NULL },
+	{ "socket-file", 0, 0, G_OPTION_ARG_FILENAME, &cl_options.socket_filename, N_("Use this socket filename for communication with a running Geany instance"), NULL },
+	{ "list-documents", 0, 0, G_OPTION_ARG_NONE, &cl_options.list_documents, N_("Return a list of open documents in a running Geany instance"), NULL },
 #endif
 	{ "line", 'l', 0, G_OPTION_ARG_INT, &cl_options.goto_line, N_("Set initial line number for the first opened file"), NULL },
 	{ "no-msgwin", 'm', 0, G_OPTION_ARG_NONE, &no_msgwin, N_("Don't show message window at startup"), NULL },
@@ -162,6 +160,7 @@ static void setup_window_position(void)
 	}
 }
 
+
 /* special things for the initial setup of the checkboxes and related stuff
  * an action on a setting is only performed if the setting is not equal to the program default
  * (all the following code is not perfect but it works for the moment) */
@@ -207,14 +206,13 @@ static void apply_settings(void)
 
 #ifdef HAVE_VTE
 	if (! vte_info.load_vte)
-	{
 		gtk_widget_hide(ui_lookup_widget(main_widgets.window, "send_selection_to_vte1"));
-		gtk_widget_hide(ui_lookup_widget(main_widgets.editor_menu, "send_selection_to_vte2"));
-	}
 #else
 	gtk_widget_hide(ui_lookup_widget(main_widgets.window, "send_selection_to_vte1"));
-	gtk_widget_hide(ui_lookup_widget(main_widgets.editor_menu, "send_selection_to_vte2"));
 #endif
+
+	if (interface_prefs.sidebar_pos != GTK_POS_LEFT)
+		ui_swap_sidebar_pos();
 }
 
 
@@ -228,8 +226,9 @@ static void main_init(void)
 	ui_widgets.open_filesel		= NULL;
 	ui_widgets.save_filesel		= NULL;
 	ui_widgets.prefs_dialog		= NULL;
-	main_status.main_window_realized= FALSE;
+	main_status.main_window_realized = FALSE;
 	file_prefs.tab_order_ltr		= FALSE;
+	file_prefs.tab_order_beside		= FALSE;
 	main_status.quitting			= FALSE;
 	ignore_callback	= FALSE;
 	app->tm_workspace		= tm_get_workspace();
@@ -268,6 +267,12 @@ static void main_init(void)
 	/* hide Page setup menu item, it isn't supported with non-GTK printing */
 	gtk_widget_hide(ui_widgets.print_page_setup);
 #endif
+
+	/* for some reason we need to set the initial size request,
+	 * otherwise the main window gets crazy dimensions */
+	gtk_widget_set_size_request(main_widgets.window, 10, 10);
+	gtk_window_set_default_size(GTK_WINDOW(main_widgets.window),
+		GEANY_WINDOW_DEFAULT_WIDTH, GEANY_WINDOW_DEFAULT_HEIGHT);
 }
 
 
@@ -318,6 +323,10 @@ static void get_line_and_column_from_filename(gchar *filename, gint *line, gint
 	if (! NZV(filename))
 		return;
 
+	/* allow to open files like "test:0" */
+	if (g_file_test(filename, G_FILE_TEST_EXISTS))
+		return;
+
 	len = strlen(filename);
 	for (i = len - 1; i >= 1; i--)
 	{
@@ -384,6 +393,26 @@ static void setup_paths(void)
 }
 
 
+/**
+ *  Checks whether the main window has been realized.
+ *  This is an easy indicator whether Geany is right now starting up (main window is not
+ *  yet realized) or whether it has finished the startup process (main window is realized).
+ *  This is because the main window is realized (i.e. actually drawn on the screen) at the
+ *  end of the startup process.
+ *
+ *  @note Maybe you want to use the @ref geany-startup-complete signal to get notified about
+ *        the completed startup process.
+ *
+ *  @return @c TRUE if the Geany main window has been realized or @c FALSE otherwise.
+ *
+ *  @since 0.19
+ **/
+gboolean main_is_realized(void)
+{
+	return main_status.main_window_realized;
+}
+
+
 /**
  *  Initialises the gettext translation system.
  *  This is a convenience function to set up gettext for internationalisation support
@@ -426,7 +455,6 @@ void main_locale_init(const gchar *locale_dir, const gchar *package)
 
 	bindtextdomain(package, l_locale_dir);
 	bind_textdomain_codeset(package, "UTF-8");
-	textdomain(package);
 	g_free(l_locale_dir);
 }
 
@@ -451,7 +479,7 @@ static void parse_command_line_options(gint *argc, gchar ***argv)
 	GError *error = NULL;
 	GOptionContext *context;
 	gint i;
-	CommandLineOptions def_clo = {FALSE, TRUE, -1, -1, FALSE};
+	CommandLineOptions def_clo = {FALSE, NULL, TRUE, -1, -1, FALSE, FALSE};
 
 	/* first initialise cl_options fields with default values */
 	cl_options = def_clo;
@@ -502,12 +530,6 @@ static void parse_command_line_options(gint *argc, gchar ***argv)
 	}
 
 	app->debug_mode = verbose_mode;
-	if (debug_mode)
-	{
-		app->debug_mode = TRUE;
-		g_warning(
-			"Command line option --debug is obsolete and will be removed in the next version.");
-	}
 
 #ifdef G_OS_WIN32
 	win32_init_debug_code();
@@ -523,21 +545,16 @@ static void parse_command_line_options(gint *argc, gchar ***argv)
 		app->configdir = g_build_filename(g_get_user_config_dir(), "geany", NULL);
 	}
 
-#ifdef GEANY_DEBUG
-	if (generate_datafiles)
-	{
-		filetypes_init_types();
-		configuration_generate_data_files();	/* currently only filetype_extensions.conf */
-		exit(0);
-	}
-#endif
 	if (generate_tags)
 	{
 		gboolean ret;
 
+		document_init_doclist();
 		filetypes_init_types();
 		filetypes_read_extensions();	/* needed for *.lang.tags filetype matching */
 		ret = symbols_generate_global_tags(*argc, *argv, ! no_preprocessing);
+		filetypes_free_types();
+		document_finalize();
 		exit(ret);
 	}
 
@@ -549,7 +566,12 @@ static void parse_command_line_options(gint *argc, gchar ***argv)
 
 #ifdef HAVE_SOCKET
 	socket_info.ignore_socket = cl_options.new_instance;
+	if (cl_options.socket_filename)
+	{
+		socket_info.file_name = cl_options.socket_filename;
+	}
 #endif
+
 #ifdef HAVE_VTE
 	vte_info.lib_vte = lib_vte;
 #endif
@@ -821,6 +843,26 @@ static void load_settings(void)
 }
 
 
+void main_load_project_from_command_line(const gchar *locale_filename, gboolean use_session)
+{
+	gchar *pfile = NULL;
+
+	if (utils_is_uri(locale_filename))
+		pfile = utils_get_path_from_uri(locale_filename);
+	else
+		pfile = g_strdup(locale_filename);
+
+	if (pfile != NULL)
+	{
+		if (use_session)
+			project_load_file_with_session(pfile);
+		else
+			project_load_file(pfile);
+	}
+	g_free(pfile);
+}
+
+
 static void load_startup_files(gint argc, gchar **argv)
 {
 	gboolean load_project_from_cl = FALSE;
@@ -836,15 +878,7 @@ static void load_startup_files(gint argc, gchar **argv)
 		{
 			if (load_project_from_cl)
 			{
-				gchar *pfile = argv[1];
-				if (utils_is_uri(argv[1]))
-					pfile = utils_get_path_from_uri(argv[1]);
-				if (pfile != NULL)
-				{
-					project_load_file(pfile);
-					if (pfile != argv[1])
-						g_free(pfile);
-				}
+				main_load_project_from_command_line(argv[1], FALSE);
 			}
 			else if (cl_options.load_session && !cl_options.new_instance)
 				load_session_project_file();
@@ -867,10 +901,28 @@ static void load_startup_files(gint argc, gchar **argv)
 }
 
 
+static gboolean send_startup_complete(gpointer data)
+{
+	g_signal_emit_by_name(geany_object, "geany-startup-complete");
+	return FALSE;
+}
+
+
+static const gchar *get_locale(void)
+{
+	const gchar *locale = "unknown";
+#ifdef HAVE_LOCALE_H
+	locale = setlocale(LC_CTYPE, NULL);
+#endif
+	return locale;
+}
+
+
 gint main(gint argc, gchar **argv)
 {
 	GeanyDocument *doc;
 	gint config_dir_result;
+	const gchar *locale;
 
 	log_handlers_init();
 
@@ -908,10 +960,12 @@ gint main(gint argc, gchar **argv)
 		socket_info.lock_socket = -1;
 		socket_info.lock_socket_tag = 0;
 		socket_info.lock_socket = socket_init(argc, argv);
+		/* Socket exists */
 		if (socket_info.lock_socket == -2)
 		{
-			/* Socket exists */
-			if (argc > 1)	/* filenames were sent to first instance, so quit */
+			/* Quit if filenames were sent to first instance or the list of open
+			 * documents has been sent */
+			if (argc > 1 || cl_options.list_documents)
 			{
 				gdk_notify_startup_complete();
 				g_free(app->configdir);
@@ -927,18 +981,18 @@ gint main(gint argc, gchar **argv)
 	}
 #endif
 
-	geany_debug("Geany %s, GTK+ %u.%u.%u, GLib %u.%u.%u",
+	locale = get_locale();
+	geany_debug("Geany %s, GTK+ %u.%u.%u, GLib %u.%u.%u (%s)",
 		main_get_version_string(),
 		gtk_major_version, gtk_minor_version, gtk_micro_version,
-		glib_major_version, glib_minor_version, glib_micro_version);
+		glib_major_version, glib_minor_version, glib_micro_version,
+		locale);
 
 	/* create the object so Geany signals can be connected in init() functions */
 	geany_object = geany_object_new();
 
 	/* inits */
 	main_init();
-	gtk_widget_set_size_request(main_widgets.window, GEANY_WINDOW_MINIMAL_WIDTH, GEANY_WINDOW_MINIMAL_HEIGHT);
-	gtk_window_set_default_size(GTK_WINDOW(main_widgets.window), GEANY_WINDOW_DEFAULT_WIDTH, GEANY_WINDOW_DEFAULT_HEIGHT);
 
 	encodings_init();
 	editor_init();
@@ -950,9 +1004,10 @@ gint main(gint argc, gchar **argv)
 #ifdef HAVE_PLUGINS
 	plugins_init();
 #endif
-	treeviews_init();
+	sidebar_init();
 	load_settings();	/* load keyfile */
 
+	highlighting_init();
 	msgwin_init();
 	build_init();
 	ui_create_insert_menu_items();
@@ -979,8 +1034,7 @@ gint main(gint argc, gchar **argv)
 	g_signal_connect(main_widgets.window, "delete-event", G_CALLBACK(on_exit_clicked), NULL);
 	g_signal_connect(main_widgets.window, "window-state-event", G_CALLBACK(on_window_state_event), NULL);
 
-	g_signal_connect(ui_lookup_widget(main_widgets.window, "textview_scribble"),
-							"motion-notify-event", G_CALLBACK(on_motion_event), NULL);
+	g_signal_connect(msgwindow.scribble, "motion-notify-event", G_CALLBACK(on_motion_event), NULL);
 
 #ifdef HAVE_VTE
 	vte_init();
@@ -1020,9 +1074,9 @@ gint main(gint argc, gchar **argv)
 
 	doc = document_get_current();
 	gtk_widget_grab_focus(GTK_WIDGET(doc->editor->sci));
-	treeviews_select_openfiles_item(doc);
+	sidebar_select_openfiles_item(doc);
 	build_menu_update(doc);
-	treeviews_update_tag_list(doc, FALSE);
+	sidebar_update_tag_list(doc, FALSE);
 
 #ifdef G_OS_WIN32
 	/* Manually realise the main window to be able to set the position but don't show it.
@@ -1043,7 +1097,7 @@ gint main(gint argc, gchar **argv)
 	{
 		socket_info.read_ioc = g_io_channel_unix_new(socket_info.lock_socket);
 		socket_info.lock_socket_tag = g_io_add_watch(socket_info.read_ioc,
-						G_IO_IN|G_IO_PRI|G_IO_ERR, socket_lock_input_cb, main_widgets.window);
+						G_IO_IN | G_IO_PRI | G_IO_ERR, socket_lock_input_cb, main_widgets.window);
 	}
 #endif
 
@@ -1058,6 +1112,10 @@ gint main(gint argc, gchar **argv)
 	}
 #endif
 
+	/* when we are really done with setting everything up and the main event loop is running,
+	 * tell other components, mainly plugins, that startup is complete */
+	g_idle_add_full(G_PRIORITY_LOW, send_startup_complete, NULL, NULL);
+
 	gtk_main();
 	return 0;
 }
@@ -1088,7 +1146,6 @@ void main_quit()
 	navqueue_free();
 	keybindings_free();
 	filetypes_save_commands();
-	filetypes_free_types();
 	highlighting_free_styles();
 	templates_free_templates();
 	msgwin_finalize();
@@ -1101,8 +1158,9 @@ void main_quit()
 	editor_snippets_free();
 	encodings_finalize();
 	toolbar_finalize();
-	treeviews_finalize();
+	sidebar_finalize();
 	configuration_finalize();
+	filetypes_free_types();
 	log_finalize();
 
 	tm_workspace_free(TM_WORK_OBJECT(app->tm_workspace));
@@ -1117,13 +1175,13 @@ void main_quit()
 	g_free(interface_prefs.msgwin_font);
 	g_free(editor_prefs.long_line_color);
 	g_free(editor_prefs.comment_toggle_mark);
+	g_free(editor_prefs.color_scheme);
 	g_free(tool_prefs.context_action_cmd);
 	g_free(template_prefs.developer);
 	g_free(template_prefs.company);
 	g_free(template_prefs.mail);
 	g_free(template_prefs.initials);
 	g_free(template_prefs.version);
-	g_free(tool_prefs.make_cmd);
 	g_free(tool_prefs.term_cmd);
 	g_free(tool_prefs.browser_cmd);
 	g_free(tool_prefs.grep_cmd);
@@ -1167,19 +1225,6 @@ void main_quit()
 }
 
 
-/* malloc compatibility code */
-#undef malloc
-void *malloc(size_t n);
-
-/* Allocate an N-byte block of memory from the heap. If N is zero, allocate a 1-byte block. */
-void *rpl_malloc(size_t n)
-{
-	if (n == 0)
-		n = 1;
-	return malloc(n);
-}
-
-
 /**
  *  Reloads most of Geany's configuration files without restarting. Currently the following
  *  files are reloaded: all template files, also new file templates and the 'New (with template)'
@@ -1193,8 +1238,6 @@ void *rpl_malloc(size_t n)
  **/
 void main_reload_configuration(void)
 {
-	guint i;
-
 	/* reload templates */
 	templates_free_templates();
 	templates_init();
@@ -1206,17 +1249,7 @@ void main_reload_configuration(void)
 	/* reload filetype extensions */
 	filetypes_read_extensions();
 
-	/* save possibly changed commands before re-reading them */
-	filetypes_save_commands();
-
-	/* reload filetype configs */
-	for (i = 0; i < filetypes_array->len; i++)
-	{
-		/* filetypes_load_config() will skip not loaded filetypes */
-		filetypes_load_config(i, TRUE);
-	}
-	documents_foreach(i)
-		document_reload_config(documents[i]);
+	filetypes_reload();
 
 	/* C tag names to ignore */
 	symbols_reload_config_files();
