@@ -1,8 +1,8 @@
 /*
  *      notebook.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2006-2009 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2006-2009 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2006-2010 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2006-2010 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: notebook.c 3808 2009-05-20 15:01:04Z ntrel $
+ * $Id: notebook.c 5131 2010-08-11 12:45:22Z ntrel $
  */
 
 /*
@@ -31,10 +31,11 @@
 #include "editor.h"
 #include "documentprivate.h"
 #include "ui_utils.h"
-#include "treeviews.h"
+#include "sidebar.h"
 #include "support.h"
 #include "callbacks.h"
 #include "utils.h"
+#include "keybindings.h"
 
 #define GEANY_DND_NOTEBOOK_TAB_TYPE	"geany_dnd_notebook_tab"
 
@@ -192,62 +193,35 @@ static void tab_bar_menu_activate_cb(GtkMenuItem *menuitem, gpointer data)
 }
 
 
-static GtkMenu *get_tab_bar_popup_menu(void)
+static void show_tab_bar_popup_menu(GdkEventButton *event)
 {
+	GtkWidget *menu_item;
 	static GtkWidget *menu = NULL;
-	GtkWidget *menu_item, *menu_item_label;
-	const GdkColor *color;
-	GeanyDocument *doc;
-	guint i, len;
-	gchar *base_name;
-	GeanyDocument *current_doc = document_get_current();
 
-	if (G_UNLIKELY(menu == NULL))
+	if (menu == NULL)
 		menu = gtk_menu_new();
 
 	/* clear the old menu items */
 	gtk_container_foreach(GTK_CONTAINER(menu), (GtkCallback) gtk_widget_destroy, NULL);
 
-	len = gtk_notebook_get_n_pages(GTK_NOTEBOOK(main_widgets.notebook));
-	for (i = 0; i < len; i++)
-	{
-		doc = document_get_from_page(i);
-		if (! DOC_VALID(doc))
-			continue;
+	ui_menu_add_document_items(GTK_MENU(menu), document_get_current(),
+		G_CALLBACK(tab_bar_menu_activate_cb));
 
-		base_name = g_path_get_basename(DOC_FILENAME(doc));
-		menu_item = gtk_menu_item_new_with_label(base_name);
-		gtk_widget_show(menu_item);
-		gtk_container_add(GTK_CONTAINER(menu), menu_item);
-		g_signal_connect(menu_item, "activate", G_CALLBACK(tab_bar_menu_activate_cb), doc);
-
-		color = document_get_status_color(doc);
-		menu_item_label = gtk_bin_get_child(GTK_BIN(menu_item));
-		gtk_widget_modify_fg(menu_item_label, GTK_STATE_NORMAL, color);
-		gtk_widget_modify_fg(menu_item_label, GTK_STATE_ACTIVE, color);
-
-		if (doc == current_doc)
-		{
-			setptr(base_name, g_strconcat("<b>", base_name, "</b>", NULL));
-			gtk_label_set_markup(GTK_LABEL(menu_item_label), base_name);
-		}
-		g_free(base_name);
-	}
 	menu_item = gtk_separator_menu_item_new();
 	gtk_widget_show(menu_item);
 	gtk_container_add(GTK_CONTAINER(menu), menu_item);
 
-	menu_item = gtk_menu_item_new_with_mnemonic(_("Close Ot_her Documents"));
+	menu_item = ui_image_menu_item_new(GTK_STOCK_CLOSE, _("Close Ot_her Documents"));
 	gtk_widget_show(menu_item);
 	gtk_container_add(GTK_CONTAINER(menu), menu_item);
 	g_signal_connect(menu_item, "activate", G_CALLBACK(on_close_other_documents1_activate), NULL);
 
-	menu_item = gtk_menu_item_new_with_mnemonic(_("C_lose All"));
+	menu_item = ui_image_menu_item_new(GTK_STOCK_CLOSE, _("C_lose All"));
 	gtk_widget_show(menu_item);
 	gtk_container_add(GTK_CONTAINER(menu), menu_item);
 	g_signal_connect(menu_item, "activate", G_CALLBACK(on_close_all1_activate), NULL);
 
-	return GTK_MENU(menu);
+	gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL, event->button, event->time);
 }
 
 
@@ -269,8 +243,7 @@ static gboolean notebook_tab_bar_click_cb(GtkWidget *widget, GdkEventButton *eve
 	}
 	else if (event->button == 3)
 	{
-		gtk_menu_popup(get_tab_bar_popup_menu(), NULL, NULL,
-			NULL, NULL, event->button, event->time);
+		show_tab_bar_popup_menu(event);
 	}
 	return FALSE;
 }
@@ -334,7 +307,7 @@ notebook_page_reordered_cb(GtkNotebook *notebook, GtkWidget *child, guint page_n
 {
 	/* Not necessary to update open files treeview if it's sorted.
 	 * Note: if enabled, it's best to move the item instead of recreating all items. */
-	/*treeviews_openfiles_update_all();*/
+	/*sidebar_openfiles_update_all();*/
 }
 
 
@@ -390,7 +363,7 @@ notebook_find_tab_num_at_pos(GtkNotebook *notebook, gint x, gint y)
 	GtkWidget *page;
 
 	/* deal with less than 2 pages */
-	switch(gtk_notebook_get_n_pages(notebook))
+	switch (gtk_notebook_get_n_pages(notebook))
 	{case 0: return -1; case 1: return 0;}
 
 	tab_pos = gtk_notebook_get_tab_pos(notebook); /* which edge */
@@ -412,9 +385,9 @@ notebook_find_tab_num_at_pos(GtkNotebook *notebook, gint x, gint y)
 		max_x = tab->allocation.x + tab->allocation.width - GTK_WIDGET(notebook)->allocation.x;
 		max_y = tab->allocation.y + tab->allocation.height - GTK_WIDGET(notebook)->allocation.y;
 
-		if (((tab_pos == GTK_POS_TOP) || (tab_pos == GTK_POS_BOTTOM)) && (x<=max_x))
+		if (((tab_pos == GTK_POS_TOP) || (tab_pos == GTK_POS_BOTTOM)) && (x <= max_x))
 			return page_num;
-		else if (((tab_pos == GTK_POS_LEFT) || (tab_pos == GTK_POS_RIGHT)) && (y<=max_y))
+		else if (((tab_pos == GTK_POS_LEFT) || (tab_pos == GTK_POS_RIGHT)) && (y <= max_y))
 			return page_num;
 
 		page_num++;
@@ -448,6 +421,8 @@ static void tab_count_changed(void)
 
 static gboolean notebook_tab_click(GtkWidget *widget, GdkEventButton *event, gpointer data)
 {
+	guint state;
+
 	/* toggle additional widgets on double click */
 	if (event->type == GDK_2BUTTON_PRESS)
 	{
@@ -463,7 +438,14 @@ static gboolean notebook_tab_click(GtkWidget *widget, GdkEventButton *event, gpo
 			GTK_WIDGET(data)));
 		return TRUE; /* stop other handlers like notebook_tab_bar_click_cb() */
 	}
-
+	/* switch last used tab on ctrl-click */
+	state = event->state & gtk_accelerator_get_default_mod_mask();
+	if (event->button == 1 && state == GDK_CONTROL_MASK)
+	{
+		keybindings_send_command(GEANY_KEY_GROUP_NOTEBOOK,
+			GEANY_KEYS_NOTEBOOK_SWITCHTABLASTUSED);
+		return TRUE;
+	}
 	return FALSE;
 }
 
@@ -484,6 +466,7 @@ gint notebook_new_tab(GeanyDocument *this)
 	GtkWidget *hbox, *ebox;
 	gint tabnum;
 	GtkWidget *page;
+	gint cur_page;
 
 	g_return_val_if_fail(this != NULL, -1);
 
@@ -516,7 +499,7 @@ gint notebook_new_tab(GeanyDocument *this)
 		image = gtk_image_new_from_stock(GTK_STOCK_CLOSE, GTK_ICON_SIZE_MENU);
 		gtk_container_add(GTK_CONTAINER(btn), image);
 
-		align = gtk_alignment_new(1.0, 0.0, 0.0, 0.0);
+		align = gtk_alignment_new(1.0, 0.5, 0.0, 0.0);
 		gtk_container_add(GTK_CONTAINER(align), btn);
 		gtk_box_pack_start(GTK_BOX(hbox), align, TRUE, TRUE, 0);
 
@@ -531,12 +514,16 @@ gint notebook_new_tab(GeanyDocument *this)
 
 	document_update_tab_label(this);
 
+	if (file_prefs.tab_order_beside)
+		cur_page = gtk_notebook_get_current_page(GTK_NOTEBOOK(main_widgets.notebook));
+	else
+		cur_page = file_prefs.tab_order_ltr ? -2 /* hack: -2 + 1 = -1, last page */ : 0;
 	if (file_prefs.tab_order_ltr)
-		tabnum = gtk_notebook_append_page_menu(GTK_NOTEBOOK(main_widgets.notebook), page,
-			ebox, NULL);
+		tabnum = gtk_notebook_insert_page_menu(GTK_NOTEBOOK(main_widgets.notebook), page,
+			ebox, NULL, cur_page + 1);
 	else
 		tabnum = gtk_notebook_insert_page_menu(GTK_NOTEBOOK(main_widgets.notebook), page,
-			ebox, NULL, 0);
+			ebox, NULL, cur_page);
 
 	tab_count_changed();
 
