@@ -1,8 +1,8 @@
 /*
  *      plugins.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2007-2009 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2007-2009 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2007-2010 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2007-2010 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -19,7 +19,7 @@
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
  *      MA 02110-1301, USA.
  *
- * $Id: plugins.c 4047 2009-07-30 11:56:25Z ntrel $
+ * $Id: plugins.c 5132 2010-08-11 12:50:38Z ntrel $
  */
 
 /* Code to manage, load and unload plugins. */
@@ -57,29 +57,14 @@
 #include "main.h"
 #include "toolbar.h"
 #include "stash.h"
+#include "symbols.h"
 #include "keyfile.h"
 #include "win32.h"
 #include "pluginutils.h"
 #include "pluginprivate.h"
 
 
-typedef struct Plugin
-{
-	GModule 	*module;
-	gchar		*filename;				/* plugin filename (/path/libname.so) */
-	PluginInfo		info;				/* plugin name, description, etc */
-	PluginFields	fields;
-	GeanyPlugin		public;				/* fields the plugin can read */
-	GeanyPluginPrivate	priv;			/* GeanyPlugin type private data, same as (*public.priv) */
-
-	GeanyKeyGroup	*key_group;
-
-	void		(*init) (GeanyData *data);			/* Called when the plugin is enabled */
-	GtkWidget*	(*configure) (GtkDialog *dialog);	/* plugin configure dialog, optional */
-	void		(*help) (void);					/* Called when the plugin should show some help, optional */
-	void		(*cleanup) (void);					/* Called when the plugin is disabled or when Geany exits */
-}
-Plugin;
+GList *active_plugin_list = NULL; /* list of only actually loaded plugins, always valid */
 
 
 static gboolean want_plugins = FALSE;
@@ -87,7 +72,6 @@ static gboolean want_plugins = FALSE;
 /* list of all available, loadable plugins, only valid as long as the plugin manager dialog is
  * opened, afterwards it will be destroyed */
 static GList *plugin_list = NULL;
-static GList *active_plugin_list = NULL; /* list of only actually loaded plugins, always valid */
 static gchar **active_plugins_pref = NULL; 	/* list of plugin filenames to load at startup */
 static GList *failed_plugins_list = NULL;	/* plugins the user wants active but can't be used */
 
@@ -99,7 +83,9 @@ static void pm_show_dialog(GtkMenuItem *menuitem, gpointer user_data);
 static PluginFuncs plugin_funcs = {
 	&plugin_add_toolbar_item,
 	&plugin_module_make_resident,
-	&plugin_signal_connect
+	&plugin_signal_connect,
+	&plugin_set_key_group,
+	&plugin_show_configure
 };
 
 static DocumentFuncs doc_funcs = {
@@ -121,7 +107,8 @@ static DocumentFuncs doc_funcs = {
 	&document_save_file_as,
 	&document_rename_file,
 	&document_get_status_color,
-	&document_get_basename_for_display
+	&document_get_basename_for_display,
+	&document_get_notebook_page
 };
 
 static EditorFuncs editor_funcs = {
@@ -131,7 +118,11 @@ static EditorFuncs editor_funcs = {
 	&editor_indicator_set_on_line,
 	&editor_indicator_clear,
 	&editor_set_indent_type,
-	&editor_get_word_at_pos
+	&editor_get_word_at_pos,
+	&editor_get_eol_char_name,
+	&editor_get_eol_char_len,
+	&editor_get_eol_char,
+	&editor_insert_text_block
 };
 
 static ScintillaFuncs scintilla_funcs = {
@@ -183,7 +174,18 @@ static SciFuncs sci_funcs = {
 	&sci_get_contents,
 	&sci_get_contents_range,
 	&sci_get_selection_contents,
-	&sci_set_font
+	&sci_set_font,
+	&sci_get_line_end_position,
+	&sci_set_target_start,
+	&sci_set_target_end,
+	&sci_replace_target,
+	&sci_set_marker_at_line,
+	&sci_delete_marker_at_line,
+	&sci_is_marker_set_at_line,
+	&sci_goto_line,
+	&sci_find_text,
+	&sci_set_line_indentation,
+	&sci_get_line_indentation
 };
 
 static TemplateFuncs template_funcs = {
@@ -209,7 +211,9 @@ static UtilsFuncs utils_funcs = {
 	&utils_open_browser,
 	&utils_string_replace_first,
 	&utils_str_middle_truncate,
-	&utils_str_remove_chars
+	&utils_str_remove_chars,
+	&utils_get_file_list_full,
+	&utils_copy_environment
 };
 
 static UIUtilsFuncs uiutils_funcs = {
@@ -225,7 +229,11 @@ static UIUtilsFuncs uiutils_funcs = {
 	&ui_lookup_widget,
 	&ui_progress_bar_start,
 	&ui_progress_bar_stop,
-	&ui_entry_add_clear_icon
+	&ui_entry_add_clear_icon,
+	&ui_menu_add_document_items,
+	&ui_widget_modify_font_from_string,
+	&ui_is_keyval_enter_or_return,
+	&ui_get_gtk_settings_integer
 };
 
 static DialogFuncs dialog_funcs = {
@@ -259,7 +267,8 @@ static EncodingFuncs encoding_funcs = {
 
 static KeybindingFuncs keybindings_funcs = {
 	&keybindings_send_command,
-	&keybindings_set_item
+	&keybindings_set_item,
+	&keybindings_get_item
 };
 
 static TagManagerFuncs tagmanager_funcs = {
@@ -276,7 +285,8 @@ static SearchFuncs search_funcs = {
 };
 
 static HighlightingFuncs highlighting_funcs = {
-	&highlighting_get_style
+	&highlighting_get_style,
+	&highlighting_set_styles
 };
 
 static FiletypeFuncs filetype_funcs = {
@@ -291,7 +301,34 @@ static NavQueueFuncs navqueue_funcs = {
 
 static MainFuncs main_funcs = {
 	&main_reload_configuration,
-	&main_locale_init
+	&main_locale_init,
+	&main_is_realized
+};
+
+static StashFuncs stash_funcs = {
+	&stash_group_new,
+	&stash_group_add_boolean,
+	&stash_group_add_integer,
+	&stash_group_add_string,
+	&stash_group_add_string_vector,
+	&stash_group_load_from_key_file,
+	&stash_group_save_to_key_file,
+	&stash_group_free,
+	&stash_group_load_from_file,
+	&stash_group_save_to_file,
+	&stash_group_add_toggle_button,
+	&stash_group_add_radio_buttons,
+	&stash_group_add_spin_button_integer,
+	&stash_group_add_combo_box,
+	&stash_group_add_combo_box_entry,
+	&stash_group_add_entry,
+	&stash_group_add_widget_property,
+	&stash_group_display,
+	&stash_group_update
+};
+
+static SymbolsFuncs symbols_funcs = {
+	&symbols_get_context_separator
 };
 
 static GeanyFunctions geany_functions = {
@@ -314,7 +351,9 @@ static GeanyFunctions geany_functions = {
 	&main_funcs,
 	&plugin_funcs,
 	&scintilla_funcs,
-	&msgwin_funcs
+	&msgwin_funcs,
+	&stash_funcs,
+	&symbols_funcs
 };
 
 static GeanyData geany_data;
@@ -428,7 +467,7 @@ plugin_check_version(GModule *module)
 				"release of Geany - recompile it.", g_module_name(module));
 			return FALSE;
 		}
-		if (result > 0)
+		if (result > GEANY_API_VERSION)
 		{
 			geany_debug("Plugin \"%s\" requires a newer version of Geany (API >= v%d).",
 				g_module_name(module), result);
@@ -464,42 +503,40 @@ static void add_callbacks(Plugin *plugin, PluginCallback *callbacks)
 }
 
 
-static void
-add_kb_group(Plugin *plugin)
+static void read_key_group(Plugin *plugin)
 {
-	guint i;
+	GeanyKeyGroupInfo *p_key_info;
+	GeanyKeyGroup **p_key_group;
 
-	if (!NZV(plugin->key_group->name))
+	g_module_symbol(plugin->module, "plugin_key_group_info", (void *) &p_key_info);
+	g_module_symbol(plugin->module, "plugin_key_group", (void *) &p_key_group);
+	if (p_key_info && p_key_group)
 	{
-		geany_debug("Plugin \"%s\" has not set a name for its keybinding group"
-			" - ignoring all keybindings!",
-			plugin->info.name);
-		return;
-	}
-	g_return_if_fail(! g_str_equal(plugin->key_group->name, keybindings_keyfile_group_name));
+		GeanyKeyGroupInfo *key_info = p_key_info;
 
-	for (i = 0; i < plugin->key_group->count; i++)
-	{
-		GeanyKeyBinding *kb = &plugin->key_group->keys[i];
-
-		if (!NZV(kb->name))
+		if (*p_key_group)
+			geany_debug("Ignoring plugin_key_group symbol for plugin '%s' - "
+				"use plugin_set_key_group() instead to allocate keybindings dynamically.",
+				plugin->info.name);
+		else
 		{
-			geany_debug("Plugin \"%s\" has not set a name for keybinding %d"
-				" - ignoring all keybindings!",
-				plugin->info.name, i);
-			plugin->key_group->count = 0;
-			break;
+			if (key_info->count)
+			{
+				GeanyKeyGroup *key_group =
+					plugin_set_key_group(&plugin->public, key_info->name, key_info->count, NULL);
+				if (key_group)
+					*p_key_group = key_group;
+			}
+			else
+				geany_debug("Ignoring plugin_key_group_info symbol for plugin '%s' - "
+					"count field is zero. Maybe use plugin_set_key_group() instead?",
+					plugin->info.name);
 		}
 	}
-	if (plugin->key_group->count == 0)
-	{
-		plugin->key_group = NULL;	/* Ignore the group (maybe the plugin has optional KB) */
-		return;
-	}
-
-	plugin->key_group->label = plugin->info.name;
-
-	g_ptr_array_add(keybinding_groups, plugin->key_group);
+	else if (p_key_info || p_key_group)
+		geany_debug("Ignoring only one of plugin_key_group[_info] symbols defined for plugin '%s'. "
+			"Maybe use plugin_set_key_group() instead?",
+			plugin->info.name);
 }
 
 
@@ -510,35 +547,32 @@ plugin_init(Plugin *plugin)
 	PluginCallback *callbacks;
 	PluginInfo **p_info;
 	PluginFields **plugin_fields;
-	GeanyData **p_geany_data;
-	GeanyFunctions **p_geany_functions;
 
-	/* set these symbols before plugin_init() is called */
+	/* set these symbols before plugin_init() is called
+	 * we don't set geany_functions and geany_data since they are set directly by plugin_new() */
 	g_module_symbol(plugin->module, "geany_plugin", (void *) &p_geany_plugin);
 	if (p_geany_plugin)
 		*p_geany_plugin = &plugin->public;
 	g_module_symbol(plugin->module, "plugin_info", (void *) &p_info);
 	if (p_info)
 		*p_info = &plugin->info;
-	g_module_symbol(plugin->module, "geany_data", (void *) &p_geany_data);
-	if (p_geany_data)
-		*p_geany_data = &geany_data;
-	g_module_symbol(plugin->module, "geany_functions", (void *) &p_geany_functions);
-	if (p_geany_functions)
-		*p_geany_functions = &geany_functions;
 	g_module_symbol(plugin->module, "plugin_fields", (void *) &plugin_fields);
 	if (plugin_fields)
 		*plugin_fields = &plugin->fields;
+	read_key_group(plugin);
 
 	/* start the plugin */
 	g_return_if_fail(plugin->init);
 	plugin->init(&geany_data);
 
-	if (p_geany_plugin && (*p_geany_plugin)->priv->resident)
-		g_module_make_resident(plugin->module);
-
 	/* store some function pointers for later use */
 	g_module_symbol(plugin->module, "plugin_configure", (void *) &plugin->configure);
+	g_module_symbol(plugin->module, "plugin_configure_single", (void *) &plugin->configure_single);
+	if (app->debug_mode && plugin->configure && plugin->configure_single)
+		g_warning("Plugin '%s' implements plugin_configure_single() unnecessarily - "
+			"only plugin_configure() will be used!",
+			plugin->info.name);
+
 	g_module_symbol(plugin->module, "plugin_help", (void *) &plugin->help);
 	g_module_symbol(plugin->module, "plugin_cleanup", (void *) &plugin->cleanup);
 	if (plugin->cleanup == NULL)
@@ -559,10 +593,6 @@ plugin_init(Plugin *plugin)
 	if (callbacks)
 		add_callbacks(plugin, callbacks);
 
-	g_module_symbol(plugin->module, "plugin_key_group", (void *) &plugin->key_group);
-	if (plugin->key_group)
-		add_kb_group(plugin);
-
 	/* remember which plugins are active */
 	active_plugin_list = g_list_append(active_plugin_list, plugin);
 
@@ -581,6 +611,8 @@ plugin_new(const gchar *fname, gboolean init_plugin, gboolean add_to_list)
 {
 	Plugin *plugin;
 	GModule *module;
+	GeanyData **p_geany_data;
+	GeanyFunctions **p_geany_functions;
 	void (*plugin_set_info)(PluginInfo*);
 
 	g_return_val_if_fail(fname, NULL);
@@ -638,6 +670,14 @@ plugin_new(const gchar *fname, gboolean init_plugin, gboolean add_to_list)
 
 	plugin = g_new0(Plugin, 1);
 
+	/* set basic fields here to allow plugins to call Geany functions in set_info() */
+	g_module_symbol(module, "geany_data", (void *) &p_geany_data);
+	if (p_geany_data)
+		*p_geany_data = &geany_data;
+	g_module_symbol(module, "geany_functions", (void *) &p_geany_functions);
+	if (p_geany_functions)
+		*p_geany_functions = &geany_functions;
+
 	/* read plugin name, etc. */
 	plugin_set_info(&plugin->info);
 	if (!NZV(plugin->info.name))
@@ -667,7 +707,7 @@ plugin_new(const gchar *fname, gboolean init_plugin, gboolean add_to_list)
 	plugin->filename = g_strdup(fname);
 	plugin->module = module;
 	plugin->public.info = &plugin->info;
-	plugin->public.priv = &plugin->priv;
+	plugin->public.priv = plugin;
 
 	if (init_plugin)
 		plugin_init(plugin);
@@ -681,7 +721,7 @@ plugin_new(const gchar *fname, gboolean init_plugin, gboolean add_to_list)
 
 static void remove_callbacks(Plugin *plugin)
 {
-	GArray *signal_ids = plugin->priv.signal_ids;
+	GArray *signal_ids = plugin->signal_ids;
 	SignalConnection *sc;
 
 	if (signal_ids == NULL)
@@ -712,9 +752,9 @@ plugin_cleanup(Plugin *plugin)
 	remove_callbacks(plugin);
 
 	if (plugin->key_group)
-		g_ptr_array_remove_fast(keybinding_groups, plugin->key_group);
+		keybindings_free_group(plugin->key_group);
 
-	widget = plugin->priv.toolbar_separator.widget;
+	widget = plugin->toolbar_separator.widget;
 	if (widget)
 		gtk_widget_destroy(widget);
 
@@ -937,7 +977,7 @@ static void on_save_settings(GKeyFile *config)
 /* called even if plugin support is disabled */
 void plugins_init(void)
 {
-	GeanyPrefGroup *group;
+	StashGroup *group;
 
 	group = stash_group_new("plugins");
 	configuration_add_pref_group(group, TRUE);
@@ -969,6 +1009,25 @@ void plugins_finalize(void)
 }
 
 
+/* Check whether there are any plugins loaded which provide a configure symbol */
+gboolean plugins_have_preferences(void)
+{
+	GList *item;
+
+	if (active_plugin_list == NULL)
+		return FALSE;
+
+	foreach_list(item, active_plugin_list)
+	{
+		Plugin *plugin = item->data;
+		if (plugin->configure != NULL || plugin->configure_single != NULL)
+			return TRUE;
+	}
+
+	return FALSE;
+}
+
+
 /* Plugin Manager */
 
 enum
@@ -996,7 +1055,18 @@ PluginManagerWidgets;
 static PluginManagerWidgets pm_widgets;
 
 
-void pm_selection_changed(GtkTreeSelection *selection, gpointer user_data)
+static void pm_update_buttons(Plugin *p)
+{
+	gboolean is_active;
+
+	is_active = is_active_plugin(p);
+	gtk_widget_set_sensitive(pm_widgets.configure_button,
+		(p->configure || p->configure_single) && is_active);
+	gtk_widget_set_sensitive(pm_widgets.help_button, p->help != NULL && is_active);
+}
+
+
+static void pm_selection_changed(GtkTreeSelection *selection, gpointer user_data)
 {
 	GtkTreeIter iter;
 	GtkTreeModel *model;
@@ -1010,7 +1080,6 @@ void pm_selection_changed(GtkTreeSelection *selection, gpointer user_data)
 		{
 			gchar *text;
 			PluginInfo *pi;
-			gboolean is_active;
 
 			pi = &p->info;
 			text = g_strdup_printf(
@@ -1020,9 +1089,7 @@ void pm_selection_changed(GtkTreeSelection *selection, gpointer user_data)
 			geany_wrap_label_set_text(GTK_LABEL(pm_widgets.description_label), text);
 			g_free(text);
 
-			is_active = is_active_plugin(p);
-			gtk_widget_set_sensitive(pm_widgets.configure_button, p->configure != NULL && is_active);
-			gtk_widget_set_sensitive(pm_widgets.help_button, p->help != NULL && is_active);
+			pm_update_buttons(p);
 		}
 	}
 }
@@ -1031,7 +1098,6 @@ void pm_selection_changed(GtkTreeSelection *selection, gpointer user_data)
 static void pm_plugin_toggled(GtkCellRendererToggle *cell, gchar *pth, gpointer data)
 {
 	gboolean old_state, state;
-	gboolean is_active;
 	gchar *file_name;
 	GtkTreeIter iter;
 	GtkTreePath *path = gtk_tree_path_new_from_string(pth);
@@ -1042,7 +1108,11 @@ static void pm_plugin_toggled(GtkCellRendererToggle *cell, gchar *pth, gpointer
 
 	gtk_tree_model_get(GTK_TREE_MODEL(pm_widgets.store), &iter,
 		PLUGIN_COLUMN_CHECK, &old_state, PLUGIN_COLUMN_PLUGIN, &p, -1);
-	g_return_if_fail(p != NULL);
+
+	/* no plugins item */
+	if (p == NULL)
+		return;
+
 	state = ! old_state; /* toggle the state */
 
 	/* save the filename of the plugin */
@@ -1057,20 +1127,25 @@ static void pm_plugin_toggled(GtkCellRendererToggle *cell, gchar *pth, gpointer
 
 	/* reload plugin module and initialize it if item is checked */
 	p = plugin_new(file_name, state, TRUE);
-	if (state)
-		keybindings_load_keyfile();		/* load shortcuts */
+	if (!p)
+	{
+		/* plugin file may no longer be on disk, or is now incompatible */
+		gtk_list_store_remove(pm_widgets.store, &iter);
+	}
+	else
+	{
+		if (state)
+			keybindings_load_keyfile();		/* load shortcuts */
 
-	/* update model */
-	gtk_list_store_set(pm_widgets.store, &iter,
-		PLUGIN_COLUMN_CHECK, state,
-		PLUGIN_COLUMN_PLUGIN, p, -1);
+		/* update model */
+		gtk_list_store_set(pm_widgets.store, &iter,
+			PLUGIN_COLUMN_CHECK, state,
+			PLUGIN_COLUMN_PLUGIN, p, -1);
 
+		/* set again the sensitiveness of the configure and help buttons */
+		pm_update_buttons(p);
+	}
 	g_free(file_name);
-
-	/* set again the sensitiveness of the configure and help buttons */
-	is_active = is_active_plugin(p);
-	gtk_widget_set_sensitive(pm_widgets.configure_button, p->configure != NULL && is_active);
-	gtk_widget_set_sensitive(pm_widgets.help_button, p->help != NULL && is_active);
 }
 
 
@@ -1138,40 +1213,7 @@ static void pm_prepare_treeview(GtkWidget *tree, GtkListStore *store)
 }
 
 
-static void configure_plugin(Plugin *p)
-{
-	GtkWidget *parent = pm_widgets.dialog;
-	GtkWidget *prefs_page, *dialog, *vbox;
-
-	dialog = gtk_dialog_new_with_buttons(p->info.name,
-		GTK_WINDOW(parent), GTK_DIALOG_DESTROY_WITH_PARENT,
-		GTK_STOCK_APPLY, GTK_RESPONSE_APPLY,
-		GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-		GTK_STOCK_OK, GTK_RESPONSE_OK, NULL);
-	gtk_widget_set_name(dialog, "GeanyDialog");
-
-	vbox = ui_dialog_vbox_new(GTK_DIALOG(dialog));
-	gtk_widget_show(vbox);
-
-	prefs_page = p->configure(GTK_DIALOG(dialog));
-
-	if (! GTK_IS_WIDGET(prefs_page))
-	{
-		geany_debug("Invalid widget returned from plugin_configure() in plugin \"%s\"!",
-			p->info.name);
-	}
-	else
-	{
-		gtk_container_add(GTK_CONTAINER(vbox), prefs_page);
-
-		/* run the dialog */
-		while (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_APPLY);
-	}
-	gtk_widget_destroy(dialog);
-}
-
-
-void pm_on_plugin_button_clicked(GtkButton *button, gpointer user_data)
+static void pm_on_plugin_button_clicked(GtkButton *button, gpointer user_data)
 {
 	GtkTreeModel *model;
 	GtkTreeSelection *selection;
@@ -1186,7 +1228,7 @@ void pm_on_plugin_button_clicked(GtkButton *button, gpointer user_data)
 		if (p != NULL)
 		{
 			if (GPOINTER_TO_INT(user_data) == PM_BUTTON_CONFIGURE)
-				configure_plugin(p);
+				plugin_show_configure(&p->public);
 			else if (GPOINTER_TO_INT(user_data) == PM_BUTTON_HELP && p->help != NULL)
 				p->help();
 		}
@@ -1247,8 +1289,7 @@ static void pm_show_dialog(GtkMenuItem *menuitem, gpointer user_data)
 	gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(swin), GTK_SHADOW_IN);
 	gtk_container_add(GTK_CONTAINER(swin), pm_widgets.tree);
 
-	label = geany_wrap_label_new(_("Below is a list of available plugins. "
-		"Select the plugins which should be loaded when Geany is started."));
+	label = geany_wrap_label_new(_("Choose which plugins should be loaded at startup:"));
 
 	pm_widgets.configure_button = gtk_button_new_from_stock(GTK_STOCK_PREFERENCES);
 	gtk_widget_set_sensitive(pm_widgets.configure_button, FALSE);
