@@ -1,8 +1,8 @@
 /*
  *      plugins.h - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2007-2008 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2007-2008 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2007-2010 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2007-2010 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -19,7 +19,7 @@
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
  *      MA 02110-1301, USA.
  *
- * $Id: plugins.h 2491 2008-04-16 15:09:52Z ntrel $
+ * $Id: plugins.h 4518 2010-01-01 22:55:18Z eht16 $
  */
 
 
@@ -28,17 +28,16 @@
 
 #ifdef HAVE_PLUGINS
 
+extern GList *active_plugin_list;
+
+
 void plugins_init(void);
 
-void plugins_free(void);
+void plugins_finalize(void);
 
-void plugins_save_prefs(GKeyFile *config);
+void plugins_load_active(void);
 
-void plugins_load_prefs(GKeyFile *config);
-
-void plugins_update_tools_menu(void);
-
-void plugins_update_document_sensitive(gboolean enabled);
+gboolean plugins_have_preferences(void);
 
 #endif
 
