@@ -1,8 +1,8 @@
 /*
  *      prefs.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2005-2009 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2006-2009 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2005-2010 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2006-2010 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -18,7 +18,12 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: prefs.c 4009 2009-07-21 22:10:42Z eht16 $
+ * $Id: prefs.c 4929 2010-05-16 18:22:24Z eht16 $
+ */
+
+/*
+ * @file prefs.h
+ * Preferences dialog.
  */
 
 /*
@@ -48,7 +53,7 @@
 #include "project.h"
 #include "editor.h"
 #include "main.h"
-#include "treeviews.h"
+#include "sidebar.h"
 #include "printing.h"
 #include "geanywraplabel.h"
 #include "templates.h"
@@ -56,14 +61,13 @@
 #include "toolbar.h"
 #include "stash.h"
 #include "keyfile.h"
+#include "filetypes.h"
+#include "win32.h"
 
 #ifdef HAVE_VTE
 # include "vte.h"
 #endif
 
-#ifdef G_OS_WIN32
-# include "win32.h"
-#endif
 
 
 GeanyPrefs prefs;
@@ -80,13 +84,14 @@ static gboolean edited = FALSE;
 static void kb_cell_edited_cb(GtkCellRendererText *cellrenderertext, gchar *path, gchar *new_text, gpointer user_data);
 static gboolean kb_keytype_dialog_response_cb(GtkWidget *dialog, GdkEventKey *event, gpointer user_data);
 static void kb_dialog_response_cb(GtkWidget *dialog, gint response, gpointer user_data);
-static gboolean kb_find_duplicate(GeanyKeyBinding *search_kb,
+static gboolean kb_find_duplicate(GtkWidget *parent, GeanyKeyBinding *search_kb,
 		guint key, GdkModifierType mods, const gchar *action);
 static void on_toolbar_show_toggled(GtkToggleButton *togglebutton, gpointer user_data);
 static void on_show_notebook_tabs_toggled(GtkToggleButton *togglebutton, gpointer user_data);
 static void on_enable_plugins_toggled(GtkToggleButton *togglebutton, gpointer user_data);
 static void on_use_folding_toggled(GtkToggleButton *togglebutton, gpointer user_data);
 static void on_open_encoding_toggled(GtkToggleButton *togglebutton, gpointer user_data);
+static void on_sidebar_visible_toggled(GtkToggleButton *togglebutton, gpointer user_data);
 static void on_prefs_print_radio_button_toggled(GtkToggleButton *togglebutton, gpointer user_data);
 static void on_prefs_print_page_header_toggled(GtkToggleButton *togglebutton, gpointer user_data);
 
@@ -102,7 +107,7 @@ PrefCallbackAction;
 /* Synchronize Stash settings with widgets (see keyfile.c - init_pref_groups()). */
 static void prefs_action(PrefCallbackAction action)
 {
-	GeanyPrefGroup *group;
+	StashGroup *group;
 	guint i;
 
 	foreach_ptr_array(group, i, pref_groups)
@@ -296,7 +301,7 @@ static void kb_init(void)
 {
 	GtkTreeIter parent, iter;
 	gsize g, i;
-	gchar *key_string;
+	gchar *key_string, *label;
 	GeanyKeyGroup *group;
 	GeanyKeyBinding *kb;
 
@@ -315,11 +320,13 @@ static void kb_init(void)
 		{
 			kb = &group->keys[i];
 
+			label = keybindings_get_label(kb);
 			key_string = gtk_accelerator_name(kb->key, kb->mods);
 			gtk_tree_store_append(store, &iter, &parent);
-			gtk_tree_store_set(store, &iter, KB_TREE_ACTION, kb->label,
+			gtk_tree_store_set(store, &iter, KB_TREE_ACTION, label,
 				KB_TREE_SHORTCUT, key_string, KB_TREE_INDEX, i, -1);
 			g_free(key_string);
+			g_free(label);
 		}
 	}
 	gtk_tree_view_expand_all(GTK_TREE_VIEW(tree));
@@ -368,6 +375,10 @@ static void prefs_init_dialog(void)
 	widget = ui_lookup_widget(ui_widgets.prefs_dialog, "check_auto_focus");
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), prefs.auto_focus);
 
+	widget = ui_lookup_widget(ui_widgets.prefs_dialog, "check_native_windows_dialogs");
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget),
+		interface_prefs.use_native_windows_dialogs);
+
 	widget = ui_lookup_widget(ui_widgets.prefs_dialog, "entry_contextaction");
 	gtk_entry_set_text(GTK_ENTRY(widget), tool_prefs.context_action_cmd);
 
@@ -375,6 +386,10 @@ static void prefs_init_dialog(void)
 
 
 	/* Interface settings */
+	widget = ui_lookup_widget(ui_widgets.prefs_dialog, "check_sidebar_visible");
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), ui_prefs.sidebar_visible);
+	on_sidebar_visible_toggled(GTK_TOGGLE_BUTTON(widget), NULL);
+
 	widget = ui_lookup_widget(ui_widgets.prefs_dialog, "check_list_symbol");
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), interface_prefs.sidebar_symbol_visible);
 
@@ -391,13 +406,15 @@ static void prefs_init_dialog(void)
 	gtk_font_button_set_font_name(GTK_FONT_BUTTON(widget), interface_prefs.editor_font);
 
 	widget = ui_lookup_widget(ui_widgets.prefs_dialog, "spin_long_line");
-	gtk_spin_button_set_value(GTK_SPIN_BUTTON(widget), editor_prefs.long_line_column);
+	gtk_spin_button_set_value(GTK_SPIN_BUTTON(widget), editor_prefs.long_line_global_column);
 
-	switch (editor_prefs.long_line_type)
+	widget = ui_lookup_widget(ui_widgets.prefs_dialog, "check_long_line");
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), editor_prefs.long_line_global_enabled);
+
+	switch (editor_prefs.long_line_global_type)
 	{
 		case 0: widget = ui_lookup_widget(ui_widgets.prefs_dialog, "radio_long_line_line"); break;
 		case 1: widget = ui_lookup_widget(ui_widgets.prefs_dialog, "radio_long_line_background"); break;
-		default: widget = ui_lookup_widget(ui_widgets.prefs_dialog, "radio_long_line_disabled"); break;
 	}
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), TRUE);
 
@@ -441,6 +458,8 @@ static void prefs_init_dialog(void)
 		case 1: widget = ui_lookup_widget(ui_widgets.prefs_dialog, "radio_toolbar_text"); break;
 		default: widget = ui_lookup_widget(ui_widgets.prefs_dialog, "radio_toolbar_imagetext"); break;
 	}
+	if (toolbar_prefs.use_gtk_default_style)
+		widget = ui_lookup_widget(ui_widgets.prefs_dialog, "radio_toolbar_style_default");
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), TRUE);
 
 	switch (toolbar_prefs.icon_size)
@@ -451,6 +470,8 @@ static void prefs_init_dialog(void)
 				widget = ui_lookup_widget(ui_widgets.prefs_dialog, "radio_toolbar_small"); break;
 		default: widget = ui_lookup_widget(ui_widgets.prefs_dialog, "radio_toolbar_verysmall"); break;
 	}
+	if (toolbar_prefs.use_gtk_default_icon)
+		widget = ui_lookup_widget(ui_widgets.prefs_dialog, "radio_toolbar_icon_default");
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), TRUE);
 
 	/* disable elements if toolbar is hidden */
@@ -465,6 +486,8 @@ static void prefs_init_dialog(void)
 		widget = ui_lookup_widget(ui_widgets.prefs_dialog, "radio_tab_left");
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), TRUE);
 
+	widget = ui_lookup_widget(ui_widgets.prefs_dialog, "check_tab_beside");
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), file_prefs.tab_order_beside);
 
 	widget = ui_lookup_widget(ui_widgets.prefs_dialog, "combo_new_encoding");
 	/* luckily the index of the combo box items match the index of the encodings array */
@@ -587,8 +610,6 @@ static void prefs_init_dialog(void)
 		(editor_prefs.autoclose_chars & GEANY_AC_DQUOTE));
 
 	/* Tools Settings */
-	if (tool_prefs.make_cmd)
-			gtk_entry_set_text(GTK_ENTRY(ui_lookup_widget(ui_widgets.prefs_dialog, "entry_com_make")), tool_prefs.make_cmd);
 
     if (tool_prefs.term_cmd)
             gtk_entry_set_text(GTK_ENTRY(ui_lookup_widget(ui_widgets.prefs_dialog, "entry_com_term")), tool_prefs.term_cmd);
@@ -741,10 +762,18 @@ on_prefs_button_clicked(GtkDialog *dialog, gint response, gpointer user_data)
 		GtkWidget *widget;
 		guint i;
 		guint autoclose_brackets[5];
+		gboolean old_invert_all = interface_prefs.highlighting_invert_all;
+		gboolean old_sidebar_pos = interface_prefs.sidebar_pos;
 
 		/* Synchronize Stash settings */
 		prefs_action(PREF_UPDATE);
 
+		if (interface_prefs.highlighting_invert_all != old_invert_all)
+			filetypes_reload();
+
+		if (interface_prefs.sidebar_pos != old_sidebar_pos)
+			ui_swap_sidebar_pos();
+
 		/* General settings */
 		/* startup */
 		widget = ui_lookup_widget(ui_widgets.prefs_dialog, "check_load_session");
@@ -775,6 +804,10 @@ on_prefs_button_clicked(GtkDialog *dialog, gint response, gpointer user_data)
 		widget = ui_lookup_widget(ui_widgets.prefs_dialog, "check_auto_focus");
 		prefs.auto_focus = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
 
+		widget = ui_lookup_widget(ui_widgets.prefs_dialog, "check_native_windows_dialogs");
+		interface_prefs.use_native_windows_dialogs =
+			gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
+
 		widget = ui_lookup_widget(ui_widgets.prefs_dialog, "entry_contextaction");
 		g_free(tool_prefs.context_action_cmd);
 		tool_prefs.context_action_cmd = g_strdup(gtk_entry_get_text(GTK_ENTRY(widget)));
@@ -783,24 +816,27 @@ on_prefs_button_clicked(GtkDialog *dialog, gint response, gpointer user_data)
 
 
 		/* Interface settings */
+		widget = ui_lookup_widget(ui_widgets.prefs_dialog, "check_sidebar_visible");
+		ui_prefs.sidebar_visible = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
+
 		widget = ui_lookup_widget(ui_widgets.prefs_dialog, "check_list_symbol");
 		interface_prefs.sidebar_symbol_visible = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
 
 		widget = ui_lookup_widget(ui_widgets.prefs_dialog, "check_list_openfiles");
 		interface_prefs.sidebar_openfiles_visible = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
 
+		widget = ui_lookup_widget(ui_widgets.prefs_dialog, "check_long_line");
+		editor_prefs.long_line_global_enabled = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
+
 		widget = ui_lookup_widget(ui_widgets.prefs_dialog, "radio_long_line_line");
-		if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget))) editor_prefs.long_line_type = 0;
+		if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget)))
+			editor_prefs.long_line_global_type = 0;
 		else
-		{
-			widget = ui_lookup_widget(ui_widgets.prefs_dialog, "radio_long_line_background");
-			if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget))) editor_prefs.long_line_type = 1;
-			else
-			{	/* now only the disabled radio remains, so disable it */
-				editor_prefs.long_line_type = 2;
-			}
-		}
-		if (editor_prefs.long_line_column == 0) editor_prefs.long_line_type = 2;
+			/* now only the "background" radio remains */
+			editor_prefs.long_line_global_type = 1;
+
+		if (editor_prefs.long_line_global_column == 0)
+			editor_prefs.long_line_global_enabled = FALSE;
 
 		widget = ui_lookup_widget(ui_widgets.prefs_dialog, "check_show_notebook_tabs");
 		interface_prefs.show_notebook_tabs = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
@@ -827,36 +863,48 @@ on_prefs_button_clicked(GtkDialog *dialog, gint response, gpointer user_data)
 		widget = ui_lookup_widget(ui_widgets.prefs_dialog, "check_toolbar_in_menu");
 		toolbar_prefs.append_to_menu = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
 
-		widget = ui_lookup_widget(ui_widgets.prefs_dialog, "radio_toolbar_imagetext");
-		if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget)))
-			toolbar_prefs.icon_style = 2;
-		else
+		widget = ui_lookup_widget(ui_widgets.prefs_dialog, "radio_toolbar_style_default");
+		toolbar_prefs.use_gtk_default_style = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
+		if (! toolbar_prefs.use_gtk_default_style)
 		{
-			widget = ui_lookup_widget(ui_widgets.prefs_dialog, "radio_toolbar_image");
+			widget = ui_lookup_widget(ui_widgets.prefs_dialog, "radio_toolbar_imagetext");
 			if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget)))
-				toolbar_prefs.icon_style = 0;
+				toolbar_prefs.icon_style = 2;
 			else
-				/* now only the text only radio remains, so set text only */
-				toolbar_prefs.icon_style = 1;
+			{
+				widget = ui_lookup_widget(ui_widgets.prefs_dialog, "radio_toolbar_image");
+				if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget)))
+					toolbar_prefs.icon_style = 0;
+				else
+					/* now only the text only radio remains, so set text only */
+					toolbar_prefs.icon_style = 1;
+			}
 		}
 
-
-		widget = ui_lookup_widget(ui_widgets.prefs_dialog, "radio_toolbar_large");
-		if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget)))
-			toolbar_prefs.icon_size = GTK_ICON_SIZE_LARGE_TOOLBAR;
-		else
-		{
-			widget = ui_lookup_widget(ui_widgets.prefs_dialog, "radio_toolbar_small");
+		widget = ui_lookup_widget(ui_widgets.prefs_dialog, "radio_toolbar_icon_default");
+		toolbar_prefs.use_gtk_default_icon = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
+		if (! toolbar_prefs.use_gtk_default_icon)
+		{	toolbar_prefs.icon_size = GTK_ICON_SIZE_LARGE_TOOLBAR;
+			widget = ui_lookup_widget(ui_widgets.prefs_dialog, "radio_toolbar_large");
 			if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget)))
-				toolbar_prefs.icon_size = GTK_ICON_SIZE_SMALL_TOOLBAR;
+				toolbar_prefs.icon_size = GTK_ICON_SIZE_LARGE_TOOLBAR;
 			else
-				toolbar_prefs.icon_size = GTK_ICON_SIZE_MENU;
+			{
+				widget = ui_lookup_widget(ui_widgets.prefs_dialog, "radio_toolbar_small");
+				if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget)))
+					toolbar_prefs.icon_size = GTK_ICON_SIZE_SMALL_TOOLBAR;
+				else
+					toolbar_prefs.icon_size = GTK_ICON_SIZE_MENU;
+			}
 		}
 
 		/* Files settings */
 		widget = ui_lookup_widget(ui_widgets.prefs_dialog, "radio_tab_right");
 		file_prefs.tab_order_ltr = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
 
+		widget = ui_lookup_widget(ui_widgets.prefs_dialog, "check_tab_beside");
+		file_prefs.tab_order_beside = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
+
 		widget = ui_lookup_widget(ui_widgets.prefs_dialog, "combo_new_encoding");
 		file_prefs.default_new_encoding = gtk_combo_box_get_active(GTK_COMBO_BOX(widget));
 
@@ -890,7 +938,7 @@ on_prefs_button_clicked(GtkDialog *dialog, gint response, gpointer user_data)
 		widget = ui_lookup_widget(ui_widgets.prefs_dialog, "spin_long_line");
 		/* note: use stash for new code - it updates spin buttons itself */
 		gtk_spin_button_update(GTK_SPIN_BUTTON(widget));
-		editor_prefs.long_line_column = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(widget));
+		editor_prefs.long_line_global_column = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(widget));
 
 		widget = ui_lookup_widget(ui_widgets.prefs_dialog, "check_folding");
 		editor_prefs.folding = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
@@ -979,9 +1027,6 @@ on_prefs_button_clicked(GtkDialog *dialog, gint response, gpointer user_data)
 		| (autoclose_brackets[4] ? GEANY_AC_DQUOTE : 0);
 
 		/* Tools Settings */
-		widget = ui_lookup_widget(ui_widgets.prefs_dialog, "entry_com_make");
-		g_free(tool_prefs.make_cmd);
-		tool_prefs.make_cmd = g_strdup(gtk_entry_get_text(GTK_ENTRY(widget)));
 
 		widget = ui_lookup_widget(ui_widgets.prefs_dialog, "entry_com_term");
 		g_free(tool_prefs.term_cmd);
@@ -1105,7 +1150,7 @@ on_prefs_button_clicked(GtkDialog *dialog, gint response, gpointer user_data)
 
 		/* apply the changes made */
 		ui_statusbar_showhide(interface_prefs.statusbar_visible);
-		treeviews_openfiles_update_all(); /* to update if full path setting has changed */
+		sidebar_openfiles_update_all(); /* to update if full path setting has changed */
 		toolbar_apply_settings();
 		toolbar_update_ui();
 		toolbar_show_hide();
@@ -1141,39 +1186,16 @@ on_prefs_button_clicked(GtkDialog *dialog, gint response, gpointer user_data)
 }
 
 
-void on_prefs_color_choosed(GtkColorButton *widget, gpointer user_data)
+static void on_color_button_choose_cb(GtkColorButton *widget, gpointer user_data)
 {
 	GdkColor color;
 
-	switch (GPOINTER_TO_INT(user_data))
-	{
-		case 1:
-		{
-			gtk_color_button_get_color(widget, &color);
-			setptr(editor_prefs.long_line_color, utils_get_hex_from_color(&color));
-			break;
-		}
-#ifdef HAVE_VTE
-		case 2:
-		{
-			g_free(vc->colour_fore);
-			vc->colour_fore = g_new0(GdkColor, 1);
-			gtk_color_button_get_color(widget, vc->colour_fore);
-			break;
-		}
-		case 3:
-		{
-			g_free(vc->colour_back);
-			vc->colour_back = g_new0(GdkColor, 1);
-			gtk_color_button_get_color(widget, vc->colour_back);
-			break;
-		}
-#endif
-	}
+	gtk_color_button_get_color(widget, &color);
+	setptr(editor_prefs.long_line_color, utils_get_hex_from_color(&color));
 }
 
 
-void on_prefs_font_choosed(GtkFontButton *widget, gpointer user_data)
+static void on_prefs_font_choosed(GtkFontButton *widget, gpointer user_data)
 {
 	const gchar *fontbtn = gtk_font_button_get_font_name(widget);
 	guint i;
@@ -1207,9 +1229,7 @@ void on_prefs_font_choosed(GtkFontButton *widget, gpointer user_data)
 			ui_widget_modify_font_from_string(msgwindow.tree_compiler, interface_prefs.msgwin_font);
 			ui_widget_modify_font_from_string(msgwindow.tree_msg, interface_prefs.msgwin_font);
 			ui_widget_modify_font_from_string(msgwindow.tree_status, interface_prefs.msgwin_font);
-			ui_widget_modify_font_from_string(
-				ui_lookup_widget(main_widgets.window, "textview_scribble"),
-				interface_prefs.msgwin_font);
+			ui_widget_modify_font_from_string(msgwindow.scribble, interface_prefs.msgwin_font);
 			break;
 		}
 		case 3:
@@ -1217,17 +1237,6 @@ void on_prefs_font_choosed(GtkFontButton *widget, gpointer user_data)
 			ui_set_editor_font(fontbtn);
 			break;
 		}
-#ifdef HAVE_VTE
-		case 4:
-		{
-			/* VTE settings */
-			if (strcmp(fontbtn, vc->font) == 0)
-				break;
-			setptr(vc->font, g_strdup(gtk_font_button_get_font_name(widget)));
-			vte_apply_user_settings();
-			break;
-		}
-#endif
 	}
 }
 
@@ -1265,13 +1274,12 @@ static void kb_cell_edited_cb(GtkCellRendererText *cellrenderertext, gchar *path
 
 		kb = kb_lookup_kb_from_iter(GTK_TREE_MODEL(store), &iter);
 
-		if (kb_find_duplicate(kb, lkey, lmods, new_text))
+		if (kb_find_duplicate(ui_widgets.prefs_dialog, kb, lkey, lmods, new_text))
 			return;
 
 		/* set the values here, because of the above check, setting it in
 		 * gtk_accelerator_parse would return a wrong key combination if it is duplicate */
-		kb->key = lkey;
-		kb->mods = lmods;
+		keybindings_update_combo(kb, lkey, lmods);
 
 		gtk_tree_store_set(store, &iter, KB_TREE_SHORTCUT, new_text, -1);
 
@@ -1311,13 +1319,12 @@ static void kb_dialog_response_cb(GtkWidget *dialog, gint response, G_GNUC_UNUSE
 
 		gtk_accelerator_parse(gtk_label_get_text(GTK_LABEL(dialog_label)), &lkey, &lmods);
 
-		if (kb_find_duplicate(kb, lkey, lmods, gtk_label_get_text(GTK_LABEL(dialog_label))))
+		if (kb_find_duplicate(dialog, kb, lkey, lmods, gtk_label_get_text(GTK_LABEL(dialog_label))))
 			return;
 
 		/* set the values here, because of the above check, setting it in
 		 * gtk_accelerator_parse would return a wrong key combination if it is duplicate */
-		kb->key = lkey;
-		kb->mods = lmods;
+		keybindings_update_combo(kb, lkey, lmods);
 
 		gtk_tree_store_set(store, &g_iter,
 			KB_TREE_SHORTCUT, gtk_label_get_text(GTK_LABEL(dialog_label)), -1);
@@ -1371,8 +1378,9 @@ static void kb_clear_tree_shortcut(gsize group_id, gsize keybinding_id)
 }
 
 
-/* test if the entered key combination is already used */
-static gboolean kb_find_duplicate(GeanyKeyBinding *search_kb,
+/* test if the entered key combination is already used
+ * returns true if cancelling duplicate */
+static gboolean kb_find_duplicate(GtkWidget *parent, GeanyKeyBinding *search_kb,
 		guint key, GdkModifierType mods, const gchar *action)
 {
 	gsize g, i;
@@ -1395,17 +1403,25 @@ static gboolean kb_find_duplicate(GeanyKeyBinding *search_kb,
 			if (kb->key == key && kb->mods == mods
 				&& ! (kb->key == search_kb->key && kb->mods == search_kb->mods))
 			{
-				if (dialogs_show_question_full(main_widgets.window, _("_Override"), GTK_STOCK_CANCEL,
+				gchar *label = keybindings_get_label(kb);
+				gint ret = dialogs_show_prompt(parent,
+					_("_Allow"), GTK_RESPONSE_APPLY,
+					GTK_STOCK_CANCEL, GTK_RESPONSE_NO,
+					_("_Override"), GTK_RESPONSE_YES,
 					_("Override that keybinding?"),
 					_("The combination '%s' is already used for \"%s\"."),
-					action, kb->label))
+					action, label);
+
+				g_free(label);
+
+				if (ret == GTK_RESPONSE_YES)
 				{
-					kb->key = 0;
-					kb->mods = 0;
+					keybindings_update_combo(kb, 0, 0);
 					kb_clear_tree_shortcut(g, i);
+					/* carry on looking for other duplicates if overriding */
 					continue;
 				}
-				return TRUE;
+				return ret == GTK_RESPONSE_NO;
 			}
 		}
 	}
@@ -1417,7 +1433,12 @@ static void on_toolbar_show_toggled(GtkToggleButton *togglebutton, gpointer user
 {
 	gboolean sens = gtk_toggle_button_get_active(togglebutton);
 
-	gtk_widget_set_sensitive(ui_lookup_widget(ui_widgets.prefs_dialog, "frame13"), sens);
+	gtk_widget_set_sensitive(
+		ui_lookup_widget(ui_widgets.prefs_dialog, "frame_toolbar_style"), sens);
+	gtk_widget_set_sensitive(
+		ui_lookup_widget(ui_widgets.prefs_dialog, "frame_toolbar_icon"), sens);
+	gtk_widget_set_sensitive(
+		ui_lookup_widget(ui_widgets.prefs_dialog, "button_customize_toolbar"), sens);
 	gtk_widget_set_sensitive(
 		ui_lookup_widget(ui_widgets.prefs_dialog, "check_toolbar_in_menu"), sens);
 }
@@ -1459,6 +1480,15 @@ static void on_open_encoding_toggled(GtkToggleButton *togglebutton, gpointer use
 }
 
 
+static void on_sidebar_visible_toggled(GtkToggleButton *togglebutton, gpointer user_data)
+{
+	gboolean sens = gtk_toggle_button_get_active(togglebutton);
+
+	gtk_widget_set_sensitive(ui_lookup_widget(ui_widgets.prefs_dialog, "check_list_openfiles"), sens);
+	gtk_widget_set_sensitive(ui_lookup_widget(ui_widgets.prefs_dialog, "check_list_symbol"), sens);
+}
+
+
 static void on_prefs_print_radio_button_toggled(GtkToggleButton *togglebutton, gpointer user_data)
 {
 	gboolean sens = gtk_toggle_button_get_active(togglebutton);
@@ -1477,6 +1507,77 @@ static void on_prefs_print_page_header_toggled(GtkToggleButton *togglebutton, gp
 }
 
 
+static void open_preferences_help(void)
+{
+	gchar *uri;
+	const gchar *label, *suffix = NULL;
+	GtkNotebook *notebook = GTK_NOTEBOOK(
+		ui_lookup_widget(ui_widgets.prefs_dialog, "notebook2"));
+	gint page_nr = gtk_notebook_get_current_page(notebook);
+	GtkWidget *page = gtk_notebook_get_nth_page(notebook, page_nr);
+
+	label = gtk_notebook_get_tab_label_text(notebook, page);
+
+	/* TODO Find a better way to map the current notebook page to the
+	 * corresponding chapter in the documentation, comparing translatable
+	 * strings is easy to break. Maybe attach an identifying string to the
+	 * tab label object. */
+	if (utils_str_equal(label, _("General")))
+		suffix = "#general-startup-preferences";
+	else if (utils_str_equal(label, _("Interface")))
+		suffix = "#interface-preferences";
+	else if (utils_str_equal(label, _("Toolbar")))
+		suffix = "#toolbar-preferences";
+	else if (utils_str_equal(label, _("Editor")))
+		suffix = "#editor-features-preferences";
+	else if (utils_str_equal(label, _("Files")))
+		suffix = "#files-preferences";
+	else if (utils_str_equal(label, _("Tools")))
+		suffix = "#tools-preferences";
+	else if (utils_str_equal(label, _("Templates")))
+		suffix = "#template-preferences";
+	else if (utils_str_equal(label, _("Keybindings")))
+		suffix = "#keybinding-preferences";
+	else if (utils_str_equal(label, _("Printing")))
+		suffix = "#printing-preferences";
+	else if (utils_str_equal(label, _("Terminal")))
+		suffix = "#terminal-vte-preferences";
+
+	uri = utils_get_help_url(suffix);
+	utils_open_browser(uri);
+	g_free(uri);
+}
+
+
+static gboolean prefs_dialog_key_press_response_cb(GtkWidget *dialog, GdkEventKey *event,
+												   gpointer data)
+{
+	gint group, keybinding;
+
+	if (keybindings_check_event(event, &group, &keybinding) != NULL)
+	{
+		if (group == GEANY_KEY_GROUP_HELP && keybinding == GEANY_KEYS_HELP_HELP)
+		{
+			open_preferences_help();
+			return TRUE;
+		}
+	}
+	return FALSE;
+}
+
+
+static gboolean prefs_dialog_button_press_event_cb(GtkWidget *dialog, GdkEventButton *event,
+												   gpointer data)
+{
+	if (event->button == 1)
+	{
+		open_preferences_help();
+		return TRUE;
+	}
+	return FALSE;
+}
+
+
 void prefs_show_dialog(void)
 {
 	if (ui_widgets.prefs_dialog == NULL)
@@ -1485,10 +1586,14 @@ void prefs_show_dialog(void)
 		GtkWidget *label;
 		guint i;
 		gchar *encoding_string;
+		GdkPixbuf *pb;
 
 		ui_widgets.prefs_dialog = create_prefs_dialog();
 		gtk_widget_set_name(ui_widgets.prefs_dialog, "GeanyPrefsDialog");
 		gtk_window_set_transient_for(GTK_WINDOW(ui_widgets.prefs_dialog), GTK_WINDOW(main_widgets.window));
+		pb = ui_new_pixbuf_from_inline(GEANY_IMAGE_LOGO);
+		gtk_window_set_icon(GTK_WINDOW(ui_widgets.prefs_dialog), pb);
+		g_object_unref(pb);	/* free our reference */
 
 		/* init the default file encoding combo box */
 		combo_new = ui_lookup_widget(ui_widgets.prefs_dialog, "combo_new_encoding");
@@ -1545,30 +1650,40 @@ void prefs_show_dialog(void)
 			"label_project_indent_warning")), label, FALSE, TRUE, 5);
 
 		/* add the clear icon to GtkEntry widgets in the dialog */
-		ui_entry_add_clear_icon(ui_lookup_widget(ui_widgets.prefs_dialog, "startup_path_entry"));
-		ui_entry_add_clear_icon(ui_lookup_widget(ui_widgets.prefs_dialog, "project_file_path_entry"));
-		ui_entry_add_clear_icon(ui_lookup_widget(ui_widgets.prefs_dialog, "extra_plugin_path_entry"));
-		ui_entry_add_clear_icon(ui_lookup_widget(ui_widgets.prefs_dialog, "entry_toggle_mark"));
-		ui_entry_add_clear_icon(ui_lookup_widget(ui_widgets.prefs_dialog, "entry_com_make"));
-		ui_entry_add_clear_icon(ui_lookup_widget(ui_widgets.prefs_dialog, "entry_com_term"));
-		ui_entry_add_clear_icon(ui_lookup_widget(ui_widgets.prefs_dialog, "entry_browser"));
-		ui_entry_add_clear_icon(ui_lookup_widget(ui_widgets.prefs_dialog, "entry_grep"));
-		ui_entry_add_clear_icon(ui_lookup_widget(ui_widgets.prefs_dialog, "entry_contextaction"));
-		ui_entry_add_clear_icon(ui_lookup_widget(ui_widgets.prefs_dialog, "entry_template_developer"));
-		ui_entry_add_clear_icon(ui_lookup_widget(ui_widgets.prefs_dialog, "entry_template_initial"));
-		ui_entry_add_clear_icon(ui_lookup_widget(ui_widgets.prefs_dialog, "entry_template_mail"));
-		ui_entry_add_clear_icon(ui_lookup_widget(ui_widgets.prefs_dialog, "entry_template_company"));
-		ui_entry_add_clear_icon(ui_lookup_widget(ui_widgets.prefs_dialog, "entry_template_version"));
-		ui_entry_add_clear_icon(ui_lookup_widget(ui_widgets.prefs_dialog, "entry_template_year"));
-		ui_entry_add_clear_icon(ui_lookup_widget(ui_widgets.prefs_dialog, "entry_template_date"));
-		ui_entry_add_clear_icon(ui_lookup_widget(ui_widgets.prefs_dialog, "entry_template_datetime"));
-		ui_entry_add_clear_icon(ui_lookup_widget(ui_widgets.prefs_dialog, "entry_print_external_cmd"));
-		ui_entry_add_clear_icon(ui_lookup_widget(ui_widgets.prefs_dialog, "entry_print_dateformat"));
+		{
+			const gchar *names[] = {
+				"startup_path_entry",
+				"project_file_path_entry",
+				"extra_plugin_path_entry",
+				"entry_toggle_mark",
+			/*	"entry_com_make", */
+				"entry_com_term",
+				"entry_browser",
+				"entry_grep",
+				"entry_contextaction",
+				"entry_template_developer",
+				"entry_template_initial",
+				"entry_template_mail",
+				"entry_template_company",
+				"entry_template_version",
+				"entry_template_year",
+				"entry_template_date",
+				"entry_template_datetime",
+				"entry_print_external_cmd",
+				"entry_print_dateformat"};
+			const gchar **name;
+
+			foreach_c_array(name, names, G_N_ELEMENTS(names))
+				ui_entry_add_clear_icon(GTK_ENTRY(ui_lookup_widget(ui_widgets.prefs_dialog, *name)));
+		}
 
 #ifdef HAVE_VTE
 		vte_append_preferences_tab();
 #endif
 
+#ifndef G_OS_WIN32
+		gtk_widget_hide(ui_lookup_widget(ui_widgets.prefs_dialog, "check_native_windows_dialogs"));
+#endif
 		ui_setup_open_button_callback(ui_lookup_widget(ui_widgets.prefs_dialog, "startup_path_button"), NULL,
 			GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER, GTK_ENTRY(ui_lookup_widget(ui_widgets.prefs_dialog, "startup_path_entry")));
 		ui_setup_open_button_callback(ui_lookup_widget(ui_widgets.prefs_dialog, "extra_plugin_path_button"), NULL,
@@ -1586,24 +1701,33 @@ void prefs_show_dialog(void)
 		g_signal_connect(ui_lookup_widget(ui_widgets.prefs_dialog, "editor_font"),
 				"font-set", G_CALLBACK(on_prefs_font_choosed), GINT_TO_POINTER(3));
 		g_signal_connect(ui_lookup_widget(ui_widgets.prefs_dialog, "long_line_color"),
-				"color-set", G_CALLBACK(on_prefs_color_choosed), GINT_TO_POINTER(1));
+				"color-set", G_CALLBACK(on_color_button_choose_cb), NULL);
 		/* file chooser buttons in the tools tab */
-		g_signal_connect(ui_lookup_widget(ui_widgets.prefs_dialog, "button_make"),
-				"clicked", G_CALLBACK(on_prefs_tools_button_clicked), ui_lookup_widget(ui_widgets.prefs_dialog, "entry_com_make"));
-		g_signal_connect(ui_lookup_widget(ui_widgets.prefs_dialog, "button_term"),
-				"clicked", G_CALLBACK(on_prefs_tools_button_clicked), ui_lookup_widget(ui_widgets.prefs_dialog, "entry_com_term"));
-		g_signal_connect(ui_lookup_widget(ui_widgets.prefs_dialog, "button_browser"),
-				"clicked", G_CALLBACK(on_prefs_tools_button_clicked), ui_lookup_widget(ui_widgets.prefs_dialog, "entry_browser"));
-		g_signal_connect(ui_lookup_widget(ui_widgets.prefs_dialog, "button_grep"),
-				"clicked", G_CALLBACK(on_prefs_tools_button_clicked), ui_lookup_widget(ui_widgets.prefs_dialog, "entry_grep"));
+		ui_setup_open_button_callback(ui_lookup_widget(ui_widgets.prefs_dialog, "button_term"),
+			NULL,
+			GTK_FILE_CHOOSER_ACTION_OPEN,
+			GTK_ENTRY(ui_lookup_widget(ui_widgets.prefs_dialog, "entry_com_term")));
+		ui_setup_open_button_callback(ui_lookup_widget(ui_widgets.prefs_dialog, "button_browser"),
+			NULL,
+			GTK_FILE_CHOOSER_ACTION_OPEN,
+			GTK_ENTRY(ui_lookup_widget(ui_widgets.prefs_dialog, "entry_browser")));
+		ui_setup_open_button_callback(ui_lookup_widget(ui_widgets.prefs_dialog, "button_grep"),
+			NULL,
+			GTK_FILE_CHOOSER_ACTION_OPEN,
+			GTK_ENTRY(ui_lookup_widget(ui_widgets.prefs_dialog, "entry_grep")));
 
 		/* tools commands */
-		g_signal_connect(ui_lookup_widget(ui_widgets.prefs_dialog, "button_contextaction"),
-			"clicked", G_CALLBACK(on_prefs_tools_button_clicked), ui_lookup_widget(ui_widgets.prefs_dialog, "entry_contextaction"));
+		ui_setup_open_button_callback(ui_lookup_widget(ui_widgets.prefs_dialog, "button_contextaction"),
+			NULL,
+			GTK_FILE_CHOOSER_ACTION_OPEN,
+			GTK_ENTRY(ui_lookup_widget(ui_widgets.prefs_dialog, "entry_contextaction")));
 
 		/* printing */
-		g_signal_connect(ui_lookup_widget(ui_widgets.prefs_dialog, "button_print_external_cmd"),
-			"clicked", G_CALLBACK(on_prefs_tools_button_clicked), ui_lookup_widget(ui_widgets.prefs_dialog, "entry_print_external_cmd"));
+		ui_setup_open_button_callback(ui_lookup_widget(ui_widgets.prefs_dialog, "button_print_external_cmd"),
+			NULL,
+			GTK_FILE_CHOOSER_ACTION_OPEN,
+			GTK_ENTRY(ui_lookup_widget(ui_widgets.prefs_dialog, "entry_print_external_cmd")));
+
 		g_signal_connect(ui_lookup_widget(ui_widgets.prefs_dialog, "radio_print_gtk"),
 			"toggled", G_CALLBACK(on_prefs_print_radio_button_toggled), NULL);
 		g_signal_connect(ui_lookup_widget(ui_widgets.prefs_dialog, "check_print_pageheader"),
@@ -1619,64 +1743,15 @@ void prefs_show_dialog(void)
 				"toggled", G_CALLBACK(on_use_folding_toggled), NULL);
 		g_signal_connect(ui_lookup_widget(ui_widgets.prefs_dialog, "check_open_encoding"),
 				"toggled", G_CALLBACK(on_open_encoding_toggled), NULL);
+		g_signal_connect(ui_lookup_widget(ui_widgets.prefs_dialog, "check_sidebar_visible"),
+				"toggled", G_CALLBACK(on_sidebar_visible_toggled), NULL);
+
+		g_signal_connect(ui_lookup_widget(ui_widgets.prefs_dialog, "button_help"),
+				"button-press-event", G_CALLBACK(prefs_dialog_button_press_event_cb), NULL);
+		g_signal_connect(ui_widgets.prefs_dialog,
+				"key-press-event", G_CALLBACK(prefs_dialog_key_press_response_cb), NULL);
 	}
 
 	prefs_init_dialog();
 	gtk_window_present(GTK_WINDOW(ui_widgets.prefs_dialog));
 }
-
-
-void
-on_prefs_tools_button_clicked           (GtkButton       *button,
-                                        gpointer         item)
-{
-#ifdef G_OS_WIN32
-	win32_show_pref_file_dialog(item);
-#else
-	GtkWidget *dialog;
-	gchar *filename, *tmp, **field;
-
-	/* initialize the dialog */
-	dialog = gtk_file_chooser_dialog_new(_("Open File"), GTK_WINDOW(ui_widgets.prefs_dialog),
-					GTK_FILE_CHOOSER_ACTION_OPEN,
-					GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-					GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT, NULL);
-	/* cut the options from the command line */
-	field = g_strsplit(gtk_entry_get_text(GTK_ENTRY(item)), " ", 2);
-	if (field[0])
-	{
-		filename = g_find_program_in_path(field[0]);
-		if (filename)
-		{
-			gtk_file_chooser_select_filename(GTK_FILE_CHOOSER(dialog), filename);
-			g_free(filename);
-		}
-	}
-
-	/* run it */
-	if (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_ACCEPT)
-	{
-		gchar *utf8_filename;
-
-		tmp = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog));
-		utf8_filename = utils_get_utf8_from_locale(tmp);
-
-		if (g_strv_length(field) > 1)
-			filename = g_strconcat(utf8_filename, " ", field[1], NULL);
-		else
-		{
-			filename = utf8_filename;
-			utf8_filename = NULL;
-		}
-		gtk_entry_set_text(GTK_ENTRY(item), filename);
-		g_free(filename);
-		g_free(tmp);
-		g_free(utf8_filename);
-	}
-
-	g_strfreev(field);
-	gtk_widget_destroy(dialog);
-#endif
-}
-
-
