@@ -1,8 +1,8 @@
 /*
  *      printing.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2007-2009 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2007-2009 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2007-2010 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2007-2010 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: printing.c 4014 2009-07-23 16:01:45Z ntrel $
+ * $Id: printing.c 5137 2010-08-11 14:26:41Z ntrel $
  */
 
 
@@ -270,7 +270,7 @@ static gint get_page_count(GtkPrintContext *context, DocInfo *dinfo)
 }
 
 
-static void add_page_header(PangoLayout *layout, cairo_t *cr, DocInfo *dinfo, gint width, gint page_nr)
+static void add_page_header(DocInfo *dinfo, cairo_t *cr, gint width, gint page_nr)
 {
 	gint ph_height = dinfo->line_height * 3;
 	gchar *data;
@@ -279,6 +279,7 @@ static void add_page_header(PangoLayout *layout, cairo_t *cr, DocInfo *dinfo, gi
 		dinfo->doc->file_name : GEANY_STRING_UNTITLED;
 	gchar *file_name = (printing_prefs.page_header_basename) ?
 		g_path_get_basename(tmp_file_name) : g_strdup(tmp_file_name);
+	PangoLayout *layout = dinfo->layout;
 
 	/* draw the frame */
 	cairo_set_line_width(cr, 0.3);
@@ -417,7 +418,7 @@ static GtkWidget *create_custom_widget(GtkPrintOperation *operation, gpointer us
 	gtk_box_pack_start(GTK_BOX(hbox10), label203, FALSE, FALSE, 0);
 
 	w->entry_print_dateformat = gtk_entry_new();
-	ui_entry_add_clear_icon(w->entry_print_dateformat);
+	ui_entry_add_clear_icon(GTK_ENTRY(w->entry_print_dateformat));
 	gtk_box_pack_start(GTK_BOX(hbox10), w->entry_print_dateformat, TRUE, TRUE, 0);
 	ui_widget_set_tooltip_text(w->entry_print_dateformat, _("Specify a format for the date and time stamp which is added to the page header on each page. You can use any conversion specifiers which can be used with the ANSI C strftime function."));
 	gtk_entry_set_text(GTK_ENTRY(w->entry_print_dateformat), printing_prefs.page_header_datefmt);
@@ -490,6 +491,9 @@ static void begin_print(GtkPrintOperation *operation, GtkPrintContext *context,
 			dinfo->styles[i][BACK] = ROTATE_RGB(scintilla_send_message(
 				dinfo->doc->editor->sci, SCI_STYLEGETBACK, i, 0));
 		}
+		/* use white background color unless foreground is white to save ink */
+		if (dinfo->styles[i][FORE] != 0xffffff)
+			dinfo->styles[i][BACK] = 0xffffff;
 		dinfo->styles[i][BOLD] =
 			scintilla_send_message(dinfo->doc->editor->sci, SCI_STYLEGETBOLD, i, 0);
 		dinfo->styles[i][ITALIC] =
@@ -526,6 +530,7 @@ static void draw_page(GtkPrintOperation *operation, GtkPrintContext *context,
 		gchar *text = g_strdup_printf(_("Page %d of %d"), page_nr, dinfo->n_pages);
 		gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(main_widgets.progressbar), fraction);
 		gtk_progress_bar_set_text(GTK_PROGRESS_BAR(main_widgets.progressbar), text);
+		g_free(text);
 	}
 
 #ifdef GEANY_PRINT_DEBUG
@@ -547,9 +552,12 @@ static void draw_page(GtkPrintOperation *operation, GtkPrintContext *context,
 	cairo_move_to(cr, 0, 0);
 
 	pango_layout_set_width(dinfo->layout, width * PANGO_SCALE);
+	pango_layout_set_alignment(dinfo->layout, PANGO_ALIGN_LEFT);
+	pango_layout_set_ellipsize(dinfo->layout, FALSE);
+	pango_layout_set_justify(dinfo->layout, FALSE);
 
 	if (printing_prefs.print_page_header)
-		add_page_header(dinfo->layout, cr, dinfo, width, page_nr);
+		add_page_header(dinfo, cr, width, page_nr);
 
 	count = 0;	/* the actual line counter for the current page, might be different from
 				 * dinfo->cur_line due to possible line breaks */
@@ -773,9 +781,9 @@ static void status_changed(GtkPrintOperation *op, gpointer data)
 {
 	gchar *filename = (data != NULL) ? data : GEANY_STRING_UNTITLED;
 	if (gtk_print_operation_get_status(op) == GTK_PRINT_STATUS_FINISHED_ABORTED)
-		msgwin_status_add(_("Printing of file %s was cancelled."), filename);
+		msgwin_status_add(_("Did not send document %s to the printing subsystem."), filename);
 	else if (gtk_print_operation_get_status(op) == GTK_PRINT_STATUS_FINISHED)
-		msgwin_status_add(_("File %s printed."), filename);
+		msgwin_status_add(_("Document %s was sent to the printing subsystem."), filename);
 }
 
 
@@ -799,6 +807,9 @@ static void printing_print_gtk(GeanyDocument *doc)
 
 	gtk_print_operation_set_unit(op, GTK_UNIT_POINTS);
 	gtk_print_operation_set_show_progress(op, TRUE);
+#if GTK_CHECK_VERSION(2, 18, 0)
+	gtk_print_operation_set_embed_page_setup(op, TRUE);
+#endif
 
 	g_signal_connect(op, "begin-print", G_CALLBACK(begin_print), dinfo);
 	g_signal_connect(op, "end-print", G_CALLBACK(end_print), dinfo);
