@@ -1,8 +1,8 @@
 /*
  *      search.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2006-2009 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2006-2009 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2006-2010 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2006-2010 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: search.c 3974 2009-07-16 14:37:35Z ntrel $
+ * $Id: search.c 5153 2010-08-12 16:51:55Z ntrel $
  */
 
 /*
@@ -45,12 +45,19 @@
 
 #include <unistd.h>
 #include <string.h>
+#include <ctype.h>
 
 #ifdef G_OS_UNIX
 # include <sys/types.h>
 # include <sys/wait.h>
 #endif
 
+#ifdef HAVE_REGEX_H
+# include <regex.h>
+#else
+# include "gnuregex.h"
+#endif
+
 
 enum
 {
@@ -91,7 +98,7 @@ static struct
 }
 settings = {0, NULL, FALSE, FALSE, FALSE, FALSE, FALSE};
 
-static GeanyPrefGroup *fif_prefs = NULL;
+static StashGroup *fif_prefs = NULL;
 
 
 static struct
@@ -99,8 +106,9 @@ static struct
 	GtkWidget	*dialog;
 	GtkWidget	*entry;
 	gboolean	all_expanded;
+	gint		position[2]; /* x, y */
 }
-find_dlg = {NULL, NULL, FALSE};
+find_dlg = {NULL, NULL, FALSE, {0, 0}};
 
 static struct
 {
@@ -108,8 +116,9 @@ static struct
 	GtkWidget	*find_entry;
 	GtkWidget	*replace_entry;
 	gboolean	all_expanded;
+	gint		position[2]; /* x, y */
 }
-replace_dlg = {NULL, NULL, NULL, FALSE};
+replace_dlg = {NULL, NULL, NULL, FALSE, {0, 0}};
 
 static struct
 {
@@ -117,8 +126,9 @@ static struct
 	GtkWidget	*dir_combo;
 	GtkWidget	*search_combo;
 	GtkWidget	*encoding_combo;
+	gint		position[2]; /* x, y */
 }
-fif_dlg = {NULL, NULL, NULL, NULL};
+fif_dlg = {NULL, NULL, NULL, NULL, {0, 0}};
 
 
 static gboolean search_read_io(GIOChannel *source, GIOCondition condition, gpointer data);
@@ -157,7 +167,7 @@ search_find_in_files(const gchar *utf8_search_text, const gchar *dir, const gcha
 
 static void init_prefs(void)
 {
-	GeanyPrefGroup *group;
+	StashGroup *group;
 
 	group = stash_group_new("search");
 	configuration_add_pref_group(group, TRUE);
@@ -165,6 +175,13 @@ static void init_prefs(void)
 		"pref_search_current_file_dir", TRUE, "check_fif_current_dir");
 	stash_group_add_boolean(group, &find_dlg.all_expanded, "find_all_expanded", FALSE);
 	stash_group_add_boolean(group, &replace_dlg.all_expanded, "replace_all_expanded", FALSE);
+	/* dialog positions */
+	stash_group_add_integer(group, &find_dlg.position[0], "position_find_x", -1);
+	stash_group_add_integer(group, &find_dlg.position[1], "position_find_y", -1);
+	stash_group_add_integer(group, &replace_dlg.position[0], "position_replace_x", -1);
+	stash_group_add_integer(group, &replace_dlg.position[1], "position_replace_y", -1);
+	stash_group_add_integer(group, &fif_dlg.position[0], "position_fif_x", -1);
+	stash_group_add_integer(group, &fif_dlg.position[1], "position_fif_y", -1);
 
 	group = stash_group_new("search");
 	fif_prefs = group;
@@ -397,7 +414,7 @@ static void create_find_dialog(void)
 	gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
 
 	entry = gtk_combo_box_entry_new_text();
-	ui_entry_add_clear_icon(gtk_bin_get_child(GTK_BIN(entry)));
+	ui_entry_add_clear_icon(GTK_ENTRY(gtk_bin_get_child(GTK_BIN(entry))));
 	gtk_label_set_mnemonic_widget(GTK_LABEL(label), entry);
 	gtk_entry_set_max_length(GTK_ENTRY(gtk_bin_get_child(GTK_BIN(entry))), 248);
 	gtk_entry_set_width_chars(GTK_ENTRY(gtk_bin_get_child(GTK_BIN(entry))), 50);
@@ -464,6 +481,13 @@ static void create_find_dialog(void)
 }
 
 
+static void set_dialog_position(GtkWidget *dialog, gint *position)
+{
+	if (position[0] >= 0)
+		gtk_window_move(GTK_WINDOW(dialog), position[0], position[1]);
+}
+
+
 void search_show_find_dialog(void)
 {
 	GeanyDocument *doc = document_get_current();
@@ -479,6 +503,7 @@ void search_show_find_dialog(void)
 		if (sel)
 			gtk_entry_set_text(GTK_ENTRY(find_dlg.entry), sel);
 
+		set_dialog_position(find_dlg.dialog, find_dlg.position);
 		gtk_widget_show_all(find_dlg.dialog);
 	}
 	else
@@ -487,12 +512,14 @@ void search_show_find_dialog(void)
 		if (! GTK_WIDGET_VISIBLE(find_dlg.dialog) && sel)
 			gtk_entry_set_text(GTK_ENTRY(find_dlg.entry), sel);
 		gtk_widget_grab_focus(find_dlg.entry);
+		set_dialog_position(find_dlg.dialog, find_dlg.position);
 		gtk_widget_show(find_dlg.dialog);
 		if (sel != NULL) /* when we have a selection, reset the entry widget's background colour */
 			ui_set_search_entry_background(find_dlg.entry, TRUE);
 		/* bring the dialog back in the foreground in case it is already open but the focus is away */
 		gtk_window_present(GTK_WINDOW(find_dlg.dialog));
 	}
+
 	g_free(sel);
 }
 
@@ -539,7 +566,7 @@ static void create_replace_dialog(void)
 	gtk_misc_set_alignment(GTK_MISC(label_replace), 0, 0.5);
 
 	entry_find = gtk_combo_box_entry_new_text();
-	ui_entry_add_clear_icon(gtk_bin_get_child(GTK_BIN(entry_find)));
+	ui_entry_add_clear_icon(GTK_ENTRY(gtk_bin_get_child(GTK_BIN(entry_find))));
 	gtk_label_set_mnemonic_widget(GTK_LABEL(label_find), entry_find);
 	gtk_entry_set_max_length(GTK_ENTRY(gtk_bin_get_child(GTK_BIN(entry_find))), 248);
 	gtk_entry_set_width_chars(GTK_ENTRY(gtk_bin_get_child(GTK_BIN(entry_find))), 50);
@@ -548,7 +575,7 @@ static void create_replace_dialog(void)
 	replace_dlg.find_entry = GTK_BIN(entry_find)->child;
 
 	entry_replace = gtk_combo_box_entry_new_text();
-	ui_entry_add_clear_icon(gtk_bin_get_child(GTK_BIN(entry_replace)));
+	ui_entry_add_clear_icon(GTK_ENTRY(gtk_bin_get_child(GTK_BIN(entry_replace))));
 	gtk_label_set_mnemonic_widget(GTK_LABEL(label_replace), entry_replace);
 	gtk_entry_set_max_length(GTK_ENTRY(gtk_bin_get_child(GTK_BIN(entry_replace))), 248);
 	gtk_entry_set_width_chars(GTK_ENTRY(gtk_bin_get_child(GTK_BIN(entry_replace))), 50);
@@ -644,6 +671,7 @@ void search_show_replace_dialog(void)
 		if (sel)
 			gtk_entry_set_text(GTK_ENTRY(replace_dlg.find_entry), sel);
 
+		set_dialog_position(replace_dlg.dialog, replace_dlg.position);
 		gtk_widget_show_all(replace_dlg.dialog);
 	}
 	else
@@ -654,6 +682,7 @@ void search_show_replace_dialog(void)
 		if (sel != NULL) /* when we have a selection, reset the entry widget's background colour */
 			ui_set_search_entry_background(replace_dlg.find_entry, TRUE);
 		gtk_widget_grab_focus(replace_dlg.find_entry);
+		set_dialog_position(replace_dlg.dialog, replace_dlg.position);
 		gtk_widget_show(replace_dlg.dialog);
 		/* bring the dialog back in the foreground in case it is already open but the focus is away */
 		gtk_window_present(GTK_WINDOW(replace_dlg.dialog));
@@ -698,7 +727,7 @@ static void create_fif_dialog(void)
 
 	dir_combo = gtk_combo_box_entry_new_text();
 	entry = gtk_bin_get_child(GTK_BIN(dir_combo));
-	ui_entry_add_clear_icon(entry);
+	ui_entry_add_clear_icon(GTK_ENTRY(entry));
 	gtk_label_set_mnemonic_widget(GTK_LABEL(label1), entry);
 	gtk_entry_set_max_length(GTK_ENTRY(entry), 248);
 	gtk_entry_set_width_chars(GTK_ENTRY(entry), 50);
@@ -713,7 +742,7 @@ static void create_fif_dialog(void)
 
 	combo = gtk_combo_box_entry_new_text();
 	entry = gtk_bin_get_child(GTK_BIN(combo));
-	ui_entry_add_clear_icon(entry);
+	ui_entry_add_clear_icon(GTK_ENTRY(entry));
 	gtk_label_set_mnemonic_widget(GTK_LABEL(label), entry);
 	gtk_entry_set_max_length(GTK_ENTRY(entry), 248);
 	gtk_entry_set_width_chars(GTK_ENTRY(entry), 50);
@@ -817,7 +846,7 @@ static void create_fif_dialog(void)
 	gtk_button_set_focus_on_click(GTK_BUTTON(check_extra), FALSE);
 
 	entry_extra = gtk_entry_new();
-	ui_entry_add_clear_icon(entry_extra);
+	ui_entry_add_clear_icon(GTK_ENTRY(entry_extra));
 	gtk_widget_set_sensitive(entry_extra, FALSE);
 	ui_widget_set_tooltip_text(entry_extra, _("Other options to pass to Grep"));
 	ui_hookup_widget(fif_dlg.dialog, entry_extra, "entry_extra");
@@ -846,7 +875,6 @@ void search_show_find_in_files_dialog(const gchar *dir)
 {
 	GtkWidget *entry; /* for child GtkEntry of a GtkComboBoxEntry */
 	GeanyDocument *doc = document_get_current();
-	GeanyEditor *editor = doc ? doc->editor : NULL;
 	gchar *sel = NULL;
 	gchar *cur_dir = NULL;
 	GeanyEncodingIndex enc_idx = GEANY_ENCODING_UTF_8;
@@ -855,13 +883,14 @@ void search_show_find_in_files_dialog(const gchar *dir)
 	{
 		create_fif_dialog();
 		gtk_widget_show_all(fif_dlg.dialog);
-		sel = editor_get_default_selection(editor, search_prefs.use_current_word, NULL);
+		if (doc)
+			sel = editor_get_default_selection(doc->editor, search_prefs.use_current_word, NULL);
 	}
 	stash_group_display(fif_prefs, fif_dlg.dialog);
 
 	/* only set selection if the dialog is not already visible, or has just been created */
-	if (! sel && ! GTK_WIDGET_VISIBLE(fif_dlg.dialog))
-		sel = editor_get_default_selection(editor, search_prefs.use_current_word, NULL);
+	if (doc && ! sel && ! GTK_WIDGET_VISIBLE(fif_dlg.dialog))
+		sel = editor_get_default_selection(doc->editor, search_prefs.use_current_word, NULL);
 
 	entry = GTK_BIN(fif_dlg.search_combo)->child;
 	if (sel)
@@ -912,6 +941,9 @@ void search_show_find_in_files_dialog(const gchar *dir)
 	else
 		gtk_widget_grab_focus(fif_dlg.search_combo);
 
+	/* set dialog window position */
+	set_dialog_position(fif_dlg.dialog, fif_dlg.position);
+
 	gtk_widget_show(fif_dlg.dialog);
 	/* bring the dialog back in the foreground in case it is already open but the focus is away */
 	gtk_window_present(GTK_WINDOW(fif_dlg.dialog));
@@ -932,9 +964,12 @@ on_find_replace_checkbutton_toggled(GtkToggleButton *togglebutton, gpointer user
 		GtkWidget *check_wordstart = ui_lookup_widget(dialog, "check_wordstart");
 		GtkToggleButton *check_case = GTK_TOGGLE_BUTTON(
 			ui_lookup_widget(dialog, "check_case"));
+		GtkWidget *check_escape = ui_lookup_widget(dialog, "check_escape");
 		static gboolean case_state = FALSE; /* state before regex enabled */
 
 		/* hide options that don't apply to regex searches */
+		gtk_widget_set_sensitive(check_escape, ! regex_set);
+
 		if (dialog == find_dlg.dialog)
 			gtk_widget_set_sensitive(ui_lookup_widget(dialog, "btn_previous"), ! regex_set);
 		else
@@ -959,7 +994,8 @@ on_find_replace_checkbutton_toggled(GtkToggleButton *togglebutton, gpointer user
 }
 
 
-/* @return Number of matches marked. */
+/* Clears markers if text is null/empty.
+ * @return Number of matches marked. */
 gint search_mark_all(GeanyDocument *doc, const gchar *search_text, gint flags)
 {
 	gint pos, count = 0;
@@ -1024,6 +1060,9 @@ static gint get_search_flags(GtkWidget *dialog)
 static void
 on_find_dialog_response(GtkDialog *dialog, gint response, gpointer user_data)
 {
+	gtk_window_get_position(GTK_WINDOW(find_dlg.dialog),
+		&find_dlg.position[0], &find_dlg.position[1]);
+
 	if (response == GTK_RESPONSE_CANCEL || response == GTK_RESPONSE_DELETE_EVENT)
 		gtk_widget_hide(find_dlg.dialog);
 	else
@@ -1043,18 +1082,22 @@ on_find_dialog_response(GtkDialog *dialog, gint response, gpointer user_data)
 
 		g_free(search_data.text);
 		search_data.text = g_strdup(gtk_entry_get_text(GTK_ENTRY(gtk_bin_get_child(GTK_BIN(user_data)))));
-		if (strlen(search_data.text) == 0 ||
-			(search_replace_escape && ! utils_str_replace_escape(search_data.text)))
+		search_data.flags = get_search_flags(find_dlg.dialog);
+
+		if (strlen(search_data.text) == 0)
 		{
+			fail:
 			utils_beep();
 			gtk_widget_grab_focus(find_dlg.entry);
 			return;
 		}
-
+		if (search_replace_escape || search_data.flags & SCFIND_REGEXP)
+		{
+			if (! utils_str_replace_escape(search_data.text, search_data.flags & SCFIND_REGEXP))
+				goto fail;
+		}
 		ui_combo_box_add_to_history(GTK_COMBO_BOX(user_data), search_data.text);
 
-		search_data.flags = get_search_flags(find_dlg.dialog);
-
 		switch (response)
 		{
 			case GEANY_RESPONSE_FIND:
@@ -1103,6 +1146,44 @@ on_replace_entry_activate(GtkEntry *entry, gpointer user_data)
 }
 
 
+static void replace_in_session(GeanyDocument *doc,
+		gint search_flags_re, gboolean search_replace_escape_re,
+		const gchar *find, const gchar *replace)
+{
+	guint n, page_count, rep_count = 0, file_count = 0;
+
+	/* replace in all documents following notebook tab order */
+	page_count = gtk_notebook_get_n_pages(GTK_NOTEBOOK(main_widgets.notebook));
+	for (n = 0; n < page_count; n++)
+	{
+		GeanyDocument *tmp_doc = document_get_from_page(n);
+		gint reps = 0;
+
+		reps = document_replace_all(tmp_doc, find, replace, search_flags_re,
+			search_replace_escape_re);
+		rep_count += reps;
+		if (reps)
+			file_count++;
+	}
+	if (file_count == 0)
+	{
+		utils_beep();
+		ui_set_statusbar(FALSE, _("No matches found for \"%s\"."), find);
+		return;
+	}
+	/* if only one file was changed, don't override that document's status message
+	 * so we don't have to translate 4 messages for ngettext */
+	if (file_count > 1)
+		ui_set_statusbar(FALSE, _("Replaced %u matches in %u documents."),
+			rep_count, file_count);
+
+	/* show which docs had replacements: */
+	gtk_notebook_set_current_page(GTK_NOTEBOOK(msgwindow.notebook), MSG_STATUS);
+
+	ui_save_buttons_toggle(doc->changed);	/* update save all */
+}
+
+
 static void
 on_replace_dialog_response(GtkDialog *dialog, gint response, gpointer user_data)
 {
@@ -1112,6 +1193,9 @@ on_replace_dialog_response(GtkDialog *dialog, gint response, gpointer user_data)
 	gboolean close_window;
 	gchar *find, *replace;
 
+	gtk_window_get_position(GTK_WINDOW(replace_dlg.dialog),
+		&replace_dlg.position[0], &replace_dlg.position[1]);
+
 	if (response == GTK_RESPONSE_CANCEL || response == GTK_RESPONSE_DELETE_EVENT)
 	{
 		gtk_widget_hide(replace_dlg.dialog);
@@ -1132,24 +1216,29 @@ on_replace_dialog_response(GtkDialog *dialog, gint response, gpointer user_data)
 	if ((response != GEANY_RESPONSE_FIND) && (search_flags_re & SCFIND_MATCHCASE)
 		&& (strcmp(find, replace) == 0))
 	{
+		fail:
 		utils_beep();
 		gtk_widget_grab_focus(replace_dlg.find_entry);
 		return;
 	}
+	if (search_flags_re & SCFIND_REGEXP)
+	{
+		if (! utils_str_replace_escape(find, TRUE) ||
+			! utils_str_replace_escape(replace, TRUE))
+			goto fail;
+	}
+	else if (search_replace_escape_re)
+	{
+		if (! utils_str_replace_escape(find, FALSE) ||
+			! utils_str_replace_escape(replace, FALSE))
+			goto fail;
+	}
 
 	ui_combo_box_add_to_history(GTK_COMBO_BOX(
 		gtk_widget_get_parent(replace_dlg.find_entry)), find);
 	ui_combo_box_add_to_history(GTK_COMBO_BOX(
 		gtk_widget_get_parent(replace_dlg.replace_entry)), replace);
 
-	if (search_replace_escape_re &&
-		(! utils_str_replace_escape(find) || ! utils_str_replace_escape(replace)))
-	{
-		utils_beep();
-		gtk_widget_grab_focus(replace_dlg.find_entry);
-		return;
-	}
-
 	switch (response)
 	{
 		case GEANY_RESPONSE_REPLACE_AND_FIND:
@@ -1162,11 +1251,10 @@ on_replace_dialog_response(GtkDialog *dialog, gint response, gpointer user_data)
 			break;
 		}
 		case GEANY_RESPONSE_REPLACE:
-		{
 			document_replace_text(doc, find, replace, search_flags_re,
 				search_backwards_re);
 			break;
-		}
+
 		case GEANY_RESPONSE_FIND:
 		{
 			gint result = document_find_text(doc, find, search_flags_re,
@@ -1175,44 +1263,18 @@ on_replace_dialog_response(GtkDialog *dialog, gint response, gpointer user_data)
 			break;
 		}
 		case GEANY_RESPONSE_REPLACE_IN_FILE:
-		{
 			if (! document_replace_all(doc, find, replace, search_flags_re,
 				search_replace_escape_re))
-			{
 				utils_beep();
-			}
 			break;
-		}
+
 		case GEANY_RESPONSE_REPLACE_IN_SESSION:
-		{
-			guint n, page_count, count = 0;
-
-			/* replace in all documents following notebook tab order */
-			page_count = gtk_notebook_get_n_pages(GTK_NOTEBOOK(main_widgets.notebook));
-			for (n = 0; n < page_count; n++)
-			{
-				GeanyDocument *tmp_doc = document_get_from_page(n);
-
-				if (document_replace_all(tmp_doc, find, replace, search_flags_re,
-					search_replace_escape_re)) count++;
-			}
-			if (count == 0)
-				utils_beep();
-
-			ui_set_statusbar(FALSE,
-				ngettext("Replaced text in %u file.",
-						 "Replaced text in %u files.", count), count);
-			/* show which docs had replacements: */
-			gtk_notebook_set_current_page(GTK_NOTEBOOK(msgwindow.notebook), MSG_STATUS);
-
-			ui_save_buttons_toggle(doc->changed);	/* update save all */
+			replace_in_session(doc, search_flags_re, search_replace_escape_re, find, replace);
 			break;
-		}
+
 		case GEANY_RESPONSE_REPLACE_IN_SEL:
-		{
 			document_replace_sel(doc, find, replace, search_flags_re, search_replace_escape_re);
 			break;
-		}
 	}
 	switch (response)
 	{
@@ -1287,6 +1349,8 @@ static void
 on_find_in_files_dialog_response(GtkDialog *dialog, gint response,
 		G_GNUC_UNUSED gpointer user_data)
 {
+	gtk_window_get_position(GTK_WINDOW(fif_dlg.dialog), &fif_dlg.position[0], &fif_dlg.position[1]);
+
 	stash_group_update(fif_prefs, fif_dlg.dialog);
 
 	if (response == GTK_RESPONSE_ACCEPT)
@@ -1378,11 +1442,12 @@ search_find_in_files(const gchar *utf8_search_text, const gchar *dir, const gcha
 
 	i++;	/* correct for tool_prefs.grep_cmd */
 	argv_prefix[i++] = g_strdup("--");
-	argv_prefix[i++] = g_strdup(search_text);
+	argv_prefix[i++] = search_text;
 
 	/* finally add the arguments(files to be searched) */
 	if (strstr(argv_prefix[1], "r"))	/* recursive option set */
 	{
+		/* Use '.' so we get relative paths in the output */
 		argv_prefix[i++] = g_strdup(".");
 		argv_prefix[i++] = NULL;
 		argv = argv_prefix;
@@ -1396,7 +1461,6 @@ search_find_in_files(const gchar *utf8_search_text, const gchar *dir, const gcha
 
 	if (argv == NULL)	/* no files */
 	{
-		g_strfreev(argv);
 		return FALSE;
 	}
 
@@ -1422,9 +1486,9 @@ search_find_in_files(const gchar *utf8_search_text, const gchar *dir, const gcha
 		msgwindow.find_in_files_dir = g_strdup(dir);
 		/* we can pass 'enc' without strdup'ing it here because it's a global const string and
 		 * always exits longer than the lifetime of this function */
-		utils_set_up_io_channel(stdout_fd, G_IO_IN|G_IO_PRI|G_IO_ERR|G_IO_HUP|G_IO_NVAL,
+		utils_set_up_io_channel(stdout_fd, G_IO_IN | G_IO_PRI | G_IO_ERR | G_IO_HUP | G_IO_NVAL,
 			TRUE, search_read_io, (gpointer) enc);
-		utils_set_up_io_channel(stderr_fd, G_IO_IN|G_IO_PRI|G_IO_ERR|G_IO_HUP|G_IO_NVAL,
+		utils_set_up_io_channel(stderr_fd, G_IO_IN | G_IO_PRI | G_IO_ERR | G_IO_HUP | G_IO_NVAL,
 			TRUE, search_read_io_stderr, (gpointer) enc);
 		g_child_watch_add(child_pid, search_close_pid, NULL);
 
@@ -1480,12 +1544,11 @@ static gchar **search_get_argv(const gchar **argv_prefix, const gchar *dir)
 }
 
 
-static gboolean search_read_io(GIOChannel *source, GIOCondition condition, gpointer data)
+static gboolean read_fif_io(GIOChannel *source, GIOCondition condition, gchar *enc, gint msg_color)
 {
 	if (condition & (G_IO_IN | G_IO_PRI))
 	{
 		gchar *msg, *utf8_msg;
-		gchar *enc = data;
 
 		while (g_io_channel_read_line(source, &msg, NULL, NULL, NULL) && msg)
 		{
@@ -1505,7 +1568,7 @@ static gboolean search_read_io(GIOChannel *source, GIOCondition condition, gpoin
 			else
 				utf8_msg = msg;
 
-			msgwin_msg_add_string(COLOR_BLACK, -1, NULL, utf8_msg);
+			msgwin_msg_add_string(msg_color, -1, NULL, utf8_msg);
 
 			if (utf8_msg != msg)
 				g_free(utf8_msg);
@@ -1519,42 +1582,15 @@ static gboolean search_read_io(GIOChannel *source, GIOCondition condition, gpoin
 }
 
 
+static gboolean search_read_io(GIOChannel *source, GIOCondition condition, gpointer data)
+{
+	return read_fif_io(source, condition, data, COLOR_BLACK);
+}
+
+
 static gboolean search_read_io_stderr(GIOChannel *source, GIOCondition condition, gpointer data)
 {
-	if (condition & (G_IO_IN | G_IO_PRI))
-	{
-		gchar *msg, *utf8_msg;
-		gchar *enc = data;
-
-		while (g_io_channel_read_line(source, &msg, NULL, NULL, NULL) && msg)
-		{
-			utf8_msg = NULL;
-
-			g_strstrip(msg);
-			/* enc is NULL when encoding is set to UTF-8, so we can skip any conversion */
-			if (enc != NULL)
-			{
-				if (! g_utf8_validate(msg, -1, NULL))
-				{
-					utf8_msg = g_convert(msg, -1, "UTF-8", enc, NULL, NULL, NULL);
-				}
-				if (utf8_msg == NULL)
-					utf8_msg = msg;
-			}
-			else
-				utf8_msg = msg;
-
-			g_warning("Find in Files: %s", utf8_msg);
-
-			if (utf8_msg != msg)
-				g_free(utf8_msg);
-			g_free(msg);
-		}
-	}
-	if (condition & (G_IO_ERR | G_IO_HUP | G_IO_NVAL))
-		return FALSE;
-
-	return TRUE;
+	return read_fif_io(source, condition, data, COLOR_DARK_RED);
 }
 
 
@@ -1562,8 +1598,7 @@ static void search_close_pid(GPid child_pid, gint status, gpointer user_data)
 {
 	/* TODO: port this also to Windows API */
 #ifdef G_OS_UNIX
-	const gchar *msg = _("Search failed (see Help->Debug Messages for details).");
-	gint color = COLOR_DARK_RED;
+	const gchar *msg = _("Search failed.");
 	gint exit_status = 1;
 
 	if (WIFEXITED(status))
@@ -1592,9 +1627,8 @@ static void search_close_pid(GPid child_pid, gint status, gpointer user_data)
 		}
 		case 1:
 			msg = _("No matches found.");
-			color = COLOR_BLUE;
 		default:
-			msgwin_msg_add_string(color, -1, NULL, msg);
+			msgwin_msg_add_string(COLOR_BLUE, -1, NULL, msg);
 			ui_set_statusbar(FALSE, "%s", msg);
 			break;
 	}
@@ -1606,6 +1640,159 @@ static void search_close_pid(GPid child_pid, gint status, gpointer user_data)
 }
 
 
+static gboolean compile_regex(regex_t *regex, const gchar *str, gint sflags)
+{
+	gint err;
+	gint rflags = REG_EXTENDED | REG_NEWLINE;
+
+	if (~sflags & SCFIND_MATCHCASE)
+		rflags |= REG_ICASE;
+	if (sflags & (SCFIND_WHOLEWORD | SCFIND_WORDSTART))
+	{
+		geany_debug("%s: Unsupported regex flags found!", G_STRFUNC);
+	}
+
+	err = regcomp(regex, str, rflags);
+	if (err != 0)
+	{
+		gchar buf[256];
+
+		regerror(err, regex, buf, sizeof buf);
+		ui_set_statusbar(FALSE, _("Bad regex: %s"), buf);
+		return FALSE;
+	}
+	return TRUE;
+}
+
+
+/* groups that don't exist are handled OK as len = end - start = (-1) - (-1) = 0 */
+static gchar *get_regex_match_string(const gchar *text, regmatch_t *pmatch, gint match_idx)
+{
+	return g_strndup(&text[pmatch[match_idx].rm_so],
+		pmatch[match_idx].rm_eo - pmatch[match_idx].rm_so);
+}
+
+
+static regmatch_t regex_matches[10];
+/* All matching text from regex_matches[0].rm_so to regex_matches[0].rm_eo */
+static gchar *regex_match_text = NULL;
+
+static gint find_regex(ScintillaObject *sci, guint pos, regex_t *regex)
+{
+	const gchar *text;
+	gint flags = 0;
+
+	g_return_val_if_fail(pos <= (guint)sci_get_length(sci), FALSE);
+
+	if (sci_get_col_from_position(sci, pos) != 0)
+		flags = REG_NOTBOL;
+	/* Warning: any SCI calls will invalidate 'text' after calling SCI_GETCHARACTERPOINTER */
+	text = (void*)scintilla_send_message(sci, SCI_GETCHARACTERPOINTER, 0, 0);
+	text += pos;
+
+	if (regexec(regex, text, G_N_ELEMENTS(regex_matches), regex_matches, flags) == 0)
+	{
+		setptr(regex_match_text, get_regex_match_string(text, regex_matches, 0));
+		return regex_matches[0].rm_so + pos;
+	}
+	setptr(regex_match_text, NULL);
+	return -1;
+}
+
+
+gint search_find_next(ScintillaObject *sci, const gchar *str, gint flags)
+{
+	regex_t regex;
+	gint ret = -1;
+	gint pos;
+
+	if (~flags & SCFIND_REGEXP)
+		return sci_search_next(sci, flags, str);
+
+	if (!compile_regex(&regex, str, flags))
+		return -1;
+
+	pos = sci_get_current_position(sci);
+	ret = find_regex(sci, pos, &regex);
+	if (ret >= 0)
+		sci_set_selection(sci, ret, regex_matches[0].rm_eo + pos);
+
+	regfree(&regex);
+	return ret;
+}
+
+
+gint search_replace_target(ScintillaObject *sci, const gchar *replace_text,
+	gboolean regex)
+{
+	GString *str;
+	gint ret = 0;
+	gint i = 0;
+
+	if (!regex)
+		return sci_replace_target(sci, replace_text, FALSE);
+
+	str = g_string_new(replace_text);
+	while (str->str[i])
+	{
+		gchar *ptr = &str->str[i];
+		gchar *grp;
+		gchar c;
+
+		if (ptr[0] != '\\')
+		{
+			i++;
+			continue;
+		}
+		c = ptr[1];
+		/* backslash or unnecessary escape */
+		if (c == '\\' || !isdigit(c))
+		{
+			g_string_erase(str, i, 1);
+			i++;
+			continue;
+		}
+		/* digit escape */
+		g_string_erase(str, i, 2);
+		/* fix match offsets by subtracting index of whole match start from the string */
+		grp = get_regex_match_string(regex_match_text - regex_matches[0].rm_so,
+			regex_matches, c - '0');
+		g_string_insert(str, i, grp);
+		i += strlen(grp);
+		g_free(grp);
+	}
+	ret = sci_replace_target(sci, str->str, FALSE);
+	g_string_free(str, TRUE);
+	return ret;
+}
+
+
+gint search_find_text(ScintillaObject *sci, gint flags, struct Sci_TextToFind *ttf)
+{
+	regex_t regex;
+	gint pos;
+	gint ret;
+
+	if (~flags & SCFIND_REGEXP)
+		return sci_find_text(sci, flags, ttf);
+
+	if (!compile_regex(&regex, ttf->lpstrText, flags))
+		return -1;
+
+	pos = ttf->chrg.cpMin;
+	ret = find_regex(sci, pos, &regex);
+	regfree(&regex);
+
+	if (ret >= 0 && ret < ttf->chrg.cpMax)
+	{
+		ttf->chrgText.cpMin = regex_matches[0].rm_so + pos;
+		ttf->chrgText.cpMax = regex_matches[0].rm_eo + pos;
+		return ret;
+	}
+	return -1;
+}
+
+
 static gint find_document_usage(GeanyDocument *doc, const gchar *search_text, gint flags)
 {
 	gchar *buffer, *short_file_name;
@@ -1624,7 +1811,7 @@ static gint find_document_usage(GeanyDocument *doc, const gchar *search_text, gi
 	{
 		gint pos, line, start, find_len;
 
-		pos = sci_find_text(doc->editor->sci, flags, &ttf);
+		pos = search_find_text(doc->editor->sci, flags, &ttf);
 		if (pos == -1)
 			break;	/* no more matches */
 		find_len = ttf.chrgText.cpMax - ttf.chrgText.cpMin;
@@ -1637,7 +1824,7 @@ static gint find_document_usage(GeanyDocument *doc, const gchar *search_text, gi
 		{
 			buffer = sci_get_line(doc->editor->sci, line);
 			msgwin_msg_add(COLOR_BLACK, line + 1, doc,
-				"%s:%d : %s", short_file_name, line + 1, g_strstrip(buffer));
+				"%s:%d: %s", short_file_name, line + 1, g_strstrip(buffer));
 			g_free(buffer);
 			prev_line = line;
 		}
@@ -1700,3 +1887,67 @@ void search_find_usage(const gchar *search_text, gint flags, gboolean in_session
 }
 
 
+/* ttf is updated to include the last match position (ttf->chrg.cpMin) and
+ * the new search range end (ttf->chrg.cpMax).
+ * Note: Normally you would call sci_start/end_undo_action() around this call. */
+/* Warning: Scintilla recommends caching replacements to do all at once to avoid
+ * performance issues with SCI_GETCHARACTERPOINTER. */
+guint search_replace_range(ScintillaObject *sci, struct Sci_TextToFind *ttf,
+		gint flags, const gchar *replace_text)
+{
+	gint count = 0;
+	const gchar *find_text = ttf->lpstrText;
+	gint start = ttf->chrg.cpMin;
+	gint end = ttf->chrg.cpMax;
+
+	g_return_val_if_fail(sci != NULL && find_text != NULL && replace_text != NULL, 0);
+	if (! *find_text)
+		return 0;
+
+	while (TRUE)
+	{
+		gint search_pos;
+		gint find_len = 0, replace_len = 0;
+
+		search_pos = search_find_text(sci, flags, ttf);
+		find_len = ttf->chrgText.cpMax - ttf->chrgText.cpMin;
+		if (search_pos == -1)
+			break;	/* no more matches */
+		if (find_len == 0 && ! NZV(replace_text))
+			break;	/* nothing to do */
+
+		if (search_pos + find_len > end)
+			break;	/* found text is partly out of range */
+		else
+		{
+			gint movepastEOL = 0;
+
+			sci_set_target_start(sci, search_pos);
+			sci_set_target_end(sci, search_pos + find_len);
+
+			if (find_len <= 0)
+			{
+				gchar chNext = sci_get_char_at(sci, sci_get_target_end(sci));
+
+				if (chNext == '\r' || chNext == '\n')
+					movepastEOL = 1;
+			}
+			replace_len = search_replace_target(sci, replace_text,
+				flags & SCFIND_REGEXP);
+			count++;
+			if (search_pos == end)
+				break;	/* Prevent hang when replacing regex $ */
+
+			/* make the next search start after the replaced text */
+			start = search_pos + replace_len + movepastEOL;
+			if (find_len == 0)
+				start = sci_get_position_after(sci, start);	/* prevent '[ ]*' regex rematching part of replaced text */
+			ttf->chrg.cpMin = start;
+			end += replace_len - find_len;	/* update end of range now text has changed */
+			ttf->chrg.cpMax = end;
+		}
+	}
+	return count;
+}
+
+
