@@ -1,8 +1,8 @@
 /*
  *      search.h - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2006-2008 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2006-2008 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2006-2010 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2006-2010 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -18,9 +18,14 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: search.h 2724 2008-06-25 11:56:58Z ntrel $
+ * $Id: search.h 4873 2010-05-05 12:50:59Z ntrel $
  */
 
+/**
+ * @file search.h
+ * Search (prefs).
+ **/
+
 
 #ifndef GEANY_SEARCH_H
 #define GEANY_SEARCH_H 1
@@ -40,11 +45,11 @@ GeanySearchData;
 extern GeanySearchData search_data;
 
 
+/** Search preferences */
 typedef struct GeanySearchPrefs
 {
-	gchar		*fif_extra_options;
 	gboolean	suppress_dialogs;
-	gboolean	use_current_word;
+	gboolean	use_current_word;		/**< Use current word for default search text */
 	gboolean	use_current_file_dir;	/* find in files directory to use on showing dialog */
 }
 GeanySearchPrefs;
@@ -62,8 +67,24 @@ void search_show_replace_dialog(void);
 
 void search_show_find_in_files_dialog(const gchar *dir);
 
+
+struct _ScintillaObject;
+struct Sci_TextToFind;
+
+gint search_find_next(struct _ScintillaObject *sci, const gchar *str, gint flags);
+
+gint search_find_text(struct _ScintillaObject *sci, gint flags, struct Sci_TextToFind *ttf);
+
 void search_find_usage(const gchar *search_text, gint flags, gboolean in_session);
 
 void search_find_selection(GeanyDocument *doc, gboolean search_backwards);
 
+gint search_mark_all(GeanyDocument *doc, const gchar *search_text, gint flags);
+
+gint search_replace_target(struct _ScintillaObject *sci, const gchar *replace_text,
+	gboolean regex);
+
+guint search_replace_range(struct _ScintillaObject *sci, struct Sci_TextToFind *ttf,
+		gint flags, const gchar *replace_text);
+
 #endif
