@@ -1,8 +1,8 @@
 /*
  *      socket.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2006-2009 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2006-2009 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2006-2010 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2006-2010 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *      Copyright 2006 Hiroyuki Yamamoto (author of Sylpheed)
  *
  *      This program is free software; you can redistribute it and/or modify
@@ -19,7 +19,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: socket.c 3708 2009-04-15 22:47:33Z eht16 $
+ * $Id: socket.c 4862 2010-04-25 17:43:39Z eht16 $
  */
 
 /*
@@ -42,7 +42,7 @@
  * The command window is only available on Windows and takes no additional data, instead it
  * writes back a Windows handle (HWND) for the main window to set it to the foreground (focus).
  *
- * At the moment the commands window, open, line and column are available.
+ * At the moment the commands window, doclist, open, line and column are available.
  *
  * About the socket files on Unix-like systems:
  * Geany creates a socket in /tmp (or any other directory returned by g_get_tmp_dir()) and
@@ -59,7 +59,6 @@
 #ifdef HAVE_SOCKET
 
 #ifndef G_OS_WIN32
-# include <string.h>
 # include <sys/time.h>
 # include <sys/types.h>
 # include <sys/socket.h>
@@ -72,6 +71,7 @@
 # include <winsock2.h>
 # include <ws2tcpip.h>
 #endif
+#include <string.h>
 #include <stdlib.h>
 #include <unistd.h>
 #include <fcntl.h>
@@ -86,6 +86,7 @@
 #include "support.h"
 #include "ui_utils.h"
 #include "utils.h"
+#include "dialogs.h"
 #include "encodings.h"
 
 
@@ -97,7 +98,7 @@
 #define SOCKET_IS_VALID(s)	(G_LIKELY((s) >= 0))
 #define INVALID_SOCKET		(-1)
 #endif
-
+#define BUFFER_LENGTH 4096
 
 struct socket_info_struct socket_info;
 
@@ -121,7 +122,7 @@ static gint socket_fd_close			(gint sock);
 
 
 
-void send_open_command(gint sock, gint argc, gchar **argv)
+static void send_open_command(gint sock, gint argc, gchar **argv)
 {
 	gint i;
 	gchar *filename;
@@ -191,6 +192,49 @@ static void remove_socket_link_full(void)
 #endif
 
 
+static void socket_get_document_list(gint sock)
+{
+	gchar doc_list[BUFFER_LENGTH];
+	gint doc_list_len;
+
+	if (sock < 0)
+		return;
+
+	socket_fd_write_all(sock, "doclist\n", 8);
+
+	doc_list_len = socket_fd_read(sock, doc_list, sizeof(doc_list));
+	if (doc_list_len >= BUFFER_LENGTH)
+		doc_list_len = BUFFER_LENGTH -1;
+	doc_list[doc_list_len] = '\0';
+	/* if we received ETX (end-of-text), there were no open files, so print only otherwise */
+	if (! utils_str_equal(doc_list, "\3"))
+		printf("%s", doc_list);
+}
+
+
+#ifndef G_OS_WIN32
+static void check_socket_permissions(void)
+{
+	struct stat socket_stat;
+
+	if (g_lstat(socket_info.file_name, &socket_stat) == 0)
+	{	/* If the user id of the process is not the same as the owner of the socket
+		 * file, then ignore this socket and start a new session. */
+		if (socket_stat.st_uid != getuid())
+		{
+			const gchar *msg = _(
+	/* TODO maybe this message needs a rewording */
+	"Geany tried to access the Unix Domain socket of another instance running as another user.\n"
+	"This is a fatal error and Geany will now quit.");
+			g_warning("%s", msg);
+			dialogs_show_msgbox(GTK_MESSAGE_ERROR, "%s", msg);
+			exit(1);
+		}
+	}
+}
+#endif
+
+
 /* (Unix domain) socket support to replace the old FIFO code
  * (taken from Sylpheed, thanks)
  * Returns the created socket, -1 if an error occurred or -2 if another socket exists and files
@@ -246,6 +290,9 @@ gint socket_init(gint argc, gchar **argv)
 	g_free(display_name);
 	g_free(hostname);
 
+	/* check whether the real user id is the same as this of the socket file */
+	check_socket_permissions();
+
 	sock = socket_fd_connect_unix(socket_info.file_name);
 	if (sock < 0)
 	{
@@ -268,6 +315,11 @@ gint socket_init(gint argc, gchar **argv)
 		send_open_command(sock, argc, argv);
 	}
 
+	if (cl_options.list_documents)
+	{
+		socket_get_document_list(sock);
+	}
+
 	socket_fd_close(sock);
 	return -2;
 }
@@ -504,16 +556,37 @@ static void handle_input_filename(const gchar *buf)
 
 	locale_filename = utils_get_locale_from_utf8(utf8_filename);
 	if (locale_filename)
-		main_handle_filename(locale_filename);
+	{
+		if (g_str_has_suffix(locale_filename, ".geany"))
+			main_load_project_from_command_line(locale_filename, TRUE);
+		else
+			main_handle_filename(locale_filename);
+	}
 	g_free(utf8_filename);
 	g_free(locale_filename);
 }
 
 
+static gchar *build_document_list(void)
+{
+	GString *doc_list = g_string_new(NULL);
+	guint i;
+	const gchar *filename;
+
+	foreach_document(i)
+	{
+		filename = DOC_FILENAME(documents[i]);
+		g_string_append(doc_list, filename);
+		g_string_append_c(doc_list, '\n');
+	}
+	return g_string_free(doc_list, FALSE);
+}
+
+
 gboolean socket_lock_input_cb(GIOChannel *source, GIOCondition condition, gpointer data)
 {
 	gint fd, sock;
-	gchar buf[4096];
+	gchar buf[BUFFER_LENGTH];
 	struct sockaddr_in caddr;
 	guint caddr_len = sizeof(caddr);
 	GtkWidget *window = data;
@@ -533,6 +606,17 @@ gboolean socket_lock_input_cb(GIOChannel *source, GIOCondition condition, gpoint
 			}
 			popup = TRUE;
 		}
+		else if (strncmp(buf, "doclist", 7) == 0)
+		{
+			gchar *doc_list = build_document_list();
+			if (NZV(doc_list))
+				socket_fd_write_all(sock, doc_list, strlen(doc_list));
+			else
+				/* send ETX (end-of-text) in case we have no open files, we must send anything
+				 * otherwise the client would hang on reading */
+				socket_fd_write_all(sock, "\3", 1);
+			g_free(doc_list);
+		}
 		else if (strncmp(buf, "line", 4) == 0)
 		{
 			while (socket_fd_gets(sock, buf, sizeof(buf)) != -1 && *buf != '.')
@@ -704,6 +788,7 @@ gint socket_fd_write(gint fd, const gchar *buf, gint len)
 #endif
 }
 
+
 #endif
 
 
