@@ -1,8 +1,8 @@
 /*
  *      stash.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2008-2009 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
- *      Copyright 2008-2009 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2008-2010 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2008-2010 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -19,35 +19,66 @@
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
  *      MA 02110-1301, USA.
  *
- * $Id: stash.c 4041 2009-07-29 17:40:20Z ntrel $
+ * $Id: stash.c 4795 2010-03-31 16:21:28Z ntrel $
  */
 
-/* Mini-library for reading/writing GKeyFile settings and synchronizing widgets with
- * C variables. */
-
-/* Terms
+/**
+ * @file stash.h
+ * Lightweight library for reading/writing @c GKeyFile settings and synchronizing widgets with
+ * C variables.
+ *
+ * Note: Stash should only depend on GLib and GTK, but currently has some minor
+ * dependencies on Geany's utils.c.
+ *
+ * @section Terms
  * 'Setting' is used only for data stored on disk or in memory.
  * 'Pref' can also include visual widget information.
  *
- * Memory Usage
+ * @section Memory Usage
  * Stash will not duplicate strings if they are normally static arrays, such as
  * keyfile group names and key names, string default values, widget_id names, property names.
  *
- * String Settings
- * String settings and other dynamically allocated settings must be initialized to NULL as they
- * will be freed before reassigning.
+ * @section String Settings
+ * String settings and other dynamically allocated settings will be initialized to NULL when
+ * added to a StashGroup (so they can safely be reassigned later).
  *
- * Widget Support
+ * @section Widget Support
  * Widgets very commonly used in configuration dialogs will be supported with their own function.
- * Widgets less commonly used such as GtkExpander or widget settings that aren't commonly needed
+ * Widgets less commonly used such as @c GtkExpander or widget settings that aren't commonly needed
  * to be persistent won't be directly supported, to keep the library lightweight. However, you can
  * use stash_group_add_widget_property() to also save these settings for any read/write widget
- * property.
+ * property. Macros could be added for common widget properties such as @c GtkExpander:"expanded".
+ *
+ * @section settings-example Settings Example
+ * @include stash-example.c
+ * @note You might want to handle the warning/error conditions differently from above.
+ *
+ * @section prefs-example GUI Prefs Example
+ * For prefs, it's the same as the above example but you add widget prefs instead of e.g.
+ * boolean settings.
+ *
+ * This example uses lookup strings for widgets as they are more flexible than widget pointers.
+ * @code
+gboolean want_handle;
+GtkWidget *parent;
+GtkWidget *my_check_button;
+
+stash_group_add_toggle_button(group, &want_handle, "handle", TRUE, "check_handle");
+...
+gtk_container_add(GTK_CONTAINER(parent), my_check_button);
+ui_hookup_widget(parent, my_check_button, "check_handle");
+...
+stash_group_display(group, parent);
+* @endcode
+* Now let the user manipulate widgets. To synchronize the Stash settings afterwards, call:
+* @code
+stash_group_update(group, parent);
+ * @endcode
  */
 
 /* Implementation Note
  * We use a GArray to hold prefs. It would be more efficient for user code to declare
- * a static array of GeanyPrefEntry structs, but we don't do this because:
+ * a static array of StashPref structs, but we don't do this because:
  *
  * * It would be more ugly (lots of casts and NULLs).
  * * Less type checking.
@@ -56,7 +87,9 @@
  * Usually the prefs code isn't what user code will spend most of its time doing, so this
  * should be efficient enough. But, if desired we could add a stash_group_set_size() function
  * to reduce reallocation.
- * */
+ *
+ * TODO: Maybe using GSlice chunks with an extra 'next' pointer would be more (memory) efficient.
+ */
 
 
 #include <gtk/gtk.h>
@@ -65,28 +98,30 @@
 #include "utils.h"		/* only for foreach_*, utils_get_setting_*(). Stash should not depend on Geany. */
 
 
-struct GeanyPrefEntry
+struct StashPref
 {
 	GType setting_type;			/* e.g. G_TYPE_INT */
 	gpointer setting;
 	const gchar *key_name;
 	gpointer default_value;
 	GType widget_type;			/* e.g. GTK_TYPE_TOGGLE_BUTTON */
-	gpointer widget_id;			/* can be GtkWidget or gchararray */
+	StashWidgetID widget_id;	/* (GtkWidget*) or (gchar*) */
 	gpointer fields;			/* extra fields */
 };
 
-struct GeanyPrefGroup
+typedef struct StashPref StashPref;
+
+struct StashGroup
 {
 	const gchar *name;			/* group name to use in the keyfile */
-	GArray *entries;			/* array of GeanyPrefEntry */
+	GArray *entries;			/* array of StashPref */
 	gboolean write_once;		/* only write settings if they don't already exist */
 	gboolean use_defaults;		/* use default values if there's no keyfile entry */
 };
 
 typedef struct EnumWidget
 {
-	gpointer widget_id;
+	StashWidgetID widget_id;
 	gint enum_id;
 }
 EnumWidget;
@@ -107,7 +142,7 @@ typedef enum PrefAction
 PrefAction;
 
 
-static void handle_boolean_setting(GeanyPrefGroup *group, GeanyPrefEntry *se,
+static void handle_boolean_setting(StashGroup *group, StashPref *se,
 		GKeyFile *config, SettingAction action)
 {
 	gboolean *setting = se->setting;
@@ -125,7 +160,7 @@ static void handle_boolean_setting(GeanyPrefGroup *group, GeanyPrefEntry *se,
 }
 
 
-static void handle_integer_setting(GeanyPrefGroup *group, GeanyPrefEntry *se,
+static void handle_integer_setting(StashGroup *group, StashPref *se,
 		GKeyFile *config, SettingAction action)
 {
 	gboolean *setting = se->setting;
@@ -143,7 +178,7 @@ static void handle_integer_setting(GeanyPrefGroup *group, GeanyPrefEntry *se,
 }
 
 
-static void handle_string_setting(GeanyPrefGroup *group, GeanyPrefEntry *se,
+static void handle_string_setting(StashGroup *group, StashPref *se,
 		GKeyFile *config, SettingAction action)
 {
 	gchararray *setting = se->setting;
@@ -163,7 +198,7 @@ static void handle_string_setting(GeanyPrefGroup *group, GeanyPrefEntry *se,
 }
 
 
-static void handle_strv_setting(GeanyPrefGroup *group, GeanyPrefEntry *se,
+static void handle_strv_setting(StashGroup *group, StashPref *se,
 		GKeyFile *config, SettingAction action)
 {
 	gchararray **setting = se->setting;
@@ -192,11 +227,11 @@ static void handle_strv_setting(GeanyPrefGroup *group, GeanyPrefEntry *se,
 }
 
 
-static void keyfile_action(SettingAction action, GeanyPrefGroup *group, GKeyFile *keyfile)
+static void keyfile_action(SettingAction action, StashGroup *group, GKeyFile *keyfile)
 {
-	GeanyPrefEntry *entry;
+	StashPref *entry;
 
-	foreach_array(GeanyPrefEntry, entry, group->entries)
+	foreach_array(StashPref, entry, group->entries)
 	{
 		/* don't overwrite write_once prefs */
 		if (group->write_once && action == SETTING_WRITE &&
@@ -227,34 +262,101 @@ static void keyfile_action(SettingAction action, GeanyPrefGroup *group, GKeyFile
 }
 
 
-void stash_group_load_from_key_file(GeanyPrefGroup *group, GKeyFile *keyfile)
+/** Reads key values from @a keyfile into the group settings.
+ * @note You should still call this even if the keyfile couldn't be loaded from disk
+ * so that all Stash settings are initialized to defaults.
+ * @param group .
+ * @param keyfile Usually loaded from a configuration file first. */
+void stash_group_load_from_key_file(StashGroup *group, GKeyFile *keyfile)
 {
 	keyfile_action(SETTING_READ, group, keyfile);
 }
 
 
-void stash_group_save_to_key_file(GeanyPrefGroup *group, GKeyFile *keyfile)
+/** Writes group settings into key values in @a keyfile.
+ * @a keyfile is usually written to a configuration file afterwards.
+ * @param group .
+ * @param keyfile . */
+void stash_group_save_to_key_file(StashGroup *group, GKeyFile *keyfile)
 {
 	keyfile_action(SETTING_WRITE, group, keyfile);
 }
 
 
-GeanyPrefGroup *stash_group_new(const gchar *name)
+/** Reads group settings from a configuration file using @c GKeyFile.
+ * @note Stash settings will be initialized to defaults if the keyfile
+ * couldn't be loaded from disk.
+ * @param group .
+ * @param filename Filename of the file to read, in locale encoding.
+ * @return @c TRUE if a key file could be loaded.
+ * @see stash_group_load_from_key_file().
+ **/
+gboolean stash_group_load_from_file(StashGroup *group, const gchar *filename)
 {
-	GeanyPrefGroup *group = g_new0(GeanyPrefGroup, 1);
+	GKeyFile *keyfile;
+	gboolean ret;
+
+	keyfile = g_key_file_new();
+	ret = g_key_file_load_from_file(keyfile, filename, 0, NULL);
+	/* even on failure we load settings to apply defaults */
+	stash_group_load_from_key_file(group, keyfile);
+
+	g_key_file_free(keyfile);
+	return ret;
+}
+
+
+/** Writes group settings to a configuration file using @c GKeyFile.
+ *
+ * @param group .
+ * @param filename Filename of the file to write, in locale encoding.
+ * @param flags Keyfile options - @c G_KEY_FILE_NONE is the most efficient.
+ * @return 0 if the file was successfully written, otherwise the @c errno of the
+ *         failed operation is returned.
+ * @see stash_group_save_to_key_file().
+ **/
+gint stash_group_save_to_file(StashGroup *group, const gchar *filename,
+		GKeyFileFlags flags)
+{
+	GKeyFile *keyfile;
+	gchar *data;
+	gint ret;
+
+	keyfile = g_key_file_new();
+	/* if we need to keep comments or translations, try to load first */
+	if (flags)
+		g_key_file_load_from_file(keyfile, filename, flags, NULL);
+
+	stash_group_save_to_key_file(group, keyfile);
+	data = g_key_file_to_data(keyfile, NULL, NULL);
+	ret = utils_write_file(filename, data);
+	g_free(data);
+	g_key_file_free(keyfile);
+	return ret;
+}
+
+
+/** Creates a new group.
+ * @param name Name used for @c GKeyFile group.
+ * @return Group. */
+StashGroup *stash_group_new(const gchar *name)
+{
+	StashGroup *group = g_new0(StashGroup, 1);
 
 	group->name = name;
-	group->entries = g_array_new(FALSE, FALSE, sizeof(GeanyPrefEntry));
+	group->entries = g_array_new(FALSE, FALSE, sizeof(StashPref));
 	group->use_defaults = TRUE;
 	return group;
 }
 
 
-void stash_group_free(GeanyPrefGroup *group)
+/** Frees a group.
+ * @param group . */
+void stash_group_free(StashGroup *group)
 {
-	GeanyPrefEntry *entry;
+	StashPref *entry;
 
-	foreach_array(GeanyPrefEntry, entry, group->entries)
+	foreach_array(StashPref, entry, group->entries)
 	{
 		if (entry->widget_type == GTK_TYPE_RADIO_BUTTON)
 			g_free(entry->fields);
@@ -271,7 +373,7 @@ void stash_group_free(GeanyPrefGroup *group)
 /* Useful so the user can edit the keyfile manually while the program is running,
  * and the setting won't be overridden.
  * @c FALSE by default. */
-void stash_group_set_write_once(GeanyPrefGroup *group, gboolean write_once)
+void stash_group_set_write_once(StashGroup *group, gboolean write_once)
 {
 	group->write_once = write_once;
 }
@@ -280,53 +382,75 @@ void stash_group_set_write_once(GeanyPrefGroup *group, gboolean write_once)
 /* When @c FALSE, Stash doesn't change the setting if there is no keyfile entry, so it
  * remains whatever it was initialized/set to by user code.
  * @c TRUE by default. */
-void stash_group_set_use_defaults(GeanyPrefGroup *group, gboolean use_defaults)
+void stash_group_set_use_defaults(StashGroup *group, gboolean use_defaults)
 {
 	group->use_defaults = use_defaults;
 }
 
 
-static GeanyPrefEntry *
-add_pref(GeanyPrefGroup *group, GType type, gpointer setting,
+static StashPref *
+add_pref(StashGroup *group, GType type, gpointer setting,
 		const gchar *key_name, gpointer default_value)
 {
-	GeanyPrefEntry entry = {type, setting, key_name, default_value, G_TYPE_NONE, NULL, NULL};
+	StashPref entry = {type, setting, key_name, default_value, G_TYPE_NONE, NULL, NULL};
 	GArray *array = group->entries;
 
+	/* init any pointer settings to NULL so they can be freed later */
+	if (type == G_TYPE_STRING ||
+		type == G_TYPE_STRV)
+		if (group->use_defaults)
+			*(gpointer**)setting = NULL;
+
 	g_array_append_val(array, entry);
 
-	return &g_array_index(array, GeanyPrefEntry, array->len - 1);
+	return &g_array_index(array, StashPref, array->len - 1);
 }
 
 
-void stash_group_add_boolean(GeanyPrefGroup *group, gboolean *setting,
+/** Adds boolean setting.
+ * @param group .
+ * @param setting Address of setting variable.
+ * @param key_name Name for key in a @c GKeyFile.
+ * @param default_value Value to use if the key doesn't exist when loading. */
+void stash_group_add_boolean(StashGroup *group, gboolean *setting,
 		const gchar *key_name, gboolean default_value)
 {
 	add_pref(group, G_TYPE_BOOLEAN, setting, key_name, GINT_TO_POINTER(default_value));
 }
 
 
-void stash_group_add_integer(GeanyPrefGroup *group, gint *setting,
+/** Adds integer setting.
+ * @param group .
+ * @param setting Address of setting variable.
+ * @param key_name Name for key in a @c GKeyFile.
+ * @param default_value Value to use if the key doesn't exist when loading. */
+void stash_group_add_integer(StashGroup *group, gint *setting,
 		const gchar *key_name, gint default_value)
 {
 	add_pref(group, G_TYPE_INT, setting, key_name, GINT_TO_POINTER(default_value));
 }
 
 
-/* The contents of @a setting will be freed before being replaced, so make sure it is
- * allocated, or initialized to @c NULL.
- * @param default_value Not duplicated. */
-void stash_group_add_string(GeanyPrefGroup *group, gchar **setting,
+/** Adds string setting.
+ * The contents of @a setting will be initialized to @c NULL.
+ * @param group .
+ * @param setting Address of setting variable.
+ * @param key_name Name for key in a @c GKeyFile.
+ * @param default_value String to copy if the key doesn't exist when loading, or @c NULL. */
+void stash_group_add_string(StashGroup *group, gchar **setting,
 		const gchar *key_name, const gchar *default_value)
 {
 	add_pref(group, G_TYPE_STRING, setting, key_name, (gpointer)default_value);
 }
 
 
-/* The contents of @a setting will be freed before being replaced, so make sure it is
- * allocated, or initialized to @c NULL.
- * @param default_value Not duplicated. */
-void stash_group_add_string_vector(GeanyPrefGroup *group, gchar ***setting,
+/** Adds string vector setting (array of strings).
+ * The contents of @a setting will be initialized to @c NULL.
+ * @param group .
+ * @param setting Address of setting variable.
+ * @param key_name Name for key in a @c GKeyFile.
+ * @param default_value Vector to copy if the key doesn't exist when loading. Usually @c NULL. */
+void stash_group_add_string_vector(StashGroup *group, gchar ***setting,
 		const gchar *key_name, const gchar **default_value)
 {
 	add_pref(group, G_TYPE_STRV, setting, key_name, (gpointer)default_value);
@@ -350,7 +474,7 @@ static void handle_toggle_button(GtkWidget *widget, gboolean *setting,
 }
 
 
-static void handle_spin_button(GtkWidget *widget, GeanyPrefEntry *entry,
+static void handle_spin_button(GtkWidget *widget, StashPref *entry,
 		PrefAction action)
 {
 	gint *setting = entry->setting;
@@ -371,7 +495,7 @@ static void handle_spin_button(GtkWidget *widget, GeanyPrefEntry *entry,
 }
 
 
-static void handle_combo_box(GtkWidget *widget, GeanyPrefEntry *entry,
+static void handle_combo_box(GtkWidget *widget, StashPref *entry,
 		PrefAction action)
 {
 	gint *setting = entry->setting;
@@ -388,7 +512,7 @@ static void handle_combo_box(GtkWidget *widget, GeanyPrefEntry *entry,
 }
 
 
-static void handle_entry(GtkWidget *widget, GeanyPrefEntry *entry,
+static void handle_entry(GtkWidget *widget, StashPref *entry,
 		PrefAction action)
 {
 	gchararray *setting = entry->setting;
@@ -406,7 +530,7 @@ static void handle_entry(GtkWidget *widget, GeanyPrefEntry *entry,
 }
 
 
-static void handle_combo_box_entry(GtkWidget *widget, GeanyPrefEntry *entry,
+static void handle_combo_box_entry(GtkWidget *widget, StashPref *entry,
 		PrefAction action)
 {
 	widget = gtk_bin_get_child(GTK_BIN(widget));
@@ -441,7 +565,7 @@ lookup_widget(GtkWidget *widget, const gchar *widget_name)
 
 
 static GtkWidget *
-get_widget(GtkWidget *owner, gpointer widget_id)
+get_widget(GtkWidget *owner, StashWidgetID widget_id)
 {
 	GtkWidget *widget = widget_id;
 
@@ -503,7 +627,7 @@ static void handle_radio_buttons(GtkWidget *owner, EnumWidget *fields,
 }
 
 
-static void handle_widget_property(GtkWidget *widget, GeanyPrefEntry *entry,
+static void handle_widget_property(GtkWidget *widget, StashPref *entry,
 		PrefAction action)
 {
 	GObject *object = G_OBJECT(widget);
@@ -525,11 +649,11 @@ static void handle_widget_property(GtkWidget *widget, GeanyPrefEntry *entry,
 }
 
 
-static void pref_action(PrefAction action, GeanyPrefGroup *group, GtkWidget *owner)
+static void pref_action(PrefAction action, StashGroup *group, GtkWidget *owner)
 {
-	GeanyPrefEntry *entry;
+	StashPref *entry;
 
-	foreach_array(GeanyPrefEntry, entry, group->entries)
+	foreach_array(StashPref, entry, group->entries)
 	{
 		GtkWidget *widget;
 
@@ -572,25 +696,36 @@ static void pref_action(PrefAction action, GeanyPrefGroup *group, GtkWidget *own
 }
 
 
-/** @param owner If non-NULL, used to lookup widgets by name. */
-void stash_group_display(GeanyPrefGroup *group, GtkWidget *owner)
+/** Applies Stash settings to widgets, usually called before displaying the widgets.
+ * The @a owner argument depends on which type you use for @ref StashWidgetID.
+ * @param group .
+ * @param owner If non-NULL, used to lookup widgets by name, otherwise
+ * widget pointers are assumed.
+ * @see stash_group_update(). */
+void stash_group_display(StashGroup *group, GtkWidget *owner)
 {
 	pref_action(PREF_DISPLAY, group, owner);
 }
 
 
-void stash_group_update(GeanyPrefGroup *group, GtkWidget *owner)
+/** Applies widget values to Stash settings, usually called after displaying the widgets.
+ * The @a owner argument depends on which type you use for @ref StashWidgetID.
+ * @param group .
+ * @param owner If non-NULL, used to lookup widgets by name, otherwise
+ * widget pointers are assumed.
+ * @see stash_group_display(). */
+void stash_group_update(StashGroup *group, GtkWidget *owner)
 {
 	pref_action(PREF_UPDATE, group, owner);
 }
 
 
-static GeanyPrefEntry *
-add_widget_pref(GeanyPrefGroup *group, GType setting_type, gpointer setting,
+static StashPref *
+add_widget_pref(StashGroup *group, GType setting_type, gpointer setting,
 		const gchar *key_name, gpointer default_value,
-		GType widget_type, gpointer widget_id)
+		GType widget_type, StashWidgetID widget_id)
 {
-	GeanyPrefEntry *entry =
+	StashPref *entry =
 		add_pref(group, setting_type, setting, key_name, default_value);
 
 	entry->widget_type = widget_type;
@@ -599,28 +734,40 @@ add_widget_pref(GeanyPrefGroup *group, GType setting_type, gpointer setting,
 }
 
 
-/* Used for GtkCheckButton or GtkToggleButton widgets.
+/** Adds a @c GtkToggleButton (or @c GtkCheckButton) widget pref.
+ * @param group .
+ * @param setting Address of setting variable.
+ * @param key_name Name for key in a @c GKeyFile.
+ * @param default_value Value to use if the key doesn't exist when loading.
+ * @param widget_id @c GtkWidget pointer or string to lookup widget later.
  * @see stash_group_add_radio_buttons(). */
-void stash_group_add_toggle_button(GeanyPrefGroup *group, gboolean *setting,
-		const gchar *key_name, gboolean default_value, gpointer widget_id)
+void stash_group_add_toggle_button(StashGroup *group, gboolean *setting,
+		const gchar *key_name, gboolean default_value, StashWidgetID widget_id)
 {
 	add_widget_pref(group, G_TYPE_BOOLEAN, setting, key_name, GINT_TO_POINTER(default_value),
 		GTK_TYPE_TOGGLE_BUTTON, widget_id);
 }
 
 
-/* @param ... pairs of widget_id, enum_id.
+/** Adds a @c GtkRadioButton widget group pref.
+ * @param group .
+ * @param setting Address of setting variable.
+ * @param key_name Name for key in a @c GKeyFile.
+ * @param default_value Value to use if the key doesn't exist when loading.
+ * @param widget_id @c GtkWidget pointer or string to lookup widget later.
+ * @param enum_id Enum value for @a widget_id.
+ * @param ... pairs of @a widget_id, @a enum_id.
  * Example (using widget lookup strings, but widget pointers can also work):
  * @code
  * enum {FOO, BAR};
  * stash_group_add_radio_buttons(group, &which_one_setting, "which_one", BAR,
  * 	"radio_foo", FOO, "radio_bar", BAR, NULL);
  * @endcode */
-void stash_group_add_radio_buttons(GeanyPrefGroup *group, gint *setting,
+void stash_group_add_radio_buttons(StashGroup *group, gint *setting,
 		const gchar *key_name, gint default_value,
-		gpointer widget_id, gint enum_id, ...)
+		StashWidgetID widget_id, gint enum_id, ...)
 {
-	GeanyPrefEntry *entry =
+	StashPref *entry =
 		add_widget_pref(group, G_TYPE_INT, setting, key_name, GINT_TO_POINTER(default_value),
 			GTK_TYPE_RADIO_BUTTON, NULL);
 	va_list args;
@@ -662,35 +809,59 @@ void stash_group_add_radio_buttons(GeanyPrefGroup *group, gint *setting,
 }
 
 
-void stash_group_add_spin_button_integer(GeanyPrefGroup *group, gint *setting,
-		const gchar *key_name, gint default_value, gpointer widget_id)
+/** Adds a @c GtkSpinButton widget pref.
+ * @param group .
+ * @param setting Address of setting variable.
+ * @param key_name Name for key in a @c GKeyFile.
+ * @param default_value Value to use if the key doesn't exist when loading.
+ * @param widget_id @c GtkWidget pointer or string to lookup widget later. */
+void stash_group_add_spin_button_integer(StashGroup *group, gint *setting,
+		const gchar *key_name, gint default_value, StashWidgetID widget_id)
 {
 	add_widget_pref(group, G_TYPE_INT, setting, key_name, GINT_TO_POINTER(default_value),
 		GTK_TYPE_SPIN_BUTTON, widget_id);
 }
 
 
-/* @see stash_group_add_combo_box_entry(). */
-void stash_group_add_combo_box(GeanyPrefGroup *group, gint *setting,
-		const gchar *key_name, gint default_value, gpointer widget_id)
+/** Adds a @c GtkComboBox widget pref.
+ * @param group .
+ * @param setting Address of setting variable.
+ * @param key_name Name for key in a @c GKeyFile.
+ * @param default_value Value to use if the key doesn't exist when loading.
+ * @param widget_id @c GtkWidget pointer or string to lookup widget later.
+ * @see stash_group_add_combo_box_entry(). */
+void stash_group_add_combo_box(StashGroup *group, gint *setting,
+		const gchar *key_name, gint default_value, StashWidgetID widget_id)
 {
 	add_widget_pref(group, G_TYPE_INT, setting, key_name, GINT_TO_POINTER(default_value),
 		GTK_TYPE_COMBO_BOX, widget_id);
 }
 
 
+/** Adds a @c GtkComboBoxEntry widget pref.
+ * @param group .
+ * @param setting Address of setting variable.
+ * @param key_name Name for key in a @c GKeyFile.
+ * @param default_value Value to use if the key doesn't exist when loading.
+ * @param widget_id @c GtkWidget pointer or string to lookup widget later. */
 /* We could maybe also have something like stash_group_add_combo_box_entry_with_menu()
  * for the history list - or should that be stored as a separate setting? */
-void stash_group_add_combo_box_entry(GeanyPrefGroup *group, gchar **setting,
-		const gchar *key_name, const gchar *default_value, gpointer widget_id)
+void stash_group_add_combo_box_entry(StashGroup *group, gchar **setting,
+		const gchar *key_name, const gchar *default_value, StashWidgetID widget_id)
 {
 	add_widget_pref(group, G_TYPE_STRING, setting, key_name, (gpointer)default_value,
 		GTK_TYPE_COMBO_BOX_ENTRY, widget_id);
 }
 
 
-void stash_group_add_entry(GeanyPrefGroup *group, gchar **setting,
-		const gchar *key_name, const gchar *default_value, gpointer widget_id)
+/** Adds a @c GtkEntry widget pref.
+ * @param group .
+ * @param setting Address of setting variable.
+ * @param key_name Name for key in a @c GKeyFile.
+ * @param default_value Value to use if the key doesn't exist when loading.
+ * @param widget_id @c GtkWidget pointer or string to lookup widget later. */
+void stash_group_add_entry(StashGroup *group, gchar **setting,
+		const gchar *key_name, const gchar *default_value, StashWidgetID widget_id)
 {
 	add_widget_pref(group, G_TYPE_STRING, setting, key_name, (gpointer)default_value,
 		GTK_TYPE_ENTRY, widget_id);
@@ -707,16 +878,22 @@ static GType object_get_property_type(GObject *object, const gchar *property_nam
 }
 
 
-/* Add a widget's read/write property to the stash group. The property will be set when calling
+/** Adds a widget's read/write property to the stash group.
+ * The property will be set when calling
  * stash_group_display(), and read when calling stash_group_update().
- * @param setting address of e.g. an integer if using an integer property.
- * @param default_value should be cast into a pointer e.g. with @c GINT_TO_POINTER().
- * @param type can be 0 if passing a GtkWidget as the @a widget_id argument to look it up from the
- * GObject data.
+ * @param group .
+ * @param setting Address of e.g. an integer if using an integer property.
+ * @param key_name Name for key in a @c GKeyFile.
+ * @param default_value Value to use if the key doesn't exist when loading.
+ * Should be cast into a pointer e.g. with @c GINT_TO_POINTER().
+ * @param widget_id @c GtkWidget pointer or string to lookup widget later.
+ * @param property_name .
+ * @param type can be @c 0 if passing a @c GtkWidget as the @a widget_id argument to look it up from the
+ * @c GObject data.
  * @warning Currently only string GValue properties will be freed before setting; patch for
- * other types - see handle_widget_property(). */
-void stash_group_add_widget_property(GeanyPrefGroup *group, gpointer setting,
-		const gchar *key_name, gpointer default_value, gpointer widget_id,
+ * other types - see @c handle_widget_property(). */
+void stash_group_add_widget_property(StashGroup *group, gpointer setting,
+		const gchar *key_name, gpointer default_value, StashWidgetID widget_id,
 		const gchar *property_name, GType type)
 {
 	if (!type)
