@@ -1,8 +1,8 @@
 /*
  *      symbols.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2006-2009 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2006-2009 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2006-2010 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2006-2010 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -18,9 +18,14 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: symbols.c 3997 2009-07-20 12:25:24Z ntrel $
+ * $Id: symbols.c 5004 2010-06-06 18:02:49Z eht16 $
  */
 
+/**
+ * @file symbols.h
+ * Tag-related functions.
+ **/
+
 /*
  * Symbol Tree and TagManager-related convenience functions.
  * TagManager parses tags for each document, and also adds them to the workspace (session).
@@ -44,7 +49,7 @@
 #include "documentprivate.h"
 #include "support.h"
 #include "msgwindow.h"
-#include "treeviews.h"
+#include "sidebar.h"
 #include "main.h"
 #include "navqueue.h"
 #include "ui_utils.h"
@@ -128,7 +133,7 @@ void symbols_reload_config_files(void)
 }
 
 
-static gsize get_tag_count()
+static gsize get_tag_count(void)
 {
 	GPtrArray *tags = tm_get_workspace()->global_tags;
 	gsize count = tags ? tags->len : 0;
@@ -172,6 +177,9 @@ void symbols_global_tags_loaded(gint file_type_idx)
 	if (cl_options.ignore_global_tags || app->tm_workspace == NULL)
 		return;
 
+	/* load config in case of custom filetypes */
+	filetypes_load_config(file_type_idx, FALSE);
+
 	load_user_tags(file_type_idx);
 
 	switch (file_type_idx)
@@ -266,6 +274,13 @@ GString *symbols_find_tags_as_string(GPtrArray *tags_array, guint tag_types, gin
 }
 
 
+/** Gets the context separator used by the tag manager for a particular file
+ * type.
+ * @param ft_id File type identifier.
+ * @return The context separator string.
+ *
+ * @since 0.19
+ */
 const gchar *symbols_get_context_separator(gint ft_id)
 {
 	switch (ft_id)
@@ -601,7 +616,6 @@ static void add_top_level_items(GeanyDocument *doc)
 				&(tv_iters.tag_struct), _("Appendix"), NULL,
 				&(tv_iters.tag_other), _("Other"), NULL,
 				NULL);
-			/*	&(tv_iters.tag_namespace), _("Other"), NULL, NULL);	*/
 			break;
 		}
 		case GEANY_FILETYPES_HASKELL:
@@ -647,18 +661,23 @@ static void add_top_level_items(GeanyDocument *doc)
 				NULL);
 			break;
 		}
+		case GEANY_FILETYPES_R:
+		{
+			tag_list_add_groups(tag_store,
+				&(tv_iters.tag_function), _("Functions"), "classviewer-method",
+				&(tv_iters.tag_struct), _("Other"), NULL,
+				NULL);
+			break;
+		}
 		case GEANY_FILETYPES_PERL:
 		{
 			tag_list_add_groups(tag_store,
-				&(tv_iters.tag_class), _("Package"), NULL,
+				&(tv_iters.tag_namespace), _("Package"), "classviewer-namespace",
 				&(tv_iters.tag_function), _("Functions"), "classviewer-method",
-				&(tv_iters.tag_member), _("My"), NULL,
-				&(tv_iters.tag_macro), _("Local"), NULL,
-				&(tv_iters.tag_variable), _("Our"), NULL,
+				&(tv_iters.tag_macro), _("Labels"), NULL,
+				&(tv_iters.tag_type), _("Constants"), NULL,
+				&(tv_iters.tag_other), _("Other"), "classviewer-other",
 				NULL);
-				/*&(tv_iters.tag_struct), _("Label"), NULL,*/
-				/*&(tv_iters.tag_namespace), _("Begin"), NULL,*/
-				/*&(tv_iters.tag_other), _("Other"), NULL, NULL);*/
 			break;
 		}
 		case GEANY_FILETYPES_PHP:
@@ -670,9 +689,6 @@ static void add_top_level_items(GeanyDocument *doc)
 				&(tv_iters.tag_macro), _("Constants"), "classviewer-macro",
 				&(tv_iters.tag_variable), _("Variables"), "classviewer-var",
 				NULL);
-				/*&(tv_iters.tag_struct), _("Label"),*/
-				/*&(tv_iters.tag_namespace), _("Begin"),*/
-				/*&(tv_iters.tag_other), _("Other"), NULL);*/
 			break;
 		}
 		case GEANY_FILETYPES_HTML:
@@ -695,6 +711,8 @@ static void add_top_level_items(GeanyDocument *doc)
 			break;
 		}
 		case GEANY_FILETYPES_REST:
+		case GEANY_FILETYPES_TXT2TAGS:
+		case GEANY_FILETYPES_ABC:
 		{
 			tag_list_add_groups(tag_store,
 				&(tv_iters.tag_namespace), _("Chapter"), NULL,
@@ -712,8 +730,6 @@ static void add_top_level_items(GeanyDocument *doc)
 				&(tv_iters.tag_member), _("Singletons"), "classviewer-struct",
 				&(tv_iters.tag_function), _("Methods"), "classviewer-method",
 				NULL);
-				/*&(tv_iters.tag_namespace), _("Begin"),*/
-				/*&(tv_iters.tag_other), _("Other"), NULL);*/
 			break;
 		}
 		case GEANY_FILETYPES_TCL:
@@ -735,37 +751,41 @@ static void add_top_level_items(GeanyDocument *doc)
 				&(tv_iters.tag_variable), _("Variables"), "classviewer-var",
 				&(tv_iters.tag_namespace), _("Imports"), "classviewer-namespace",
 				NULL);
-				/*&(tv_iters.tag_macro), _("Mixin"),*/
-				/*&(tv_iters.tag_variable), _("Variables"),*/
-				/*&(tv_iters.tag_struct), _("Members"),*/
-				/*&(tv_iters.tag_namespace), _("Begin"),*/
-				/*&(tv_iters.tag_other), _("Other"), NULL);*/
 			break;
 		}
 		case GEANY_FILETYPES_VHDL:
 		{
 			tag_list_add_groups(tag_store,
-				&(tv_iters.tag_function), _("Functions"), "classviewer-method",
-				/*&(tv_iters.tag_class), _("Constants"),*/
-				/*&(tv_iters.tag_member), _("Members"),*/
-				/*&(tv_iters.tag_macro), _("Macros"),*/
+				&(tv_iters.tag_namespace), _("Package"), "classviewer-namespace",
+				&(tv_iters.tag_class), _("Entities"), "classviewer-class",
+				&(tv_iters.tag_struct), _("Architectures"), "classviewer-struct",
+				&(tv_iters.tag_type), _("Types"), "classviewer-other",
+				&(tv_iters.tag_function), _("Functions / Procedures"), "classviewer-method",
+				&(tv_iters.tag_variable), _("Variables / Signals"), "classviewer-var",
+				&(tv_iters.tag_member), _("Processes / Components"), "classviewer-member",
+				&(tv_iters.tag_other), _("Other"), "classviewer-other",
+				NULL);
+			break;
+		}
+		case GEANY_FILETYPES_VERILOG:
+		{
+			tag_list_add_groups(tag_store,
+				&(tv_iters.tag_type), _("Events"), "classviewer-macro",
+				&(tv_iters.tag_class), _("Modules"), "classviewer-class",
+				&(tv_iters.tag_function), _("Functions / Tasks"), "classviewer-method",
 				&(tv_iters.tag_variable), _("Variables"), "classviewer-var",
+				&(tv_iters.tag_other), _("Other"), "classviewer-other",
 				NULL);
-				/*&(tv_iters.tag_namespace), _("Namespaces"),*/
-				/*&(tv_iters.tag_struct), _("Signals"),*/
-				/*&(tv_iters.tag_other), _("Other"), NULL);*/
 			break;
 		}
 		case GEANY_FILETYPES_JAVA:
 		{
 			tag_list_add_groups(tag_store,
-				&(tv_iters.tag_namespace), _("Package"), NULL,
+				&(tv_iters.tag_namespace), _("Package"), "classviewer-namespace",
 				&(tv_iters.tag_struct), _("Interfaces"), "classviewer-struct",
 				&(tv_iters.tag_class), _("Classes"), "classviewer-class",
 				&(tv_iters.tag_function), _("Methods"), "classviewer-method",
 				&(tv_iters.tag_member), _("Members"), "classviewer-member",
-				/*&(tv_iters.tag_macro), _("Macros"),*/
-				/*&(tv_iters.tag_variable), _("Variables"),*/
 				&(tv_iters.tag_other), _("Other"), "classviewer-other",
 				NULL);
 			break;
@@ -967,7 +987,7 @@ static gchar *get_symbol_tooltip(GeanyDocument *doc, const TMTag *tag)
 
 
 /* find the last word in "foo::bar::blah", e.g. "blah" */
-const gchar *get_parent_name(const TMTag *tag, filetype_id ft_id)
+static const gchar *get_parent_name(const TMTag *tag, filetype_id ft_id)
 {
 	const gchar *scope = tag->atts.entry.scope;
 	const gchar *separator = symbols_get_context_separator(ft_id);
@@ -1059,6 +1079,21 @@ static GtkTreeIter *get_tag_type_iter(TMTagType tag_type, filetype_id ft_id)
 }
 
 
+static GdkPixbuf *get_child_icon(GtkTreeStore *tree_store, GtkTreeIter *parent)
+{
+	GdkPixbuf *icon = NULL;
+
+	if (parent == &tv_iters.tag_other)
+	{
+		return get_tag_icon("classviewer-var");
+	}
+	/* copy parent icon */
+	gtk_tree_model_get(GTK_TREE_MODEL(tree_store), parent,
+		SYMBOLS_COLUMN_ICON, &icon, -1);
+	return icon;
+}
+
+
 static void add_tree_tag(GeanyDocument *doc, const TMTag *tag, GHashTable *parent_hash)
 {
 	filetype_id ft_id = FILETYPE_ID(doc->file_type);
@@ -1072,14 +1107,11 @@ static void add_tree_tag(GeanyDocument *doc, const TMTag *tag, GHashTable *paren
 		const gchar *name;
 		const gchar *parent_name = get_parent_name(tag, ft_id);
 		GtkTreeIter iter;
-		GtkTreeIter *icon_iter = NULL, *child = NULL;
-		GdkPixbuf *icon = NULL;
+		GtkTreeIter *child = NULL;
+ 		GdkPixbuf *icon = NULL;
 
-		child = &iter;
-		icon_iter = (parent != &tv_iters.tag_other) ? parent : &tv_iters.tag_variable;
-
-		gtk_tree_model_get(GTK_TREE_MODEL(tree_store), icon_iter,
-			SYMBOLS_COLUMN_ICON, &icon, -1);
+ 		child = &iter;
+		icon = get_child_icon(tree_store, parent);
 
 		if (parent_name)
 		{
@@ -1219,6 +1251,12 @@ static gint tree_sort_func(GtkTreeModel *model, GtkTreeIter *a, GtkTreeIter *b,
 		}
 		else
 		{
+			/* this is what g_strcmp0() does */
+			if (! astr)
+				return -(astr != bstr);
+			if (! bstr)
+				return astr != bstr;
+
 			cmp = strcmp(astr, bstr);
 
 			/* sort duplicate 'ScopeName::OverloadedTagName' items by line as well */
@@ -1237,10 +1275,10 @@ static gint tree_sort_func(GtkTreeModel *model, GtkTreeIter *a, GtkTreeIter *b,
 
 static void sort_tree(GtkTreeStore *store, gboolean sort_by_name)
 {
-	gtk_tree_sortable_set_sort_func(GTK_TREE_SORTABLE(store), 1, tree_sort_func,
+	gtk_tree_sortable_set_sort_func(GTK_TREE_SORTABLE(store), SYMBOLS_COLUMN_NAME, tree_sort_func,
 		GINT_TO_POINTER(sort_by_name), NULL);
 
-	gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(store), 1, GTK_SORT_ASCENDING);
+	gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(store), SYMBOLS_COLUMN_NAME, GTK_SORT_ASCENDING);
 }
 
 
@@ -1300,10 +1338,6 @@ static GeanyFiletype *detect_global_tags_filetype(const gchar *utf8_filename)
 		ft = filetypes_detect_from_extension(shortname);
 	}
 	g_free(shortname);
-
-	if (ft == NULL || ! filetype_has_tags(ft))
-		return NULL;
-
 	return ft;
 }
 
@@ -1337,6 +1371,9 @@ int symbols_generate_global_tags(int argc, char **argv, gboolean want_preprocess
 			g_printerr(_("Unknown filetype extension for \"%s\".\n"), tags_file);
 			return 1;
 		}
+		/* load config in case of custom filetypes */
+		filetypes_load_config(ft->id, FALSE);
+
 		/* load ignore list for C/C++ parser */
 		if (ft->id == GEANY_FILETYPES_C || ft->id == GEANY_FILETYPES_CPP)
 			load_c_ignore_tags();
@@ -1445,18 +1482,6 @@ static GHashTable *get_tagfile_hash(const GSList *file_list)
 }
 
 
-static void utils_slist_add_path(GSList *list, const gchar *path)
-{
-	GSList *node;
-
-	for (node = list; node != NULL; node = g_slist_next(node))
-	{
-		setptr(node->data,
-			g_build_path(G_DIR_SEPARATOR_S, path, node->data, NULL));
-	}
-}
-
-
 static GHashTable *init_user_tags(void)
 {
 	GSList *file_list = NULL, *list = NULL;
@@ -1469,12 +1494,10 @@ static GHashTable *init_user_tags(void)
 	{
 		utils_mkdir(dir, FALSE);
 	}
-	file_list = utils_get_file_list(dir, NULL, NULL);
-	utils_slist_add_path(file_list, dir);
+	file_list = utils_get_file_list_full(dir, TRUE, TRUE, NULL);
 
 	dir = utils_build_path(app->datadir, "tags", NULL);
-	list = utils_get_file_list(dir, NULL, NULL);
-	utils_slist_add_path(list, dir);
+	list =  utils_get_file_list_full(dir, TRUE, TRUE, NULL);
 	file_list = g_slist_concat(file_list, list);
 
 	lang_hash = get_tagfile_hash(file_list);
@@ -1487,15 +1510,16 @@ static GHashTable *init_user_tags(void)
 
 static void load_user_tags(filetype_id ft_id)
 {
-	static guchar tags_loaded[GEANY_MAX_BUILT_IN_FILETYPES] = {0};
+	static guchar *tags_loaded = NULL;
 	static GHashTable *lang_hash = NULL;
 	GList *fnames;
 	const GList *node;
 	GeanyFiletype *ft = filetypes[ft_id];
 
 	g_return_if_fail(ft_id > 0);
-	g_return_if_fail(ft_id < GEANY_MAX_BUILT_IN_FILETYPES);
 
+	if (!tags_loaded)
+		tags_loaded = g_new0(guchar, filetypes_array->len);
 	if (tags_loaded[ft_id])
 		return;
 	tags_loaded[ft_id] = TRUE;	/* prevent reloading */
@@ -1732,7 +1756,7 @@ gint symbols_get_current_function(GeanyDocument *doc, const gchar **tagname)
 	tm_file = doc->tm_file;
 
 	/* if the document has no changes, get the previous function name from TM */
-	if(! doc->changed && tm_file != NULL && tm_file->tags_array != NULL)
+	if (! doc->changed && tm_file != NULL && tm_file->tags_array != NULL)
 	{
 		const TMTag *tag = (const TMTag*) tm_get_current_function(tm_file->tags_array, line);
 
