@@ -1,8 +1,8 @@
 /*
  *      templates.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2005-2009 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2006-2009 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2005-2010 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2006-2010 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: templates.c 4088 2009-08-15 17:33:20Z eht16 $
+ * $Id: templates.c 4917 2010-05-15 13:14:38Z eht16 $
  */
 
 /*
@@ -39,13 +39,16 @@
 #include "ui_utils.h"
 #include "toolbar.h"
 #include "geanymenubuttonaction.h"
+#include "project.h"
 
 
 GeanyTemplatePrefs template_prefs;
 
-static GtkWidget *new_with_template_menu = NULL;	/* File menu submenu */
+static GtkWidget *new_with_template_menu = NULL;	/* submenu used for both file menu and toolbar */
 
 
+/* TODO: implement custom insertion templates, put these into files in data/templates */
+
 /* default templates, only for initial tempate file creation on first start of Geany */
 static const gchar templates_gpl_notice[] = "\
 This program is free software; you can redistribute it and/or modify\n\
@@ -117,134 +120,22 @@ static const gchar templates_changelog[] = "\
 \n\
  * \n\n\n";
 
-static const gchar templates_filetype_none[] = "";
-
-static const gchar templates_filetype_c[] = "{fileheader}\n\n\
-#include <stdio.h>\n\
-\n\
-int main(int argc, char** argv)\n\
-{\n\
-	\n\
-	return 0;\n\
-}\n\
-";
-
-static const gchar templates_filetype_cpp[] = "{fileheader}\n\n\
-#include <iostream>\n\
-\n\
-int main(int argc, char** argv)\n\
-{\n\
-	\n\
-	return 0;\n\
-}\n\
-";
-
-static const gchar templates_filetype_d[] = "{fileheader}\n\n\
-import std.stdio;\n\
-\n\
-int main(char[][] args)\n\
-{\n\
-	\n\
-	return 0;\n\
-}\n\
-";
-
-static const gchar templates_filetype_php[] = "\
-<?php\n\
-{fileheader}\
-?>\n\n\
-<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Strict//EN\"\n\
-  \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd\">\n\
-<html xmlns=\"http://www.w3.org/1999/xhtml\" xml:lang=\"en\" lang=\"en\">\n\
-\n\
-<head>\n\
-	<title>{untitled}</title>\n\
-	<meta http-equiv=\"content-type\" content=\"text/html;charset=utf-8\" />\n\
-	<meta name=\"generator\" content=\"{geanyversion}\" />\n\
-</head>\n\
-\n\
-<body>\n\
-	\n\
-</body>\n\
-</html>\n\
-";
-
-static const gchar templates_filetype_html[] = "{fileheader}\n\
-<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Strict//EN\"\n\
-  \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd\">\n\
-<html xmlns=\"http://www.w3.org/1999/xhtml\" xml:lang=\"en\" lang=\"en\">\n\
-\n\
-<head>\n\
-	<title>{untitled}</title>\n\
-	<meta http-equiv=\"content-type\" content=\"text/html;charset=utf-8\" />\n\
-	<meta name=\"generator\" content=\"{geanyversion}\" />\n\
-</head>\n\
-\n\
-<body>\n\
-	\n\
-</body>\n\
-</html>\n\
-";
-
-static const gchar templates_filetype_pascal[] = "{fileheader}\n\n\
-program {untitled};\n\
-\n\
-uses crt;\n\
-var i : byte;\n\
-\n\
-BEGIN\n\
-	\n\
-	\n\
-END.\n\
-";
-
-static const gchar templates_filetype_java[] = "{fileheader}\n\n\
-public class {untitled} {\n\
-\n\
-	public static void main (String args[]) {\
-		\n\
-		\n\
-	}\n\
-}\n\
-";
-
-static const gchar templates_filetype_ruby[] = "{fileheader}\n\n\
-class StdClass\n\
-	def initialize\n\
-		\n\
-	end\n\
-end\n\
-\n\
-x = StdClass.new\n\
-";
-
-static const gchar templates_filetype_python[] = "#!/usr/bin/env python\n\
-# -*- coding: utf-8 -*-\n#\n\
-{fileheader}\n\n\
-\n\
-def main():\n\
-	\n\
-	return 0\n\
-\n\
-if __name__ == '__main__':\n\
-	main()\n\
-";
-
-static const gchar templates_filetype_latex[] = "\
-\\documentclass[a4paper]{article}\n\
-\\usepackage[T1]{fontenc}\n\
-\\usepackage[utf8]{inputenc}\n\
-\\usepackage{lmodern}\n\
-\\usepackage{babel}\n\
-\\begin{document}\n\
-\n\
-\\end{document}\n\
-";
-
 static gchar *templates[GEANY_MAX_TEMPLATES];
+
+/* We should probably remove filetype templates support soon - users can use custom
+ * file templates instead. */
 static gchar *ft_templates[GEANY_MAX_BUILT_IN_FILETYPES] = {NULL};
 
 
+static void replace_static_values(GString *text);
+static gchar *get_template_fileheader(GeanyFiletype *ft);
+
+/* called by templates_replace_common */
+static void templates_replace_default_dates(GString *text);
+static void templates_replace_command(GString *text, const gchar *file_name,
+	const gchar *file_type, const gchar *func_name);
+
+
 /* some simple macros to reduce code size and make the code readable */
 #define TEMPLATES_GET_FILENAME(shortname) \
 	g_strconcat(app->configdir, \
@@ -274,6 +165,7 @@ static void create_template_file_if_necessary(const gchar *filename, const gchar
 	}
 }
 
+
 /* FIXME the callers should use GStrings instead of char arrays */
 static gchar *replace_all(gchar *text, const gchar *year, const gchar *date, const gchar *datetime)
 {
@@ -285,7 +177,11 @@ static gchar *replace_all(gchar *text, const gchar *year, const gchar *date, con
 	str = g_string_new(text);
 
 	g_free(text);
-	templates_replace_all(str, year, date, datetime);
+	templates_replace_valist(str,
+		"{year}", year,
+		"{date}", date,
+		"{datetime}", datetime,
+		NULL);
 
 	return g_string_free(str, FALSE);
 }
@@ -341,43 +237,6 @@ static void init_ft_templates(const gchar *year, const gchar *date, const gchar
 		gchar *shortname = g_strconcat("filetype.", ext, NULL);
 		gchar *fname = TEMPLATES_GET_FILENAME(shortname);
 
-		switch (ft_id)
-		{
-			case GEANY_FILETYPES_NONE:
-				create_template_file_if_necessary(fname, templates_filetype_none);
-				break;
-			case GEANY_FILETYPES_C:
-				create_template_file_if_necessary(fname, templates_filetype_c);
-				break;
-			case GEANY_FILETYPES_CPP:
-				create_template_file_if_necessary(fname, templates_filetype_cpp);
-				break;
-			case GEANY_FILETYPES_D:
-				create_template_file_if_necessary(fname, templates_filetype_d);
-				break;
-			case GEANY_FILETYPES_JAVA:
-				create_template_file_if_necessary(fname, templates_filetype_java);
-				break;
-			case GEANY_FILETYPES_PASCAL:
-				create_template_file_if_necessary(fname, templates_filetype_pascal);
-				break;
-			case GEANY_FILETYPES_PHP:
-				create_template_file_if_necessary(fname, templates_filetype_php);
-				break;
-			case GEANY_FILETYPES_HTML:
-				create_template_file_if_necessary(fname, templates_filetype_html);
-				break;
-			case GEANY_FILETYPES_RUBY:
-				create_template_file_if_necessary(fname, templates_filetype_ruby);
-				break;
-			case GEANY_FILETYPES_PYTHON:
-				create_template_file_if_necessary(fname, templates_filetype_python);
-				break;
-			case GEANY_FILETYPES_LATEX:
-				create_template_file_if_necessary(fname, templates_filetype_latex);
-				break;
-			default: break;
-		}
 		TEMPLATES_READ_FILE(fname, &ft_templates[ft_id]);
 		ft_templates[ft_id] = replace_all(ft_templates[ft_id], year, date, datetime);
 
@@ -389,8 +248,7 @@ static void init_ft_templates(const gchar *year, const gchar *date, const gchar
 
 
 static void
-on_new_with_template                   (GtkMenuItem     *menuitem,
-                                        gpointer         user_data)
+on_new_with_filetype_template(GtkMenuItem *menuitem, gpointer user_data)
 {
 	GeanyFiletype *ft = user_data;
 	gchar *template = templates_get_template_new_file(ft);
@@ -400,30 +258,69 @@ on_new_with_template                   (GtkMenuItem     *menuitem,
 }
 
 
-/* template items for the new file menu */
-static void create_new_menu_items(GtkWidget *toolbar_new_file_menu)
+/* TODO: remove filetype template support after 0.19 */
+static gboolean create_new_filetype_items(void)
 {
 	GSList *node;
+	gboolean ret = FALSE;
+	GtkWidget *menu = NULL;
 
 	foreach_slist(node, filetypes_by_title)
 	{
 		GeanyFiletype *ft = node->data;
-		GtkWidget *tmp_menu, *tmp_button;
-		const gchar *label = ft->title;
+		GtkWidget *item;
 
-		if (ft_templates[ft->id] == NULL)
+		if (ft->id >= GEANY_MAX_BUILT_IN_FILETYPES || ft_templates[ft->id] == NULL)
 			continue;
 
-		tmp_menu = gtk_menu_item_new_with_label(label);
-		gtk_widget_show(tmp_menu);
-		gtk_container_add(GTK_CONTAINER(new_with_template_menu), tmp_menu);
-		g_signal_connect(tmp_menu, "activate", G_CALLBACK(on_new_with_template), ft);
-
-		tmp_button = gtk_menu_item_new_with_label(label);
-		gtk_widget_show(tmp_button);
-		gtk_container_add(GTK_CONTAINER(toolbar_new_file_menu), tmp_button);
-		g_signal_connect(tmp_button, "activate", G_CALLBACK(on_new_with_template), ft);
+		if (!menu)
+		{
+			item = gtk_menu_item_new_with_label(_("Old"));
+			menu = gtk_menu_new();
+			gtk_menu_item_set_submenu(GTK_MENU_ITEM(item), menu);
+			gtk_widget_show_all(item);
+			gtk_container_add(GTK_CONTAINER(new_with_template_menu), item);
+		}
+		item = gtk_menu_item_new_with_label(ft->title);
+		gtk_widget_show(item);
+		gtk_container_add(GTK_CONTAINER(menu), item);
+		g_signal_connect(item, "activate", G_CALLBACK(on_new_with_filetype_template), ft);
+		ret = TRUE;
 	}
+	return ret;
+}
+
+
+void templates_replace_common(GString *template, const gchar *fname,
+							  GeanyFiletype *ft, const gchar *func_name)
+{
+	gchar *shortname;
+
+	if (fname == NULL)
+	{
+		if (!ft->extension)
+			shortname = g_strdup(GEANY_STRING_UNTITLED);
+		else
+			shortname = g_strconcat(GEANY_STRING_UNTITLED, ".", ft->extension, NULL);
+	}
+	else
+		shortname = g_path_get_basename(fname);
+
+	templates_replace_valist(template,
+		"{filename}", shortname,
+		"{project}", app->project ? app->project->name : "",
+		"{description}", app->project ? app->project->description : "",
+		NULL);
+	g_free(shortname);
+
+	templates_replace_default_dates(template);
+	templates_replace_command(template, fname, ft->name, func_name);
+	/* Bug: command results could have {ob} {cb} strings in! */
+	/* replace braces last */
+	templates_replace_valist(template,
+		"{ob}", "{",
+		"{cb}", "}",
+		NULL);
 }
 
 
@@ -438,20 +335,19 @@ static gchar *get_template_from_file(const gchar *locale_fname, const gchar *doc
 	if (content != NULL)
 	{
 		gchar *file_header;
-		gchar *year = utils_get_date_time(template_prefs.year_format, NULL);
-		gchar *date = utils_get_date_time(template_prefs.date_format, NULL);
-		gchar *datetime = utils_get_date_time(template_prefs.datetime_format, NULL);
 
 		template = g_string_new(content);
 
-		file_header = templates_get_template_fileheader(FILETYPE_ID(ft), doc_filename);
-		templates_replace_all(template, year, date, datetime);
-		utils_string_replace_all(template, "{filename}", doc_filename);
-		utils_string_replace_all(template, "{fileheader}", file_header);
+		file_header = get_template_fileheader(ft);
+		templates_replace_valist(template,
+			"{fileheader}", file_header,
+			NULL);
+		templates_replace_common(template, doc_filename, ft, NULL);
 
-		utils_free_pointers(5, year, date, datetime, file_header, content, NULL);
+		utils_free_pointers(2, file_header, content, NULL);
+		return g_string_free(template, FALSE);
 	}
-	return g_string_free(template, FALSE);
+	return NULL;
 }
 
 
@@ -461,120 +357,137 @@ on_new_with_file_template(GtkMenuItem *menuitem, G_GNUC_UNUSED gpointer user_dat
 	gchar *fname = ui_menu_item_get_text(menuitem);
 	GeanyFiletype *ft;
 	gchar *template;
-	gchar *extension = strrchr(fname, '.'); /* easy way to get the file extension */
+	const gchar *extension = strrchr(fname, '.'); /* easy way to get the file extension */
 	gchar *new_filename = g_strconcat(GEANY_STRING_UNTITLED, extension, NULL);
+	gchar *path;
 
 	ft = filetypes_detect_from_extension(fname);
 	setptr(fname, utils_get_locale_from_utf8(fname));
-	/* fname is just the basename from the menu item, so prepend the custom files path */
-	setptr(fname, g_build_path(G_DIR_SEPARATOR_S, app->configdir, GEANY_TEMPLATES_SUBDIR,
-		"files", fname, NULL));
-	template = get_template_from_file(fname, new_filename, ft);
-	g_free(fname);
 
-	document_new_file(new_filename, ft, template);
+	/* fname is just the basename from the menu item, so prepend the custom files path */
+	path = g_build_path(G_DIR_SEPARATOR_S, app->configdir, GEANY_TEMPLATES_SUBDIR,
+		"files", fname, NULL);
+	template = get_template_from_file(path, new_filename, ft);
+	if (!template)
+	{
+		/* try the system path */
+		g_free(path);
+		path = g_build_path(G_DIR_SEPARATOR_S, app->datadir, GEANY_TEMPLATES_SUBDIR,
+			"files", fname, NULL);
+		template = get_template_from_file(path, new_filename, ft);
+	}
+	if (template)
+		document_new_file(new_filename, ft, template);
+	else
+	{
+		setptr(fname, utils_get_utf8_from_locale(fname));
+		ui_set_statusbar(TRUE, _("Could not find file '%s'."), fname);
+	}
 	g_free(template);
+	g_free(path);
 	g_free(new_filename);
+	g_free(fname);
 }
 
 
-static void add_file_item(gpointer data, gpointer user_data)
+static void add_file_item(const gchar *fname, GtkWidget *menu)
 {
-	GtkWidget *tmp_menu, *tmp_button;
-	GtkWidget *toolbar_new_file_menu = user_data;
+	GtkWidget *tmp_button;
 	gchar *label;
 
-	g_return_if_fail(data);
+	g_return_if_fail(fname);
+	g_return_if_fail(menu);
 
-	label = utils_get_utf8_from_locale(data);
-
-	tmp_menu = gtk_menu_item_new_with_label(label);
-	gtk_widget_show(tmp_menu);
-	gtk_container_add(GTK_CONTAINER(new_with_template_menu), tmp_menu);
-	g_signal_connect(tmp_menu, "activate", G_CALLBACK(on_new_with_file_template), NULL);
+	label = utils_get_utf8_from_locale(fname);
 
 	tmp_button = gtk_menu_item_new_with_label(label);
 	gtk_widget_show(tmp_button);
-	gtk_container_add(GTK_CONTAINER(toolbar_new_file_menu), tmp_button);
+	gtk_container_add(GTK_CONTAINER(menu), tmp_button);
 	g_signal_connect(tmp_button, "activate", G_CALLBACK(on_new_with_file_template), NULL);
 
 	g_free(label);
 }
 
 
-static gint compare_filenames_by_filetype(gconstpointer a, gconstpointer b)
+static gboolean add_custom_template_items(void)
 {
-	GeanyFiletype *ft_a = filetypes_detect_from_extension(a);
-	GeanyFiletype *ft_b = filetypes_detect_from_extension(b);
+	GSList *list = utils_get_config_files(GEANY_TEMPLATES_SUBDIR G_DIR_SEPARATOR_S "files");
+	GSList *node;
 
-	/* sort by filetype name first */
-	if (G_LIKELY(ft_a != ft_b))
+	foreach_slist(node, list)
 	{
-		/* None filetypes should come first */
-		if (G_UNLIKELY(ft_a->id == GEANY_FILETYPES_NONE))
-			return -1;
-		if (G_UNLIKELY(ft_b->id == GEANY_FILETYPES_NONE))
-			return 1;
+		gchar *fname = node->data;
 
-		return utils_str_casecmp(ft_a->name, ft_b->name);
+		add_file_item(fname, new_with_template_menu);
+		g_free(fname);
 	}
-	return utils_str_casecmp(a, b);
-}
-
-
-static gboolean add_custom_template_items(GtkWidget *toolbar_new_file_menu)
-{
-	gchar *path = g_build_path(G_DIR_SEPARATOR_S, app->configdir, GEANY_TEMPLATES_SUBDIR,
-		"files", NULL);
-	GSList *list = utils_get_file_list(path, NULL, NULL);
-
-	if (list == NULL)
-	{
-		utils_mkdir(path, FALSE);
-		return FALSE;
-	}
-	list = g_slist_sort(list, compare_filenames_by_filetype);
-	g_slist_foreach(list, add_file_item, toolbar_new_file_menu);
-	g_slist_foreach(list, (GFunc) g_free, NULL);
 	g_slist_free(list);
-	g_free(path);
-	return TRUE;
+	return list != NULL;
 }
 
 
-static void create_file_template_menus(void)
+static void create_file_template_menu(void)
 {
-	GtkWidget *sep1, *sep2 = NULL;
-	GtkWidget *toolbar_new_file_menu = NULL;
+	GtkWidget *sep = NULL;
 
-	new_with_template_menu = ui_lookup_widget(main_widgets.window, "menu_new_with_template1_menu");
-	toolbar_new_file_menu = gtk_menu_new();
-	/* we hold our own ref on the menu in case it is not used in the toolbar */
-	g_object_ref(toolbar_new_file_menu);
+	new_with_template_menu = gtk_menu_new();
 
-	create_new_menu_items(toolbar_new_file_menu);
-
-	sep1 = gtk_separator_menu_item_new();
-	gtk_container_add(GTK_CONTAINER(new_with_template_menu), sep1);
-	sep2 = gtk_separator_menu_item_new();
-	gtk_container_add(GTK_CONTAINER(toolbar_new_file_menu), sep2);
-
-	if (add_custom_template_items(toolbar_new_file_menu))
+	if (add_custom_template_items())
 	{
-		gtk_widget_show(sep1);
-		gtk_widget_show(sep2);
+		sep = gtk_separator_menu_item_new();
+		gtk_container_add(GTK_CONTAINER(new_with_template_menu), sep);
 	}
-
+	if (create_new_filetype_items() && sep)
+	{
+		gtk_widget_show(sep);
+	}
+	/* unless the file menu is showing, menu should be in the toolbar widget */
 	geany_menu_button_action_set_menu(GEANY_MENU_BUTTON_ACTION(
-		toolbar_get_action_by_name("New")), toolbar_new_file_menu);
+		toolbar_get_action_by_name("New")), new_with_template_menu);
 }
 
 
+static void on_file_menu_show(GtkWidget *item)
+{
+	geany_menu_button_action_set_menu(
+		GEANY_MENU_BUTTON_ACTION(toolbar_get_action_by_name("New")), NULL);
+	item = ui_lookup_widget(main_widgets.window, "menu_new_with_template1");
+	gtk_menu_item_set_submenu(GTK_MENU_ITEM(item), new_with_template_menu);
+}
+
+
+static void on_file_menu_hide(GtkWidget *item)
+{
+	item = ui_lookup_widget(main_widgets.window, "menu_new_with_template1");
+	gtk_menu_item_set_submenu(GTK_MENU_ITEM(item), NULL);
+	geany_menu_button_action_set_menu(
+		GEANY_MENU_BUTTON_ACTION(toolbar_get_action_by_name("New")), new_with_template_menu);
+}
+
+
+/* reload templates if any file in the templates path is saved */
+static void on_document_save(G_GNUC_UNUSED GObject *object, GeanyDocument *doc)
+{
+	const gchar *path = utils_build_path(app->configdir, GEANY_TEMPLATES_SUBDIR, NULL);
+
+	g_return_if_fail(NZV(doc->real_path));
+
+	if (strncmp(doc->real_path, path, strlen(path)) == 0)
+	{
+		/* reload templates */
+		templates_free_templates();
+		templates_init();
+	}
+}
+
+
+/* warning: also called when reloading template settings */
 void templates_init(void)
 {
 	gchar *year = utils_get_date_time(template_prefs.year_format, NULL);
 	gchar *date = utils_get_date_time(template_prefs.date_format, NULL);
 	gchar *datetime = utils_get_date_time(template_prefs.datetime_format, NULL);
+	static gboolean init_done = FALSE;
 
 	init_general_templates(year, date, datetime);
 	init_ft_templates(year, date, datetime);
@@ -583,7 +496,23 @@ void templates_init(void)
 	g_free(datetime);
 	g_free(year);
 
-	create_file_template_menus();
+	create_file_template_menu();
+	/* we hold our own ref for the menu as it has no parent whilst being moved */
+	g_object_ref(new_with_template_menu);
+
+	/* only connect signals to persistent objects once */
+	if (!init_done)
+	{
+		GtkWidget *item;
+		/* reparent the template menu as needed */
+		item = ui_lookup_widget(main_widgets.window, "file1");
+		item = gtk_menu_item_get_submenu(GTK_MENU_ITEM(item));
+		g_signal_connect(item, "show", G_CALLBACK(on_file_menu_show), NULL);
+		g_signal_connect(item, "hide", G_CALLBACK(on_file_menu_hide), NULL);
+
+		g_signal_connect(geany_object, "document-save", G_CALLBACK(on_document_save), NULL);
+	}
+	init_done = TRUE;
 }
 
 
@@ -591,11 +520,12 @@ void templates_init(void)
  * e.g. indent = 8 prints " *     here comes the text of the line"
  * indent is meant to be the whole amount of characters before the real line content follows, i.e.
  * 6 characters are filled with whitespace when the comment characters include " *" */
+/* TODO make this function operating on a GString */
 static gchar *make_comment_block(const gchar *comment_text, gint filetype_idx, guint indent)
 {
-	gchar *frame_start;		/* to add before comment_text */
-	gchar *frame_end;		/* to add after comment_text */
-	gchar *line_prefix;		/* to add before every line in comment_text */
+	gchar *frame_start;			/* to add before comment_text */
+	gchar *frame_end;			/* to add after comment_text */
+	const gchar *line_prefix;	/* to add before every line in comment_text */
 	gchar *result;
 	gchar *tmp;
 	gchar *prefix;
@@ -664,109 +594,113 @@ static gchar *make_comment_block(const gchar *comment_text, gint filetype_idx, g
 }
 
 
-gchar *templates_get_template_licence(gint filetype_idx, gint licence_type)
+gchar *templates_get_template_licence(GeanyDocument *doc, gint licence_type)
 {
-	if (licence_type != GEANY_TEMPLATE_GPL && licence_type != GEANY_TEMPLATE_BSD)
-		return NULL;
+	GString *template;
+	gchar *result = NULL;
 
-	return make_comment_block(templates[licence_type], filetype_idx, 8);
-}
+	g_return_val_if_fail(doc != NULL, NULL);
+	g_return_val_if_fail(licence_type == GEANY_TEMPLATE_GPL || licence_type == GEANY_TEMPLATE_BSD, NULL);
 
+	template = g_string_new(templates[licence_type]);
+	replace_static_values(template);
+	templates_replace_default_dates(template);
+	templates_replace_command(template, DOC_FILENAME(doc), doc->file_type->name, NULL);
 
-gchar *templates_get_template_fileheader(gint filetype_idx, const gchar *fname)
-{
-	gchar *template = g_strdup(templates[GEANY_TEMPLATE_FILEHEADER]);
-	gchar *shortname;
-	gchar *result;
-	gchar *date = utils_get_date_time(template_prefs.datetime_format, NULL);
-	filetype_id ft_id = filetype_idx;
-	GeanyFiletype *ft = filetypes[ft_id];
+	result = make_comment_block(template->str, FILETYPE_ID(doc->file_type), 8);
 
-	filetypes_load_config(ft_id, FALSE);	/* load any user extension setting */
+	g_string_free(template, TRUE);
 
-	if (fname == NULL)
-	{
-		if (ft_id == GEANY_FILETYPES_NONE)
-			shortname = g_strdup(GEANY_STRING_UNTITLED);
-		else
-			shortname = g_strconcat(GEANY_STRING_UNTITLED, ".", ft->extension, NULL);
-	}
-	else
-		shortname = g_path_get_basename(fname);
-
-	utils_str_replace_all(&template, "{filename}", shortname);
-	utils_str_replace_all(&template, "{gpl}", templates[GEANY_TEMPLATE_GPL]);
-	utils_str_replace_all(&template, "{bsd}", templates[GEANY_TEMPLATE_BSD]);
-	utils_str_replace_all(&template, "{datetime}", date);
-
-	result = make_comment_block(template, ft_id, 8);
-
-	g_free(template);
-	g_free(shortname);
-	g_free(date);
 	return result;
 }
 
 
-static gchar *get_file_template(GeanyFiletype *ft)
+static gchar *get_template_fileheader(GeanyFiletype *ft)
 {
-	filetype_id ft_id = FILETYPE_ID(ft);
+	GString *template = g_string_new(templates[GEANY_TEMPLATE_FILEHEADER]);
+	gchar *result;
 
-	return g_strdup(ft_templates[ft_id]);
+	filetypes_load_config(ft->id, FALSE);	/* load any user extension setting */
+
+	templates_replace_valist(template,
+		"{gpl}", templates[GEANY_TEMPLATE_GPL],
+		"{bsd}", templates[GEANY_TEMPLATE_BSD],
+		NULL);
+
+	/* we don't replace other wildcards here otherwise they would get done twice for files */
+	result = make_comment_block(template->str, ft->id, 8);
+	g_string_free(template, TRUE);
+	return result;
+}
+
+
+/* TODO change the signature to take a GeanyDocument? this would break plugin API/ABI */
+gchar *templates_get_template_fileheader(gint filetype_idx, const gchar *fname)
+{
+	GeanyFiletype *ft = filetypes[filetype_idx];
+	gchar *str = get_template_fileheader(ft);
+	GString *template = g_string_new(str);
+
+	g_free(str);
+	templates_replace_common(template, fname, ft, NULL);
+	return g_string_free(template, FALSE);
 }
 
 
 gchar *templates_get_template_new_file(GeanyFiletype *ft)
 {
-	gchar *ft_template = NULL;
+	GString *ft_template;
 	gchar *file_header = NULL;
 
+	g_return_val_if_fail(ft != NULL, NULL);
+	g_return_val_if_fail(ft->id < GEANY_MAX_BUILT_IN_FILETYPES, NULL);
+
+	ft_template = g_string_new(ft_templates[ft->id]);
 	if (FILETYPE_ID(ft) == GEANY_FILETYPES_NONE)
-		return get_file_template(ft);
+	{
+		replace_static_values(ft_template);
+	}
+	else
+	{	/* file template only used for new files */
+		file_header = get_template_fileheader(ft);
+		templates_replace_valist(ft_template, "{fileheader}", file_header, NULL);
+	}
+	templates_replace_common(ft_template, NULL, ft, NULL);
 
-	file_header = templates_get_template_fileheader(ft->id, NULL);	/* file template only used for new files */
-	ft_template = get_file_template(ft);
-	utils_str_replace_all(&ft_template, "{fileheader}", file_header);
 	g_free(file_header);
-	return ft_template;
+	return g_string_free(ft_template, FALSE);
 }
 
 
-gchar *templates_get_template_generic(gint template)
+gchar *templates_get_template_function(GeanyDocument *doc, const gchar *func_name)
 {
-	return g_strdup(templates[template]);
-}
-
-
-gchar *templates_get_template_function(gint filetype_idx, const gchar *func_name)
-{
-	gchar *template = g_strdup(templates[GEANY_TEMPLATE_FUNCTION]);
-	gchar *date = utils_get_date_time(template_prefs.date_format, NULL);
-	gchar *datetime = utils_get_date_time(template_prefs.datetime_format, NULL);
 	gchar *result;
+	GString *text;
 
-	utils_str_replace_all(&template, "{date}", date);
-	utils_str_replace_all(&template, "{datetime}", datetime);
-	utils_str_replace_all(&template, "{functionname}", (func_name) ? func_name : "");
+	func_name = (func_name != NULL) ? func_name : "";
+	text = g_string_new(templates[GEANY_TEMPLATE_FUNCTION]);
 
-	result = make_comment_block(template, filetype_idx, 3);
+	templates_replace_valist(text, "{functionname}", func_name, NULL);
+	templates_replace_default_dates(text);
+	templates_replace_command(text, DOC_FILENAME(doc), doc->file_type->name, func_name);
 
-	g_free(template);
-	g_free(date);
-	g_free(datetime);
+	result = make_comment_block(text->str, doc->file_type->id, 3);
+
+	g_string_free(text, TRUE);
 	return result;
 }
 
 
-gchar *templates_get_template_changelog(void)
+gchar *templates_get_template_changelog(GeanyDocument *doc)
 {
-	gchar *date = utils_get_date_time(template_prefs.datetime_format, NULL);
-	gchar *result = g_strdup(templates[GEANY_TEMPLATE_CHANGELOG]);
+	GString *result = g_string_new(templates[GEANY_TEMPLATE_CHANGELOG]);
+	const gchar *file_type_name = (doc != NULL) ? doc->file_type->name : "";
 
-	utils_str_replace_all(&result, "{date}", date);
+	replace_static_values(result);
+	templates_replace_default_dates(result);
+	templates_replace_command(result, DOC_FILENAME(doc), file_type_name, NULL);
 
-	g_free(date);
-	return result;
+	return g_string_free(result, FALSE);
 }
 
 
@@ -774,8 +708,11 @@ void templates_free_templates(void)
 {
 	gint i;
 	GList *children, *item;
-	GtkWidget *toolbar_new_file_menu = geany_menu_button_action_get_menu(
-					GEANY_MENU_BUTTON_ACTION(toolbar_get_action_by_name("New")));
+
+	/* disconnect the menu from the action widget, so destroying the items below doesn't
+	 * trigger rebuilding of the menu on each item destroy */
+	geany_menu_button_action_set_menu(
+		GEANY_MENU_BUTTON_ACTION(toolbar_get_action_by_name("New")), NULL);
 
 	for (i = 0; i < GEANY_MAX_TEMPLATES; i++)
 	{
@@ -786,26 +723,20 @@ void templates_free_templates(void)
 		g_free(ft_templates[i]);
 	}
 	/* destroy "New with template" sub menu items (in case we want to reload the templates) */
-	children = gtk_container_get_children(GTK_CONTAINER(toolbar_new_file_menu));
-	foreach_list(item, children)
-	{
-		gtk_widget_destroy(GTK_WIDGET(item->data));
-	}
-	g_object_unref(toolbar_new_file_menu);
 	children = gtk_container_get_children(GTK_CONTAINER(new_with_template_menu));
 	foreach_list(item, children)
 	{
 		gtk_widget_destroy(GTK_WIDGET(item->data));
 	}
+	g_list_free(children);
+
+	g_object_unref(new_with_template_menu);
+	new_with_template_menu = NULL;
 }
 
 
-void templates_replace_all(GString *text, const gchar *year, const gchar *date,
-						   const gchar *datetime)
+static void replace_static_values(GString *text)
 {
-	utils_string_replace_all(text, "{year}", year);
-	utils_string_replace_all(text, "{date}", date);
-	utils_string_replace_all(text, "{datetime}", datetime);
 	utils_string_replace_all(text, "{version}", template_prefs.version);
 	utils_string_replace_all(text, "{initial}", template_prefs.initials);
 	utils_string_replace_all(text, "{developer}", template_prefs.developer);
@@ -815,3 +746,122 @@ void templates_replace_all(GString *text, const gchar *year, const gchar *date,
 	utils_string_replace_all(text, "{geanyversion}", "Geany " VERSION);
 }
 
+
+/* Replaces all static template wildcards (version, mail, company, name, ...)
+ * plus those wildcard, value pairs which are passed, e.g.
+ *
+ * templates_replace_valist(text, "{some_wildcard}", "some value",
+ *      "{another_wildcard}", "another value", NULL);
+ *
+ * The argument list must be terminated with NULL. */
+void templates_replace_valist(GString *text, const gchar *first_wildcard, ...)
+{
+	va_list args;
+	const gchar *key, *value;
+
+	g_return_if_fail(text != NULL);
+
+	va_start(args, first_wildcard);
+
+	key = first_wildcard;
+	value = va_arg(args, gchar*);
+
+	while (key != NULL)
+	{
+		utils_string_replace_all(text, key, value);
+
+		key = va_arg(args, gchar*);
+		if (key == NULL || text == NULL)
+			break;
+		value = va_arg(args, gchar*);
+	}
+	va_end(args);
+
+	replace_static_values(text);
+}
+
+
+static void templates_replace_default_dates(GString *text)
+{
+	gchar *year = utils_get_date_time(template_prefs.year_format, NULL);
+	gchar *date = utils_get_date_time(template_prefs.date_format, NULL);
+	gchar *datetime = utils_get_date_time(template_prefs.datetime_format, NULL);
+
+	g_return_if_fail(text != NULL);
+
+	templates_replace_valist(text,
+		"{year}", year,
+		"{date}", date,
+		"{datetime}", datetime,
+		NULL);
+
+	utils_free_pointers(3, year, date, datetime, NULL);
+}
+
+
+static gchar *run_command(const gchar *command, const gchar *file_name,
+						  const gchar *file_type, const gchar *func_name)
+{
+	gchar *result = NULL;
+	gchar **argv;
+
+	if (g_shell_parse_argv(command, NULL, &argv, NULL))
+	{
+		GError *error = NULL;
+		gchar **env;
+
+		file_name = (file_name != NULL) ? file_name : "";
+		file_type = (file_type != NULL) ? file_type : "";
+		func_name = (func_name != NULL) ? func_name : "";
+
+		env = utils_copy_environment(NULL,
+			"GEANY_FILENAME", file_name,
+			"GEANY_FILETYPE", file_type,
+			"GEANY_FUNCNAME", func_name,
+			NULL);
+		if (! utils_spawn_sync(NULL, argv, env, G_SPAWN_SEARCH_PATH,
+				NULL, NULL, &result, NULL, NULL, &error))
+		{
+			g_warning("templates_replace_command: %s", error->message);
+			g_error_free(error);
+			return NULL;
+		}
+		g_strfreev(argv);
+		g_strfreev(env);
+	}
+	return result;
+}
+
+
+static void templates_replace_command(GString *text, const gchar *file_name,
+							   const gchar *file_type, const gchar *func_name)
+{
+	gchar *match = NULL;
+	gchar *wildcard = NULL;
+	gchar *cmd;
+	gchar *result;
+
+	g_return_if_fail(text != NULL);
+
+	while ((match = strstr(text->str, "{command:")) != NULL)
+	{
+		cmd = match;
+		while (*match != '}' && *match != '\0')
+			match++;
+
+		wildcard = g_strndup(cmd, match - cmd + 1);
+		cmd = g_strndup(wildcard + 9, strlen(wildcard) - 10);
+
+		result = run_command(cmd, file_name, file_type, func_name);
+		if (result != NULL)
+		{
+			utils_string_replace_first(text, wildcard, result);
+			g_free(result);
+		}
+		else
+			utils_string_replace_first(text, wildcard, "");
+
+		g_free(wildcard);
+		g_free(cmd);
+	}
+}
