@@ -1,8 +1,8 @@
 /*
  *      templates.h - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2005-2008 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2006-2008 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2005-2010 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2006-2010 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -18,9 +18,15 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: templates.h 2150 2008-01-06 18:11:57Z eht16 $
+ * $Id: templates.h 4852 2010-04-22 13:01:45Z ntrel $
  */
 
+/**
+ * @file templates.h
+ * Templates (prefs).
+ **/
+
+
 #ifndef GEANY_TEMPLATES_H
 #define GEANY_TEMPLATES_H 1
 
@@ -34,21 +40,43 @@ enum
 	GEANY_MAX_TEMPLATES
 };
 
+
+/** Template preferences. */
+typedef struct GeanyTemplatePrefs
+{
+	gchar			*developer;	/**< Name */
+	gchar			*company;	/**< Company */
+	gchar			*mail;		/**< Email */
+	gchar			*initials;	/**< Initials */
+	gchar			*version;	/**< Initial version */
+	gchar			*year_format;
+	gchar			*date_format;
+	gchar			*datetime_format;
+}
+GeanyTemplatePrefs;
+
+extern GeanyTemplatePrefs template_prefs;
+
+
 struct filetype;
 
 void templates_init(void);
 
 gchar *templates_get_template_fileheader(gint filetype_idx, const gchar *fname);
 
-gchar *templates_get_template_new_file(struct filetype *ft);
+gchar *templates_get_template_new_file(GeanyFiletype *ft);
 
-gchar *templates_get_template_changelog(void);
+gchar *templates_get_template_changelog(GeanyDocument *doc);
 
-gchar *templates_get_template_generic(gint template);
+gchar *templates_get_template_function(GeanyDocument *doc, const gchar *func_name);
 
-gchar *templates_get_template_function(gint filetype_idx, const gchar *func_name);
+gchar *templates_get_template_licence(GeanyDocument *doc, gint licence_type);
 
-gchar *templates_get_template_licence(gint filetype_idx, gint licence_type);
+void templates_replace_common(GString *template, const gchar *fname,
+	GeanyFiletype *ft, const gchar *func_name);
+
+void templates_replace_valist(GString *text,
+	const gchar *first_wildcard, ...) G_GNUC_NULL_TERMINATED;
 
 void templates_free_templates(void);
 
