@@ -1,8 +1,8 @@
 /*
  *      toolbar.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2009 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2009 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2009-2010 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2009-2010 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -18,13 +18,14 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: toolbar.c 4012 2009-07-23 11:00:32Z ntrel $
+ * $Id: toolbar.c 4941 2010-05-22 19:39:03Z eht16 $
  */
 
-
-/** @file toolbar.c
- * Utility functions to create the toolbar.
+/**
+ * @file toolbar.h
+ * Toolbar (prefs).
  */
+/* Utility functions to create the toolbar */
 
 #include "geany.h"
 #include "support.h"
@@ -287,9 +288,10 @@ static GtkWidget *toolbar_reload(const gchar *markup)
 	if (main_status.main_window_realized)
 	{
 		GeanyDocument *doc = document_get_current();
+		gboolean doc_changed = (doc != NULL) ? doc->changed : FALSE;
 
 		ui_document_buttons_update();
-		ui_save_buttons_toggle(doc->changed); /* update save all */
+		ui_save_buttons_toggle(doc_changed); /* update save all */
 		ui_update_popup_reundo_items(doc);
 
 		toolbar_apply_settings();
@@ -315,6 +317,24 @@ static GtkWidget *toolbar_reload(const gchar *markup)
 }
 
 
+static void toolbar_notify_style_cb(GObject *object, GParamSpec *arg1, gpointer data)
+{
+	const gchar *arg_name = g_param_spec_get_name(arg1);
+	gint value;
+
+	if (toolbar_prefs.use_gtk_default_style && utils_str_equal(arg_name, "gtk-toolbar-style"))
+	{
+		value = ui_get_gtk_settings_integer(arg_name, toolbar_prefs.icon_style);
+		gtk_toolbar_set_style(GTK_TOOLBAR(main_widgets.toolbar), value);
+	}
+	else if (toolbar_prefs.use_gtk_default_icon && utils_str_equal(arg_name, "gtk-toolbar-size"))
+	{
+		value = ui_get_gtk_settings_integer(arg_name, toolbar_prefs.icon_size);
+		gtk_toolbar_set_icon_size(GTK_TOOLBAR(main_widgets.toolbar), value);
+	}
+}
+
+
 GtkWidget *toolbar_init(void)
 {
 	GtkWidget *toolbar;
@@ -323,6 +343,7 @@ GtkWidget *toolbar_init(void)
 	GtkAction *action_build;
 	GtkAction *action_searchentry;
 	GtkAction *action_gotoentry;
+	GtkSettings *gtk_settings;
 
 	uim = gtk_ui_manager_new();
 	group = gtk_action_group_new("GeanyToolbar");
@@ -331,17 +352,27 @@ GtkWidget *toolbar_init(void)
 	gtk_action_group_add_actions(group, ui_entries, ui_entries_n, NULL);
 
 	/* Create our custom actions */
-	action_new = geany_menu_button_action_new("New", NULL, _("Create a new file"), GTK_STOCK_NEW);
+	action_new = geany_menu_button_action_new(
+		"New", NULL,
+		_("Create a new file"),
+		_("Create a new file from a template"),
+		GTK_STOCK_NEW);
 	g_signal_connect(action_new, "button-clicked", G_CALLBACK(on_toolbutton_new_clicked), NULL);
 	gtk_action_group_add_action(group, action_new);
 
 	action_open = geany_menu_button_action_new(
-		"Open", NULL, _("Open an existing file"), GTK_STOCK_OPEN);
+		"Open", NULL,
+		_("Open an existing file"),
+		_("Open a recent file"),
+		GTK_STOCK_OPEN);
 	g_signal_connect(action_open, "button-clicked", G_CALLBACK(on_toolbutton_open_clicked), NULL);
 	gtk_action_group_add_action(group, action_open);
 
 	action_build = geany_menu_button_action_new(
-		"Build", NULL, _("Build the current file"), GEANY_STOCK_BUILD);
+		"Build", NULL,
+		_("Build the current file"),
+		_("Choose more build actions"),
+		GEANY_STOCK_BUILD);
 	g_signal_connect(action_build, "button-clicked",
 		G_CALLBACK(build_toolbutton_build_clicked), NULL);
 	gtk_action_group_add_action(group, action_build);
@@ -364,6 +395,13 @@ GtkWidget *toolbar_init(void)
 
 	toolbar = toolbar_reload(NULL);
 
+	gtk_settings = gtk_widget_get_settings(GTK_WIDGET(toolbar));
+	if (gtk_settings != NULL)
+	{
+		g_signal_connect(gtk_settings, "notify::gtk-toolbar-style",
+			G_CALLBACK(toolbar_notify_style_cb), NULL);
+	}
+
 	return toolbar;
 }
 
@@ -372,17 +410,20 @@ void toolbar_update_ui(void)
 {
 	static GtkWidget *hbox_menubar = NULL;
 	static GtkWidget *menubar = NULL;
-	static GtkWidget *menubar_toolbar_separator = NULL;
+	GtkWidget *menubar_toolbar_separator = NULL;
 	GtkWidget *parent;
+	GtkToolItem *first_item;
 
 	if (menubar == NULL)
 	{	/* cache widget pointers */
 		hbox_menubar = ui_lookup_widget(main_widgets.window, "hbox_menubar");
 		menubar = ui_lookup_widget(main_widgets.window, "menubar1");
-
-		menubar_toolbar_separator = GTK_WIDGET(gtk_separator_tool_item_new());
-		gtk_toolbar_insert(GTK_TOOLBAR(main_widgets.toolbar),
-			GTK_TOOL_ITEM(menubar_toolbar_separator), 0);
+	}
+	/* the separator between the menubar and the toolbar */
+	first_item = gtk_toolbar_get_nth_item(GTK_TOOLBAR(main_widgets.toolbar), 0);
+	if (first_item != NULL && GTK_IS_SEPARATOR_TOOL_ITEM(first_item))
+	{
+		gtk_widget_destroy(GTK_WIDGET(first_item));
 	}
 
 	parent = gtk_widget_get_parent(main_widgets.toolbar);
@@ -405,6 +446,12 @@ void toolbar_update_ui(void)
 		}
 		else
 			gtk_box_pack_start(GTK_BOX(hbox_menubar), main_widgets.toolbar, TRUE, TRUE, 0);
+
+		/* the separator between the menubar and the toolbar */
+		menubar_toolbar_separator = GTK_WIDGET(gtk_separator_tool_item_new());
+		gtk_widget_show(menubar_toolbar_separator);
+		gtk_toolbar_insert(GTK_TOOLBAR(main_widgets.toolbar),
+			GTK_TOOL_ITEM(menubar_toolbar_separator), 0);
 	}
 	else
 	{
@@ -429,8 +476,6 @@ void toolbar_update_ui(void)
 			gtk_box_reorder_child(GTK_BOX(box), main_widgets.toolbar, 1);
 		}
 	}
-	/* the separator between the menubar and the toolbar */
-	ui_widget_show_hide(menubar_toolbar_separator, toolbar_prefs.append_to_menu);
 	/* we need to adjust the packing flags for the menubar to expand it if it is alone in the
 	 * hbox and not expand it if the toolbar is appended */
 	gtk_box_set_child_packing(GTK_BOX(hbox_menubar), menubar,
@@ -473,8 +518,9 @@ gint toolbar_get_insert_position(void)
 
 void toolbar_finalize(void)
 {
-	g_object_unref(geany_menu_button_action_get_menu(
-					GEANY_MENU_BUTTON_ACTION(toolbar_get_action_by_name("Open"))));
+	GeanyMenubuttonAction *open_action = GEANY_MENU_BUTTON_ACTION(toolbar_get_action_by_name("Open"));
+	g_object_unref(geany_menu_button_action_get_menu(open_action));
+	geany_menu_button_action_set_menu(open_action, NULL);
 
 	/* unref'ing the GtkUIManager object will destroy all its widgets unless they were ref'ed */
 	g_object_unref(uim);
@@ -494,39 +540,38 @@ void toolbar_show_hide(void)
 }
 
 
+/* sets the icon style of the toolbar */
+static void toolbar_set_icon_style(void)
+{
+	gint icon_style;
+
+	icon_style = toolbar_prefs.icon_style;
+
+	if (toolbar_prefs.use_gtk_default_style)
+		icon_style = ui_get_gtk_settings_integer("gtk-toolbar-style", toolbar_prefs.icon_style);
+
+	gtk_toolbar_set_style(GTK_TOOLBAR(main_widgets.toolbar), icon_style);
+}
+
+
+/* sets the icon size of the toolbar */
+static void toolbar_set_icon_size(void)
+{
+	gint icon_size;
+
+	icon_size = toolbar_prefs.icon_size;
+
+	if (toolbar_prefs.use_gtk_default_icon)
+		icon_size = ui_get_gtk_settings_integer("gtk-toolbar-icon-size", toolbar_prefs.icon_size);
+
+	gtk_toolbar_set_icon_size(GTK_TOOLBAR(main_widgets.toolbar), icon_size);
+}
+
+
 void toolbar_apply_settings(void)
 {
-	/* sets the icon style of the toolbar */
-	switch (toolbar_prefs.icon_style)
-	{
-		case GTK_TOOLBAR_BOTH:
-		{
-			/*gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(ui_lookup_widget(main_widgets.window, "images_and_text1")), TRUE);*/
-			gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(ui_lookup_widget(ui_widgets.toolbar_menu, "images_and_text2")), TRUE);
-			break;
-		}
-		case GTK_TOOLBAR_ICONS:
-		{
-			/*gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(ui_lookup_widget(main_widgets.window, "images_only1")), TRUE);*/
-			gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(ui_lookup_widget(ui_widgets.toolbar_menu, "images_only2")), TRUE);
-			break;
-		}
-		case GTK_TOOLBAR_TEXT:
-		{
-			/*gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(ui_lookup_widget(main_widgets.window, "text_only1")), TRUE);*/
-			gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(ui_lookup_widget(ui_widgets.toolbar_menu, "text_only2")), TRUE);
-			break;
-		}
-	}
-	gtk_toolbar_set_style(GTK_TOOLBAR(main_widgets.toolbar), toolbar_prefs.icon_style);
-
-	/* sets the icon size of the toolbar, use user preferences (.gtkrc) if not set */
-	if (toolbar_prefs.icon_size == GTK_ICON_SIZE_SMALL_TOOLBAR ||
-		toolbar_prefs.icon_size == GTK_ICON_SIZE_LARGE_TOOLBAR ||
-		toolbar_prefs.icon_size == GTK_ICON_SIZE_MENU)
-	{
-		gtk_toolbar_set_icon_size(GTK_TOOLBAR(main_widgets.toolbar), toolbar_prefs.icon_size);
-	}
+	toolbar_set_icon_style();
+	toolbar_set_icon_size();
 }
 
 
@@ -583,6 +628,7 @@ static void tb_editor_handler_start_element(GMarkupParseContext *context, const
 	}
 }
 
+
 static const GMarkupParser tb_editor_xml_parser =
 {
 	tb_editor_handler_start_element, NULL, NULL, NULL, NULL
@@ -870,7 +916,7 @@ static void tb_editor_available_items_deleted_cb(GtkTreeModel *model, GtkTreePat
 }
 
 
-static TBEditorWidget *tb_editor_create_dialog(void)
+static TBEditorWidget *tb_editor_create_dialog(GtkWindow *parent)
 {
 	GtkWidget *dialog, *vbox, *hbox, *vbox_buttons, *button_add, *button_remove;
 	GtkWidget *swin_available, *swin_used, *tree_available, *tree_used, *label;
@@ -878,8 +924,11 @@ static TBEditorWidget *tb_editor_create_dialog(void)
 	GtkTreeViewColumn *column;
 	TBEditorWidget *tbw = g_new(TBEditorWidget, 1);
 
+	if (parent == NULL)
+		parent = GTK_WINDOW(main_widgets.window);
+
 	dialog = gtk_dialog_new_with_buttons(_("Customize Toolbar"),
-				GTK_WINDOW(main_widgets.window),
+				parent,
 				GTK_DIALOG_DESTROY_WITH_PARENT,
 				GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE, NULL);
 	vbox = ui_dialog_vbox_new(GTK_DIALOG(dialog));
@@ -999,7 +1048,7 @@ static TBEditorWidget *tb_editor_create_dialog(void)
 }
 
 
-void toolbar_configure(void)
+void toolbar_configure(GtkWindow *parent)
 {
 	gchar *markup;
 	const gchar *name;
@@ -1018,7 +1067,7 @@ void toolbar_configure(void)
 	all_items = gtk_action_group_list_actions(group);
 
 	/* create the GUI */
-	tbw = tb_editor_create_dialog();
+	tbw = tb_editor_create_dialog(parent);
 
 	/* fill the stores */
 	gtk_list_store_insert_with_values(tbw->store_available, NULL, -1,
