@@ -1,8 +1,8 @@
 /*
  *      ui_utils.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2006-2009 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2006-2009 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2006-2010 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2006-2010 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: ui_utils.c 3898 2009-06-25 17:21:45Z eht16 $
+ * $Id: ui_utils.c 5149 2010-08-12 12:51:51Z ntrel $
  */
 
 /** @file ui_utils.h
@@ -28,6 +28,8 @@
 #include "geany.h"
 
 #include <string.h>
+#include <ctype.h>
+#include <gdk/gdkkeysyms.h>
 
 #include "ui_utils.h"
 #include "prefs.h"
@@ -41,7 +43,7 @@
 #include "callbacks.h"
 #include "encodings.h"
 #include "images.c"
-#include "treeviews.h"
+#include "sidebar.h"
 #include "win32.h"
 #include "project.h"
 #include "editor.h"
@@ -70,6 +72,8 @@ static struct
 	GtkWidget	*undo_items[3];
 	GtkWidget	*save_buttons[4];
 	GtkWidget	*config_files_menu;
+	GtkWidget	*commands_menu;
+	GtkWidget	*format_menu;
 }
 widgets;
 
@@ -132,16 +136,16 @@ static void set_statusbar(const gchar *text, gboolean allow_override)
 }
 
 
-/** Display text on the statusbar.
+/** Displays text on the statusbar.
  * @param log Whether the message should be recorded in the Status window.
  * @param format A @c printf -style string. */
 void ui_set_statusbar(gboolean log, const gchar *format, ...)
 {
-	gchar string[512];
+	gchar *string;
 	va_list args;
 
 	va_start(args, format);
-	g_vsnprintf(string, 512, format, args);
+	string = g_strdup_vprintf(format, args);
 	va_end(args);
 
 	if (! prefs.suppress_status_messages)
@@ -149,6 +153,8 @@ void ui_set_statusbar(gboolean log, const gchar *format, ...)
 
 	if (log || prefs.suppress_status_messages)
 		msgwin_status_add("%s", string);
+
+	g_free(string);
 }
 
 
@@ -185,8 +191,8 @@ void ui_update_statusbar(GeanyDocument *doc, gint pos)
 			col = 0;
 
 		/* Status bar statistics: col = column, sel = selection. */
-		g_string_printf(stats_str, _("line: %d\t col: %d\t sel: %d\t "),
-			(line + 1), col,
+		g_string_printf(stats_str, _("line: %d / %d\t col: %d\t sel: %d\t "),
+			(line + 1), sci_get_line_count(doc->editor->sci), col,
 			sci_get_selected_text_length(doc->editor->sci) - 1);
 
 		g_string_append(stats_str,
@@ -646,6 +652,7 @@ static void init_document_widgets(void)
 	add_doc_widget("menu_select_all1");
 	add_doc_widget("insert_date1");
 	add_doc_widget("menu_format1");
+	add_doc_widget("commands2");
 	add_doc_widget("menu_open_selected_file1");
 	add_doc_widget("page_setup1");
 	add_doc_widget("find1");
@@ -677,6 +684,8 @@ static void init_document_widgets(void)
 	add_doc_toolitem("Color");
 	add_doc_toolitem("Goto");
 	add_doc_toolitem("GotoEntry");
+	add_doc_toolitem("Replace");
+	add_doc_toolitem("Print");
 }
 
 
@@ -702,7 +711,7 @@ static void on_doc_sensitive_widget_destroy(GtkWidget *widget, G_GNUC_UNUSED gpo
 }
 
 
-/** Add a widget to the list of widgets that should be set sensitive/insensitive
+/** Adds a widget to the list of widgets that should be set sensitive/insensitive
  * when some documents are present/no documents are open.
  * It will be removed when the widget is destroyed.
  * @param widget The widget to add.
@@ -764,7 +773,7 @@ void ui_sidebar_show_hide(void)
 
 void ui_document_show_hide(GeanyDocument *doc)
 {
-	gchar *widget_name;
+	const gchar *widget_name;
 	GtkWidget *item;
 	const GeanyIndentPrefs *iprefs;
 
@@ -870,7 +879,7 @@ static gboolean have_tango_icon_theme(void)
 /* Note: remember to unref the pixbuf once an image or window has added a reference. */
 GdkPixbuf *ui_new_pixbuf_from_inline(gint img)
 {
-	switch(img)
+	switch (img)
 	{
 		case GEANY_IMAGE_LOGO:
 			return gdk_pixbuf_new_from_inline(-1, aladin_inline, FALSE, NULL);
@@ -1109,6 +1118,7 @@ static void recent_file_loaded(const gchar *utf8_filename, GeanyRecentFiles *grf
 	item = g_list_find_custom(children, utf8_filename, (GCompareFunc) find_recent_file_item);
 	if (item != NULL)
 		gtk_widget_destroy(GTK_WIDGET(item->data));
+	g_list_free(children);
 
 	if (grf->toolbar != NULL)
 	{
@@ -1116,6 +1126,7 @@ static void recent_file_loaded(const gchar *utf8_filename, GeanyRecentFiles *grf
 		item = g_list_find_custom(children, utf8_filename, (GCompareFunc) find_recent_file_item);
 		if (item != NULL)
 			gtk_widget_destroy(GTK_WIDGET(item->data));
+		g_list_free(children);
 	}
 	/* now prepend a new menuitem for the filename,
 	 * first for the recent files menu in the menu bar */
@@ -1158,6 +1169,8 @@ static void update_recent_menu(GeanyRecentFiles *grf)
 			item = g_list_next(item);
 		}
 	}
+	g_list_free(children);
+
 	/* create item for the menu bar menu */
 	tmp = gtk_menu_item_new_with_label(filename);
 	gtk_widget_show(tmp);
@@ -1178,6 +1191,8 @@ static void update_recent_menu(GeanyRecentFiles *grf)
 				item = g_list_next(item);
 			}
 		}
+		g_list_free(children);
+
 		/* create item for the tool bar menu */
 		tmp = gtk_menu_item_new_with_label(filename);
 		gtk_widget_show(tmp);
@@ -1193,7 +1208,7 @@ void ui_toggle_editor_features(GeanyUIEditorFeatures feature)
 	gint i, max = gtk_notebook_get_n_pages(GTK_NOTEBOOK(main_widgets.notebook));
 	GeanyDocument *doc;
 
-	for(i = 0; i < max; i++)
+	for (i = 0; i < max; i++)
 	{
 		doc = document_get_from_page(i);
 
@@ -1238,43 +1253,36 @@ void ui_update_view_editor_menu_items(void)
 GtkWidget *ui_frame_new_with_alignment(const gchar *label_text, GtkWidget **alignment)
 {
 	GtkWidget *label, *align;
-	GtkWidget *frame = gtk_frame_new (NULL);
-	gchar *label_markup;
+	GtkWidget *frame = gtk_frame_new(NULL);
 
-	gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_NONE);
+	gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_NONE);
 
-	align = gtk_alignment_new (0.5, 0.5, 1, 1);
-	gtk_container_add (GTK_CONTAINER (frame), align);
-	gtk_alignment_set_padding (GTK_ALIGNMENT (align), 0, 0, 12, 0);
+	align = gtk_alignment_new(0.5, 0.5, 1, 1);
+	gtk_container_add(GTK_CONTAINER(frame), align);
+	gtk_alignment_set_padding(GTK_ALIGNMENT(align), 0, 0, 12, 0);
 
-	label_markup = g_strconcat("<b>", label_text, "</b>", NULL);
-	label = gtk_label_new (label_markup);
-	gtk_frame_set_label_widget (GTK_FRAME (frame), label);
-	gtk_label_set_use_markup (GTK_LABEL (label), TRUE);
-	g_free(label_markup);
+	label = ui_label_new_bold(label_text);
+	gtk_frame_set_label_widget(GTK_FRAME(frame), label);
 
 	*alignment = align;
 	return frame;
 }
 
 
-const gint BUTTON_BOX_BORDER = 5;
-
-/** Convenience function for getting a fixed border for dialogs that doesn't
- * increase the button box border.
+/** Makes a fixed border for dialogs without increasing the button box border.
  * @param dialog The parent container for the @c GtkVBox.
  * @return The packed @c GtkVBox. */
 GtkWidget *ui_dialog_vbox_new(GtkDialog *dialog)
 {
 	GtkWidget *vbox = gtk_vbox_new(FALSE, 12);	/* need child vbox to set a separate border. */
 
-	gtk_container_set_border_width(GTK_CONTAINER(vbox), BUTTON_BOX_BORDER);
+	gtk_container_set_border_width(GTK_CONTAINER(vbox), 6);
 	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), vbox);
 	return vbox;
 }
 
 
-/** Create a @c GtkButton with custom text and a stock image, aligned like
+/** Creates a @c GtkButton with custom text and a stock image similar to
  * @c gtk_button_new_from_stock().
  * @param stock_id A @c GTK_STOCK_NAME string.
  * @param text Button label text, can include mnemonics.
@@ -1282,24 +1290,18 @@ GtkWidget *ui_dialog_vbox_new(GtkDialog *dialog)
  */
 GtkWidget *ui_button_new_with_image(const gchar *stock_id, const gchar *text)
 {
-	GtkWidget *image, *label, *align, *hbox, *button;
+	GtkWidget *image, *button;
 
-	hbox = gtk_hbox_new(FALSE, 2);
+	button = gtk_button_new_with_mnemonic(text);
+	gtk_widget_show(button);
 	image = gtk_image_new_from_stock(stock_id, GTK_ICON_SIZE_BUTTON);
-	label = gtk_label_new_with_mnemonic(text);
-	gtk_box_pack_start(GTK_BOX(hbox), image, FALSE, FALSE, 0);
-	gtk_box_pack_end(GTK_BOX(hbox), label, FALSE, FALSE, 0);
-
-	button = gtk_button_new();
-	align = gtk_alignment_new(0.5, 0.5, 0.0, 0.0);
-	gtk_container_add(GTK_CONTAINER(align), hbox);
-	gtk_container_add(GTK_CONTAINER(button), align);
-	gtk_widget_show_all(align);
+	gtk_button_set_image(GTK_BUTTON(button), image);
+	/* note: image is shown by gtk */
 	return button;
 }
 
 
-/** Create a @c GtkImageMenuItem with a stock image and a custom label.
+/** Creates a @c GtkImageMenuItem with a stock image and a custom label.
  * @param stock_id Stock image ID, e.g. @c GTK_STOCK_OPEN.
  * @param label Menu item label, can include mnemonics.
  * @return The new @c GtkImageMenuItem.
@@ -1324,11 +1326,12 @@ static void entry_clear_icon_release_cb(GtkEntry *entry, gint icon_pos,
 	if (event->button.button == 1 && icon_pos == 1)
 	{
 		gtk_entry_set_text(entry, "");
+		gtk_widget_grab_focus(GTK_WIDGET(entry));
 	}
 }
 
 
-/** Convenience function to add a small clear icon to the right end of the passed @a entry.
+/** Adds a small clear icon to the right end of the passed @a entry.
  *  A callback to clear the contents of the GtkEntry is automatically added.
  *
  *  This feature is only available with GTK 2.16 but implemented as a runtime check,
@@ -1339,7 +1342,7 @@ static void entry_clear_icon_release_cb(GtkEntry *entry, gint icon_pos,
  *
  *  @since 0.16
  */
-void ui_entry_add_clear_icon(GtkWidget *entry)
+void ui_entry_add_clear_icon(GtkEntry *entry)
 {
 	if (gtk_check_version(2, 15, 2) == NULL)
 	{
@@ -1445,7 +1448,7 @@ void ui_update_tab_status(GeanyDocument *doc)
 	gtk_widget_modify_fg(doc->priv->tab_label, GTK_STATE_NORMAL, color);
 	gtk_widget_modify_fg(doc->priv->tab_label, GTK_STATE_ACTIVE, color);
 
-	treeviews_openfiles_update(doc);
+	sidebar_openfiles_update(doc);
 }
 
 
@@ -1488,7 +1491,7 @@ static gboolean tree_view_find(GtkTreeView *treeview, TVMatchCallback cb, gboole
 	while (TRUE)
 	{
 		gtk_tree_selection_select_iter(treesel, &iter);
-		if (cb())
+		if (cb(0))
 			break;	/* found next message */
 
 		if (! tree_model_iter_get_next(model, &iter, down))
@@ -1521,12 +1524,18 @@ gboolean ui_tree_view_find_previous(GtkTreeView *treeview, TVMatchCallback cb)
 }
 
 
-void ui_widget_modify_font_from_string(GtkWidget *wid, const gchar *str)
+/**
+ * Modifies the font of a widget using gtk_widget_modify_font().
+ *
+ * @param widget The widget.
+ * @param str The font name as expected by pango_font_description_from_string().
+ */
+void ui_widget_modify_font_from_string(GtkWidget *widget, const gchar *str)
 {
 	PangoFontDescription *pfd;
 
 	pfd = pango_font_description_from_string(str);
-	gtk_widget_modify_font(wid, pfd);
+	gtk_widget_modify_font(widget, pfd);
 	pango_font_description_free(pfd);
 }
 
@@ -1610,6 +1619,11 @@ static gchar *run_file_chooser(const gchar *title, GtkFileChooserAction action,
 		if (g_path_is_absolute(locale_path) && g_file_test(locale_path, G_FILE_TEST_IS_DIR))
 			gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(dialog), locale_path);
 	}
+	else if (action == GTK_FILE_CHOOSER_ACTION_OPEN)
+	{
+		if (g_path_is_absolute(locale_path))
+			gtk_file_chooser_set_filename(GTK_FILE_CHOOSER(dialog), locale_path);
+	}
 	g_free(locale_path);
 
 	if (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_OK)
@@ -1634,21 +1648,36 @@ static void ui_path_box_open_clicked(GtkButton *button, gpointer user_data)
 	GtkEntry *entry =
 		(GtkEntry *) g_object_get_data(G_OBJECT(path_box), "entry");
 	const gchar *title = g_object_get_data(G_OBJECT(path_box), "title");
-	gchar *utf8_path;
+	gchar *utf8_path = NULL;
 
 	/* TODO: extend for other actions */
-	g_return_if_fail(action == GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER);
+	g_return_if_fail(action == GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER ||
+					 action == GTK_FILE_CHOOSER_ACTION_OPEN);
 
 	if (title == NULL)
 		title = (action == GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER) ?
 			_("Select Folder") : _("Select File");
 
+	if (action == GTK_FILE_CHOOSER_ACTION_OPEN)
+	{
 #ifdef G_OS_WIN32
-	utf8_path = win32_show_project_folder_dialog(ui_widgets.prefs_dialog, title,
+		utf8_path = win32_show_file_dialog(GTK_WINDOW(ui_widgets.prefs_dialog), title,
 						gtk_entry_get_text(GTK_ENTRY(entry)));
 #else
-	utf8_path = run_file_chooser(title, action, gtk_entry_get_text(GTK_ENTRY(entry)));
+		utf8_path = run_file_chooser(title, action, gtk_entry_get_text(GTK_ENTRY(entry)));
 #endif
+	}
+	else if (action == GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER)
+	{
+		gchar *path = g_path_get_dirname(gtk_entry_get_text(GTK_ENTRY(entry)));
+#ifdef G_OS_WIN32
+		utf8_path = win32_show_folder_dialog(ui_widgets.prefs_dialog, title,
+						gtk_entry_get_text(GTK_ENTRY(entry)));
+#else
+		utf8_path = run_file_chooser(title, action, path);
+#endif
+		g_free(path);
+	}
 
 	if (utf8_path != NULL)
 	{
@@ -1671,7 +1700,7 @@ void ui_statusbar_showhide(gboolean state)
 }
 
 
-/** Pack all @c GtkWidgets passed after the row argument into a table, using
+/** Packs all @c GtkWidgets passed after the row argument into a table, using
  * one widget per cell. The first widget is not expanded as the table grows,
  * as this is usually a label.
  * @param table
@@ -1707,7 +1736,7 @@ static void on_config_file_clicked(GtkWidget *widget, gpointer user_data)
 		document_open_file(file_name, FALSE, ft, NULL);
 	else
 	{
-		gchar *utf8 = utils_get_utf8_from_locale(file_name);
+		gchar *utf8_filename = utils_get_utf8_from_locale(file_name);
 		gchar *base_name = g_path_get_basename(file_name);
 		gchar *global_file = g_build_filename(app->datadir, base_name, NULL);
 		gchar *global_content = NULL;
@@ -1717,8 +1746,9 @@ static void on_config_file_clicked(GtkWidget *widget, gpointer user_data)
 		if (g_file_test(global_file, G_FILE_TEST_EXISTS))
 			g_file_get_contents(global_file, &global_content, NULL, NULL);
 
-		document_new_file(utf8, ft, global_content);
-		utils_free_pointers(4, utf8, base_name, global_file, global_content, NULL);
+		document_new_file(utf8_filename, ft, global_content);
+
+		utils_free_pointers(4, utf8_filename, base_name, global_file, global_content, NULL);
 	}
 }
 
@@ -1764,10 +1794,8 @@ static void create_config_files_menu(void)
 
 	widgets.config_files_menu = menu = gtk_menu_new();
 
-	item = ui_image_menu_item_new(GTK_STOCK_FILE, _("C_onfiguration Files"));
-	gtk_widget_show(item);
+	item = ui_lookup_widget(main_widgets.window, "configuration_files1");
 	gtk_menu_item_set_submenu(GTK_MENU_ITEM(item), menu);
-	gtk_container_add(GTK_CONTAINER(main_widgets.tools_menu), item);
 
 	/* sort menu after all items added */
 	g_idle_add(sort_menu, widgets.config_files_menu);
@@ -1813,6 +1841,24 @@ void ui_init_toolbar_widgets(void)
 }
 
 
+void ui_swap_sidebar_pos(void)
+{
+	GtkWidget *pane = ui_lookup_widget(main_widgets.window, "hpaned1");
+	GtkWidget *left = gtk_paned_get_child1(GTK_PANED(pane));
+	GtkWidget *right = gtk_paned_get_child2(GTK_PANED(pane));
+	GtkWidget *box = ui_lookup_widget(main_widgets.window, "vbox1");
+
+	/* reparenting avoids scintilla problem with middle click paste */
+	gtk_widget_reparent(left, box);
+	gtk_widget_reparent(right, box);
+	gtk_widget_reparent(right, pane);
+	gtk_widget_reparent(left, pane);
+
+	gtk_paned_set_position(GTK_PANED(pane), pane->allocation.width
+		- gtk_paned_get_position(GTK_PANED(pane)));
+}
+
+
 static void init_recent_files(void)
 {
 	GtkWidget *toolbar_recent_files_menu;
@@ -1831,8 +1877,45 @@ static void init_recent_files(void)
 }
 
 
+static void ui_menu_move(GtkWidget *menu, GtkWidget *old, GtkWidget *new)
+{
+	g_object_ref(menu);
+	gtk_menu_item_set_submenu(GTK_MENU_ITEM(old), NULL);
+	gtk_menu_item_set_submenu(GTK_MENU_ITEM(new), menu);
+	g_object_unref(menu);
+}
+
+
+static void on_editor_menu_show(GtkWidget *item)
+{
+	GtkWidget *popup = ui_lookup_widget(main_widgets.editor_menu, "commands1");
+	GtkWidget *bar = ui_lookup_widget(main_widgets.window, "commands2");
+
+	ui_menu_move(widgets.commands_menu, bar, popup);
+
+	popup = ui_lookup_widget(main_widgets.editor_menu, "menu_format2");
+	bar = ui_lookup_widget(main_widgets.window, "menu_format1");
+	ui_menu_move(widgets.format_menu, bar, popup);
+}
+
+
+static void on_editor_menu_hide(GtkWidget *item)
+{
+	GtkWidget *popup = ui_lookup_widget(main_widgets.editor_menu, "commands1");
+	GtkWidget *bar = ui_lookup_widget(main_widgets.window, "commands2");
+
+	ui_menu_move(widgets.commands_menu, popup, bar);
+
+	popup = ui_lookup_widget(main_widgets.editor_menu, "menu_format2");
+	bar = ui_lookup_widget(main_widgets.window, "menu_format1");
+	ui_menu_move(widgets.format_menu, popup, bar);
+}
+
+
 void ui_init(void)
 {
+	GtkWidget *item;
+
 	init_recent_files();
 
 	ui_widgets.statusbar = ui_lookup_widget(main_widgets.window, "statusbar");
@@ -1859,6 +1942,16 @@ void ui_init(void)
 	widgets.undo_items[0] = ui_lookup_widget(main_widgets.editor_menu, "undo1");
 	widgets.undo_items[1] = ui_lookup_widget(main_widgets.window, "menu_undo2");
 
+	item = ui_lookup_widget(main_widgets.window, "menu_format1");
+	widgets.format_menu = gtk_menu_item_get_submenu(GTK_MENU_ITEM(item));
+	item = ui_lookup_widget(main_widgets.window, "commands2");
+	widgets.commands_menu = gtk_menu_item_get_submenu(GTK_MENU_ITEM(item));
+
+	/* reparent edit submenus as needed */
+	item = main_widgets.editor_menu;
+	g_signal_connect(item, "show", G_CALLBACK(on_editor_menu_show), NULL);
+	g_signal_connect(item, "hide", G_CALLBACK(on_editor_menu_hide), NULL);
+
 	ui_init_toolbar_widgets();
 	init_document_widgets();
 	create_config_files_menu();
@@ -1943,7 +2036,7 @@ void ui_widget_set_tooltip_text(GtkWidget *widget, const gchar *text)
 }
 
 
-/** This function returns a widget from a name in a component, usually created by Glade.
+/** Returns a widget from a name in a component, usually created by Glade.
  * Call it with the toplevel widget in the component (i.e. a window/dialog),
  * or alternatively any widget in the component, and the name of the widget
  * you want returned.
@@ -1981,7 +2074,6 @@ GtkWidget *ui_lookup_widget(GtkWidget *widget, const gchar *widget_name)
 }
 
 
-
 /* Progress Bar */
 static guint progress_bar_timer_id = (guint) -1;
 
@@ -2088,3 +2180,131 @@ void ui_menu_sort_by_label(GtkMenu *menu)
 	g_list_free(list);
 }
 
+
+/* return value is for macros */
+GtkWidget *ui_label_set_markup(GtkLabel *label, const gchar *format, ...)
+{
+	va_list a;
+	gchar *text;
+
+	va_start(a, format);
+	text = g_strdup_vprintf(format, a);
+	va_end(a);
+
+	gtk_label_set_text(label, text);
+	gtk_label_set_use_markup(label, TRUE);
+	g_free(text);
+	return GTK_WIDGET(label);
+}
+
+
+GtkWidget *ui_label_new_bold(const gchar *text)
+{
+	GtkWidget *label;
+	gchar *label_text;
+
+	label_text = g_markup_escape_text(text, -1);
+	label = ui_label_set_markup(GTK_LABEL(gtk_label_new(NULL)), "<b>%s</b>", label_text);
+	g_free(label_text);
+	return label;
+}
+
+
+/** Adds a list of document items to @a menu.
+ * @param menu Menu.
+ * @param active Which document to highlight, or @c NULL.
+ * @param callback is used for each menu item's @c "activate" signal and will be passed
+ * the corresponding document pointer as @c user_data.
+ * @warning You should check @c doc->is_valid in the callback.
+ * @since 0.19 */
+void ui_menu_add_document_items(GtkMenu *menu, GeanyDocument *active, GCallback callback)
+{
+	GtkWidget *menu_item, *menu_item_label;
+	const GdkColor *color;
+	GeanyDocument *doc;
+	guint i, len;
+	gchar *base_name, *label;
+
+	len = gtk_notebook_get_n_pages(GTK_NOTEBOOK(main_widgets.notebook));
+	for (i = 0; i < len; i++)
+	{
+		doc = document_get_from_page(i);
+		if (! DOC_VALID(doc))
+			continue;
+
+		base_name = g_path_get_basename(DOC_FILENAME(doc));
+		menu_item = gtk_menu_item_new_with_label(base_name);
+		gtk_widget_show(menu_item);
+		gtk_container_add(GTK_CONTAINER(menu), menu_item);
+		g_signal_connect(menu_item, "activate", callback, doc);
+
+		color = document_get_status_color(doc);
+		menu_item_label = gtk_bin_get_child(GTK_BIN(menu_item));
+		gtk_widget_modify_fg(menu_item_label, GTK_STATE_NORMAL, color);
+		gtk_widget_modify_fg(menu_item_label, GTK_STATE_ACTIVE, color);
+
+		if (doc == active)
+		{
+			label = g_markup_escape_text(base_name, -1);
+			ui_label_set_markup(GTK_LABEL(menu_item_label), "<b>%s</b>", label);
+			g_free(label);
+		}
+
+		g_free(base_name);
+	}
+}
+
+
+/** Checks whether the passed @a keyval is the Enter or Return key.
+ * There are three different Enter/Return key values
+ * (@c GDK_Return, @c GDK_ISO_Enter, @c GDK_KP_Enter).
+ * This is just a convenience function.
+ * @param keyval A keyval.
+ * @return @c TRUE if @a keyval is the one of the Enter/Return key values, otherwise @c FALSE.
+ * @since 0.19 */
+gboolean ui_is_keyval_enter_or_return(guint keyval)
+{
+	return (keyval == GDK_Return || keyval == GDK_ISO_Enter|| keyval == GDK_KP_Enter);
+}
+
+
+/** Reads an integer from the GTK default settings registry
+ * (see http://library.gnome.org/devel/gtk/stable/GtkSettings.html).
+ * @param property_name The property to read.
+ * @param default_value The default value in case the value could not be read.
+ * @return The value for the property if it exists, otherwise the @a default_value.
+ * @since 0.19 */
+gint ui_get_gtk_settings_integer(const gchar *property_name, gint default_value)
+{
+	if (g_object_class_find_property(G_OBJECT_GET_CLASS(G_OBJECT(
+		gtk_settings_get_default())), property_name))
+	{
+		gint value;
+		g_object_get(G_OBJECT(gtk_settings_get_default()), property_name, &value, NULL);
+		return value;
+	}
+	else
+		return default_value;
+}
+
+
+void ui_editable_insert_text_callback(GtkEditable *editable, gchar *new_text,
+									  gint new_text_len, gint *position, gpointer data)
+{
+	gboolean stop_signal = FALSE;
+	const gchar c = *new_text;
+
+	/* allow inserting '+' and '-' as the first character */
+	if (position != NULL && *position == 0)
+	{
+		if (c != '+' && c != '-' && ! isdigit(c))
+			stop_signal = TRUE;
+	}
+	/* don't insert any text when it is not a digit */
+	else if (! isdigit(c))
+		stop_signal = TRUE;
+
+	if (stop_signal)
+		g_signal_stop_emission_by_name(editable, "insert-text");
+}
+
