@@ -1,8 +1,8 @@
 /*
  *      utils.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2005-2009 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2006-2009 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2005-2010 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2006-2010 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: utils.c 4014 2009-07-23 16:01:45Z ntrel $
+ * $Id: utils.c 4884 2010-05-08 13:10:33Z eht16 $
  */
 
 /*
@@ -139,14 +139,14 @@ gint utils_get_line_endings(const gchar* buffer, glong size)
 		}
 		else if (buffer[i] == 0x0d)
 		{
-			if (i >= (size-1))
+			if (i >= (size - 1))
 			{
 				/* Last char, CR */
 				cr++;
 			}
 			else
 			{
-				if (buffer[i+1] != 0x0a)
+				if (buffer[i + 1] != 0x0a)
 				{
 					/* CR */
 					cr++;
@@ -214,14 +214,14 @@ gboolean utils_is_opening_brace(gchar c, gboolean include_angles)
 
 
 /**
- *  Write the given @a text into a file with @a filename.
+ *  Writes the given @a text into a file with @a filename.
  *  If the file doesn't exist, it will be created.
  *  If it already exists, it will be overwritten.
  *
  *  @param filename The filename of the file to write, in locale encoding.
  *  @param text The text to write into the file.
  *
- *  @return 0 if the directory was successfully created, otherwise the @c errno of the
+ *  @return 0 if the file was successfully written, otherwise the @c errno of the
  *          failed operation is returned.
  **/
 gint utils_write_file(const gchar *filename, const gchar *text)
@@ -292,11 +292,25 @@ gchar *utils_find_open_xml_tag(const gchar sel[], gint size, gboolean check_tag)
 	else
 		cur = &sel[size - 1];
 
-	cur--; /* Skip past the > */
+	/* Skip to the character before the closing brace */
 	while (cur > begin)
 	{
-		if (*cur == '<') break;
-		else if (! check_tag && *cur == '>') break;
+		if (*cur == '>')
+			break;
+		--cur;
+	}
+	--cur;
+	/* skip whitespace */
+	while (cur > begin && isspace(*cur))
+		cur--;
+	if (*cur == '/')
+		return NULL; /* we found a short tag which doesn't need to be closed */
+	while (cur > begin)
+	{
+		if (*cur == '<')
+			break;
+		else if (! check_tag && *cur == '>')
+			break;
 		--cur;
 	}
 
@@ -508,7 +522,7 @@ gboolean utils_str_equal(const gchar *a, const gchar *b)
 
 
 /**
- *  Remove the extension from @a filename and return the result in a newly allocated string.
+ *  Removes the extension from @a filename and return the result in a newly allocated string.
  *
  *  @param filename The filename to operate on.
  *
@@ -641,7 +655,7 @@ gint utils_strpos(const gchar *haystack, const gchar *needle)
 			{
 				for (j = 1; (j < needle_length); j++)
 				{
-					if (haystack[i+j] == needle[j])
+					if (haystack[i + j] == needle[j])
 					{
 						if (pos == -1)
 							pos = i;
@@ -660,7 +674,7 @@ gint utils_strpos(const gchar *haystack, const gchar *needle)
 
 
 /**
- *  This is a convenience function to retrieve a formatted date/time string from strftime().
+ *  Retrieves a formatted date/time string from strftime().
  *  This function should be preferred to directly calling strftime() since this function
  *  works on UTF-8 encoded strings.
  *
@@ -729,7 +743,7 @@ gchar *utils_get_initials(const gchar *name)
 
 
 /**
- *  Convenience function for g_key_file_get_integer() to add a default value argument.
+ *  Wraps g_key_file_get_integer() to add a default value argument.
  *
  *  @param config A GKeyFile object.
  *  @param section The group name to look in for the key.
@@ -760,7 +774,7 @@ gint utils_get_setting_integer(GKeyFile *config, const gchar *section, const gch
 
 
 /**
- *  Convenience function for g_key_file_get_boolean() to add a default value argument.
+ *  Wraps g_key_file_get_boolean() to add a default value argument.
  *
  *  @param config A GKeyFile object.
  *  @param section The group name to look in for the key.
@@ -791,7 +805,7 @@ gboolean utils_get_setting_boolean(GKeyFile *config, const gchar *section, const
 
 
 /**
- *  Convenience function for g_key_file_get_string() to add a default value argument.
+ *  Wraps g_key_file_get_string() to add a default value argument.
  *
  *  @param config A GKeyFile object.
  *  @param section The group name to look in for the key.
@@ -806,15 +820,13 @@ gchar *utils_get_setting_string(GKeyFile *config, const gchar *section, const gc
 								const gchar *default_value)
 {
 	gchar *tmp;
-	GError *error = NULL;
 
 	if (G_UNLIKELY(config == NULL))
 		return g_strdup(default_value);
 
-	tmp = g_key_file_get_string(config, section, key, &error);
-	if (G_UNLIKELY(error))
+	tmp = g_key_file_get_string(config, section, key, NULL);
+	if (G_UNLIKELY(!tmp))
 	{
-		g_error_free(error);
 		return g_strdup(default_value);
 	}
 	return tmp;
@@ -913,7 +925,7 @@ gchar *utils_make_human_readable_str(guint64 size, gulong block_size,
 		{
 			f = fmt_tenths;
 			++u;
-			frac = ((((gint)(val % 1024)) * 10) + (1024/2)) / 1024;
+			frac = ((((gint)(val % 1024)) * 10) + (1024 / 2)) / 1024;
 			val /= 1024;
 		}
 		if (frac >= 10)
@@ -1047,8 +1059,9 @@ gchar **utils_read_file_in_array(const gchar *filename)
 
 
 /* Contributed by Stefan Oltmanns, thanks.
- * Replaces \\, \r, \n, \t and \uXXX by their real counterparts */
-gboolean utils_str_replace_escape(gchar *string)
+ * Replaces \\, \r, \n, \t and \uXXX by their real counterparts.
+ * keep_backslash is used for regex strings to leave '\\' and '\?' in place */
+gboolean utils_str_replace_escape(gchar *string, gboolean keep_backslash)
 {
 	gsize i, j, len;
 	guint unicodechar;
@@ -1068,6 +1081,8 @@ gboolean utils_str_replace_escape(gchar *string)
 			switch (string[i])
 			{
 				case '\\':
+					if (keep_backslash)
+						string[j++] = '\\';
 					string[j] = '\\';
 					break;
 				case 'n':
@@ -1086,57 +1101,58 @@ gboolean utils_str_replace_escape(gchar *string)
 					{
 						return FALSE;
 					}
-					if (isdigit(string[i-1])) string[j] = string[i-1]-48;
-					else if (isxdigit(string[i-1])) string[j] = tolower(string[i-1])-87;
+					if (isdigit(string[i - 1])) string[j] = string[i - 1] - 48;
+					else if (isxdigit(string[i - 1])) string[j] = tolower(string[i - 1])-87;
 					else return FALSE;
 					string[j] <<= 4;
-					if (isdigit(string[i])) string[j] |= string[i]-48;
+					if (isdigit(string[i])) string[j] |= string[i] - 48;
 					else if (isxdigit(string[i])) string[j] |= tolower(string[i])-87;
 					else return FALSE;
 					break;
 #endif
 				case 'u':
+				{
 					i += 2;
 					if (i >= strlen(string))
 					{
 						return FALSE;
 					}
-					if (isdigit(string[i-1])) unicodechar = string[i-1]-48;
-					else if (isxdigit(string[i-1])) unicodechar = tolower(string[i-1])-87;
+					if (isdigit(string[i - 1])) unicodechar = string[i - 1] - 48;
+					else if (isxdigit(string[i - 1])) unicodechar = tolower(string[i - 1])-87;
 					else return FALSE;
 					unicodechar <<= 4;
-					if (isdigit(string[i])) unicodechar |= string[i]-48;
+					if (isdigit(string[i])) unicodechar |= string[i] - 48;
 					else if (isxdigit(string[i])) unicodechar |= tolower(string[i])-87;
 					else return FALSE;
-					if (((i+2) < strlen(string)) && (isdigit(string[i+1]) || isxdigit(string[i+1]))
-						&& (isdigit(string[i+2]) || isxdigit(string[i+2])))
+					if (((i + 2) < strlen(string)) && (isdigit(string[i + 1]) || isxdigit(string[i + 1]))
+						&& (isdigit(string[i + 2]) || isxdigit(string[i + 2])))
 					{
 						i += 2;
 						unicodechar <<= 8;
-						if (isdigit(string[i-1])) unicodechar |= ((string[i-1]-48)<<4);
-						else unicodechar |= ((tolower(string[i-1])-87) << 4);
-						if (isdigit(string[i])) unicodechar |= string[i]-48;
+						if (isdigit(string[i - 1])) unicodechar |= ((string[i - 1] - 48) << 4);
+						else unicodechar |= ((tolower(string[i - 1])-87) << 4);
+						if (isdigit(string[i])) unicodechar |= string[i] - 48;
 						else unicodechar |= tolower(string[i])-87;
 					}
-					if (((i+2) < strlen(string)) && (isdigit(string[i+1]) || isxdigit(string[i+1]))
-						&& (isdigit(string[i+2]) || isxdigit(string[i+2])))
+					if (((i + 2) < strlen(string)) && (isdigit(string[i + 1]) || isxdigit(string[i + 1]))
+						&& (isdigit(string[i + 2]) || isxdigit(string[i + 2])))
 					{
 						i += 2;
 						unicodechar <<= 8;
-						if (isdigit(string[i-1])) unicodechar |= ((string[i-1]-48) << 4);
-						else unicodechar |= ((tolower(string[i-1])-87) << 4);
-						if (isdigit(string[i])) unicodechar |= string[i]-48;
+						if (isdigit(string[i - 1])) unicodechar |= ((string[i - 1] - 48) << 4);
+						else unicodechar |= ((tolower(string[i - 1])-87) << 4);
+						if (isdigit(string[i])) unicodechar |= string[i] - 48;
 						else unicodechar |= tolower(string[i])-87;
 					}
-					if(unicodechar < 0x80)
+					if (unicodechar < 0x80)
 					{
 						string[j] = unicodechar;
 					}
 					else if (unicodechar < 0x800)
 					{
-						string[j] = (unsigned char) ((unicodechar >> 6)| 0xC0);
+						string[j] = (unsigned char) ((unicodechar >> 6) | 0xC0);
 						j++;
-						string[j] = (unsigned char) ((unicodechar & 0x3F)| 0x80);
+						string[j] = (unsigned char) ((unicodechar & 0x3F) | 0x80);
 					}
 					else if (unicodechar < 0x10000)
 					{
@@ -1161,8 +1177,12 @@ gboolean utils_str_replace_escape(gchar *string)
 						return FALSE;
 					}
 					break;
+				}
 				default:
-					return FALSE;
+					/* unnecessary escapes are allowed */
+					if (keep_backslash)
+						string[j++] = '\\';
+					string[j] = string[i];
 			}
 		}
 		else
@@ -1316,8 +1336,8 @@ gchar **utils_strv_new(const gchar *first, ...)
 
 
 /**
- *  Create a directory if it doesn't already exist.
- *  Create intermediate parent directories as needed, too.
+ *  Creates a directory if it doesn't already exist.
+ *  Creates intermediate parent directories as needed, too.
  *  The permissions of the created directory are set 0700.
  *
  *  @param path The path of the directory to create, in locale encoding.
@@ -1342,47 +1362,76 @@ gint utils_mkdir(const gchar *path, gboolean create_parent_dirs)
 
 
 /**
- *  Gets a sorted list of files from the specified directory.
- *  Locale encoding is expected for path and used for the file list. The list and the data
- *  in the list should be freed after use.
+ * Gets a list of files from the specified directory.
+ * Locale encoding is expected for @a path and used for the file list. The list and the data
+ * in the list should be freed after use, e.g.:
+ * @code
+ * g_slist_foreach(list, (GFunc) g_free, NULL);
+ * g_slist_free(list); @endcode
  *
- *  @param path The path of the directory to scan, in locale encoding.
- *  @param length The location to store the number of non-@c NULL data items in the list,
- *                unless @c NULL.
- *  @param error The is the location for storing a possible error, or @c NULL.
+ * @note If you don't need a list you should use the foreach_dir() macro instead -
+ * it's more efficient.
  *
- *  @return A newly allocated list or @c NULL if no files found. The list and its data should be
- *          freed when no longer needed.
+ * @param path The path of the directory to scan, in locale encoding.
+ * @param full_path Whether to include the full path for each filename in the list. Obviously this
+ * will use more memory.
+ * @param sort Whether to sort alphabetically (UTF-8 safe).
+ * @param error The location for storing a possible error, or @c NULL.
+ *
+ * @return A newly allocated list or @c NULL if no files were found. The list and its data should be
+ * freed when no longer needed.
+ * @see utils_get_file_list().
  **/
-GSList *utils_get_file_list(const gchar *path, guint *length, GError **error)
+GSList *utils_get_file_list_full(const gchar *path, gboolean full_path, gboolean sort, GError **error)
 {
 	GSList *list = NULL;
-	guint len = 0;
 	GDir *dir;
+	const gchar *filename;
 
 	if (error)
 		*error = NULL;
-	if (length)
-		*length = 0;
 	g_return_val_if_fail(path != NULL, NULL);
 
 	dir = g_dir_open(path, 0, error);
 	if (dir == NULL)
 		return NULL;
 
-	while (1)
+	foreach_dir(filename, dir)
 	{
-		const gchar *filename = g_dir_read_name(dir);
-		if (filename == NULL)
-			break;
-
-		list = g_slist_insert_sorted(list, g_strdup(filename), (GCompareFunc) utils_str_casecmp);
-		len++;
+		list = g_slist_append(list, full_path ?
+			g_build_path(G_DIR_SEPARATOR_S, path, filename, NULL) : g_strdup(filename));
 	}
 	g_dir_close(dir);
+	/* sorting last is quicker than on insertion */
+	if (sort)
+		list = g_slist_sort(list, (GCompareFunc) utils_str_casecmp);
+	return list;
+}
+
+
+/**
+ * Gets a sorted list of files from the specified directory.
+ * Locale encoding is expected for @a path and used for the file list. The list and the data
+ * in the list should be freed after use, e.g.:
+ * @code
+ * g_slist_foreach(list, (GFunc) g_free, NULL);
+ * g_slist_free(list); @endcode
+ *
+ * @param path The path of the directory to scan, in locale encoding.
+ * @param length The location to store the number of non-@c NULL data items in the list,
+ *               unless @c NULL.
+ * @param error The location for storing a possible error, or @c NULL.
+ *
+ * @return A newly allocated list or @c NULL if no files were found. The list and its data should be
+ *         freed when no longer needed.
+ * @see utils_get_file_list_full().
+ **/
+GSList *utils_get_file_list(const gchar *path, guint *length, GError **error)
+{
+	GSList *list = utils_get_file_list_full(path, FALSE, TRUE, error);
 
 	if (length)
-		*length = len;
+		*length = g_slist_length(list);
 	return list;
 }
 
@@ -1453,7 +1502,7 @@ static guint utils_string_replace_helper(GString *haystack, const gchar *needle,
  * @param needle The string which should be replaced.
  * @param replace The replacement for @a needle.
  *
- * @return amount of replacements done
+ * @return Number of replacements made.
  **/
 guint utils_string_replace_all(GString *haystack, const gchar *needle, const gchar *replace)
 {
@@ -1462,7 +1511,7 @@ guint utils_string_replace_all(GString *haystack, const gchar *needle, const gch
 
 
 /**
- * Convenience function to replace only the first occurrence of @a needle in @a haystack
+ * Replaces only the first occurrence of @a needle in @a haystack
  * with @a replace.
  * For details, see utils_string_replace_all().
  *
@@ -1470,7 +1519,7 @@ guint utils_string_replace_all(GString *haystack, const gchar *needle, const gch
  * @param needle The string which should be replaced.
  * @param replace The replacement for @a needle.
  *
- * @return amount of replacements done
+ * @return Number of replacements made.
  *
  *  @since 0.16
  */
@@ -1514,7 +1563,7 @@ static gboolean check_error(GError **error)
 
 
 /**
- *  This is a wrapper function for g_spawn_sync() and internally calls this function on Unix-like
+ *  Wraps g_spawn_sync() and internally calls this function on Unix-like
  *  systems. On Win32 platforms, it uses the Windows API.
  *
  *  @param dir The child's current working directory, or @a NULL to inherit parent's.
@@ -1562,7 +1611,7 @@ gboolean utils_spawn_sync(const gchar *dir, gchar **argv, gchar **env, GSpawnFla
 
 
 /**
- *  This is a wrapper function for g_spawn_async() and internally calls this function on Unix-like
+ *  Wraps g_spawn_async() and internally calls this function on Unix-like
  *  systems. On Win32 platforms, it uses the Windows API.
  *
  *  @param dir The child's current working directory, or @a NULL to inherit parent's.
@@ -1719,15 +1768,22 @@ void utils_tidy_path(gchar *filename)
 	const gchar *c, *needle;
 	gchar *tmp;
 	gssize pos;
+	gboolean preserve_double_backslash = FALSE;
 
 	g_return_if_fail(g_path_is_absolute(filename));
 
+	if (str->len >= 2 && strncmp(str->str, "\\\\", 2) == 0)
+		preserve_double_backslash = TRUE;
+
 	/* replace "/./" and "//" */
 	utils_string_replace_all(str, G_DIR_SEPARATOR_S "." G_DIR_SEPARATOR_S, G_DIR_SEPARATOR_S);
 	utils_string_replace_all(str, G_DIR_SEPARATOR_S G_DIR_SEPARATOR_S, G_DIR_SEPARATOR_S);
 
-	/* replace "/.." */
-	needle = G_DIR_SEPARATOR_S "..";
+	if (preserve_double_backslash)
+		g_string_prepend(str, "\\");
+
+	/* replace "/../" */
+	needle = G_DIR_SEPARATOR_S ".." G_DIR_SEPARATOR_S;
 	while (1)
 	{
 		c = strstr(str->str, needle);
@@ -1739,9 +1795,11 @@ void utils_tidy_path(gchar *filename)
 			if (pos <= 3)
 				break;	/* bad path */
 
-			g_string_erase(str, pos, strlen(needle));	/* erase "/.." */
+			/* replace "/../" */
+			g_string_erase(str, pos, strlen(needle));
+			g_string_insert_c(str, pos, G_DIR_SEPARATOR);
 
-			tmp = g_strndup(str->str, pos);	/* path up to "/.." */
+			tmp = g_strndup(str->str, pos);	/* path up to "/../" */
 			c = g_strrstr(tmp, G_DIR_SEPARATOR_S);
 			g_return_if_fail(c);
 
@@ -1756,12 +1814,8 @@ void utils_tidy_path(gchar *filename)
 }
 
 
-/* @warning Doesn't include null terminating character. */
-#define foreach_str(char_ptr, string) \
-	for (char_ptr = string; *char_ptr; char_ptr++)
-
 /**
- *  Replace or remove characters from a string in place.
+ *  Removes characters from a string, in place.
  *
  *  @param string String to search.
  *  @param chars Characters to remove.
@@ -1790,3 +1844,166 @@ gchar *utils_str_remove_chars(gchar *string, const gchar *chars)
 }
 
 
+static void utils_slist_remove_next(GSList *node)
+{
+	GSList *old = node->next;
+
+	g_return_if_fail(old);
+
+	node->next = old->next;
+	g_slist_free_1(old);
+}
+
+
+/* Gets list of sorted filenames with no path and no duplicates from user and system config */
+GSList *utils_get_config_files(const gchar *subdir)
+{
+	gchar *path = g_build_path(G_DIR_SEPARATOR_S, app->configdir, subdir, NULL);
+	GSList *list = utils_get_file_list_full(path, FALSE, FALSE, NULL);
+	GSList *syslist, *node;
+
+	if (!list)
+	{
+		utils_mkdir(path, FALSE);
+	}
+	setptr(path, g_build_path(G_DIR_SEPARATOR_S, app->datadir, subdir, NULL));
+	syslist = utils_get_file_list_full(path, FALSE, FALSE, NULL);
+	/* merge lists */
+	list = g_slist_concat(list, syslist);
+
+	list = g_slist_sort(list, (GCompareFunc) utils_str_casecmp);
+	/* remove duplicates (next to each other after sorting) */
+	foreach_slist(node, list)
+	{
+		if (node->next && utils_str_equal(node->next->data, node->data))
+		{
+			g_free(node->next->data);
+			utils_slist_remove_next(node);
+		}
+	}
+	g_free(path);
+	return list;
+}
+
+
+/* Suffix can be NULL or a string which should be appended to the Help URL like
+ * an anchor link, e.g. "#some_anchor". */
+gchar *utils_get_help_url(const gchar *suffix)
+{
+	gint skip;
+	gchar *uri;
+
+#ifdef G_OS_WIN32
+	skip = 8;
+	uri = g_strconcat("file:///", app->docdir, "/Manual.html", NULL);
+	g_strdelimit(uri, "\\", '/'); /* replace '\\' by '/' */
+#else
+	skip = 7;
+	uri = g_strconcat("file://", app->docdir, "index.html", NULL);
+#endif
+
+	if (! g_file_test(uri + skip, G_FILE_TEST_IS_REGULAR))
+	{	/* fall back to online documentation if it is not found on the hard disk */
+		g_free(uri);
+		uri = g_strconcat(GEANY_HOMEPAGE, "manual/", VERSION, "/index.html", NULL);
+	}
+
+	if (suffix != NULL)
+	{
+		setptr(uri, g_strconcat(uri, suffix, NULL));
+	}
+
+	return uri;
+}
+
+
+static gboolean str_in_array(const gchar **haystack, const gchar *needle)
+{
+	const gchar **p;
+
+	for (p = haystack; *p != NULL; ++p)
+	{
+		if (utils_str_equal(*p, needle))
+			return TRUE;
+	}
+	return FALSE;
+}
+
+
+/**
+ * Copies the current environment into a new array.
+ * @a exclude_vars is a @c NULL-terminated array of variable names which should be not copied.
+ * All further arguments are key, value pairs of variables which should be added to
+ * the environment.
+ *
+ * The argument list must be @c NULL-terminated.
+ *
+ * @param exclude_vars @c NULL-terminated array of variable names to exclude.
+ * @param first_varname Name of the first variable to copy into the new array.
+ * @param ... Key-value pairs of variable names and values, @c NULL-terminated.
+ *
+ * @return The new environment array.
+ **/
+gchar **utils_copy_environment(const gchar **exclude_vars, const gchar *first_varname, ...)
+{
+	gchar **result;
+	gchar **p;
+	gchar **env;
+	va_list args;
+	const gchar *key, *value;
+	guint n, o;
+
+	/* get all the environ variables */
+	env = g_listenv();
+
+	/* count the additional variables */
+	va_start(args, first_varname);
+	for (o = 1; va_arg(args, gchar*) != NULL; o++);
+	va_end(args);
+	/* the passed arguments should be even (key, value pairs) */
+	g_return_val_if_fail(o % 2 == 0, NULL);
+
+	o /= 2;
+
+	/* create an array large enough to hold the new environment */
+	n = g_strv_length(env);
+	/* 'n + o + 1' could leak a little bit when exclude_vars is set */
+	result = g_new(gchar *, n + o + 1);
+
+	/* copy the environment */
+	for (n = 0, p = env; *p != NULL; ++p)
+	{
+		/* copy the variable */
+		value = g_getenv(*p);
+		if (G_LIKELY(value != NULL))
+		{
+			/* skip excluded variables */
+			if (exclude_vars != NULL && str_in_array(exclude_vars, *p))
+				continue;
+
+			result[n++] = g_strconcat(*p, "=", value, NULL);
+		}
+	}
+	g_strfreev(env);
+
+	/* now add additional variables */
+	va_start(args, first_varname);
+	key = first_varname;
+	value = va_arg(args, gchar*);
+	while (key != NULL)
+	{
+		result[n++] = g_strconcat(key, "=", value, NULL);
+
+		key = va_arg(args, gchar*);
+		if (key == NULL)
+			break;
+		value = va_arg(args, gchar*);
+	}
+	va_end(args);
+
+	result[n] = NULL;
+
+	return result;
+}
+
+
