@@ -1,8 +1,8 @@
 /*
  *      vte.h - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2005-2009 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2006-2009 Nick Treleaven <<nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2005-2010 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2006-2010 Nick Treleaven <<nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: vte.h 3545 2009-02-04 19:52:44Z eht16 $
+ * $Id: vte.h 4518 2010-01-01 22:55:18Z eht16 $
  */
 
 
@@ -52,6 +52,7 @@ typedef struct
 	gboolean skip_run_script;
 	gboolean enable_bash_keys;
 	gboolean cursor_blinks;
+	gboolean send_selection_unsafe;
 	gint scrollback_lines;
 	gchar *emulation;
 	gchar *shell;
@@ -76,6 +77,7 @@ void vte_cwd(const gchar *filename, gboolean force);
 
 void vte_append_preferences_tab(void);
 
+void vte_send_selection_to_vte(void);
 
 #endif
 
