@@ -1,5 +1,5 @@
 /*
-*   $Id: make.c 2497 2008-04-17 18:06:31Z eht16 $
+*   $Id: make.c 3441 2009-01-03 16:22:28Z eht16 $
 *
 *   Copyright (c) 2000-2005, Darren Hiebert
 *
@@ -110,8 +110,12 @@ static void newMacro (vString *const name)
 static void newMacroFromDefine (vString *const name)
 {
 	/* name is something like "define JAVAHPP_RULE", find the space and jump to the next char */
-	vStringCopyS (name, strchr (vStringValue (name), ' ') + 1);
-	makeSimpleTag (name, MakeKinds, K_MACRO);
+	char *name_val = strchr (vStringValue (name), ' ');
+
+	if (name_val != NULL) {
+		vStringCopyS (name, name_val + 1);
+		makeSimpleTag (name, MakeKinds, K_MACRO);
+	}
 }
 
 static void readIdentifier (const int first, vString *const id)
