@@ -51,7 +51,11 @@
     ValaParser, \
     ActionScriptParser, \
     NsisParser, \
-    MarkdownParser
+    MarkdownParser, \
+    Txt2tagsParser, \
+    AbcParser, \
+    VerilogParser, \
+    RParser
 /*
 langType of each parser
  0	CParser
@@ -91,6 +95,10 @@ langType of each parser
 34  ActionScriptParser
 35  NsisParser
 36  MarkdownParser
+37  Txt2tagsParser
+38  AbcParser
+39  Verilog
+40	RParser
 */
 #endif	/* _PARSERS_H */
 
