@@ -147,14 +147,17 @@ gboolean tm_workspace_load_global_tags(const char *tags_file, gint mode)
 	TMTag *tag;
 	gboolean format_pipe = FALSE;
 
-	if (NULL == (fp = g_fopen(tags_file, "r")))
-		return FALSE;
 	if (NULL == theWorkspace)
 		return FALSE;
+	if (NULL == (fp = g_fopen(tags_file, "r")))
+		return FALSE;
 	if (NULL == theWorkspace->global_tags)
 		theWorkspace->global_tags = g_ptr_array_new();
 	if ((NULL == fgets((gchar*) buf, BUFSIZ, fp)) || ('\0' == *buf))
+	{
+		fclose(fp);
 		return FALSE; /* early out on error */
+	}
 	else
 	{	/* We read the first line for the format specification. */
 		if (buf[0] == '#' && strstr((gchar*) buf, "format=pipe") != NULL)
@@ -223,7 +226,7 @@ static void write_includes_file(FILE *fp, GList *includes_files)
 		size_t size;
 
 		size = fwrite(str, str_len, 1, fp);
-		free(str);
+		g_free(str);
 		node = g_list_next (node);
 	}
 }
@@ -281,7 +284,11 @@ gboolean tm_workspace_create_global_tags(const char *config_dir, const char *pre
 #endif
 
 	if (NULL == theWorkspace || NULL == (fp = g_fopen(temp_file, "w")))
+	{
+		g_free(temp_file);
+		g_free(temp_file2);
 		return FALSE;
+	}
 
 	includes_files_hash = g_hash_table_new_full (tm_file_inode_hash,
 												 g_direct_equal,
@@ -295,6 +302,7 @@ gboolean tm_workspace_create_global_tags(const char *config_dir, const char *pre
 	{
  		int dirty_len = strlen(includes[idx_inc]);
 		char *clean_path = g_malloc(dirty_len - 1);
+
 		strncpy(clean_path, includes[idx_inc] + 1, dirty_len - 1);
 		clean_path[dirty_len - 2] = 0;
 
@@ -324,7 +332,7 @@ gboolean tm_workspace_create_global_tags(const char *config_dir, const char *pre
 			}
 		}
 		globfree(&globbuf);
-		free(clean_path);
+		g_free(clean_path);
   	}
   	else
 #endif
